c***********************************************************************
c The loop functions used for the NLO SM and charged Higgs 
c contributions taken from hep-ph/0112305
c***********************************************************************

      Double Precision Function BBKU_f01(x)

c Equation (12) of Misiak and Urban.

      Implicit NONE

c Inputs
      Double Precision x

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f01=3.D0/4.D0
      Else
         BBKU_f01=(3.D0*x**2+2.D0*x)/(2.D0*(x-1.D0)**2)*Log(x)
     &        +(x**2-6.D0*x)/(2.D0*(x-1.D0))
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f02(x)

      Implicit NONE

c Inputs
      Double Precision x

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f02=-1.D0/2.D0
      Else
         BBKU_f02=x/((x-1.D0)**2)*LOG(x)
     &        -x/(x-1.D0)
      End If

      Return
      End

c************************************************************************

      Double Precision Function BBKU_f02pr(x)

      Implicit NONE
      
c Inputs
      Double Precision x

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f02pr=-1.D0/6.D0
      Else
         BBKU_f02pr=-(x+1.D0)/((x-1.D0)**3)*Log(x)
     &        +2.D0/((x-1.D0)**2)
      End If

      Return
      End

c************************************************************************

      Double Precision Function BBKU_f01Pr(x)

      Implicit NONE

c Inputs
      Double Precision x

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f01pr=5.D0/6.D0
      Else
         BBKU_f01pr=-(4.D0*x+1.D0)/((x-1.D0)**3)*Log(x)
     &        +(x**3+7.D0*x-8.D0)/(2.D0*(x-1.D0)**3)
      End If

      Return
      End

c*************************************************************************

      Double Precision Function BBKU_f11(x)

      Implicit NONE

c Inputs
      Double Precision x,tmp
      Double Complex z

c Functions
      Double Complex DiLog

      z=DCMPLX(x,0.D0)

      tmp=DBLE(DiLog(1.D0-1.D0/z))

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f11=35.D0/9.D0
      Else
         BBKU_f11=-4.D0*x*(4.D0+x**2)/((x-1.D0)**2)*tmp
     &        -4.D0*x*(23.D0+14.D0*x+3.D0*x**2)/(3.D0*(x-1.D0)**3)
     &        *Log(x)
     &        +4.D0*x*(29.D0+7.D0*x+4.D0*x**2)/(3.D0*(x-1.D0)**2)
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f110(x)

      Implicit NONE

c Inputs
      Double Precision x,tmp
      Double Complex z

c Functions
      Double Complex DiLog

      z=DCMPLX(x,0.D0)

      tmp=DBLE(DiLog(1.D0-1.D0/z))

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f110=-16.D0/9.D0
      Else
         BBKU_f110=-(8.D0*x)/((x-1.D0)**2)*tmp
     &        -4.D0*x*(17.D0-x)/(3.D0*(x-1.D0)**3)*Log(x)
     &        +4.D0*x*(19.D0-3.D0*x)/(3.D0*(x-1.D0)**2)
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f114(x,y)

      Implicit NONE

c Inputs
      Double Precision x,y

c Local
      Double Complex z1,z2
      Double Precision tmp1,tmp2

c Functions
      Double Complex DiLog

      z1=DCMPLX(1.D0/x,0.D0)
      z2=DCMPLX(y/x,0.D0)

      tmp1=DBLE(DiLog(1.D0-z1))
      tmp2=DBLE(DiLog(1.D0-z2))

      If (ABS(x-1.D0).lt.1.D-3.and.ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f114=-4.D0/9.D0
      Else If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f114=8.D0*(-9.D0*y**2+14.D0*y-5.D0)/(3.D0*(y-1.D0)**3)
     &        +8.D0*y*(7.D0*y-3.D0)/(3.D0*(y-1.D0)**3)*LOG(y)
     &        +8.D0/(y-1.D0)*tmp2
      Else If (ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f114=8.D0*x*(x+7.D0)/(3.D0*(x-1.D0)**2)
     &        +8.D0*(-5.D0*x-3.D0)*x/(3.D0*(x-1.D0)**3)*LOG(x)
      Else If (ABS(x-y).lt.1.D-3) Then
         BBKU_f114=8.D0*x*(5.D0*x-9.D0)/(3.D0*(x-1.D0)**2)
     &        +8.D0*(7.D0-3.D0*x)/(3.D0*(x-1.D0)**3)*LOG(x)
     &        -8.D0*x/(x-1.D0)*tmp1
      Else
         BBKU_f114=32.D0*x**2/(3.D0*(x-1.D0)*(x-y))
     &        -8.D0*x**2*(7.D0*x*(1.D0+y)-11.D0*y-3.D0*x**2)
     &        /(3.D0*(x-1.D0)**2*(x-y)**2)*LOG(x)
     &        -8.D0*x*y*(3.D0*x-7.D0*y)/(3.D0*(x-y)**2*(y-1.D0))*LOG(y)
     &        -8.D0*x/(y-1.D0)*tmp1+8.D0*x/(y-1.D0)*tmp2
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f03pr(x,y)

c The derivative w.r.t x of f^(0)_3

      Implicit NONE

c Inputs
      Double Precision x,y

      If (ABS(x-1.D0).lt.1.D-3.and.ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f03pr=-1.D0/6.D0
      Else If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f03pr=(1.D0-y**2)/(2.D0*(y-1.D0)**3)+y/(y-1.D0)**3*LOG(y)
      Else If (ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f03pr=2.D0/((x-1.D0)**2)-(x+1.D0)/((x-1.D0)**3)*LOG(x)
      Else If (ABS(x-y).lt.1.D-3) Then
         BBKU_f03pr=-(x+1.D0)/(2.D0*(x-1.D0)**2*x)
     &        +1.D0/((x-1.D0)**3)*LOG(x)
      Else
         BBKU_f03pr=(y-x**2)/((x-1.D0)**2*(x-y)**2)*LOG(x)
     &        +y/((x-y)**2*(y-1.D0))*LOG(y)+1.D0/((x-1.D0)*(x-y))
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f07pr(x,y)

c The derivative w.r.t x of f^(0)_7

      Implicit NONE

c Inputs
      Double Precision x,y

      If (ABS(x-1.D0).lt.1.D-3.and.ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f07pr=-1.D0/6.D0
      Else If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f07pr=(1.D0-y**2)/(2.D0*(y-1.D0)**3)+y/(y-1.D0)**3*LOG(y)
      Else If (ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f07pr=2.D0/((x-1.D0)**2)-(x+1.D0)/((x-1.D0)**3)*LOG(x)
      Else If (ABS(x-y).lt.1.D-3) Then
         BBKU_f07pr=-(x+1.D0)/(2.D0*(x-1.D0)**2*x)
     &        +1.D0/((x-1.D0)**3)*LOG(x)
      Else
         BBKU_f07pr=(y-x**2)/((x-1.D0)**2*(x-y)**2)*LOG(x)
     &        +y/((x-y)**2*(y-1.D0))*LOG(y)+1.D0/((x-1.D0)*(x-y))
      End If

      Return
      End
c***********************************************************************

      Double Precision Function BBKU_f12(x)

      Implicit NONE

c Inputs
      Double Precision x

c Local
      Double Complex z1
      Double Precision tmp1

c Functions
      Double Complex DiLog

      z1=DCMPLX(1.D0/x,0.D0)

      tmp1=DBLE(DiLog(1.D0-z1))

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f12=26.D0/9.D0
      Else
         BBKU_f12=32.D0*x*(3.D0-x)/(3.D0*(x-1.D0)**2)
     &        -8.D0*x*(11.D0-3.D0*x)/(3.D0*(x-1.D0)**3)*LOG(x)
     &        -8.D0*x*(2.D0-x)/((x-1.D0)**2)*tmp1
      End If
      
      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f111(x,y)

      Implicit NONE

c Inputs
      Double Precision x,y

c Local
      Double Complex z1,z2
      Double Precision tmp1,tmp2

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

c Functions
      Double Complex DiLog

      z1=DCMPLX(1.D0/x,0.D0)
      z2=DCMPLX(y/x,0.D0)

      tmp1=DBLE(DiLog(1.D0-z1))
      tmp2=DBLE(DiLog(1.D0-z2))

      If (ABS(x-1.D0).lt.1.D-3.and.ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f111=4.D0/9.D0*(-31.D0+3.D0*PI**2)
      Else If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f111=4.D0*(PI**2*(y-1.D0)**2+2.D0*(1.D0-5.D0*y)*y)
     &        /(3.D0*(y-1.D0)**2*y)
     &        +8.D0*(7.D0*y-3.D0)/(3.D0*(y-1.D0)**3)
     &        +8.D0/((y-1.D0)*y)*tmp2
      Else If (ABS(y-1.D0).lt.1.D-3) Then
         BBKU_f111=4.D0*(PI**2*(x-1.D0)**2-6.D0*x+22.D0)*x
     &        /(3.D0*(x-1.D0)**2)
     &        +(-24.D0*x**3+40.D0*x**2-80.D0*x)
     &        /(3.D0*(x-1.D0)**3)*LOG(x)
     &        -8.D0*tmp1*x
      Else If (ABS(y-x).lt.1.D-3) Then
         BBKU_f111=4.D0*(PI**2*(x-1.D0)**2+2.D0*(x-5.D0))
     &        /(3.D0*(x-1.D0)**2)
     &        +8.D0*x*(7.D0-3.D0*x)/(3.D0*(x-1.D0)**3)*LOG(x)
     &        -8.D0*x/(x-1.D0)*tmp1
      Else
         BBKU_f111=4.D0*x*(8.D0*y+(x-1.D0)*(x-y)*PI**2)
     &        /(3.D0*y*(x-1.D0)*(x-y))
     &        -8.D0*x*(x**2-7.D0*y+3.D0*x*(1.D0+y))
     &        /(3.D0*(x-y)**2*(x-1.D0)**2)*LOG(x)
     &        -8.D0*x*(3.D0*x-7.D0*y)/(3.D0*(x-y)**2*(y-1.D0))*LOG(y)
     &        -8.D0*x/(y-1.D0)*tmp1
     &        -8.D0*x/(x*(x-1.D0))*tmp2
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BBKU_f17(x)

      Implicit NONE

c Inputs
      Double Precision x

c Local
      Double Complex z1
      Double Precision tmp1

c Functions
      Double Complex DiLog

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

      z1=DCMPLX(1.D0/x,0.D0)

      tmp1=DBLE(DiLog(1.D0-z1))

      If (ABS(x-1.D0).lt.1.D-3) Then
         BBKU_f17=4.D0/9.D0*(-52.D0+3.D0*PI**2)
      Else
         BBKU_f17=4.D0*x*(27.D0-11.D0*x+(x-1.D0)**2*PI**2)
     &        /(3.D0*(x-1.D0)**2)
     &        -4.D0*x*(37.D0-33.D0*x+12.D0*x**2)
     &        /(3.D0*(x-1.D0)**3)*LOG(x)
     &        -8.D0*x*(2.D0-2.D0*x+x**2)/((x-1.D0)**2)*tmp1
      End If
      Return
      End
