c**********************************************************************
c The Passarino-Veltman functions B0,B1,C0,C00,C11,C12,D0,D00 in the
c limit of vanishing external momenta.
c NOTE - I use Mathematica to calculate the various limits.
c I use the notation of LoopTools / FeynCalc when defining Loop
c integrals (see the LoopTools manual for details). With the following
c exception :
c C_{\mu}=p1_{\mu} C11+ p2_{\mu} C12
c As opposed to :
c C_{\mu}=p1_{\mu} C1 + (p1+p2)_{\mu} C2
c**********************************************************************

      Double Precision Function PV_B0(xtmp,ytmp,mu)

c The Passarino-Veltman function B0.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,mu

c Local
      Double Precision x,y

      x=xtmp**2
      y=ytmp**2

      If (x.eq.y) Then
         PV_B0=-LOG(x/mu**2)
      Else If (x.eq.0.D0) Then
         PV_B0=-Log(y/mu**2)+1.D0
      Else If (y.eq.0.D0) Then
         PV_B0=-Log(x/mu**2)+1.D0
      Else
         PV_B0=-LOG(x/mu**2)+1.D0+y/(x-y)*LOG(y/x)
      End If

      Return
      End

c**********************************************************************

      Double Precision Function PV_B1(xtmp,ytmp,mu)

c The Passarino-Veltman function B1.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,mu

c Local
      Double Precision x,y

      x=xtmp**2
      y=ytmp**2

      If (x.eq.y) Then
         PV_B1=LOG(x/mu**2)/2.D0
      Else If (y.eq.0.D0) Then
         PV_B1=LOG(x/mu**2)/2.D0-3.D0/4.D0
      Else If (x.eq.0.D0) Then
         PV_B1=LOG(y/mu**2)/2.D0-1.D0/4.D0
      Else
         PV_B1=LOG(x/mu**2)/2.D0-1.D0/4.D0
     &        +(y**2-2.D0*x*y)/(2.D0*(x-y)**2)*LOG(y/x)
     &        -x/(2.D0*(x-y))
      End If
      
      Return
      End

c**********************************************************************

      Double Precision Function PV_C0(xtmp,ytmp,ztmp)

c The Passarino-Veltman function C0.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,ztmp

c Local
      Double Precision x,y,z

      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (x.eq.y.and.y.eq.z) Then
         PV_C0=-1.D0/(2.D0*x)
      Else If (x.eq.0.D0) Then
         If (y.eq.z) Then
            PV_C0=-1.D0/y
         Else
            PV_C0=LOG(z/y)/(y-z)
         End If
      Else If (y.eq.0.D0) Then
         If (x.eq.z) Then
            PV_C0=-1.D0/x
         Else
            PV_C0=LOG(z/x)/(x-z)
         End If
      Else If (z.eq.0.D0) Then
         If (x.eq.y) Then
            PV_C0=-1.D0/x
         Else
            PV_C0=LOG(y/x)/(x-y)
         End If
      Else If (x.eq.y) Then
         PV_C0=(-x+z-z*LOG(z/x))/(x-z)**2
      Else If (x.eq.z) Then
         PV_C0=(-x+y-y*LOG(y/x))/(x-y)**2
      Else If (y.eq.z) Then
         PV_C0=(-y+x+x*Log(y/x))/(x-y)**2
      Else
         PV_C0=y/((x-y)*(y-z))*LOG(y/x)+z/((x-z)*(z-y))*LOG(z/x)
      End If

      Return
      End

c**********************************************************************

      Double Precision Function PV_C00(xtmp,ytmp,ztmp,mu)

c The Passarino-Veltman function C00.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,ztmp,mu

c Local
      Double Precision x,y,z

      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (x.eq.y.and.y.eq.z) Then
         PV_C00=-LOG(x/mu**2)
      Else If (x.eq.0.D0) Then
         If (y.eq.z) Then
            PV_C00=1.D0/2.D0-LOG(y/mu**2)
         Else
            PV_C00=-LOG(y/mu**2)+z*Log(z/y)/(y-z)+3.D0/2.D0
         End If
      Else If (y.eq.0.D0) Then
         If (x.eq.z) Then
            PV_C00=1.D0/2.D0-LOG(x/mu**2)
         Else
            PV_C00=-LOG(x/mu**2)+z*Log(z/x)/(x-z)+3.D0/2.D0
         End If
      Else If (z.eq.0.D0) Then
         If (x.eq.y) Then
            PV_C00=1.D0/2.D0-LOG(x/mu**2)
         Else
            PV_C00=-LOG(x/mu**2)+z*Log(y/x)/(x-y)+3.D0/2.D0
         End If
      Else If (x.eq.y) Then
         PV_C00=-LOG(x/mu**2)
     &        +(x**2-4.D0*z*x+3.D0*z**2)/(2.D0*(x-z)**2)
     &        -z**2*LOG(z/x)/(x-z)**2
      Else If (x.eq.z) Then
         PV_C00=-LOG(x/mu**2)
     &        +(x**2-4.D0*y*x+3.D0*y**2)/(2.D0*(x-y)**2)
     &        -y**2*LOG(y/x)/(x-y)**2
      Else If (y.eq.z) Then
         PV_C00=-LOG(x/mu**2)
     &        +(y**2-4.D0*y*x+3.D0*x**2)/(2.D0*(x-y)**2)
     &        +(2.D0*x-y)*y*LOG(y/x)/(x-y)**2
      Else
         PV_C00=3.D0/2.D0-LOG(x/mu**2)
     &        +y**2*LOG(y/x)/((x-y)*(y-z))
     &        +z**2*LOG(z/x)/((x-z)*(z-y))
      End If

      PV_C00=1.D0/4.D0*PV_C00

      Return
      End

c**********************************************************************

      Double Precision Function PV_C11(xtmp,ytmp,ztmp)

c The Passarino-Veltman function C11.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,ztmp

c Local
      Double Precision x,y,z

c Functions
      Double Precision PV_C0

      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (x.eq.y.and.y.eq.z) Then
         PV_C11=1.D0/(3.D0*x)
      Else If (x.eq.0.D0) Then
         If (y.eq.z) Then
            PV_C11=1.D0/(2.D0*y)
         Else
            PV_C11=LOG(z/y)/(2.D0*(z-y))
         End If
      Else If (y.eq.0.D0) Then
         If (x.eq.z) Then
            PV_C11=3.D0/(4.D0*x)
         Else
            PV_C11=1.D0/(2.D0*(z-x))
     &           +(z-2.D0*x)*LOG(z/x)/(2.D0*(x-z)**2)
         End If
      Else If (z.eq.0.D0) Then
         If (y.eq.x) Then
            PV_C11=3.D0/(4.D0*x)
         Else
            PV_C11=1.D0/(2.D0*(y-x))
     &           +(y-2.D0*x)*LOG(y/x)/(2.D0*(x-y)**2)
         End If
      Else If (x.eq.y) Then
         PV_C11=(3.D0*x**2-4.D0*z*x+z**2
     &        +2.D0*(2.D0*x-z)*z*LOG(z/x))/(4.D0*(x-z)**3)
      Else If (x.eq.z) Then
         PV_C11=(3.D0*x**2-4.D0*y*x+y**2
     &        +2.D0*(2.D0*x-y)*y*LOG(y/x))/(4.D0*(x-y)**3)
      Else If (y.eq.z) Then
         PV_C11=(-3.D0*x**2+4.D0*y*x-y**2
     &        -2.D0*x**2*LOG(y/x))/(2.D0*(x-y)**3)
      Else
         PV_C11=-x/(2.D0*(x-y)*(x-z))-y**2/(2.D0*(x-y)**2*(y-z))
     &        -z**2/(2.D0*(x-z)**2*(z-y))*Log(z/x)
     &        -PV_C0(xtmp,ytmp,ztmp)
      End If

      Return
      End

c**********************************************************************

      Double Precision Function PV_C12(xtmp,ytmp,ztmp)

c The Passarino-Veltman function C12.

      Implicit NONE

c Inputs
      Double Precision xtmp,ytmp,ztmp

c Local
      Double Precision x,y,z

c Functions
      Double Precision PV_C0

      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (x.eq.y.and.y.eq.z) Then
         PV_C12=1.D0/(6.D0*x)
      Else If (x.eq.0.D0) Then
         If (y.eq.z) Then
            PV_C12=1.D0/(4.D0*y)
         Else
            PV_C12=1.D0/(2.D0*(z-y))
     &           -y*Log(z/y)/(2.D0*(y-z)**2)
         End If
      Else If (y.eq.0.D0) Then
         If (x.eq.z) Then
            PV_C12=1.D0/(4.D0*x)
         Else
            PV_C12=1.D0/(2.D0*(z-x))
     &           -x*LOG(z/x)/(2.D0*(x-z)**2)
         End If
      Else If (z.eq.0.D0) Then
         If (x.eq.y) Then
            PV_C12=1.D0/(2.D0*x)
         Else
            PV_C12=LOG(y/x)/(2.D0*(y-x))
         End If
      Else If (x.eq.y) Then
         PV_C12=(x**2-z**2+2.D0*z*LOG(z/x))/(2.D0*(x-z)**3)
      Else If (x.eq.z) Then
         PV_C12=(x**2-4.D0*y*x+3.D0*y**2
     &        -2.D0*y**2*LOG(y/x))/(4.D0*(x-y)**3)
      Else If (y.eq.z) Then
         PV_C12=(-3.D0*x**2+4.D0*y*x-y**2
     &        -2.D0*x**2*LOG(y/x))/(4.D0*(x-y)**3)
      Else
         PV_C12=-PV_C0(xtmp,ytmp,ztmp)-z/(2.D0*(x-z)*(z-y))
     &        +(y-2.D0*z+x)*z**2/(2.D0*(y-z)**2*(x-z)**2)*LOG(z/x)
     &        -y*(2.D0*z-y)/(2.D0*(x-y)*(y-z)**2)*LOG(y/x)
      End If

      Return
      End

c**********************************************************************

      Double Precision Function PV_D0(wtmp,xtmp,ytmp,ztmp)

c The Passarino-Veltman function D0.

      Implicit NONE

c Inputs
      Double Precision wtmp,xtmp,ytmp,ztmp

c Local
      Double Precision w,x,y,z

      w=wtmp**2
      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (w.eq.x.and.w.eq.y.and.w.eq.z) Then
         PV_D0=1.D0/(6.D0*x**2)
      Else If (w.eq.0.D0) Then
         If (x.eq.y.and.x.eq.z) Then
            PV_D0=1.D0/(2.D0*x**2)
         Else If (x.eq.y) Then
            PV_D0=1.D0/(x*(z-x))
     &           -LOG(z/x)/((x-z)**2)
         Else If (x.eq.z) Then
            PV_D0=1.D0/(y*(y-x))
     &           -LOG(y/x)/((x-y)**2)
         Else If (y.eq.z) Then
            PV_D0=1.D0/(y*(x-y))
     &           -LOG(x/y)/((y-x)**2)
         Else
            PV_D0=Log(y/x)/((x-y)*(y-z))
     &           +Log(z/x)/((y-z)*(z-x))
         End If
      Else If (z.eq.0.D0) Then
         If (x.eq.y.and.x.eq.z) Then
            PV_D0=1.D0/(2.D0*x**2)
         Else If (w.eq.x) Then
            PV_D0=1.D0/(x*(y-x))
     &           -LOG(y/x)/((y-x)**2)
         Else If (w.eq.y) Then
            PV_D0=1.D0/(y*(x-y))
     &           -LOG(x/y)/((x-y)**2)
         Else If (x.eq.y) Then
            PV_D0=1.D0/(x*(w-x))
     &           -LOG(w/x)/((w-x)**2)
         Else
            PV_D0=Log(x/w)/((w-x)*(x-y))
     &           +Log(y/w)/((x-y)*(y-w))
         End If
      Else If (y.eq.0.D0) Then
         If (x.eq.y.and.x.eq.z) Then
            PV_D0=1.D0/(2.D0*x**2)
         Else If (w.eq.x) Then
            PV_D0=1.D0/(x*(z-x))
     &           -LOG(z/x)/((x-z)**2)
         Else If (w.eq.z) Then
            PV_D0=1.D0/(z*(x-z))
     &           -LOG(x/z)/((z-x)**2)
         Else If (x.eq.z) Then
            PV_D0=1.D0/(x*(w-x))
     &           -LOG(w/x)/((w-x)**2)
         Else
            PV_D0=Log(x/w)/((w-x)*(x-z))
     &           +Log(z/w)/((x-z)*(z-w))
         End If
      Else If (x.eq.0.D0) Then
         If (w.eq.y.and.w.eq.z) Then
            PV_D0=1.D0/(2.D0*y**2)
         Else If (w.eq.y) Then
            PV_D0=1.D0/(y*(z-y))
     &           -LOG(z/y)/((z-y)**2)
         Else If (w.eq.z) Then
            PV_D0=1.D0/(z*(y-z))
     &           -LOG(y/z)/((y-z)**2)
         Else If (y.eq.z) Then
            PV_D0=1.D0/(y*(w-y))
     &           -LOG(w/y)/((w-y)**2)
         Else
            PV_D0=Log(y/w)/((w-y)*(y-z))
     &           +Log(z/w)/((y-z)*(z-w))
         End If
      Else If (w.eq.x.and.w.eq.y) Then
         PV_D0=(x+z)/(2.D0*x*(x-z)**2)
     &        +z*Log(z/x)/((x-z)**3)
      Else If (w.eq.x.and.w.eq.z) Then
         PV_D0=(x+y)/(2.D0*x*(x-y)**2)
     &        +y*Log(y/x)/((x-y)**3)
      Else If (w.eq.y.and.w.eq.z) Then
         PV_D0=(x+y)/(2.D0*y*(x-y)**2)
     &        -x*Log(x/y)/((x-y)**3)
      Else If (x.eq.y.and.x.eq.z) Then
         PV_D0=(x+w)/(2.D0*x*(w-x)**2)
     &        +w*Log(x/w)/((w-x)**3)
      Else If (w.eq.x.and.y.eq.z) Then
         PV_D0=-2.D0/(x-y)**2
     &        -(x+y)*LOG(y/x)/((x-y)**3)
      Else If (w.eq.y.and.z.eq.x) Then
         PV_D0=-2.D0/(x-y)**2
     &        -(x+y)*LOG(x/y)/((y-x)**3)
      Else If (w.eq.z.and.y.eq.x) Then
         PV_D0=-2.D0/(x-z)**2
     &        -(x+z)*LOG(x/z)/((z-x)**3)
      Else If (w.eq.x) Then
         PV_D0=1.D0/((y-x)*(x-z))
     &        +y*Log(y/x)/((x-y)**2*(z-y))
     &        +z*Log(z/x)/((x-z)**2*(y-z))
      Else If (w.eq.y) Then
         PV_D0=1.D0/((x-y)*(y-z))
     &        +z*Log(z/y)/((x-z)*(y-z)**2)
     &        +x*Log(x/y)/((x-y)**2*(z-x))
      Else If (w.eq.z) Then
         PV_D0=1.D0/((x-z)*(z-y))
     &        +y*Log(y/z)/((x-y)*(y-z)**2)
     &        +x*Log(x/z)/((y-x)*(x-z)**2)
      Else If (x.eq.y) Then
         PV_D0=1.D0/((w-x)*(x-z))
     &        +z*Log(z/w)/((w-z)*(x-z)**2)
     &        +(x**2-w*z)*Log(x/w)/((w-x)**2*(x-z)**2)
      Else If (x.eq.z) Then
         PV_D0=1.D0/((w-x)*(x-y))
     &        +y*Log(y/w)/((w-y)*(x-y)**2)
     &        +(x**2-w*y)*Log(x/w)/((w-x)**2*(x-y)**2)
      Else If (y.eq.z) Then
         PV_D0=1.D0/((w-y)*(y-x))
     &        +x*Log(x/w)/((w-x)*(x-y)**2)
     &        +(y**2-w*x)*Log(y/w)/((w-y)**2*(x-y)**2)
      Else
         PV_D0=x*LOG(x/w)/((w-x)*(x-y)*(x-z))
     &        +y*LOG(y/w)/((w-y)*(y-x)*(y-z))
     &        +z*LOG(z/w)/((w-z)*(z-x)*(z-y))
      End If

      Return
      End

c**********************************************************************

      Double Precision Function PV_D00(wtmp,xtmp,ytmp,ztmp)

c The Passarino-Veltman function D00.

      Implicit NONE

c Inputs
      Double Precision wtmp,xtmp,ytmp,ztmp

c Local
      Double Precision w,x,y,z

      w=wtmp**2
      x=xtmp**2
      y=ytmp**2
      z=ztmp**2

      If (w.eq.x.and.w.eq.y.and.w.eq.z) Then
         PV_D00=-1.D0/(3.D0*x)
      Else If (w.eq.0.D0) Then
         If (x.eq.y.and.x.eq.z) Then
            PV_D00=-1.D0/(2.D0*x)
         Else If (x.eq.y) Then
            PV_D00=1.D0/(z-x)
     &           -z*LOG(z/x)/((x-z)**2)
         Else If (x.eq.z) Then
            PV_D00=1.D0/(y-x)
     &           -y*LOG(y/x)/((x-y)**2)
         Else If (y.eq.z) Then
            PV_D00=1.D0/(x-y)
     &           -x*LOG(x/y)/((x-y)**2)
         Else
            PV_D00=y*Log(y/x)/((x-y)*(y-z))
     &           +z*Log(z/x)/((y-z)*(z-x))
         End If
      Else If (z.eq.0.D0) Then
         If (w.eq.x.and.w.eq.y) Then
            PV_D00=-1.D0/(2.D0*x)
         Else If (w.eq.x) Then
            PV_D00=1.D0/(y-x)
     &           -y*LOG(y/x)/((x-y)**2)
         Else If (w.eq.y) Then
            PV_D00=1.D0/(x-y)
     &           -x*LOG(x/y)/((x-y)**2)
         Else If (x.eq.y) Then
            PV_D00=1.D0/(w-x)
     &           -w*LOG(w/x)/((w-x)**2)
         Else
            PV_D00=x*Log(x/w)/((w-x)*(x-y))
     &           +y*Log(y/w)/((x-y)*(y-w))
         End If
      Else If (y.eq.0.D0) Then
         If (w.eq.x.and.w.eq.y) Then
            PV_D00=-1.D0/(2.D0*x)
         Else If (w.eq.x) Then
            PV_D00=1.D0/(z-x)
     &           -z*LOG(z/x)/((x-z)**2)
         Else If (w.eq.z) Then
            PV_D00=1.D0/(x-z)
     &           -x*LOG(x/z)/((x-z)**2)
         Else If (x.eq.z) Then
            PV_D00=1.D0/(w-x)
     &           -w*LOG(w/x)/((w-x)**2)
         Else
            PV_D00=x*Log(x/w)/((w-x)*(x-z))
     &           +z*Log(z/w)/((x-z)*(z-w))
         End If
      Else If (x.eq.0.D0) Then
         If (w.eq.x.and.w.eq.y) Then
            PV_D00=-1.D0/(2.D0*y)
         Else If (w.eq.y) Then
            PV_D00=1.D0/(z-y)
     &           -z*LOG(z/y)/((y-z)**2)
         Else If (w.eq.z) Then
            PV_D00=1.D0/(y-z)
     &           -y*LOG(y/z)/((y-z)**2)
         Else If (y.eq.z) Then
            PV_D00=1.D0/(w-y)
     &           -w*LOG(w/y)/((w-y)**2)
         Else
            PV_D00=y*Log(y/w)/((w-y)*(y-z))
     &           +z*Log(z/w)/((y-z)*(z-w))
         End If
      Else If (w.eq.x.and.w.eq.y) Then
         PV_D00=(3.D0*z-x)/(2.D0*(x-z)**2)
     &        +z**2*Log(z/x)/((x-z)**3)
      Else If (w.eq.x.and.w.eq.z) Then
         PV_D00=(3.D0*y-x)/(2.D0*(x-y)**2)
     &        +y**2*Log(y/x)/((x-y)**3)
      Else If (w.eq.y.and.w.eq.z) Then
         PV_D00=(3.D0*x**2-4.D0*y*x+y**2)/(2.D0*(x-y)**3)
     &        +x**2*Log(x/y)/((y-x)**3)
      Else If (x.eq.y.and.x.eq.z) Then
         PV_D00=(3.D0*w**2-4.D0*x*w+x**2)/(2.D0*(w-x)**3)
     &        +w**2*Log(x/w)/((w-x)**3)
      Else If (w.eq.x.and.y.eq.z) Then
         PV_D00=-(x+y)/(x-y)**2
     &        -2.D0*x*y*LOG(y/x)/((x-y)**3)
      Else If (w.eq.y.and.z.eq.x) Then
         PV_D00=-(x+y)/(x-y)**2
     &        -2.D0*x*y*LOG(x/y)/((y-x)**3)
      Else If (w.eq.z.and.y.eq.x) Then
         PV_D00=-(x+z)/(x-z)**2
     &        -2.D0*x*z*LOG(x/z)/((z-x)**3)
      Else If (w.eq.x) Then
         PV_D00=x/((x-y)*(z-x))
     &        +y**2*Log(y/x)/((x-y)**2*(z-y))
     &        +z**2*Log(z/x)/((x-z)**2*(y-z))
      Else If (w.eq.y) Then
         PV_D00=y/((x-y)*(y-z))
     &        +z**2*Log(z/y)/((y-z)**2*(x-z))
     &        +x**2*Log(x/y)/((y-x)**2*(z-x))
      Else If (w.eq.z) Then
         PV_D00=z/((z-x)*(y-z))
     &        +x**2*Log(x/z)/((z-x)**2*(y-x))
     &        +z**2*Log(y/z)/((z-y)**2*(x-y))
      Else If (x.eq.y) Then
         PV_D00=y/((x-y)*(y-z))
     &        +z**2*Log(z/y)/((y-z)**2*(x-z))
     &        +x**2*Log(x/y)/((y-x)**2*(z-x))
      Else If (x.eq.z) Then
         PV_D00=x/((w-x)*(x-y))
     &        +x*(w*(x-2.D0*y)+x*y)*Log(x/w)/((w-x)**2*(x-y)**2)
     &        +y**2*Log(y/w)/((w-y)*(x-y)**2)
      Else If (y.eq.z) Then
         PV_D00=y/((w-y)*(y-x))
     &        +y*((w+x)*y-2*w*x)*Log(y/w)/((w-y)**2*(x-y)**2)
     &        +x**2*Log(x/w)/((w-x)*(x-y)**2)
      Else
         PV_D00=x**2*Log(x/w)/((w-x)*(x-y)*(x-z))
     &        +y**2*Log(y/w)/((w-y)*(y-x)*(y-z))
     &        +z**2*Log(z/w)/((w-z)*(z-x)*(z-y))
      End If

      PV_D00=1.D0/4.D0*PV_D00

      Return
      End
