c**********************************************************************
c Functions describing the running of the various coupling constants
c and masses. Also includes a function that converts a given pole mass
c to a running mass.
c**********************************************************************
      
      Double Precision Function alphaEM(mu)

c Function that determine the running U(1)_EM coupling via the
c formula A.22 of Phys.Rev.D46,3945 given an input value at the
c Z pole.

      Implicit NONE

      Include 'insm.h'

c Inputs
c The relevant scale
      Double Precision mu
      
c Local
      Integer i,j
      Integer nu_0,nd_0,ne_0
      Integer nu,nd,ne
      Integer delnu,delnd,delne
      Integer nutmp,ndtmp,netmp
      Double Precision tmp1,tmp2,altmp,mu_tmp
      Double Precision ThMass(18,2)

c Functions
      Double Precision EMBe0

      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)


c If alpha_EM is measured at the Z-pole we have     
      nu_0=2
      nd_0=3
      ne_0=3

c Find the number of light leptons and quarks present at the scale mu
      nu=0
      nd=0
      ne=0
      Do i=1,3
         If (mu.gt.m_up(i)) nu=nu+1
         If (mu.gt.m_do(i)) nd=nd+1
         If (mu.gt.m_el(i)) ne=ne+1
      End Do

      delnu=nu-nu_0
      delnd=nd-nd_0
      delne=ne-ne_0

c If the required scale is below the starting scale
      If (delnu.lt.0.or.delnd.lt.0.or.delne.lt.0) Then
c Find the masses lying between the two scales
         If (delnd.ne.0) Then
            Do i=1,ABS(delnd)
               ThMass(i,1)=m_do(nd_0-i+1)
               ThMass(i,2)=1.D0
            End Do
         End If
         If (delnu.ne.0) Then
            Do i=1,ABS(delnu)
               ThMass(i+ABS(delnd),1)=m_up(nu_0-i+1)
               ThMass(i+ABS(delnd),2)=2.D0
            End Do
         End If
         If (delne.ne.0) Then
            Do i=1,ABS(delne)
               ThMass(i+ABS(delnd)+ABS(delnu),1)=m_el(ne_0-i+1)
               ThMass(i+ABS(delnd)+ABS(delnu),2)=3.D0
            End Do
         End If
c Sort the masses highest to lowest
         Do i=1,ABS(delne)+ABS(delnd)+ABS(delnu)-1
            Do j=i+1,ABS(delne)+ABS(delnd)+ABS(delnu)
               If (ThMass(i,1).lt.ThMass(j,1)) Then
                  tmp1=ThMass(j,1)
                  tmp2=ThMass(j,2)
                  ThMass(j,1)=ThMass(i,1)
                  ThMass(j,2)=ThMass(i,2)
                  ThMass(i,1)=tmp1
                  ThMass(i,2)=tmp2
               End If
            End Do
         End Do
c Calculate alphaEM
         altmp=alphaEM_0
         mu_tmp=mualphaEM_0
         nutmp=nu_0
         ndtmp=nd_0
         netmp=ne_0
         Do i=1,ABS(delne)+ABS(delnd)+ABS(delnu)
            tmp1=altmp/(1.D0-EMBe0(nutmp,ndtmp,netmp)
     &           *altmp/(2.D0*PI)*LOG(mu_tmp/ThMass(i,1)))
            altmp=tmp1
            If (ThMass(i,2).eq.1.D0) Then
               ndtmp=ndtmp-1
            Else If (ThMass(i,2).eq.2.D0) Then
               nutmp=nutmp-1
            Else If (ThMass(i,2).eq.3.D0) Then 
               netmp=netmp-1
            End If
            mu_tmp=ThMass(i,1)
         End Do
         alphaEM=altmp/(1.D0-EMBe0(nutmp,ndtmp,netmp)
     &        *altmp/(2.D0*PI)*LOG(mu_tmp/mu))
c If the starting scale is lower than the required scale
      Else If (delnu.gt.0.or.delnd.gt.0.or.delne.gt.0) Then
         If (delnd.ne.0) Then
            Do i=1,ABS(delnd)
               ThMass(i,1)=m_do(nd_0+i-1)
               ThMass(i,2)=1.D0
            End Do
         End If
         If (delnu.ne.0) Then
            Do i=1,ABS(delnu)
               ThMass(i+ABS(delnd),1)=m_up(nu_0+i-1)
               ThMass(i+ABS(delnd),2)=2.D0
            End Do
         End If
         If (delne.ne.0) Then
            Do i=1,ABS(delne)
               ThMass(i+ABS(delnd)+ABS(delnu),1)=m_el(ne_0+i-1)
               ThMass(i+ABS(delne)+ABS(delnu),2)=3.D0
            End Do
         End If
c Sort the masses lowest to highest
         Do i=1,ABS(delne)+ABS(delnd)+ABS(delnu)-1
            Do j=i+1,ABS(delne)+ABS(delnd)+ABS(delnu)
               If (ThMass(i,1).gt.ThMass(j,1)) Then
                  tmp1=ThMass(j,1)
                  tmp2=ThMass(j,2)
                  ThMass(j,1)=ThMass(i,1)
                  ThMass(j,2)=ThMass(i,2)
                  ThMass(i,1)=tmp1
                  ThMass(i,2)=tmp2
               End If
            End Do
         End Do
c Calculate alphaEM
         altmp=alphaEM_0
         mu_tmp=mualphaEM_0
         nutmp=nu_0
         ndtmp=nd_0
         netmp=ne_0
         Do i=1,ABS(delne)+ABS(delnd)+ABS(delnu)
            tmp1=altmp/(1.D0-EMBe0(nutmp,ndtmp,netmp)
     &           *altmp/(2.D0*PI)*LOG(mu_tmp/ThMass(i,1)))
            altmp=tmp1
            If (ThMass(i,2).eq.1.D0) Then
               ndtmp=ndtmp+1
            Else If (ThMass(i,2).eq.2.D0) Then
               nutmp=nutmp+1
            Else If (ThMass(i,2).eq.3.D0) Then 
               netmp=netmp+1
            End If
            mu_tmp=ThMass(i,1)
         End Do
         alphaEM=altmp/(1.D0-EMBe0(nutmp,ndtmp,netmp)
     &        *altmp/(2.D0*PI)*LOG(mu_tmp/mu))
      Else
         alphaEM=alphaEM_0/(1D0-EMBe0(nu,nd,ne)*alphaEM_0/(2.D0*PI)
     &        *LOG(mualphaEM_0/mu))
      End If 
      
      Return
      End

c***********************************************************************

      Double Precision Function alpha1(mu)

c Running coupling of the U(1)_Y coupling constant

      Implicit NONE

      Include 'insm.h'

c Input
      Double Precision mu

c Local 
      Double Precision b1

      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      
      b1=-33.0D0/5.0D0
      alpha1=alpha1_0
     &     /(1.0D0+(b1*alpha1_0*(LOG(mu**2/mualpha1_0**2))/(4.0D0*PI)))

      Return
      End

c******************************************************************
      Double Precision Function alpha2(mu)

c Running coupling of the SU(2)_L coupling constant

      Implicit NONE

      Include 'insm.h'

c Input
      Double Precision mu

c Local
      Double Precision b2

      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)


      b2=-1.0D0
      alpha2=alpha2_0
     &     /(1.0D0+(b2*alpha2_0*(LOG(mu**2/mualpha2_0**2))/(4.0D0*PI)))

      Return
      End

c******************************************************************
      Double Precision Function alpha3(mu)

c Running coupling of the SU(3) coupling constant

      Implicit NONE

      Include'insm.h'

c Input
      Double Precision mu

c Local
      Double Precision b3

      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)


      b3=3.0D0
      alpha3=alpha3_0
     &     /(1.0D0+(b3*alpha3_0*(LOG(mu**2/mualpha3_0**2))/(4.0D0*PI)))

      Return

      End

c***********************************************************************
      Double Precision Function EMBe0(nu,nd,ne)

      Implicit NONE

      Integer nu,nd,ne

      EMBe0=-(16.D0/9.D0*nu+4.D0/9.D0*nd+4.D0/3.D0*ne)
      
      Return
      End
