c***********************************************************************

      Subroutine bsgbrsub(debug,iOrd,WilsonC,WilsonCp)

c Subroutine that calculates the branching ratio for the decay :
c \overline{B}\to X_s\gamma using the formulae of hep-ph/0104034 and
c the magic numbers of hep-ph/0203135. Taking into account the comments 
c of hep-ph/0312260
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'bsg.h'

c Inputs
      Integer debug,iOrd

c The LO and NLO Wilson coefficients in the basis of Gambino and Misiak
c NOTE - C7 and C8 are NOT the effective coefficients commonly found in
c literature.
      Double Complex WilsonC(2,8), WilsonCp(2,8)

c Local
      Integer i,j

      Double Complex tmp1,tmp2,tmp3,tmp4

      Double Complex epsSM,eps11,eps12

      Double Complex KCtot,KtTot,epstot,KtTotpr,epstotpr

      Double Complex At1New,Ft1New,Et0New
      Double Complex At0New,Ft0New,At0Newp,Ft0Newp
      
      Double Complex Ceffp(8), Ceff(8)

      Double Precision BE_0,NE_0,PE_0,ordfac
      Double Precision x,z,eta5,mtEW,KMeta,GM_rfac

c Functions
      Double Precision alphaEM,alphaStr,MSrmass
      Double Precision GM_A0, GM_F0, GM_E0
      Double Precision GM_A0pr, GM_F0pr, GM_A1, GM_F1

      Double Complex GM_afun, GM_bfun



c Some useful quantities

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         ordfac=1.D0
c      Else
c         ordfac=0.D0
c      End If

      eta5=alphaStr(muew,5,4)/alphaStr(mub,5,4)
      x=MSrmass(muew,6,5,4)**2/mw**2
      z=((MSrmass(mub,4,5,4)+MSrmass(mu_up(2),4,5,4))/2.D0)**2/m_b1s**2
      mtEW=MSrmass(muew,6,5,4)
      KMeta=(KM(1,3)*DCONJG(KM(1,2)))/(KM(3,2)*DCONJG(KM(3,2)))
      
c The factor of r that appears in various formula using eqn. 71 of
c Hurth, Lunghi and Porod (hep-ph/0312260)

      GM_rfac=0.578D0
     &     *(alphaStr(mz,5,4)/0.1185D0)**(-1.D0)
     &     *(m_b1s/4.69D0)**(0.23D0)
     &     *(MSrmass(mu_up(2),4,4,4)/1.25D0)**(-0.003D0)
     &     *(muew/165.D0)**(-0.08D0)
     &     *(mub/4.69D0)**(0.006D0)

c***********************************************************************
c Perturbative Contributions (The function P(E_0))
c***********************************************************************
c I use the parameterization of Hurth, Lunghi and Porod. (eqn 18)

c***********************************************************************
c Charm quark contributions (eqn 19)

      KCtot=DCMPLX(0.D0,0.D0)


      Do i=1,8
         KCtot=KCtot+eta5**(GMa(i))*(GMd(i)
     &        +ordfac*alphaStr(mub,5,4)/(4.D0*PI)
     &        *(2.D0*23.D0/3.D0*GMa(i)*GMd(i)
     &        *(LOG(m_b1s/mub)+eta5*LOG(muew/mw))+GMdt(i)+GMdtn(i)*eta5
     &        +GMdta(i)*GM_afun(z)+GMdtb(i)*GM_bfun(z)
     &        +GMdtpi(i)*DCMPLX(0.D0,1.D0)*PI)
     &        +KMeta*alphaStr(mub,5,4)/(4.D0*PI)*(GMdta(i)*GM_bfun(z)
     &        +GMdtb(i)*GM_bfun(z)))
      End Do

c***********************************************************************
c Top quark and 'new physics' contributions

      At0New=GM_A0(x)-2.D0*WilsonC(1,7)
      Ft0New=GM_F0(x)-2.D0*WilsonC(1,8)
      At1New=GM_A1(x)-2.D0*WilsonC(2,7)
      Ft1New=GM_F1(x)-2.D0*WilsonC(2,8)
      Et0New=GM_E0(x)+WilsonC(2,4)

c Formula 3.8 of Gambino and Misiak

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,8
         tmp4=tmp4+Et0New*GMe(i)*eta5**(GMa(i)+11.D0/23.D0)
      End Do
      
      tmp1=(1.D0-2.D0/9.D0*ordfac*alphaStr(m_b1s,5,4)**2)
     &     *(-1.D0/2.D0*eta5**(4.D0/23.D0)*At0New
     &     +4.D0/3.D0*(eta5**(4.D0/23.D0)-eta5**(2.D0/23.D0))*Ft0New)

      tmp2=ordfac*alphaStr(muew,5,4)/PI*LOG(muew/MSrmass(muew,6,5,4))
     &     *4.D0*x*(-1.D0/2.D0*eta5**(4.D0/23.D0)*GM_A0pr(x)
     &     +4.D0/3.D0*(eta5**(4.D0/23.D0)-eta5**(2.D0/23.D0))
     &     *GM_F0pr(x))

      tmp3=eta5**(4.D0/23.D0)*(-1.D0/2.D0*eta5*At1New
     &     +(12523.D0/3174.D0-7411.D0/4761.D0*eta5-2.D0/9.D0*PI**2)
     &     *At0New
     &     -4.D0/3.D0*LOG(m_b1s/mub)*At0New
     &     -4.D0/3.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_A0(x)
     &     +4.D0/3.D0*eta5*Ft1New
     &     +(-50092.D0/4761.D0+1110842.D0/357075.D0*eta5
     &     +16.D0/27.D0*PI**2)*Ft0New
     &     +32.D0/9.D0*LOG(m_b1s/mub)*Ft0New
     &     +32.D0/9.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_F0(x))
     &     +eta5**(2.D0/23.D0)*(-4.D0/3.D0*eta5*Ft1New
     &     +(2745458.D0/357075.D0-38890.D0/14283.D0*eta5
     &     -4.D0/9.D0*PI*(PI+DCMPLX(0.D0,1.D0)))*Ft0New
     &     -16.D0/9.D0*LOG(m_b1s/mub)*Ft0New
     &     -16.D0/9.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_F0(x))

      KtTot=tmp1+tmp2+ordfac*alphaStr(mub,5,4)/(4.D0*PI)*(tmp3+tmp4)

c***********************************************************************
c Electroweak Corrections

      epsSM=0.0071D0

      eps11=alphaEM(mz)/alphaStr(mub,5,4)
     &     *(88.D0/575.D0*eta5**(16.D0/23.D0)
     &     -40.D0/69.D0*eta5**(-7.D0/23.D0)
     &     +32.D0/75.D0*eta5**(-9.D0/23.D0))
     &     -alphaEM(mz)/PI*LOG(mz/mub)*GM_rfac*eta5**(4.D0/23.D0)

      eps12=alphaEM(mz)/alphaStr(mub,5,4)
     &     *(-704.D0/1725.D0*eta5**(16.D0/23.D0)
     &     +640.D0/1449.D0*eta5**(14.D0/23.D0)
     &     +32.D0/1449.D0*eta5**(-7.D0/23.D0)
     &     -32.D0/575.D0*eta5**(-9.D0/23.D0))
     &     -alphaEM(mz)/PI*LOG(mz/mub)*GM_rfac*8.D0/3.D0
     &     *(eta5**(2.D0/23.D0)-eta5**(4.D0/23.D0))

      epsTot=ordfac*(epsSM+eps11*WilsonC(1,7)+eps12*WilsonC(1,8))

c***********************************************************************
c Top quark and 'new physics' contributions

      At0New=-2.D0*WilsonCp(1,7)
      Ft0New=-2.D0*WilsonCp(1,8)
      At1New=-2.D0*WilsonCp(2,7)
      Ft1New=-2.D0*WilsonCp(2,8)
      Et0New=WilsonCp(2,4)

c Formula 3.8 of Gambino and Misiak

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,8
         tmp4=tmp4+Et0New*GMe(i)*eta5**(GMa(i)+11.D0/23.D0)
      End Do
      
      tmp1=(1.D0-ordfac*2.D0/9.D0*alphaStr(m_b1s,5,4)**2)
     &     *(-1.D0/2.D0*eta5**(4.D0/23.D0)*At0New
     &     +4.D0/3.D0*(eta5**(4.D0/23.D0)-eta5**(2.D0/23.D0))*Ft0New)

      tmp3=eta5**(4.D0/23.D0)*(-1.D0/2.D0*eta5*At1New
     &     +(12523.D0/3174.D0-7411.D0/4761.D0*eta5-2.D0/9.D0*PI**2)
     &     *At0New
     &     -4.D0/3.D0*LOG(m_b1s/mub)*At0New
     &     +4.D0/3.D0*eta5*Ft1New
     &     +(-50092.D0/4761.D0+1110842.D0/357075.D0*eta5
     &     +16.D0/27.D0*PI**2)*Ft0New
     &     +32.D0/9.D0*LOG(m_b1s/mub)*Ft0New)
     &     +eta5**(2.D0/23.D0)*(-4.D0/3.D0*eta5*Ft1New
     &     +(2745458.D0/357075.D0-38890.D0/14283.D0*eta5
     &     -4.D0/9.D0*PI*(PI+DCMPLX(0.D0,1.D0)))*Ft0New
     &     -16.D0/9.D0*LOG(m_b1s/mub)*Ft0New)

      KtTotPr=tmp1+tmp2+ordfac*alphaStr(mub,5,4)/(4.D0*PI)*(tmp3+tmp4)

c***********************************************************************
c Electroweak Corrections

      eps11=alphaEM(mz)/alphaStr(mub,5,4)
     &     *(88.D0/575.D0*eta5**(16.D0/23.D0)
     &     -40.D0/69.D0*eta5**(-7.D0/23.D0)
     &     +32.D0/75.D0*eta5**(-9.D0/23.D0))
     &     -alphaEM(mz)/PI*LOG(mz/mub)*GM_rfac*eta5**(4.D0/23.D0)

      eps12=alphaEM(mz)/alphaStr(mub,5,4)
     &     *(-704.D0/1725.D0*eta5**(16.D0/23.D0)
     &     +640.D0/1449.D0*eta5**(14.D0/23.D0)
     &     +32.D0/1449.D0*eta5**(-7.D0/23.D0)
     &     -32.D0/575.D0*eta5**(-9.D0/23.D0))
     &     -alphaEM(mz)/PI*LOG(mz/mub)*GM_rfac*8.D0/3.D0
     &     *(eta5**(2.D0/23.D0)-eta5**(4.D0/23.D0))

      epsTotpr=ordfac*(eps11*WilsonCp(1,7)+eps12*WilsonCp(1,8))

c***********************************************************************
c Bremstrahlung Corrections
c Equation E.1 of Gambino and Misiak

      At0New=GM_A0(x)-2.D0*WilsonC(1,7)
      Ft0New=GM_F0(x)-2.D0*WilsonC(1,8)

      At0Newp=-2.D0*WilsonCp(1,7)
      Ft0Newp=-2.D0*WilsonCp(1,8)

      Ceff(1)=eta5**GMa(3)-eta5**GMa(4)
      Ceff(2)=2.D0/3.D0*eta5**GMa(3)+1.D0/3.D0*eta5**GMa(4)
      Ceff(3)=2.D0/63.D0*eta5**GMa(3)-1.D0/27.D0*eta5**GMa(4)
     &     -0.0659D0*eta5**GMa(5)+0.0595D0*eta5**GMa(6)
     &     -0.0218D0*eta5**GMa(7)+0.0335D0*eta5**GMa(8)
      Ceff(4)=1.D0/21.D0*eta5**GMa(3)+1.D0/9.D0*eta5**GMa(4)
     &     +0.0237D0*eta5**GMa(5)-0.0173D0*eta5**GMa(6)
     &     -0.1336D0*eta5**GMa(7)-0.0316D0*eta5**GMa(8)
      Ceff(5)=-1.D0/126.D0*eta5**GMa(3)+1.D0/108.D0*eta5**GMa(4)
     &     +0.0094D0*eta5**GMa(5)-0.0100D0*eta5**GMa(6)
     &     +0.0010D0*eta5**GMa(7)-0.0017D0*eta5**GMa(8)
      Ceff(6)=-1.D0/84.D0*eta5**GMa(3)-1.D0/36.D0*eta5**GMa(4)
     &     +0.0108D0*eta5**GMa(5)+0.0163D0*eta5**GMa(6)
     &     +0.0103D0*eta5**GMa(7)+0.0023D0*eta5**GMa(8)
      Ceff(7)=(-4.D0/3.D0*Ft0New+42678.D0/30253.D0)*eta5**GMa(1)
     &     +(-1.D0/2.D0*At0New+4.D0/3.D0*Ft0New
     &     -86697.D0/103460.D0)*eta5**GMa(2)
     &     -3.D0/7.D0*eta5**GMa(3)-1.D0/14.D0*eta5**GMa(4)
     &     -0.6494D0*eta5**GMa(5)-0.0380D0*eta5**GMa(6)
     &     -0.0185D0*eta5**GMa(7)-0.0057D0*eta5**GMa(8)
      Ceff(8)=(-1.D0/2.D0*Ft0New+64017.D0/121012.D0)*eta5**GMa(1)
     &     -0.9135D0*eta5**GMa(5)+0.0873D0*eta5**GMa(6)
     &     -0.0571D0*eta5**GMa(7)+0.0209D0*eta5**GMa(8)

c Primed Operators
      Do i=1,6
         Ceffp(i)=DCMPLX(0.D0,0.D0)
      End Do

      Ceffp(7)=(-4.D0/3.D0*Ft0Newp)*eta5**GMa(1)
     &     +(-1.D0/2.D0*At0Newp+4.D0/3.D0*Ft0Newp)*eta5**GMa(2)
      Ceffp(8)=(-1.D0/2.D0*Ft0Newp)*eta5**GMa(1)

      tmp1=DCMPLX(0.D0,0.D0)
      Do i=1,8
         Do j=1,8
            tmp1=tmp1+GMphi(i,j)*Ceff(i)*DCONJG(Ceff(i))
     &           +GMphi(i,j)*Ceffp(i)*DCONJG(Ceffp(i))
         End Do
      End Do

      BE_0=ordfac*alphaStr(mub,5,4)/PI*DBLE(tmp1)

c***********************************************************************
c The function P(E_0) (equation 18) Hurth, Lunghi and Porod.

c      KcTot = (-0.60979537,-0.0302965509)
      PE_0=ABS(KcTot
     &     +(1.D0+ordfac*alphaStr(muEW,5,4)/PI*LOG(muEW**2/mtEW**2))
     &     *GM_rfac*KtTot+epstot)**2
     &     +ABS((1.D0+ordfac*alphaStr(muEW,5,4)/PI*LOG(muEW**2/mtEW**2))
     &     *GM_rfac*KtTotpr+epstotpr)**2
     &     +BE_0
      
!      PE_0= 0.149669375

c Non perturbative effects (equation 16)
      NE_0=ordfac*0.0036D0

      BSGbr=GMBtoleps*ABS(DCONJG(KM(3,2))*KM(3,3)/KM(2,3))**2
     &     *6.D0*OSalphaEM/(PI*GMCfac)
     &     *(PE_0+NE_0)

c Find the SM value of the branching ratio
      Call bsgbrSMsub(debug,iOrd)

      BSGbrrat=BSGbr/BSGbrSM


      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Branching Ratio'
         Write(*,*) '***************************************'
         Write(*,*) 'SUSY: ', BSGbr
         Write(*,*) 'SM: ', BSGbrSM
         Write(*,*) 'SUSY/SM: ', BSGbrrat
         Write(*,*) '***************************************'
      End If

      Return
      End

c***********************************************************************

      Subroutine bsgbrSMsub(debug,iOrd)

c Subroutine that calculates the branching ratio for the decay :
c \overline{B}\to X_s\gamma using the formulae of hep-ph/0104034 and
c the magic numbers of hep-ph/0203135. Taking into account the comments 
c of hep-ph/0312260
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'bsg.h'

c Inputs
      Integer debug,iOrd

c Local
      Integer i,j

      Double Complex tmp1,tmp2,tmp3,tmp4

      Double Complex epsSM

      Double complex KCtot,KtTot,epstot

      Double Complex At1New,Ft1New,Et0New
      Double Complex At0New,Ft0New
      
      Double Complex Ceff(8)

      Double Precision BE_0,NE_0,PE_0,ordfac
      Double Precision x,z,eta5,mtEW,KMeta,GM_rfac

c Functions
      Double Precision alphaStr,MSrmass
      Double Precision GM_A0, GM_F0, GM_E0
      Double Precision GM_A0pr, GM_F0pr, GM_A1, GM_F1

      Double Complex GM_afun, GM_bfun

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         ordfac=1.D0
c      Else
c         ordfac=0.D0
c      End If

c Some useful quantities
c      print*,MSrmass(mu_up(2),4,5,4)
c      print*,MSrmass(mub,4,5,4)
c      print*,GM_bfun(z)
c      print*,GM_afun(z)


      eta5=alphaStr(muew,5,4)/alphaStr(mub,5,4)
      x=MSrmass(muew,6,5,4)**2/mw**2
      z=((MSrmass(mub,4,5,4)+MSrmass(mu_up(2),4,5,4))/2.D0)**2/m_b1s**2
c      z = ((MSrmass(mub,4,5,4)+1.25d0)/2.D0)**2/m_b1s**2
      mtEW=MSrmass(muew,6,5,4)
      KMeta=(KM(1,3)*DCONJG(KM(1,2)))/(KM(3,2)*DCONJG(KM(3,2)))
      
c The factor of r that appears in various formula using eqn. 71 of
c Hurth, Lunghi and Porod (hep-ph/0312260)

      GM_rfac=0.578D0
     &     *(alphaStr(mz,5,4)/0.1185D0)**(-1.D0)
     &     *(m_b1s/4.69D0)**(0.23D0)
     &     *(MSrmass(mu_up(2),4,4,4)/1.25D0)**(-0.003D0)
     &     *(muew/165.D0)**(-0.08D0)
     &     *(mub/4.69D0)**(0.006D0)

c***********************************************************************
c Perturbative Contributions (The function P(E_0)) (Eqn. 18 HLP)
c***********************************************************************
c Charm quark contributions (eqn 19)

      KCtot=DCMPLX(0.D0,0.D0)

      Do i=1,8
         KCtot=KCtot+eta5**(GMa(i))*(GMd(i)
     &        +ordfac*alphaStr(mub,5,4)/(4.D0*PI)
     &        *(2.D0*23.D0/3.D0*GMa(i)*GMd(i)
     &        *(LOG(m_b1s/mub)+eta5*LOG(muew/mw))+GMdt(i)+GMdtn(i)*eta5
     &        +GMdta(i)*GM_afun(z)+GMdtb(i)*GM_bfun(z)
     &        +GMdtpi(i)*DCMPLX(0.D0,1.D0)*PI)
     &        +KMeta*alphaStr(mub,5,4)/(4.D0*PI)*(GMdta(i)*GM_bfun(z)
     &        +GMdtb(i)*GM_bfun(z)))
      End Do

c***********************************************************************
c Top quark contributions

      At0New=GM_A0(x)
      Ft0New=GM_F0(x)
      At1New=GM_A1(x)
      Ft1New=GM_F1(x)
      Et0New=GM_E0(x)

c Formula 3.8 of Gambino and Misiak

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,8
         tmp4=tmp4+Et0New*GMe(i)*eta5**(GMa(i)+11.D0/23.D0)
      End Do
      
      tmp1=(1.D0-2.D0/9.D0*alphaStr(m_b1s,5,4)**2*ordfac)
     &     *(-1.D0/2.D0*eta5**(4.D0/23.D0)*At0New
     &     +4.D0/3.D0*(eta5**(4.D0/23.D0)-eta5**(2.D0/23.D0))*Ft0New)

      tmp2=ordfac*alphaStr(muew,5,4)/PI*LOG(muew/MSrmass(muew,6,5,4))
     &     *4.D0*x*(-1.D0/2.D0*eta5**(4.D0/23.D0)*GM_A0pr(x)
     &     +4.D0/3.D0*(eta5**(4.D0/23.D0)-eta5**(2.D0/23.D0))
     &     *GM_F0pr(x))

      tmp3=eta5**(4.D0/23.D0)*(-1.D0/2.D0*eta5*At1New
     &     +(12523.D0/3174.D0-7411.D0/4761.D0*eta5-2.D0/9.D0*PI**2)
     &     *At0New
     &     -4.D0/3.D0*LOG(m_b1s/mub)*At0New
     &     -4.D0/3.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_A0(x)
     &     +4.D0/3.D0*eta5*Ft1New
     &     +(-50092.D0/4761.D0+1110842.D0/357075.D0*eta5
     &     +16.D0/27.D0*PI**2)*Ft0New
     &     +32.D0/9.D0*LOG(m_b1s/mub)*Ft0New
     &     +32.D0/9.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_F0(x))
     &     +eta5**(2.D0/23.D0)*(-4.D0/3.D0*eta5*Ft1New
     &     +(2745458.D0/357075.D0-38890.D0/14283.D0*eta5
     &     -4.D0/9.D0*PI*(PI+DCMPLX(0.D0,1.D0)))*Ft0New
     &     -16.D0/9.D0*LOG(m_b1s/mub)*Ft0New
     &     -16.D0/9.D0*eta5*LOG(muew/MSrmass(muew,6,5,4))*GM_F0(x))

      KtTot=tmp1+tmp2+ordfac*alphaStr(mub,5,4)/(4.D0*PI)*(tmp3+tmp4)

c***********************************************************************
c Electroweak Corrections

      epsSM=0.0071D0

      epsTot=ordfac*epsSM

c***********************************************************************
c Bremstrahlung Corrections
c Equation E.1 of Gambino and Misiak

      At0New=GM_A0(x)
      Ft0New=GM_F0(x)

      Ceff(1)=eta5**GMa(3)-eta5**GMa(4)
      Ceff(2)=2.D0/3.D0*eta5**GMa(3)+1.D0/3.D0*eta5**GMa(4)
      Ceff(3)=2.D0/63.D0*eta5**GMa(3)-1.D0/27.D0*eta5**GMa(4)
     &     -0.0659D0*eta5**GMa(5)+0.0595D0*eta5**GMa(6)
     &     -0.0218D0*eta5**GMa(7)+0.0335D0*eta5**GMa(8)
      Ceff(4)=1.D0/21.D0*eta5**GMa(3)+1.D0/9.D0*eta5**GMa(4)
     &     +0.0237D0*eta5**GMa(5)-0.0173D0*eta5**GMa(6)
     &     -0.1336D0*eta5**GMa(7)-0.0316D0*eta5**GMa(8)
      Ceff(5)=-1.D0/126.D0*eta5**GMa(3)+1.D0/108.D0*eta5**GMa(4)
     &     +0.0094D0*eta5**GMa(5)-0.0100D0*eta5**GMa(6)
     &     +0.0010D0*eta5**GMa(7)-0.0017D0*eta5**GMa(8)
      Ceff(6)=-1.D0/84.D0*eta5**GMa(3)-1.D0/36.D0*eta5**GMa(4)
     &     +0.0108D0*eta5**GMa(5)+0.0163D0*eta5**GMa(6)
     &     +0.0103D0*eta5**GMa(7)+0.0023D0*eta5**GMa(8)
      Ceff(7)=(-4.D0/3.D0*Ft0New+42678.D0/30253.D0)*eta5**GMa(1)
     &     +(-1.D0/2.D0*At0New+4.D0/3.D0*Ft0New
     &     -86697.D0/103460.D0)*eta5**GMa(2)
     &     -3.D0/7.D0*eta5**GMa(3)-1.D0/14.D0*eta5**GMa(4)
     &     -0.6494D0*eta5**GMa(5)-0.0380D0*eta5**GMa(6)
     &     -0.0185D0*eta5**GMa(7)-0.0057D0*eta5**GMa(8)
      Ceff(8)=(-1.D0/2.D0*Ft0New+64017.D0/121012.D0)*eta5**GMa(1)
     &     -0.9135D0*eta5**GMa(5)+0.0873D0*eta5**GMa(6)
     &     -0.0571D0*eta5**GMa(7)+0.0209D0*eta5**GMa(8)

      tmp1=DCMPLX(0.D0,0.D0)
      Do j=1,8
         Do i=1,j
            tmp1=tmp1+GMphi(i,j)*Ceff(i)*DCONJG(Ceff(i))
         End Do
      End Do

      BE_0=ordfac*alphaStr(mub,5,4)/PI*DBLE(tmp1)

c***********************************************************************
c The function P(E_0) (equation 18) Hurth, Lunghi and Porod.

      PE_0=ABS(KcTot
     &     +(1.D0+ordfac*alphaStr(muEW,5,4)/PI*LOG(muEW**2/mtEW**2))
     &     *GM_rfac*KtTot+epstot)**2
     &     +BE_0

c Non perturbative effects (equation 16)
      NE_0=0.0036D0*ordfac


      BSGbrSM=GMBtoleps*ABS((DCONJG(KM(3,2))*KM(3,3))/KM(2,3))**2
     &     *6.D0*OSalphaEM/(PI*GMCfac)
     &     *(PE_0+NE_0)

      Return
      End

