c**********************************************************************

      Subroutine bsgWC(debug,iOrd,WilsonC,WilsonCp)

c Subroutine that calculates the various contributions to the Wilson
c coefficients relevant to the decay b\to s\gamma. Matching conditions
c etc. are generally taken from hep-ph/0308102 except for the NLO SM
c and Higgs contributions which are taken from hep-ph/9904413 due to 
c a typo concerning the definitions of the Wilson coefficients in the
c appendix of hep-ph/0308102. Neutral Higgs contributions are
c calculated using the matching condition hep-ph/0210145 (originally
c discussed in hep-ph/0207036) if undesired simply comment out the
c relevant lines.
c**********************************************************************

      Implicit NONE

c Common blocks
      Include 'const.h'
      Include 'insm.h'
      Include 'insoft.h'
      Include 'vars.h'
      Include 'vertex.h'
      Include 'bsg.h'

c Inputs
      Integer debug,iOrd

c Output
      Double Complex WilsonC(2,8), WilsonCp(2,8)

c Local
      Integer i,j
      Double Precision x,ctb2,gwEW,gwSU,c2fac
      Double Complex tmp1,tmp2,tmp3
      Double Complex SUSYWilcLO(2), SUSYWilcNLO(2)
      Double Complex SUSYWilcPLO(2), SUSYWilcPNLO(2)
      Double Complex SMc7EV,SMc8EV,SMc7prEV,SMc8prEV

c Functions
      Double Precision MSrmass,alphaStr,alpha2

c Loop functions
      Double Precision OR_F71,OR_F72,OR_F81,OR_F82,BMU_GEW7,BMU_DeltaEW7
      Double Precision BMU_GEW8,BMU_DeltaEW8,BMU_GCH7,BMU_DeltaCH7
      Double Precision BMU_GCH8,BMU_DeltaCH8,BMU_ECH
      Double Precision OR_H71,OR_H72,OR_H73,OR_H74,OR_H75,OR_H76
      Double Precision OR_H71p,OR_H72p,OR_H73p,OR_H74p,OR_H75p,OR_H76p
      Double Precision OR_H71pp,OR_H72pp,OR_H73pp,OR_H74pp,OR_H75pp
      Double Precision OR_H76pp
      Double Precision OR_H81,OR_H82,OR_H83,OR_H84,OR_H85,OR_H86
      Double Precision OR_H81p,OR_H82p,OR_H83p,OR_H84p,OR_H85p,OR_H86p
      Double Precision OR_H81pp,OR_H82pp,OR_H83pp,OR_H84pp,OR_H85pp
      Double Precision OR_H86pp

c Paramters
      Double Precision OR_Lambda7, OR_Lambda8
      Parameter(OR_Lambda7=0.8333333333333333D0,OR_Lambda8=0.5D0)

      gwEW=SQRT(alpha2(muew)*4.D0*PI)
      gwSU=SQRT(alpha2(mususy)*4.D0*PI)
      c2fac=4.D0/3.D0

c**********************************************************************
c Initialise Wilson Coefficients
      Do i=1,2
         Do j=1,8
            SMBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            SMBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            CHBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            CHBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            GINOBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            GINOBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            CINOBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            CINOBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            NINOBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            NINOBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            NHBSGWilc(i,j)=DCMPLX(0.D0,0.D0)
            NHBSGWilcp(i,j)=DCMPLX(0.D0,0.D0)
            WilsonC(i,j)=DCMPLX(0.D0,0.D0)
            WilsonCp(i,j)=DCMPLX(0.D0,0.D0)
         End Do
      End Do

c**********************************************************************
c SM Contributions
c NOTE LO and NLO results are treated in bsgbrsub.f therefore these
c results are for comparison etc only.      

      x=MSrmass(muew,6,5,4)**2/mw**2

c For the Wilson coefficient C7 :

c LO Standard Model Result - Eqn 4.4
      SMBSGWilc(1,7)=OR_F71(x)

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor
      SMc7EV=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqL(2,2,3))*delCHqqL(2,3,3)
     &     +DCONJG(delCHqqL(2,2,3))*CHqqL(2,3,3))
     &     *OR_F71(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqL(2,2,3))*delCHqqR(2,3,3)
     &     +DCONJG(delCHqqL(2,2,3))*CHqqR(2,3,3))
     &     *OR_F72(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         SMc7EV=SMc7EV/alphaStr(muew,5,4)*4.D0*PI
      Else
         SMc7EV=DCMPLX(0.D0,0.D0)
      End If

c NLO gluon result E.1

      SMBSGWilc(2,7)=BMU_GEW7(x)
     &     +BMU_DeltaEW7(x)*Log((muew/MSrmass(muew,6,5,4))**2)
     &     +SMc7EV

c For the Wilson coefficient C8 :

c LO Standard Model Result - Eqn 4.4
      SMBSGWilc(1,8)=OR_F81(x)
 
c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor
      SMc8EV=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqL(2,2,3))*delCHqqL(2,3,3)
     &     +DCONJG(delCHqqL(2,2,3))*CHqqL(2,3,3))
     &     *OR_F81(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqL(2,2,3))*delCHqqR(2,3,3)
     &     +DCONJG(delCHqqL(2,2,3))*CHqqR(2,3,3))
     &     *OR_F82(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         SMc8EV=SMc8EV/alphaStr(muew,5,4)*4.D0*PI
      Else
         SMc8EV=DCMPLX(0.D0,0.D0)
      End If

c NLO gluon result E.1

         SMBSGWilc(2,8)=BMU_GEW8(x)
     &        +BMU_DeltaEW8(x)*Log((muew/MSrmass(muew,6,5,4))**2)
     &        +SMc8EV

c For the Wilson coefficient C7pr :

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor
      SMc7prEV=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqR(2,2,3))*delCHqqR(2,3,3)
     &     +DCONJG(delCHqqR(2,2,3))*CHqqR(2,3,3))
     &     *OR_F71(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqR(2,2,3))*delCHqqL(2,3,3)
     &     +DCONJG(delCHqqR(2,2,3))*CHqqL(2,3,3))
     &     *OR_F72(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         SMc7prEV=SMc7prEV/alphaStr(muew,5,4)*4.D0*PI
      Else
         SMc7prEV=DCMPLX(0.D0,0.D0)
      End If

      SMBSGWilcp(2,7)=SMc7prEV

c For the Wilson coefficient C8pr :

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor
      SMc8prEV=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqR(2,2,3))*delCHqqR(2,3,3)
     &     +DCONJG(delCHqqR(2,2,3))*CHqqR(2,3,3))
     &     *OR_F81(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqR(2,2,3))*delCHqqL(2,3,3)
     &     +DCONJG(delCHqqR(2,2,3))*CHqqL(2,3,3))
     &     *OR_F82(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         SMc8prEV=SMc8prEV/alphaStr(muew,5,4)*4.D0*PI
      Else
         SMc8prEV=DCMPLX(0.D0,0.D0)
      End If

      SMBSGWilcp(2,8)=SMc8prEV

c**********************************************************************
c Charged Higgs Contributions

      x=MSrmass(muew,6,5,4)**2/mhc**2
      ctb2=1.D0/(tanb**2)

c For the Wilson coefficient C4 :
      CHBSGWilc(2,4)=BMU_ECH(x,ctb2)

c For the Wilson coefficient C7 :

c LO Result - Eqn 4.4
      CHBSGWilc(1,7)=ctb2/(3.D0)*OR_F71(x)+OR_F72(x)

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor hence
c the factor alphaS(muew)/4PI

      tmp1=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqL(1,2,3))*delCHqqL(1,3,3)
     &     +DCONJG(delCHqqL(1,2,3))*CHqqL(1,3,3))
     &     *OR_F71(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqL(1,2,3))*delCHqqR(1,3,3)
     &     +DCONJG(delCHqqL(1,2,3))*CHqqR(1,3,3))
     &     *OR_F72(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then      
         tmp1=tmp1/alphaStr(muew,5,4)*4.D0*PI
      Else
         tmp1=DCMPLX(0.D0,0.D0)
      End If

c NLO contributions including the gluon result

      CHBSGWilc(2,7)=BMU_GCH7(x,ctb2)
     &     +BMU_DeltaCH7(x,ctb2)*Log((muew/MSrmass(muew,6,5,4))**2)+tmp1

c For the Wilson coefficient C8 :

c LO Standard Model Result - Eqn 4.4
      CHBSGWilc(1,8)=ctb2/(3.D0)*OR_F81(x)+OR_F82(x)

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)Cor

      tmp1=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqL(1,2,3))*delCHqqL(1,3,3)
     &     +DCONJG(delCHqqL(1,2,3))*CHqqL(1,3,3))
     &     *OR_F81(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqL(1,2,3))*delCHqqR(1,3,3)
     &     +DCONJG(delCHqqL(1,2,3))*CHqqR(1,3,3))
     &     *OR_F82(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then      
         tmp1=tmp1/alphaStr(muew,5,4)*4.D0*PI
      Else
         tmp1=DCMPLX(0.D0,0.D0)
      End If

c NLO contributions including the gluon result

      CHBSGWilc(2,8)=BMU_GCH8(x,ctb2)
     &     +BMU_DeltaCH8(x,ctb2)*Log((muew/MSrmass(muew,6,5,4))**2)+tmp1

c For the Wilson coefficient C7pr :

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within delCHqq(L/R)
      tmp1=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqR(1,2,3))*delCHqqR(1,3,3)
     &     +DCONJG(delCHqqR(1,2,3))*CHqqR(1,3,3))
     &     *OR_F71(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqR(1,2,3))*delCHqqL(1,3,3)
     &     +DCONJG(delCHqqR(1,2,3))*CHqqL(1,3,3))
     &     *OR_F72(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then      
         tmp1=tmp1/alphaStr(muew,5,4)*4.D0*PI
      Else
         tmp1=DCMPLX(0.D0,0.D0)
      End If

      CHBSGWilcp(2,7)=tmp1
      
c For the Wilson coefficient C8pr :

c LO result using effective vertices - Eqn 4.7
c NOTE - The strong coupling is absorbed within CHqq(L/R)
      tmp1=2.D0/(3.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &     *mw**2/MSrmass(muew,6,5,4)**2
     &     *((DCONJG(CHqqR(1,2,3))*delCHqqR(1,3,3)
     &     +DCONJG(delCHqqR(1,2,3))*CHqqR(1,3,3))
     &     *OR_F81(x)
     &     +3.D0*MSrmass(muew,6,5,4)/MSrmass(muew,5,5,4)
     &     *(DCONJG(CHqqR(1,2,3))*delCHqqL(1,3,3)
     &     +DCONJG(delCHqqR(1,2,3))*CHqqL(1,3,3))
     &     *OR_F82(x))

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then      
         tmp1=tmp1/alphaStr(muew,5,4)*4.D0*PI
      Else
         tmp1=DCMPLX(0.D0,0.D0)
      End If

      CHBSGWilcp(2,8)=tmp1
      
c***********************************************************************
c Supersymmetric Contributions
c NOTE : All these contributions are evaluated at the SUSY scale m_susy
c and need to be first evolved down to the scale mu_w via QCD RGE flow.
c***********************************************************************
c C7
c Chargino Contributions

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*DCONJG(KM(3,2))*KM(3,3))
     &           *(mw**2/MC(i)**2)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqL(3,i,j)
     &           *OR_H71(MUpS(j)**2/MC(i)**2))
            tmp3=tmp3
     &           +1.D0/(gwSU**2*DCONJG(KM(3,2))*KM(3,3))
     &           *(mw**2/MC(i)**2)
     &           *(MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqR(3,i,j))
     &           *(OR_H72(MUpS(j)**2/MC(i)**2)+OR_Lambda7))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)*((DCONJG(cqsqL(2,i,j))*cqsqL(3,i,j)
     &           *(OR_H71p(MUpS(j)**2/MC(i)**2)
     &           +OR_H71pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2)))
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqR(3,i,j)
     &           *(OR_H72p(MUpS(j)**2/MC(i)**2)
     &           +OR_H72pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2))))
         End Do
      End Do

      CINOBSGWilc(1,7)=tmp1+tmp3
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         CINOBSGWilc(2,7)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)      
      Do i=1,4
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *(DCONJG(nqsqL(2,i,j))*nqsqL(3,i,j)
     &           *OR_H73(MDoS(j)**2/MN(i)**2)
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqL(2,i,j))*nqsqR(3,i,j))
     &           *(OR_H74(MDoS(j)**2/MN(i)**2)))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *((DCONJG(nqsqL(2,i,j))*nqsqL(3,i,j)
     &           *(OR_H73p(MDoS(j)**2/MN(i)**2)
     &           +OR_H73pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2)))
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqL(2,i,j))*nqsqR(3,i,j)
     &           *(OR_H74p(MDoS(j)**2/MN(i)**2)
     &           +OR_H74pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2))))
         End Do
      End Do

      NINOBSGWilc(1,7)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         NINOBSGWilc(2,7)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)      
      Do j=1,6
         tmp1=tmp1
     &        +c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *(DCONJG(gqsqL(2,j))*gqsqL(3,j)
     &        *OR_H75(MDoS(j)**2/MGl**2)
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *DCONJG(gqsqL(2,j))*gqsqR(3,j)
     &        *OR_H76(MDoS(j)**2/MGl**2))
         tmp2=tmp2+c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *((DCONJG(gqsqL(2,j))*gqsqL(3,j)
     &        *(OR_H75p(MDoS(j)**2/MGl**2)
     &        +OR_H75pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2)))
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqL(2,j))*gqsqR(3,j)
     &        *(OR_H76p(MDoS(j)**2/MGl**2)
     &        +OR_H76pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2))))
      End Do

      GINOBSGWilc(1,7)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         GINOBSGWilc(2,7)=tmp2
      End If

c C8
c Chargino Contributions

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqL(3,i,j)
     &           *OR_H81(MUpS(j)**2/MC(i)**2)
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqR(3,i,j))
     &           *(OR_H82(MUpS(j)**2/MC(i)**2)+OR_Lambda8))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *((DCONJG(cqsqL(2,i,j))*cqsqL(3,i,j)
     &           *(OR_H81p(MUpS(j)**2/MC(i)**2)
     &           +OR_H81pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2)))
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqL(2,i,j))*cqsqR(3,i,j)
     &           *(OR_H82p(MUpS(j)**2/MC(i)**2)
     &           +OR_H82pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2))))
         End Do
      End Do

      CINOBSGWilc(1,8)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         CINOBSGWilc(2,8)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *DCONJG(nqsqL(2,i,j))*nqsqL(3,i,j)
     &           *OR_H83(MDoS(j)**2/MN(i)**2)
            tmp3=tmp3
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *MN(i)/MSrmass(mususy,5,6,4)
     &           *DCONJG(nqsqL(2,i,j))*nqsqR(3,i,j)
     &           *OR_H84(MDoS(j)**2/MN(i)**2)
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *((DCONJG(nqsqL(2,i,j))*nqsqL(3,i,j)
     &           *(OR_H83p(MDoS(j)**2/MN(i)**2)
     &           +OR_H83pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2)))
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqL(2,i,j))*nqsqR(3,i,j)
     &           *(OR_H84p(MDoS(j)**2/MN(i)**2)
     &           +OR_H84pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2))))
         End Do
      End Do
      NINOBSGWilc(1,8)=tmp1+tmp3
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         NINOBSGWilc(2,8)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do j=1,6
         tmp1=tmp1
     &        +c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *(DCONJG(gqsqL(2,j))*gqsqL(3,j)
     &        *OR_H85(MDoS(j)**2/MGl**2)
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqL(2,j))*gqsqR(3,j))
     &        *(OR_H86(MDoS(j)**2/MGl**2)))
         tmp2=tmp2+c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *((DCONJG(gqsqL(2,j))*gqsqL(3,j)
     &        *(OR_H85p(MDoS(j)**2/MGl**2)
     &        +OR_H85pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2)))
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqL(2,j))*gqsqR(3,j)
     &        *(OR_H86p(MDoS(j)**2/MGl**2)
     &        +OR_H86pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2))))
      End Do
      GINOBSGWilc(1,8)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         GINOBSGWilc(2,8)=tmp2
      End If

c C7PR
c Chargino Contributions

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqR(3,i,j)
     &           *OR_H71(MUpS(j)**2/MC(i)**2)
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqL(3,i,j))
     &           *(OR_H72(MUpS(j)**2/MC(i)**2)+OR_Lambda7))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *((DCONJG(cqsqR(2,i,j))*cqsqR(3,i,j)
     &           *(OR_H71p(MUpS(j)**2/MC(i)**2)
     &           +OR_H71pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2)))
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqL(3,i,j)
     &           *(OR_H72p(MUpS(j)**2/MC(i)**2)
     &           +OR_H72pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2))))
         End Do
      End Do
      CINOBSGWilcp(1,7)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         CINOBSGWilcp(2,7)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqR(3,i,j)
     &           *OR_H73(MDoS(j)**2/MN(i)**2)
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqL(3,i,j))
     &           *(OR_H74(MDoS(j)**2/MN(i)**2)))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *((DCONJG(nqsqR(2,i,j))*nqsqR(3,i,j)
     &           *(OR_H73p(MDoS(j)**2/MN(i)**2)
     &           +OR_H73pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2)))
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqL(3,i,j)
     &           *(OR_H74p(MDoS(j)**2/MN(i)**2)
     &           +OR_H74pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2))))
         End Do
      End Do
      NINOBSGWilcp(1,7)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         NINOBSGWilcp(2,7)=tmp2
      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do j=1,6
         tmp1=tmp1
     &        +c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *(DCONJG(gqsqR(2,j))*gqsqR(3,j)
     &        *OR_H75(MDoS(j)**2/MGl**2)
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqR(2,j))*gqsqL(3,j))
     &        *(OR_H76(MDoS(j)**2/MGl**2)))
         tmp2=tmp2+c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *((DCONJG(gqsqR(2,j))*gqsqR(3,j)
     &        *(OR_H75p(MDoS(j)**2/MGl**2)
     &        +OR_H75pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2)))
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqR(2,j))*gqsqL(3,j)
     &        *(OR_H76p(MDoS(j)**2/MGl)
     &        +OR_H76pp(MDoS(j)**2/MGl)
     &        *LOG(mususy**2/MDoS(j)**2))))
      End Do
      GINOBSGWilcp(1,7)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         GINOBSGWilcp(2,7)=tmp2
      End If

c C8primed
c Chargino Contributions
      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqR(3,i,j)
     &           *OR_H81(MUpS(j)**2/MC(i)**2)
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqL(3,i,j))
     &           *(OR_H82(MUpS(j)**2/MC(i)**2)+OR_Lambda8))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MC(i)**2)
     &           *((DCONJG(cqsqR(2,i,j))*cqsqR(3,i,j)
     &           *(OR_H81p(MUpS(j)**2/MC(i)**2)
     &           +OR_H81pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2)))
     &           +MC(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(cqsqR(2,i,j))*cqsqL(3,i,j)
     &           *(OR_H82p(MUpS(j)**2/MC(i)**2)
     &           +OR_H82pp(MUpS(j)**2/MC(i)**2)
     &           *LOG(mususy**2/MUpS(j)**2))))
         End Do
      End Do
      CINOBSGWilcp(1,8)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         CINOBSGWilcp(2,8)=tmp2
      End If


c Neutralino Contributions

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            tmp1=tmp1
     &           +1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqR(3,i,j)
     &           *OR_H83(MDoS(j)**2/MN(i)**2)
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqL(3,i,j))
     &           *(OR_H84(MDoS(j)**2/MN(i)**2)))
            tmp2=tmp2+1.D0/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &           *(mw**2/MN(i)**2)
     &           *((DCONJG(nqsqR(2,i,j))*nqsqR(3,i,j)
     &           *(OR_H83p(MDoS(j)**2/MN(i)**2)
     &           +OR_H83pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2)))
     &           +MN(i)/MSrmass(mususy,5,6,4)
     &           *(DCONJG(nqsqR(2,i,j))*nqsqL(3,i,j)
     &           *(OR_H84p(MDoS(j)**2/MN(i)**2)
     &           +OR_H84pp(MDoS(j)**2/MN(i)**2)
     &           *LOG(mususy**2/MDoS(j)**2))))
         End Do
      End Do
      NINOBSGWilcp(1,8)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         NINOBSGWilcp(2,8)=tmp2
      End If

c Gluino Contributions

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      Do j=1,6
         tmp1=tmp1
     &        +c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *(DCONJG(gqsqR(2,j))*gqsqR(3,j)
     &        *OR_H85(MDoS(j)**2/MGl**2)
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqR(2,j))*gqsqL(3,j))
     &        *(OR_H86(MDoS(j)**2/MGl**2)))
         tmp2=tmp2+c2fac/(gwSU**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(mw**2/MGl**2)
     &        *((DCONJG(gqsqR(2,j))*gqsqR(3,j)
     &        *(OR_H85p(MDoS(j)**2/MGl**2)
     &        +OR_H85pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2)))
     &        +MGl/MSrmass(mususy,5,6,4)
     &        *(DCONJG(gqsqR(2,j))*gqsqL(3,j)
     &        *(OR_H86p(MDoS(j)**2/MGl**2)
     &        +OR_H86pp(MDoS(j)**2/MGl**2)
     &        *LOG(mususy**2/MDoS(j)**2))))
      End Do
      GINOBSGWilcp(1,8)=tmp1
      If (iOrd.eq.2.or.iOrd.eq.3) Then
         GINOBSGWilcp(2,8)=tmp2
      End If

c***********************************************************************
c RGE evolution

c Evolution from mususy to mu_ew

      Do i=1,2
         SUSYWilcLO(i)=GINOBSGWilc(1,i+6)+NINOBSGWilc(1,i+6)
     &        +CINOBSGWilc(1,i+6)
         SUSYWilcNLO(i)=GINOBSGWilc(2,i+6)+NINOBSGWilc(2,i+6)
     &        +CINOBSGWilc(2,i+6)
      End Do

c Primed operators
      Do i=1,2
         SUSYWilcpLO(i)=GINOBSGWilcp(1,i+6)+NINOBSGWilcp(1,i+6)
     &        +CINOBSGWilcp(1,i+6)
         SUSYWilcpNLO(i)=GINOBSGWilcp(2,i+6)+NINOBSGWilcp(2,i+6)
     &        +CINOBSGWilcp(2,i+6)
      End Do

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'SUSY Contributions before evolution'
         Write(*,*) 'c7 LO :',SUSYWilcLO(1)
         Write(*,*) 'c8 LO:',SUSYWilcLO(2)
         Write(*,*) 'c7p LO:',SUSYWilcpLO(1)
         Write(*,*) 'c8p LO:',SUSYWilcpLO(2)
         Write(*,*) 'c7 NLO:',SUSYWilcNLO(1)
         Write(*,*) 'c8 NLO:',SUSYWilcNLO(2)
         Write(*,*) 'c7p NLO:',SUSYWilcpNLO(1)
         Write(*,*) 'c8p NLO:',SUSYWilcpNLO(2)
      End If

      Call BSGwcev6to5(debug,SUSYWilcLO,SUSYWilcNLO,mususy,muew)
      Call BSGwcev6to5(debug,SUSYWilcpLO,SUSYWilcpNLO,mususy,muew)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'SUSY Contributions after evolution'
         Write(*,*) 'c7 LO :',SUSYWilcLO(1)
         Write(*,*) 'c8 LO:',SUSYWilcLO(2)
         Write(*,*) 'c7p LO:',SUSYWilcpLO(1)
         Write(*,*) 'c8p LO:',SUSYWilcpLO(2)
         Write(*,*) 'c7 NLO:',SUSYWilcNLO(1)
         Write(*,*) 'c8 NLO:',SUSYWilcNLO(2)
         Write(*,*) 'c7p NLO:',SUSYWilcpNLO(1)
         Write(*,*) 'c8p NLO:',SUSYWilcpNLO(2)
      End If

      If (iOrd.eq.0) Then
         Do i=1,2
            SUSYWilcNLO(i)=DCMPLX(0.D0,0.D0)
            SUSYWilcpNLO(i)=DCMPLX(0.D0,0.D0)
         End Do
      End If

c***********************************************************************
c Neutral Higgs contributions

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         NHBSGWilc(2,7)=MW**2/(18.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(SHqqRCor(1,2,3)*SHqqLCor(1,3,3)
     &        /mHh0**2
     &        +SHqqRCor(2,2,3)*SHqqLCor(2,3,3)
     &        /mLh0**2
     &        +PHqqRCor(1,2,3)*PHqqLCor(1,3,3)
     &        /ma0**2
     &        +PHqqRCor(2,2,3)*PHqqLCor(2,3,3)
     &        /mz**2)
     &        /alphaStr(muEW,5,4)*4.D0*PI
         
         NHBSGWilc(2,8)=MW**2/(18.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(SHqqRCor(1,2,3)*SHqqLCor(1,3,3)
     &        /mHh0**2
     &        +SHqqRCor(2,2,3)*SHqqLCor(2,3,3)
     &        /mLh0**2
     &        +PHqqRCor(1,2,3)*PHqqLCor(1,3,3)
     &        /ma0**2
     &        +PHqqRCor(2,2,3)*PHqqLCor(2,3,3)
     &        /mz**2)
     &        /alphaStr(muEW,5,4)*4.D0*PI

         NHBSGWilcp(2,7)=MW**2/(18.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(SHqqLCor(1,2,3)*SHqqRCor(1,3,3)
     &        /mHh0**2
     &        +SHqqLCor(2,2,3)*SHqqRCor(2,3,3)
     &        /mLh0**2
     &        +PHqqLCor(1,2,3)*PHqqRCor(1,3,3)
     &        /ma0**2
     &        +PHqqLCor(2,2,3)*PHqqRCor(2,3,3)
     &        /mz**2)
     &        /alphaStr(muEW,5,4)*4.D0*PI
         
         NHBSGWilcp(2,8)=MW**2/(18.D0*gwEW**2*(DCONJG(KM(3,2))*KM(3,3)))
     &        *(SHqqLCor(1,2,3)*SHqqRCor(1,3,3)
     &        /mHh0**2
     &        +SHqqLCor(2,2,3)*SHqqRCor(2,3,3)
     &        /mLh0**2
     &        +PHqqLCor(1,2,3)*PHqqRCor(1,3,3)
     &        /ma0**2
     &        +PHqqLCor(2,2,3)*PHqqRCor(2,3,3)
     &        /mz**2)
     &        /alphaStr(muEW,5,4)*4.D0*PI

      End If

c***********************************************************************
c Debug Writes

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Standard Model Contributions'
         Write(*,*) 'c7 LO :',SMBSGWilc(1,7)
         Write(*,*) 'c8 LO:',SMBSGWilc(1,8)
         Write(*,*) 'c7p LO:',SMBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',SMBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',SMBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',SMBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',SMBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',SMBSGWilcp(2,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Charged Higgs Contributions'
         Write(*,*) 'c7 LO :',CHBSGWilc(1,7)
         Write(*,*) 'c8 LO:',CHBSGWilc(1,8)
         Write(*,*) 'c7p LO:',CHBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',CHBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',CHBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',CHBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',CHBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',CHBSGWilcp(2,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Chargino Contributions'
         Write(*,*) 'c7 LO :',CINOBSGWilc(1,7)
         Write(*,*) 'c8 LO:',CINOBSGWilc(1,8)
         Write(*,*) 'c7p LO:',CINOBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',CINOBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',CINOBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',CINOBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',CINOBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',CINOBSGWilcp(2,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Neutralino Contributions'
         Write(*,*) 'c7 LO :',NINOBSGWilc(1,7)
         Write(*,*) 'c8 LO:',NINOBSGWilc(1,8)
         Write(*,*) 'c7p LO:',NINOBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',NINOBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',NINOBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',NINOBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',NINOBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',NINOBSGWilcp(2,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Gluino Contributions'
         Write(*,*) 'c7 LO :',GINOBSGWilc(1,7)
         Write(*,*) 'c8 LO:',GINOBSGWilc(1,8)
         Write(*,*) 'c7p LO:',GINOBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',GINOBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',GINOBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',GINOBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',GINOBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',GINOBSGWilcp(2,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Neutral Higgs Contributions'
         Write(*,*) 'c7 LO :',NHBSGWilc(1,7)
         Write(*,*) 'c8 LO:',NHBSGWilc(1,8)
         Write(*,*) 'c7p LO:',NHBSGWilcp(1,7)
         Write(*,*) 'c8p LO:',NHBSGWilcp(1,8)
         Write(*,*) 'c7 NLO:',NHBSGWilc(2,7)
         Write(*,*) 'c8 NLO:',NHBSGWilc(2,8)
         Write(*,*) 'c7p NLO:',NHBSGWilcp(2,7)
         Write(*,*) 'c8p NLO:',NHBSGWilcp(2,8)
      End If

c***********************************************************************
c Total Wilson coefficients

      WilsonC(1,7)=CHBSGWilc(1,7)+SUSYWilcLO(1)+NHBSGWilc(1,7)
      WilsonC(1,8)=CHBSGWilc(1,8)+SUSYWilcLO(2)+NHBSGWilc(1,8)

      WilsonCp(1,7)=CHBSGWilcp(1,7)+SUSYWilcpLO(1)+NHBSGWilcp(1,7)
      WilsonCp(1,8)=CHBSGWilcp(1,8)+SUSYWilcpLO(2)+NHBSGWilcp(1,8)

      WilsonC(2,4)=CHBSGWilc(2,4)+NINOBSGWilc(2,4)+GINOBSGWilc(2,4)
     &     +CINOBSGWilc(2,4)+NHBSGWilc(2,4)
      WilsonC(2,7)=CHBSGWilc(2,7)+SUSYWilcNLO(1)+NHBSGWilc(2,7)
     &     +SMc7EV
      WilsonC(2,8)=CHBSGWilc(2,8)+SUSYWilcNLO(2)+NHBSGWilc(2,8)
     &     +SMc8EV
      WilsonCp(2,4)=CHBSGWilcp(2,4)+NINOBSGWilcp(2,4)
     &     +GINOBSGWilcp(2,4)+CINOBSGWilcp(2,4)+NHBSGWilcp(2,4)
      WilsonCp(2,7)=CHBSGWilcp(2,7)+SUSYWilcpNLO(1)+NHBSGWilcp(2,7)
     &     +SMc7prEV
      WilsonCp(2,8)=CHBSGWilcp(2,8)+SUSYWilcpNLO(2)+NHBSGWilcp(2,8)
     &     +SMc8prEV

      Do i=1,2
         Do j=1,8
            BSMBSGWilc(i,j)=WilsonC(i,j)
            BSMBSGWilcp(i,j)=WilsonCp(i,j)
         End Do
      End Do

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'BSM Contributions'
         Write(*,*) 'c7 LO :',WilsonC(1,7)
         Write(*,*) 'c8 LO:',WilsonC(1,8)
         Write(*,*) 'c7p LO:',WilsonCp(1,7)
         Write(*,*) 'c8p LO:',WilsonCp(1,8)
         Write(*,*) 'c7 NLO:',WilsonC(2,7)
         Write(*,*) 'c8 NLO:',WilsonC(2,8)
         Write(*,*) 'c7p NLO:',WilsonCp(2,7)
         Write(*,*) 'c8p NLO:',WilsonCp(2,8)
      End If


      Return
      End
