c***********************************************************************

      Subroutine BSGwcev6to5(debug,WilsonCLO,WilsonCNLO,mu_High,mu_low)

c Subroutine that given the relevant Wilson coefficients evolves them
c from the energy scale mu0 to the energy scale mu1 using the RGEs
c with 6 active flavours.
c***********************************************************************

      Implicit NONE

      Include 'vars.h'

c Inputs
      Integer debug

c The start point and end point for the RG evolution
      Double Precision mu_high, mu_low

c The array WilsonC0 contains the LO Wilson coefficients at the scale
c mu_high on output it contains the LO Wilson coefficients at the 
c scale mu_low. The array Wilson C1 contains the NLO contributions 
c to the Wilson coefficients and hence only undergoes LO evolution. On
c output it contains the NLO contributions after evolution.
c They should have the following form :
c WilsonC(1) : c7
c WilsonC(2) : c8
      Double Complex WilsonCLO(2), WilsonCNLO(2)

c Local
      Integer i,j
      Double Precision eta6
      Double Complex tmp1,tmp2
      
c The LO and NLO evolution matrices
      Double Precision Uevol0(2,2)
      Double Precision Uevol1(2,2)

c Temporary arrays in which the coefficients are stored during evolution
      Double Complex WilsonCLO_0(2), WilsonCNLO_0(2)
      
c Functions
      Double Precision alphaStr

c Expressions for U_6 are taken from 4.10, 4.11 of hep-ph/0308102
c evaluated in Mathematica and retaining terms up to O(alphaS).

      eta6=alphaStr(mu_high,6,4)/alphaStr(mu_low,6,4)
      
      Do i=1,2
         Do j=1,2
            Uevol0(i,j)=0.D0
            Uevol1(i,j)=0.D0
         End Do
      End Do

      Uevol0(1,1)=eta6**(16.D0/21.D0)
      Uevol0(1,2)=8.D0/3.D0*(eta6**(2.D0/3.D0)-eta6**(16.D0/21.D0))
      Uevol0(2,1)=0.D0
      Uevol0(2,2)=eta6**(2.D0/3.D0)

      Uevol1(1,1)=(440.D0/49.D0*(eta6**(37.D0/21.D0)
     &     -eta6**(16.D0/21.D0)))
      Uevol1(1,2)=(584.D0/27.D0*eta6**(5.D0/3.D0)
     &     +3520.D0/147.D0*eta6**(16.D0/21.D0)
     &     -76256.D0/3381.D0*eta6**(37.D0/21.D0)
     &     -14296.D0/621.D0*eta6**(2.D0/3.D0))
      Uevol1(2,1)=0.D0
      Uevol1(2,2)=(73.D0/9.D0*(eta6**(5.D0/3.D0)-eta6**(2.D0/3.D0)))

      Do i=1,2
         WilsonCLO_0(i)=WilsonCLO(i)
         WilsonCNLO_0(i)=WilsonCNLO(i)
      End Do

      Do i=1,2
         tmp1=DCMPLX(0.D0,0.D0)
         tmp2=DCMPLX(0.D0,0.D0)
         Do j=1,2
            tmp1=tmp1+Uevol0(i,j)*WilsonCLO_0(j)
            tmp2=tmp2+Uevol0(i,j)*WilsonCNLO_0(j)
     &           +Uevol1(i,j)*WilsonCLO_0(j)
         End Do
         WilsonCLO(i)=tmp1
         WilsoncNLO(i)=tmp2
      End Do
      
      Return
      End

