c***********************************************************************

      Subroutine bsmbrsub(debug,genq,WilsonC,WilsonCp)

c Subroutine that given the relevant Wilson coefficients calculates the 
c relevant branching ratio of the decay $\overline{B}_s\to\mu\mu$ using
c formula 3.6 of hep-ph/0204225
c***********************************************************************

      Implicit NONE

      Include 'insm.h'
      Include 'const.h'
      Include 'vars.h'
      Include 'bsm.h'

c Input
      Integer debug,genq
c The total Wilson coefficients
      Double Complex WilsonC(3), WilsonCp(3)

c Local
      Double Precision m_hat
      Double Complex F_S,F_P,F_A,F_ASM

c Functions
      Double Precision alphaEM,MSrmass

      m_hat=m_el(2)/BSMm_B(genq)

c***********************************************************************

      F_S=BSMm_B(genq)*(WilsonC(2)*MSrmass(mub,5,5,4)
     &     -WilsonCp(2)*MSrmass(mub,3,5,4))
     &     /(MSrmass(mub,5,5,4)+MSrmass(mub,3,5,4))
      F_P=BSMm_B(genq)*(WilsonC(3)*MSrmass(mub,5,5,4)
     &     -WilsonCp(3)*MSrmass(mub,3,5,4))
     &     /(MSrmass(mub,5,5,4)+MSrmass(mub,3,5,4))
      F_A=WilsonC(1)-WilsonCp(1)
      F_ASM=SMBSMWilc(genq,1)



      BSMbr(genq)=GFermi**2*alphaEM(mub)**2
     &     *BSMm_B(genq)**3*BSMf_B(genq)**2*BSMtau_B(genq)
     &     /(64.D0*PI**3)*ABS(KM(3,3)*DCONJG(KM(3,genq)))**2
     &     *SQRT(1.D0-4.D0*m_hat**2)*((1.D0-4.D0*m_hat**2)
     &     *ABS(F_S)**2+ABS(F_P+2.D0*m_hat*F_A)**2)



      BSMbrSM(genq)=GFermi**2*alphaEM(mub)**2
     &     *BSMm_B(genq)**3*BSMf_B(genq)**2*BSMtau_B(genq)
     &     /(64.D0*PI**3)*ABS(KM(3,3)*DCONJG(KM(3,genq)))**2
     &     *SQRT(1.D0-4.D0*m_hat**2)*(ABS(2.D0*m_hat*F_ASM)**2)

      BSMbrrat(genq)=BSMbr(genq)/BSMbrSM(genq)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Branching Ratio'
         Write(*,*) '***************************************'
         Write(*,*) 'SUSY: ', BSMbr(genq)
         Write(*,*) 'SM: ', BSMbrSM(genq)
         Write(*,*) 'SUSY/SM: ', BSMbrrat(genq)
      End If

      Return
      End
