c**********************************************************************

      Subroutine bsmwc(debug,genq,iOrd,WilsonC,WilsonCp)

c Subroutine that calculates the various contributions to the Wilson
c coefficients relevant to the decay \overline{B}_{q}\to ll
c**********************************************************************

      Implicit NONE

      Include 'insm.h'
      Include 'insoft.h'
      Include 'const.h'
      Include 'vars.h'
      Include 'vertex.h'
      Include 'bsm.h'

c Inputs
      Integer debug,genq,iOrd

c Outputs
      Double Complex WilsonC(3), WilsonCp(3)

c Local
      Integer i,j,k,l
      Double Precision x,y,z,corfac
      Double Precision m_bEW,m_qEW,m_bSUSY,m_qSUSY
      Double Precision mtEW, m_upEW(3)
      Double Precision MS(2),MP(2),MCHi(2)
      Double Complex tmp1,tmp2,tmp3,tmp4,tmpBox
      Double Complex kappa

c Functions
      Double Precision alphaEM,alphaStr,MSrmass
c Passarino Veltman Functions
      Double Precision PV_B0,PV_B1,PV_C0,PV_C00,PV_D0,PV_D00
c Loop functions
      Double Precision BBKU_f01pr,BBKU_f02pr,BBKU_f07pr,BBKU_f03pr
      Double Precision BBKU_f114,BBKU_f111, BBKU_f02, BBKU_f01
      Double Precision BBKU_f110,BBKU_f11, BBKU_f12,BBKU_f17


      mtEW=MSrmass(muew,6,5,4)
      m_bEW=MSrmass(muew,5,5,4)
      m_qEW=MSrmass(muew,(2*genq-1),5,4)
      m_bSUSY=MSrmass(mususy,5,5,4)
      m_qSUSY=MSrmass(mususy,(2*genq-1),5,4)
      

      MS(1)=mHh0
      MS(2)=mLh0
      MP(1)=mA0
      MP(2)=mz
      MCHi(1)=mhc
      MCHi(2)=mw
      
      Do i=1,3
         m_upEW(i)=MSrmass(muew,2*i,5,4)
      End Do

      kappa=SQRT(2.D0)/(4.D0*alphaEM(muew)*4.D0*PI*GFermi
     &     *KM(3,3)*DCONJG(KM(3,genq)))
      
      Do i=1,3
         SMBSMWilc(genq,i)=DCMPLX(0.D0,0.D0)
         SMBSMWilcp(genq,i)=DCMPLX(0.D0,0.D0)
         CHBSMWilc(genq,i)=DCMPLX(0.D0,0.D0)
         CHBSMWilcp(genq,i)=DCMPLX(0.D0,0.D0)
         CINOBSMWilc(genq,i)=DCMPLX(0.D0,0.D0)
         CINOBSMWilcp(genq,i)=DCMPLX(0.D0,0.D0)
         NINOBSMWilc(genq,i)=DCMPLX(0.D0,0.D0)
         NINOBSMWilcp(genq,i)=DCMPLX(0.D0,0.D0)
         EVBSMWilc(genq,i)=DCMPLX(0.D0,0.D0)
         EVBSMWilcp(genq,i)=DCMPLX(0.D0,0.D0)
      End Do


c**********************************************************************
c Wilson Coefficients due to corrected SUSY vertices

c Z penguins
      EVBSMWilc(genq,1)=-16.D0*PI**2*kappa*ZleleA(2)
     &     *ZdqqLCor(3,genq)/mz**2
      EVBSMWilcp(genq,1)=-16.D0*PI**2*kappa*ZleleA(2)
     &     *ZdqqRCor(3,genq)/mz**2

c Higgs Penguins

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,2
         tmp1=tmp1+SHqqRCor(i,3,genq)*SHll(i,2)/MS(i)**2
         tmp2=tmp2+SHqqLCor(i,3,genq)*SHll(i,2)/MS(i)**2
         tmp3=tmp3+PHqqRCor(i,3,genq)*PHll(i,2)/MP(i)**2
         tmp4=tmp4+PHqqLCor(i,3,genq)*PHll(i,2)/MP(i)**2
      End Do

      EVBSMWilc(genq,2)=16.D0*PI**2*kappa/m_bEW*tmp1
      EVBSMWilcp(genq,2)=16.D0*PI**2*kappa/m_qEW*tmp2
      EVBSMWilc(genq,3)=16.D0*PI**2*kappa/m_bEW*tmp3
      EVBSMWilcp(genq,3)=16.D0*PI**2*kappa/m_qEW*tmp4

c**********************************************************************
c Box diagrams
c**********************************************************************

c Charginos

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox+1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslR(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 +2.D0*DCONJG(clslL(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End Do
      End Do
      CINOBSMWilc(genq,1)=-kappa*tmpBox
      
      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox-1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslL(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 +2.D0*DCONJG(clslR(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End Do
      End Do
      CINOBSMWilcp(genq,1)=-kappa*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox+1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslL(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 +4.D0*DCONJG(clslR(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End do
      End Do
      CINOBSMWilc(genq,2)=-kappa/m_bSUSY*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox+1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslR(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 +4.D0*DCONJG(clslL(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End do
      End Do
      CINOBSMWilcp(genq,2)=-kappa/m_qSUSY*(tmpBox)

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox+1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslL(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 -4.D0*DCONJG(clslR(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqL(genq,k,j))*cqsqR(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End do
      End Do
      CINOBSMWilc(genq,3)=-kappa/m_bSUSY*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,2
         Do j=1,6
            Do k=1,2
               Do l=1,3
                  tmpBox=tmpBox-1.D0/(4.D0)*(MC(k)*MC(i)
     &                 *DCONJG(clslR(2,i,l))*clslL(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D0(MC(i),MUpS(j),MC(k),MNeS(l))
     &                 -4.D0*DCONJG(clslL(2,i,l))*clslR(2,k,l)
     &                 *DCONJG(cqsqR(genq,k,j))*cqsqL(3,i,j)
     &                 *PV_D00(MC(i),MUpS(j),MC(k),MNeS(l)))
               End Do
            End Do
         End do
      End Do
      CINOBSMWilcp(genq,3)=-kappa/m_qSUSY*tmpBox

c**********************************************************************
c  Neutralinos

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox+1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslR(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +2.D0*DCONJG(nlslL(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -MN(k)/MN(i)*DCONJG(nlslL(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -2.D0*DCONJG(nlslR(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End Do
      End Do
      NINOBSMWilc(genq,1)=-kappa*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox-1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslL(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +2.D0*DCONJG(nlslR(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -MN(k)*MN(i)*DCONJG(nlslR(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -2.D0*DCONJG(nlslL(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End Do
      End Do
      NINOBSMWilcp(genq,1)=-kappa*(tmpBox)

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox+1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslL(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +4.D0*DCONJG(nlslR(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +MN(k)*MN(i)*DCONJG(nlslL(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +4.D0*DCONJG(nlslR(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End do
      End Do
      NINOBSMWilc(genq,2)=-kappa/m_bSUSY*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox+1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslR(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +4.D0*DCONJG(nlslL(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +MN(k)*MN(i)*DCONJG(nlslR(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +4.D0*DCONJG(nlslL(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End do
      End Do
      NINOBSMWilcp(genq,2)=-kappa/m_qSUSY*tmpBox

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox+1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslL(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -4.D0*DCONJG(nlslR(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +MN(k)*MN(i)*DCONJG(nlslL(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -4.D0*DCONJG(nlslR(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqL(genq,k,j))*nqsqR(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End do
      End Do
      NINOBSMWilc(genq,3)=-kappa/m_bSUSY*(tmpBox)

      tmpBox=DCMPLX(0.D0,0.D0)
      Do i=1,4
         Do j=1,6
            Do k=1,4
               Do l=1,6
                  tmpBox=tmpBox-1.D0/(4.D0)*(MN(k)*MN(i)
     &                 *DCONJG(nlslR(2,i,l))*nlslL(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -4.D0*DCONJG(nlslL(2,i,l))*nlslR(2,k,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l))
     &                 +MN(k)*MN(i)*DCONJG(nlslR(2,k,l))*nlslL(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D0(MN(i),MDoS(j),MN(k),MElS(l))
     &                 -4.D0*DCONJG(nlslL(2,k,l))*nlslR(2,i,l)
     &                 *DCONJG(nqsqR(genq,k,j))*nqsqL(3,i,j)
     &                 *PV_D00(MN(i),MDoS(j),MN(k),MElS(l)))
               End Do
            End Do
         End do
      End Do
      NINOBSMWilcp(genq,3)=-kappa/m_qSUSY*tmpBox

c***********************************************************************
c Electroweak scale contributions
c***********************************************************************
c Charged Higgs Contributions
c***********************************************************************

c LO matching conditions

c For the axial operator (i.e contributions from Z-penguins and Box
c diagrams)
      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,3
         tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &        *DCONJG(CHqqL(1,genq,i))*CHqqL(1,3,i)
     &        *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &           *DCONJG(delCHqqL(1,genq,i))*CHqqL(1,3,i)
     &           *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
            tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &           *DCONJG(CHqqL(1,genq,i))*delCHqqL(1,3,i)
     &           *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
         End If
         tmp2=tmp2+ZleleA(2)/(mz**2)
     &        *(2.D0*ZuqqR(i)*DCONJG(CHqqL(1,genq,i))*CHqqL(1,3,i)
     &        *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &        -1.D0/4.D0)
     &        -ZuqqL(i)*DCONJG(CHqqL(1,genq,i))*CHqqL(1,3,i)
     &        *m_upEW(i)*m_upEW(i)
     &        *PV_C0(mhc,m_upEW(i),m_upEW(i)))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp2=tmp2+ZleleA(2)/(mz**2)
     &           *(2.D0*ZuqqR(i)*DCONJG(delCHqqL(1,genq,i))*CHqqL(1,3,i)
     &           *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &           -1.D0/4.D0)
     &           -ZuqqL(i)*DCONJG(delCHqqL(1,genq,i))*CHqqL(1,3,i)
     &           *m_upEW(i)*m_upEW(i)
     &           *PV_C0(mhc,m_upEW(i),m_upEW(i)))
            tmp2=tmp2+ZleleA(2)/(mz**2)
     &           *(2.D0*ZuqqR(i)*DCONJG(CHqqL(1,genq,i))*delCHqqL(1,3,i)
     &           *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &           -1.D0/4.D0)
     &           -ZuqqL(i)*DCONJG(CHqqL(1,genq,i))*delCHqqL(1,3,i)
     &           *m_upEW(i)*m_upEW(i)
     &           *PV_C0(mhc,m_upEW(i),m_upEW(i)))
         End If
         tmp3=tmp3-1.D0/2.D0*DCONJG(CHqqL(1,genq,i))*CHqqL(1,3,i)
     &        *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &        *PV_D00(mhc,mhc,m_upEW(i),0.D0)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp3=tmp3-1.D0/2.D0*DCONJG(delCHqqL(1,genq,i))*CHqqL(1,3,i)
     &           *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &           *PV_D00(mhc,mhc,m_upEW(i),0.D0)
            tmp3=tmp3-1.D0/2.D0*DCONJG(CHqqL(1,genq,i))*delCHqqL(1,3,i)
     &           *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &           *PV_D00(mhc,mhc,m_upEW(i),0.D0)
         End If
         tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqL(3)
     &        *DCONJG(CHqqL(1,genq,i))*CHqqL(1,3,i)
     &        *PV_B1(m_upEW(i),mhc,mususy)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqL(3)
     &           *DCONJG(delCHqqL(1,genq,i))*CHqqL(1,3,i)
     &           *PV_B1(m_upEW(i),mhc,mususy)
            tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqL(3)
     &           *DCONJG(CHqqL(1,genq,i))*delCHqqL(1,3,i)
     &           *PV_B1(m_upEW(i),mhc,mususy)
         End If
      End Do
      
      CHBSMWilc(genq,1)=-kappa*(tmp1+tmp2+tmp3+tmp4)

c For the primed axial operator (i.e contributions from Z-penguins and
c Box diagrams)

      Do i=1,3
         tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &        *DCONJG(CHqqR(1,genq,i))*CHqqR(1,3,i)
     &        *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &           *DCONJG(delCHqqR(1,genq,i))*CHqqR(1,3,i)
     &           *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
            tmp1=tmp1+ZleleA(2)/(mz**2)*ZCHi(1)
     &           *DCONJG(CHqqR(1,genq,i))*delCHqqR(1,3,i)
     &           *2.D0*PV_C00(m_upEW(i),mhc,mhc,mususy)
         End If
         tmp2=tmp2+ZleleA(2)/(mz**2)
     &        *(2.D0*ZuqqL(i)*DCONJG(CHqqR(1,genq,i))*CHqqR(1,3,i)
     &        *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &        -1.D0/4.D0)
     &        -ZuqqR(i)*DCONJG(CHqqR(1,genq,i))*CHqqR(1,3,i)
     &        *m_upEW(i)*m_upEW(i)
     &        *PV_C0(mhc,m_upEW(i),m_upEW(i)))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp2=tmp2+ZleleA(2)/(mz**2)
     &           *(2.D0*ZuqqL(i)*DCONJG(delCHqqR(1,genq,i))*CHqqR(1,3,i)
     &           *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &           -1.D0/4.D0)
     &           -ZuqqR(i)*DCONJG(delCHqqR(1,genq,i))*CHqqR(1,3,i)
     &           *m_upEW(i)*m_upEW(i)
     &           *PV_C0(mhc,m_upEW(i),m_upEW(i)))
            tmp2=tmp2+ZleleA(2)/(mz**2)
     &        *(2.D0*ZuqqL(i)*DCONJG(CHqqR(1,genq,i))*delCHqqR(1,3,i)
     &           *(PV_C00(mhc,m_upEW(i),m_upEW(i),mususy)
     &           -1.D0/4.D0)
     &           -ZuqqR(i)*DCONJG(CHqqR(1,genq,i))*delCHqqR(1,3,i)
     &           *m_upEW(i)*m_upEW(i)
     &           *PV_C0(mhc,m_upEW(i),m_upEW(i)))
         End If
         tmp3=tmp3-1.D0/2.D0*DCONJG(CHqqR(1,genq,i))*CHqqR(1,3,i)
     &        *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &        *PV_D00(mhc,mhc,m_upEW(i),0.D0)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp3=tmp3-1.D0/2.D0*DCONJG(delCHqqR(1,genq,i))*CHqqR(1,3,i)
     &           *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &           *PV_D00(mhc,mhc,m_upEW(i),0.D0)
            tmp3=tmp3-1.D0/2.D0*DCONJG(CHqqR(1,genq,i))*delCHqqR(1,3,i)
     &           *DCONJG(CHnulR(1,2))*CHnulR(1,2)
     &           *PV_D00(mhc,mhc,m_upEW(i),0.D0)
         End If
         tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqR(3)
     &        *DCONJG(CHqqR(1,genq,i))*CHqqR(1,3,i)
     &        *PV_B1(m_upEW(i),mhc,mususy)
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqR(3)
     &           *DCONJG(delCHqqR(1,genq,i))*CHqqR(1,3,i)
     &           *PV_B1(m_upEW(i),mhc,mususy)
            tmp4=tmp4+ZleleA(2)/(mz**2)*ZdqqR(3)
     &           *DCONJG(CHqqR(1,genq,i))*delCHqqR(1,3,i)
     &           *PV_B1(m_upEW(i),mhc,mususy)
         End If
      End Do

      CHBSMWilcp(genq,1)=-kappa*(tmp1+tmp2+tmp3+tmp4)
      
c For the scalar operator (i.e contributions from Higgs penguins and 
c Box diagrams)
      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,2
         Do j=1,2
            Do k=1,3
               Do l=1,2
                  tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                 *DCONJG(CHqqL(l,genq,k))*CHqqR(j,3,k)*m_upEW(k)
     &                 *PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                    *DCONJG(CHqqL(l,genq,k))*delCHqqR(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                     tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                    *DCONJG(delCHqqL(l,genq,k))*CHqqR(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  End If
               End Do
               tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqL(i,k)
     &              *DCONJG(CHqqL(j,genq,k))*CHqqR(j,3,k)
     &              *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &              -1.D0/8.D0)
     &              +m_upEW(k)*m_upEW(k)*SHuqqR(i,k)
     &              *DCONJG(CHqqL(j,genq,k))*CHqqR(k,3,k)
     &              *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqL(i,k)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*SHuqqR(i,k)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
                  tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqL(i,k)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*SHuqqR(i,k)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               End If
               tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &              *SHqqR(i,3)*DCONJG(CHqqL(j,genq,k))*CHqqR(j,3,k)
     &              *PV_B0(m_upEW(k),MCHi(j),mususy)
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &                 *SHqqR(i,3)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(j,3,k)
     &                     *PV_B0(m_upEW(k),MCHi(j),mususy)
                  tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &                 *SHqqR(i,3)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
               End If
            End Do
         End Do
      End Do
      
      Do i=1,3
         tmp4=tmp4+(2.D0
     &        *(DCONJG(WqqL(2,i))*CHqqR(1,3,i)
     &        *DCONJG(CHnulR(1,2))*WnulL(2))
     &        *PV_D00(mhc,mw,m_upEW(i),0.D0))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+(2.D0
     &           *(DCONJG(CHqqL(1,genq,i))*WqqRCor(3,i)
     &           *DCONJG(WnulL(2))*CHnulR(1,2))
     &           *PV_D00(mhc,mw,m_upEW(i),0.D0))
            tmp4=tmp4+(2.D0
     &              *(DCONJG(WqqL(2,i))*delCHqqR(1,3,i)
     &           *DCONJG(CHnulR(1,2))*WnulL(2))
     &           *PV_D00(mhc,mw,m_upEW(i),0.D0))
         End If
      End Do

      CHBSMWilc(genq,2)=-kappa/m_bEW*(tmp1+tmp2+tmp3+tmp4)

c For the primed scalar operator (i.e contributions from Higgs penguins
c and Box diagrams)
      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,2
         Do j=1,2
            Do k=1,3
               Do l=1,2
                  tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                 *DCONJG(CHqqR(l,genq,k))*CHqqL(j,3,k)*m_upEW(k)
     &                 *PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                    *DCONJG(delCHqqR(l,genq,k))*CHqqL(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                     tmp1=tmp1+SHll(i,2)/(MS(i)**2)*SHCHi(i,j,l)
     &                    *DCONJG(CHqqR(l,genq,k))*delCHqqL(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  End If
               End Do
               tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqR(i,k)
     &              *DCONJG(CHqqR(j,genq,k))*CHqqL(j,3,k)
     &              *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &              +m_upEW(k)*m_upEW(k)*SHuqqL(i,k)
     &              *DCONJG(CHqqR(j,genq,k))*CHqqL(k,3,k)
     &              *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqR(i,k)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*SHuqqL(i,k)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
                  tmp2=tmp2+SHll(i,2)/(MS(i)**2)*(4.D0*SHuqqR(i,k)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*SHuqqL(i,k)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               End If
               tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &              *SHqqL(i,3)*DCONJG(CHqqR(j,genq,k))*CHqqL(j,3,k)
     &              *PV_B0(m_upEW(k),MCHi(j),mususy)
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &                 *SHqqL(i,3)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
                  tmp3=tmp3-SHll(i,2)/(MS(i)**2)*m_upEW(j)/m_bEW
     &                 *SHqqL(i,3)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
               End If
            End Do
         End Do
      End Do

      Do i=1,3
            tmp4=tmp4+(2.D0
     &           *(DCONJG(CHqqR(1,genq,i))*WqqL(3,i)
     &           *DCONJG(WnulL(2))*CHnulR(1,2))
     &           *PV_D00(mhc,mw,m_upEW(i),0.D0))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+(2.D0
     &           *(DCONJG(delCHqqR(1,genq,i))*WqqL(3,i)
     &           *DCONJG(WnulL(2))*CHnulR(1,2)
     &           +DCONJG(WqqRCor(2,i))*CHqqL(1,3,i)
     &           *DCONJG(CHnulR(1,2))*WnulL(2))
     &           *PV_D00(mhc,mw,m_upEW(i),0.D0))
         End If
      End Do

      CHBSMWilcp(genq,2)=-kappa/m_qEW*(tmp1+tmp2+tmp3+tmp4)

c For the pseudoscalar operator (i.e contributions from Higgs penguins and 
c Box diagrams)
      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,2
         Do j=1,2
            Do k=1,3
               Do l=1,2
                  tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                 *DCONJG(CHqqL(l,genq,k))*CHqqR(j,3,k)*m_upEW(k)
     &                 *PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                    *DCONJG(delCHqqL(l,genq,k))*CHqqR(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                     tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                    *DCONJG(CHqqL(l,genq,k))*delCHqqR(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  End If
               End Do
               tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqL(i,k)
     &              *DCONJG(CHqqL(j,genq,k))*CHqqR(j,3,k)
     &              *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &              -1.D0/8.D0)
     &              +m_upEW(k)*m_upEW(k)*PHuqqR(i,k)
     &              *DCONJG(CHqqL(j,genq,k))*CHqqR(k,3,k)
     &              *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqL(i,k)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*PHuqqR(i,k)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
                  tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqL(i,k)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*PHuqqR(i,k)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               End If
               tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &              *PHqqR(i,3)*DCONJG(CHqqL(j,genq,k))*CHqqR(j,3,k)
     &              *PV_B0(m_upEW(k),MCHi(j),mususy)
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &                 *PHqqR(i,3)
     &                 *DCONJG(delCHqqL(j,genq,k))*CHqqR(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
                  tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &                 *PHqqR(i,3)
     &                 *DCONJG(CHqqL(j,genq,k))*delCHqqR(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
               End If
            End Do
         End Do
      End Do

      Do i=1,3
         tmp4=tmp4-(2.D0
     &        *(DCONJG(WqqL(2,i))*CHqqR(1,3,i)
     &        *DCONJG(CHnulR(1,2))*WnulL(2))
     &        *PV_D00(mhc,mw,m_upEW(i),0.D0))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+(2.D0
     &           *(DCONJG(CHqqL(1,genq,i))*WqqRCor(3,i)
     &           *DCONJG(WnulL(2))*CHnulR(1,2)
     &           -DCONJG(WqqL(2,i))*delCHqqR(1,3,i)
     &           *DCONJG(CHnulR(1,2))*WnulL(2))
     &           *PV_D00(mhc,mw,m_upEW(i),0.D0))
         End If
      End Do

      tmp4=DCMPLX(0.D0,0.D0)

      CHBSMWilc(genq,3)=-kappa/m_bEW*(tmp1+tmp2+tmp3+tmp4)

c For the primed pseudoscalar operator (i.e contributions from Higgs
c penguins and Box diagrams)

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)

      Do i=1,2
         Do j=1,2
            Do k=1,3
               Do l=1,2
                  tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                 *DCONJG(CHqqR(l,genq,k))*CHqqL(j,3,k)*m_upEW(k)
     &                 *PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                    *DCONJG(delCHqqR(l,genq,k))*CHqqL(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                     tmp1=tmp1+PHll(i,2)/(MP(i)**2)*PHCHi(i,j,l)
     &                    *DCONJG(CHqqR(l,genq,k))*delCHqqL(j,3,k)
     &                    *m_upEW(k)*PV_C0(m_upEW(k),MCHi(j),MCHi(l))
                  End If
               End Do
               tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqR(i,k)
     &              *DCONJG(CHqqR(j,genq,k))*CHqqL(j,3,k)
     &              *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &              -1.D0/8.D0)
     &              +m_upEW(k)*m_upEW(k)*PHuqqL(i,k)
     &              *DCONJG(CHqqR(j,genq,k))*CHqqL(k,3,k)
     &              *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqR(i,k)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*PHuqqL(i,k)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
                  tmp2=tmp2+PHll(i,2)/(MP(i)**2)*(4.D0*PHuqqR(i,k)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(j,3,k)
     &                 *(PV_C00(m_upEW(k),MCHi(j),m_upEW(k),mususy)
     &                 -1.D0/8.D0)
     &                 +m_upEW(k)*m_upEW(k)*PHuqqL(i,k)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(k,3,k)
     &                 *PV_C0(m_upEW(k),MCHi(j),m_upEW(k)))
               End If
               tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &              *PHqqL(i,3)*DCONJG(CHqqR(j,genq,k))*CHqqL(j,3,k)
     &              *PV_B0(m_upEW(k),MCHi(j),mususy)
               If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                  tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &                 *PHqqL(i,3)
     &                 *DCONJG(delCHqqR(j,genq,k))*CHqqL(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
                  tmp3=tmp3-PHll(i,2)/(MP(i)**2)*m_upEW(j)/m_bEW
     &                 *PHqqL(i,3)
     &                 *DCONJG(CHqqR(j,genq,k))*delCHqqL(j,3,k)
     &                 *PV_B0(m_upEW(k),MCHi(j),mususy)
               End If
            End Do
         End Do
      End Do

      Do i=1,3
         tmp4=tmp4+(2.D0
     &        *(DCONJG(CHqqR(1,genq,i))*WqqL(3,i)
     &        *DCONJG(WnulL(2))*CHnulR(1,2))
     &        *PV_D00(mhc,mw,m_upEW(j),0.D0))
         If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
            tmp4=tmp4+(2.D0
     &           *(DCONJG(delCHqqR(1,genq,i))*WqqL(3,i)
     &           *DCONJG(WnulL(2))*CHnulR(1,2)
     &           -DCONJG(WqqRCor(2,i))*CHqqL(1,3,i)
     &           *DCONJG(CHnulR(1,2))*WnulL(2))
     &           *PV_D00(mhc,mw,m_upEW(j),0.D0))
         End If
      End Do

      CHBSMWilcp(genq,3)=-kappa/m_qEW*(tmp1+tmp2+tmp3+tmp4)

c NLO Contributions
c I use the NLO matching conditions of hep-ph/0112305 note the factor
c correcting for the change in notation. The Wilson coefficients
c for the scalar and pseudoscalar operators are only valid in the 
c large tanb regime. Though this is the only region in which they
c are important anyway.

      y=mtEW**2/mhc**2
      x=mtEW**2/mw**2
      z=mhc**2/mw**2

      corfac=1.D0/sw2

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *mhc**2/(8.D0*mw**2*tanb**2)*(y*BBKU_f12(y)
     &     +8.D0*y*BBKU_f02pr(y)*LOG(muew**2/mtEW**2))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         CHBSMWilc(genq,1)=CHBSMWilc(genq,1)+tmp1
c      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *(-m_bEW*m_qEW*tanb**2/(8.D0*mw**2)
     &     *(BBKU_f12(y)+8.D0*(BBKU_f02(y)+y*BBKU_f02pr(y))
     &     *LOG(muew**2/mtEW**2))
     &     -m_bEW*m_qEW*m_el(2)**2*tanb**4
     &     /(16.D0*mw**2*mhc**2)*(BBKU_f17(y)
     &     +8.D0*(BBKU_f02(y)+y*BBKU_f02pr(y))*LOG(muew**2/mtEW**2)))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         CHBSMWilcp(genq,1)=CHBSMWilcp(genq,1)+tmp1
c      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=-corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *m_el(2)*tanb**2/(4.D0*mw**2)
     &     *(BBKU_f114(x,z)-BBKU_f111(x,z)+8.D0*x*(x*BBKU_f03pr(x,z)
     &     -x*BBKU_f07pr(x,z)*LOG(muew**2/mtEW**2)))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         CHBSMWilc(genq,2)=CHBSMWilc(genq,2)+tmp1
c      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=-corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *m_el(2)*tanb**2/(4.D0*mw**2)
     &     *(BBKU_f114(x,z)-BBKU_f111(x,z)+8.D0*x*(x*BBKU_f03pr(x,z)
     &     -x*BBKU_f07pr(x,z)*LOG(muew**2/mtEW**2)))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         CHBSMWilcp(genq,2)=CHBSMWilcp(genq,2)+tmp1
c      End If

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *m_el(2)*tanb**2/(4.D0*mw**2)
     &     *(BBKU_f114(x,z)-BBKU_f111(x,z)+8.D0*x*(x*BBKU_f03pr(x,z)
     &     -x*BBKU_f07pr(x,z)*LOG(muew**2/mtEW**2)))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then     
         CHBSMWilc(genq,3)=CHBSMWilc(genq,3)+tmp1
c      End If         

      tmp1=DCMPLX(0.D0,0.D0)
      tmp1=-corfac*alphaStr(muEW,5,4)/(4.D0*PI)
     &     *m_el(2)*tanb**2/(4.D0*mw**2)
     &     *(BBKU_f114(x,z)-BBKU_f111(x,z)+8.D0*x*(x*BBKU_f03pr(x,z)
     &     -x*BBKU_f07pr(x,z)*LOG(muew**2/mtEW**2)))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         CHBSMWilcp(genq,3)=CHBSMWilcp(genq,3)+tmp1
c      End If

c***********************************************************************
c Standard Model Contributions
c***********************************************************************
c For the axial operator we use the NLO matching condition of 
c hep-ph/0112305 note the factor correcting for the change
c in notation

      x=mtEW**2/mw**2

      SMBSMWilc(genq,1)=1.D0/4.D0*(BBKU_f02(x)-BBKU_f01(x))

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         SMBSMWilc(genq,1)=SMBSMWilc(genq,1)
     &        +1.D0/4.D0*alphaStr(muEW,5,4)/(4.D0*PI)
     &        *((BBKU_f110(x)-BBKU_f11(x)
     &        +8.D0*x
     &        *(BBKU_f02pr(x)-BBKU_f01pr(x))*LOG(muew**2/mtEW**2)))
c      End If

      SMBSMWilc(genq,1)=corfac*SMBSMWilc(genq,1)

      SMBSMWilc(genq,2)=DCMPLX(0.D0,0.D0)
      SMBSMWilc(genq,3)=DCMPLX(0.D0,0.D0)
      SMBSMWilcp(genq,1)=DCMPLX(0.D0,0.D0)
      SMBSMWilcp(genq,2)=DCMPLX(0.D0,0.D0)
      SMBSMWilcp(genq,3)=DCMPLX(0.D0,0.D0)

c***********************************************************************
c Debug Writes

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Effective Vertex Contributions'
         Write(*,*) 'cA :',EVBSMWilc(genq,1)
         Write(*,*) 'cS :',EVBSMWilc(genq,2)
         Write(*,*) 'cP :',EVBSMWilc(genq,3)
         Write(*,*) 'cAp :',EVBSMWilcp(genq,1)
         Write(*,*) 'cSp :',EVBSMWilcp(genq,2)
         Write(*,*) 'cPp :',EVBSMWilcp(genq,3)
         Write(*,*) '***************************************'
         Write(*,*) 'Neutralino Box Diagram Contributions'
         Write(*,*) 'cA :',NINOBSMWilc(genq,1)
         Write(*,*) 'cS :',NINOBSMWilc(genq,2)
         Write(*,*) 'cP :',NINOBSMWilc(genq,3)
         Write(*,*) 'cAp :',NINOBSMWilcp(genq,1)
         Write(*,*) 'cSp :',NINOBSMWilcp(genq,2)
         Write(*,*) 'cPp :',NINOBSMWilcp(genq,3)
         Write(*,*) '***************************************'
         Write(*,*) 'Chargino Box Diagram Contributions'
         Write(*,*) 'cA :',CINOBSMWilc(genq,1)
         Write(*,*) 'cS :',CINOBSMWilc(genq,2)
         Write(*,*) 'cP :',CINOBSMWilc(genq,3)
         Write(*,*) 'cAp :',CINOBSMWilcp(genq,1)
         Write(*,*) 'cSp :',CINOBSMWilcp(genq,2)
         Write(*,*) 'cPp :',CINOBSMWilcp(genq,3)
         Write(*,*) '***************************************'
         Write(*,*) 'Charged Higgs Contributions'
         Write(*,*) 'cA :',CHBSMWilc(genq,1)
         Write(*,*) 'cS :',CHBSMWilc(genq,2)
         Write(*,*) 'cP :',CHBSMWilc(genq,3)
         Write(*,*) 'cAp :',CHBSMWilcp(genq,1)
         Write(*,*) 'cSp :',CHBSMWilcp(genq,2)
         Write(*,*) 'cPp :',CHBSMWilcp(genq,3)
         Write(*,*) '***************************************'
         Write(*,*) 'Standard Model Contributions'
         Write(*,*) 'cA :',SMBSMWilc(genq,1)
         Write(*,*) 'cS :',SMBSMWilc(genq,2)
         Write(*,*) 'cP :',SMBSMWilc(genq,3)
         Write(*,*) 'cAp :',SMBSMWilcp(genq,1)
         Write(*,*) 'cSp :',SMBSMWilcp(genq,2)
         Write(*,*) 'cPp :',SMBSMWilcp(genq,3)
      End If

c***********************************************************************
c Total Wilson coefficients

      Do i=1,3
         WilsonC(i)=DCMPLX(0.D0,0.D0)
         WilsonCp(i)=DCMPLX(0.D0,0.D0)
      End Do

      Do i=1,3
         WilsonC(i)=SMBSMWilc(genq,i)+CHBSMWilc(genq,i)
     &        +NINOBSMWilc(genq,i)+CINOBSMWilc(genq,i)+EVBSMWilc(genq,i)
         WilsonCp(i)=SMBSMWilcp(genq,i)+CHBSMWilcp(genq,i)
     &        +NINOBSMWilcp(genq,i)+CINOBSMWilcp(genq,i)
     &        +EVBSMWilcp(genq,i)
      End Do

      Do i=1,3
         BSMBSMWilc(genq,i)=CHBSMWilc(genq,i)
     &        +NINOBSMWilc(genq,i)+CINOBSMWilc(genq,i)+EVBSMWilc(genq,i)
         BSMBSMWilcp(genq,i)=CHBSMWilcp(genq,i)
     &        +NINOBSMWilcp(genq,i)+CINOBSMWilcp(genq,i)
     &        +EVBSMWilcp(genq,i)
      End Do

      Return
      End
