!***********************************************************************


      subroutine BD(Flag, InP, InN, Outsoft, BDecay, Out_err)


! Program that reads the relevant softterms and SM variables and calls
! the relevant B Decay routines.
!***********************************************************************


      use parameters


      Implicit NONE

      Include 'insoft-f90.h'
      Include 'insm-f90.h'
      Include 'bsm-f90.h'
      Include 'bsg-f90.h'
      Include 'bb-f90.h'
      Include 'vars-f90.h'


     Type(FLAGS), INTENT(IN) :: Flag
     Type(Input_params), INTENT(IN) :: InP
     Type(Nuisance_params), INTENT(IN) :: InN
     Type(Softout), INTENT(IN)  :: Outsoft
     Type(BDecay_Out), INTENT(OUT) :: BDecay
     Type(Error_Out), INTENT(INOUT):: Out_err



      integer :: i,j,k
      integer :: debug,iresum,iEWcor,iOrd,isoft,genq, softerr,bderr
      integer :: nflag,sgnmu, sick(5),CKMt
      real (8) :: sugpar(4),tmp1,tmp2,tmp3,tmp4,alfinv,MN1(4)
      complex (8) :: ZN1(4,4)
 
      debug = 0 !for internal debugging


      mz = 91.1876d0


!All masses are at MZ
      m_up(1)= outsoft%sm%mUp 
      m_up(2)= outsoft%sm%mCharm
      m_up(3)= InN%mtop 

      m_do(1)= outsoft%sm%mDown
      m_do(2)= outsoft%sm%mStrange
      m_do(3)= InN%mBottom 

      m_el(1) = outsoft%sm%mElectron
      m_el(2) = outsoft%sm%mMuon
      m_el(3) = outsoft%sm%mTau


      if (debug) then
       print*,'SM parameters: '
       print*,'m_up= ',m_up(1)
       print*,'m_charm= ',m_up(2)
       print*,'m_top= ',m_up(3)
       print*,'m_down= ',m_do(1)
       print*,'m_strange= ',m_do(2)
       print*,'m_bottom= ',m_do(3)
       print*,'m_elec= ',m_el(1)
       print*,'m_muon= ',m_el(2)
       print*,'m_tau= ',m_el(3)
      endif



!scales

      mu_up(1)= 2.d0
      mu_up(2)= m_up(2)
      mu_up(3)= 0.d0 

      mu_do(1)= 2.d0
      mu_do(2)= 2.d0 
      mu_do(3)= InN%mbottom !M_b^{1S}=4.69\pm 0.03$ Ge

!
      alphaEM_0 = InN%alphamz
      alphas_0 = InN%alphasmz


      mualphas_0 = mz
      mualphaEM_0 = mz


! Read in some additional parameters: temporary...

      GFermi = 1.16637d-5 
      sw2 =  0.23113d0
      mw = 80.410d0
      
      CKMt = 1   
      tmp1 =  0.2265d0 
      tmp2 =  0.801d0
      tmp3 =  0.189d0
      tmp4 =  0.358d0


      If(CKMt.eq.0) Then
         Call CKMcalc(debug, tmp1, tmp2, tmp3, tmp4)
      Else
         Call CKMWolf(debug, tmp1, tmp2, tmp3, tmp4)
      End If

      alpha1_0 = (5.D0/3.D0)*alphaEM_0/(1.D0-sw2)
      alpha2_0 = alphaEM_0/sw2
      alpha3_0 = alphas_0
      mualpha1_0 = mualphaEM_0
      mualpha2_0 = mualphaEM_0
      mualpha3_0 = mualphaS_0


! Set SUSY parameters
      tanb = InP%tanb
      alpha_eff = outsoft%higg%halpha
      mLh0 = outsoft%higg%mlh
      mHh0 = outsoft%higg%mhh
      mA0 = outsoft%higg%ma
      mhc = outsoft%higg%mch
      muH = outsoft%soft%mu

      mGl = outsoft%mgluino 

      if (debug) then 
        print*,' '
        print*,' Higgs sector '
        print*,' tanb ',tanb
        print*,' alpha ',alpha_eff
        print*,' mu ', muH 
        print*,' mGl ', mGl
        print*,' mA0 ', mA0
        print*,' mlh ', mLh0
        print*,' mHh0 ',mHh0
        print*,' mhc ', mhc
      endif 

     
! init soft-matrices
      Do i=1,3
       Do j=1,3
         AUp(i,j) = (0.d0,0.d0)
         ADo(i,j) = (0.d0,0.d0)
         ALe(i,j) = (0.d0,0.d0)
         MSoftQuL(i,j) = (0.d0,0.d0)
         MSoftUpR(i,j) = (0.d0,0.d0)
         MSoftDoR(i,j) = (0.d0,0.d0)   
         MSoftLeL(i,j) = (0.d0,0.d0)
         MSoftElR(i,j) = (0.d0,0.d0)
        enddo
      enddo    



      Do i=1,3      
        ALe(i, i) = Outsoft%soft%asofte(i)
        AUp(i, i) = Outsoft%soft%asoftu(i)
        ADo(i, i) = Outsoft%soft%asoftd(i)
        MSoftQuL(i,i) = Outsoft%soft%mass2q(i)
        MSoftUpR(i,i) = Outsoft%soft%mass2u(i)
        MSoftDoR(i,i) = Outsoft%soft%mass2d(i)
        MSoftLeL(i,i) = Outsoft%soft%mass2l(i)
        MSoftElR(i,i) = Outsoft%soft%mass2e(i)
      enddo



      if (debug) then 
        print*,' '
        print*,'Trilinears '
        print*,'ALe ', Outsoft%soft%asofte
        print*,'AUp ', Outsoft%soft%asoftu
        print*,'ADo ', Outsoft%soft%asoftd
      endif

      
!new stuff  

      if (debug) then 
        print*,' '
        print*,'Soft-Masses '
        print*,' MSoftQu ',Outsoft%soft%mass2q
        print*,' MSoftUpR ',Outsoft%soft%mass2u
        print*,' MSoftDoR ',Outsoft%soft%mass2d
        print*,' MSoftLeL ',Outsoft%soft%mass2l
        print*,' MSoftelR ',Outsoft%soft%mass2e
      endif



      mn1 = outsoft%neut%mneut


      ZN1 = outsoft%neut%mixchi


      MC = outsoft%charg%mcha
     


      ZCU(1, 1) = cos(outsoft%charg%thetaL)
      ZCU(2, 2) = ZCU(1, 1)
      ZCU(1, 2) = sin(outsoft%charg%thetaL)
      ZCU(2, 1) = - ZCU(1, 2);


      ZCV(1, 1) = cos(outsoft%charg%thetaR)
      ZCV(2, 2) = ZCV(1, 1)
      ZCV(1, 2) = sin(outsoft%charg%thetaR)
      ZCV(2, 1) = - ZCV(1, 2);



! Convert from negative Neutralino masses with a real mixing matrix to
! positive definate Neutralino masses with a complex mixing matrix
      Do i=1,4
         Do j=1,4
            If (MN1(i).lt.0.D0) Then
               ZN(i,j) = ZN1(i,j)*DCMPLX(0.D0,1.D0)
            Else
               ZN(i,j) = ZN1(i,j)*DCMPLX(1.D0,0.D0)
            End If
         End Do
         If (MN1(i).lt.0.D0) Then
            MN(i) = -MN1(i)
         Else
            MN(i) = MN1(i)
         End If
      End Do

      if (debug) then 
        print*,' '
        print*,' Neutralinos + Charginos '
        print*,' MN' ,MN
        print*,' ZN' ,ZN
        print*,' MC' ,MC 
        print*,' ZCU',ZCU    
        print*,' ZCV',ZCV
      endif
      

!*********************************************************************** 
! Now that everything's been read in, call the routines. First set some
! options.
! Number of iterations used in iterative procedure (0 if i0rd=0)
      iresum = 100
! Include electroweak contributions in iterative procedure
      iEWcor = 1
! Order of calculation 0,3 or 4 recommended
! i0rd=0 leading order
! iOrd=1 leading order MC's + iterative procedure
! iOrd=2 NLO MC's no iterative procedure
! iOrd=3 NLO MC's + iterative proceudre
! iOrd=4 NLO MC's for SM + Charged Higgs contributions LO otherwise
! + iterative procedure
      iOrd = 4
! Bare basis soft terms. If input is SoftSusy isoft=0
      isoft = 0 !Yukawas diagonal
! Generation for B_q-B_q mixing and B_q->\mu\mu, genq=1 untested
      genq = 2
! Error flag
      bderr = 0
 

if(Out_err%muSqWrongSign.ne.0 .or. Out_err%b.ne.0 .or. &
     Out_err%noRhoConvergence.ne.0 .or. Out_err%tachyon.ne.0 .or. & 
     Out_err%nonperturbative.ne.0 .or. Out_err%noMuConvergence.ne.0 .or. &
     Out_err%higgsUfb.ne.0 .or. Out_err%irqfp.ne.0 .or. &
     Out_err%badConvergence.ne.0.or.Out_err%unphys.ne.0) then

     BDecay%delMB = 1.0d+40
else

! Setup strong coupling routines etc
      Call BDinit(debug, 1)


 ! Some scales
! Bottom mass scale
      mub = mu_do(3)
! Electroweak scale
      muew = mu_up(3)
! SUSY scale
      mususy = mgl

     
! Setup the remaining input parameters
      Call BDinit(debug,2)
! Call the routines
      Call BDmain(debug, iresum, iEWcor, iOrd, isoft, genq, bderr)  
! Output results


      BDecay%delMB  = delMB(genq)

      if(isnan(BDecay%delMB).or.bderr.ne.0) BDecay%delMB = 0.d0
!BR(bsgamma) and BR(bsmm) are swicth off
!      BDec_out%BSGbr = BSGbr
!      BDec_out%BSMbr  =  BSMbr(genq)

      if (Flag%Debug) then 
        write(*,*) ' Delta(MB) = ', delMB(genq)
!        write(*,*) ' BR(b-> s g) = ',BSGbr
!        write(*,*) ' BR(Bs -> mu^+ mu^-) = ',BSMbr(genq)
      endif
      
      Out_err%bd = bderr
endif

end subroutine BD 
