c***********************************************************************

      Subroutine barepara(debug,iresum,iEWcor,isoft,iOrd,bderr)

c Subroutine that determines the bare up and down mass matrices and the
c effective CKM matrix and other related quantities. The method
c here roughly follows that of Buras et. al. (section 2) hep-ph/0207241
c generalized to GFM using the method discussed in hep-ph/0308102
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insm.h'
      Include 'insoft.h'
      Include 'rs.h'
      Include 'vars.h'

c Inputs
      Integer debug, iresum,iEWcor,isoft,iOrd,bderr

c Local
      Integer i,j,k
      Double Complex MIDoS(6,6), MIUpS(6,6)
      Double Complex deltamd(3,3), deltamu(3,3), deltaKM(3,3)
      Double Precision beta, gw

c Functions
      Double Precision alpha2,MSrmass

      beta=ATAN(tanb)
      gw=SQRT(alpha2(mususy)*4.D0*PI)

c Initialise the up and down squark mass matrix
      Do i=1,6
         Do j=1,6
            MIUpS(i,j)=DCMPLX(0.D0,0.D0)
            MIDoS(i,j)=DCMPLX(0.D0,0.D0)
         End Do
      End Do

c The initial iteration sets mdbare and mubare equal to their respective
c physical values at the SUSY scale, set bare basis soft terms.
      Do i=1,3
         Do j=1,3
            BBMSoftQuL(i,j)=MSoftQuL(i,j)
            BBMSoftUpR(i,j)=MSoftUpR(i,j)
            BBMSoftDoR(i,j)=MSoftDoR(i,j)
            BBAUp(i,j)=AUp(i,j)
            BBADo(i,j)=ADo(i,j)
            mdbare(i,j)=DCMPLX(0.D0,0.D0)
            mubare(i,j)=DCMPLX(0.D0,0.D0)
            KMbare(i,j)=KM(i,j)
         End Do
         mdbare(i,i)=DCMPLX(MSrmass(mususy,2*i-1,6,4),0.D0)
         mubare(i,i)=DCMPLX(MSrmass(mususy,2*i,6,4),0.D0)
      End Do

      Do i=1,3
         Do j=1,3
            SigmadVL(i,j)=DCMPLX(0.D0,0.D0)
            SigmadVR(i,j)=DCMPLX(0.D0,0.D0)
            SigmadmL(i,j)=DCMPLX(0.D0,0.D0)
            SigmauVL(i,j)=DCMPLX(0.D0,0.D0)
            SigmauVR(i,j)=DCMPLX(0.D0,0.D0)
            SigmaumL(i,j)=DCMPLX(0.D0,0.D0)
         End Do
      End Do

c Set the original mass matrix and find the mixing matrices and masses

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Down Squark Mass Matrix'
      End If
      Call MassMatrix(debug,1,6,MIDoS)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Up Squark Mass Matrix'
      End If
      Call MassMatrix(debug,2,6,MIUpS)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Down Squark Diagonalization'
      End If
      Call MixMat(debug,6,MIDoS,ZSD,MDoS)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Up Squark Diagonalization'
      End If
      Call MixMat(debug,6,MIUpS,ZSU,MUpS)

c At this point we have masses and mixing matrices for the up and down
c squarks, and if the user doesn't wish to include resummation effects
c no further calculations are necessary.

      If (iresum.ne.0) Then
c Number of iterations
         Do i=1,iresum

c The beginning of the resummation effects calculation. The resummation
c effects are taken into account in this order :
c bottom mass effects
c up quark mass effects
c Bare CKM - Physical CKM effects

c Calculate the various vertices
            Call rsvertices(debug,iOrd)

c Calculate the self energies
            Call c_Sigmas(debug,iEWcor)

c Calculate the bare down mass matrix
            
            Call c_deltamd(debug,deltamd,iEWcor)

c ruiz
            if (any(abs(deltamd(:,:)).gt. 1.d10)) then 
             bderr = 1
             return 
            endif     


            Do j=1,3
               Do k=1,3
                  mdbare(j,k)=MSrmass(mususy,2*j-1,6,4)*Unit3(j,k)
     &                 -deltamd(j,k)
               End Do
            End Do

c Calculate the bare up mass matrix

            Call c_deltamu(debug,deltamu,iEWcor)

cruiz
            if (any(abs(deltamu(:,:)).gt. 1.d10)) then 
             bderr = 1
             return
            endif

            Do j=1,3
               Do k=1,3
                  mubare(j,k)=MSrmass(mususy,2*j,6,4)*Unit3(j,k)
     &                 -deltamu(j,k)
               End Do
            End Do

            If (isoft.eq.0) Then
               Call trsoft(debug)
            End If

c Calculate the bare CKM matrix used in SUSY Feynman rules etc
            Call bareCKM(debug,deltaKM,iEWcor)

            Do j=1,3
               Do k=1,3
                  KMbare(j,k)=KM(j,k)-SQRT(2.D0)/gw*deltaKM(j,k)
               End Do
            End Do

c Calculate the new squark masses to be used in the next iteration
            Call MassMatrix(0,1,6,MIDoS)
            Call MixMat(0,6,MIDoS,ZSD,MDoS)

            Call MassMatrix(0,2,6,MIUpS)
            Call MixMat(0,6,MIUpS,ZSU,MUpS)

         End Do
      End If

      Return
      End
