c**********************************************************************
c Subroutines that calculate the vertices used in calculating 
c self-energies and bare CKM effects used in the iterative procedure. 
c I use the Feynman rules of hep-ph/9511250 with the following 
c differences in notation.
c \Gamma_{uL,R}  -->  \Gamma_{uL,R}^{*}
c A_u            -->  -A_u
c Yd             -->  -Yd
c v_i/SQRT{2}    -->  v_i
c Z_{+}^{ij}     -->  V^{ji*}
c Z_{-}^{ij}     -->  U^{ji}
c Z_N            -->  N^{ji*}
c**********************************************************************

      Subroutine rsvertices(debug,iOrd)
c**********************************************************************

      Implicit NONE

c Inputs
      Integer debug,iOrd

      Include 'const.h'
      Include 'insm.h'
      Include 'insoft.h'
      Include 'vars.h'
      Include 'vertex.h'

c Local
      Integer i,j,k,l,m,n
      Double Precision beta,thw,gsQG,gwSU,gwSQ,gwQH,gwEW
      Double Precision ordfac
      Double Complex tmp,tmp1,tmp2,tmp3,tmp4
      Double Complex SDLeft(3,3),SDRight(3,3),SULeft(3,3),SURight(3,3)

c Functions 
      Double Precision alphaStr,alpha2

      beta=ATAN(tanb)
      thw=ASIN(SQRT(sw2))

c**********************************************************************
c Conversions between DR-bar and MS-bar quantities
c**********************************************************************

      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         ordfac=1.D0
      Else
         ordfac=0.D0
      End If


c NOTE in the conversion between MS-bar and DR-bar quantities I only
c retain O(alphaS) terms
      beta=ATAN(tanb)
      thw=ASIN(DSQRT(sw2))

c Gluino MS-bar coupling C(f)=4/3 C(G)=3
      gsQG=DSQRT(alphaStr(mususy,6,4)*4.D0*PI)
     &     *(1.D0+3.D0*ordfac*alphaStr(mususy,6,4)/(6.D0*PI)
     &     -ordfac*alphaStr(mususy,6,4)/(6.D0*PI))

c SU(2)_L MS-bar coupling at mususy
      gwSU=DSQRT(alpha2(mususy)*4.D0*PI)

c The gauge coupling that appears in the gaugino interactions involing
c squarks 
      gwSQ=DSQRT(alpha2(mususy)*4.D0*PI)
     &     *(1.D0-ordfac*alphaStr(mususy,6,4)/(6.D0*PI))

c The gauge coupling that appears in the Yukawa interation involving
c squarks and Higgsinos
      gwQH=DSQRT(alpha2(mususy)*4.D0*PI)
     &     *(1.D0-ordfac*alphaStr(mususy,6,4)/(2.D0*PI))

c SU(2)_L MS-bar coupling at muew
      gwEW=DSQRT(alpha2(muew)*4.D0*PI)

c**********************************************************************
c The matrices S_L and S_R that appear when the heavy particles are
c integrated out.
c**********************************************************************

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         Do i=1,3
            Do j=1,3
               SDLeft(i,j)=Unit3(i,j)+1.D0/2.D0*SigmadVL(i,j)
               SDRight(i,j)=Unit3(i,j)+1.D0/2.D0*SigmadVR(i,j)
               SULeft(i,j)=Unit3(i,j)+1.D0/2.D0*SigmauVL(i,j)
               SURight(i,j)=Unit3(i,j)+1.D0/2.D0*SigmauVR(i,j)
            End Do
         End Do
      Else
         Do i=1,3
            Do j=1,3
               SDLeft(i,j)=Unit3(i,j)
               SDRight(i,j)=Unit3(i,j)
               SULeft(i,j)=Unit3(i,j)
               SURight(i,j)=Unit3(i,j)
            End Do
         End Do
      End If

c***********************************************************************
c Down-Quark-Squark-Gaugino/Higgsino Vertices
c***********************************************************************

c Gluino Vertices

c Note that colour algebra factors need to be included when calculating
c Wilson coefficients

      Do i=1,3
         Do j=1,6
            tmp1=DCMPLX(0.D0,0.D0)
            tmp2=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp1=tmp1-DSQRT(2.D0)*gsQG*ZSD(j,k)*SDLeft(k,i)
               tmp2=tmp2+DSQRT(2.D0)*gsQG*ZSD(j,k+3)*SDRight(k,i)
            End Do
            gqsqL(i,j)=tmp1
            gqsqR(i,j)=tmp2
         End Do
      End Do

c Chargino Vertices
      Do i=1,3
         Do j=1,2
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1+gwQH*ZSU(k,m+3)*mubare(m,l)
     &                       *KMbare(l,n)*SDLeft(n,i)*DCONJG(ZCV(j,2))
     &                       /(DSQRT(2.D0)*mw*SIN(beta))
                        tmp2=tmp2+gwQH
     &                       *ZSU(k,m)*KMbare(m,l)*DCONJG(mdbare(n,l))
     &                       *SDRight(n,i)*DCONJG(ZCU(j,2))
     &                       /(DSQRT(2.D0)*mw*COS(beta))
                     End Do
                     tmp3=tmp3-gwSQ*ZSU(k,l)*DCONJG(ZCV(j,1))
     &                    *KMbare(l,m)*SDLeft(m,i)
                  End Do
               End Do
               cqsqL(i,j,k)=tmp1+tmp3
               cqsqR(i,j,k)=tmp2
            End Do
         End Do
      End Do

c Neutralino Vertices
      Do i=1,3
         Do j=1,4
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1-gwQH/(DSQRT(2.D0)*mw*COS(beta))
     &                    *ZSD(k,l+3)*mdbare(l,m)*SDLeft(m,i)
     &                    *DCONJG(ZN(j,3))
                     tmp2=tmp2-gwQH/(DSQRT(2.D0)*mw*COS(beta))*ZSD(k,l)
     &                    *DCONJG(mdbare(i,l))*SDRight(m,i)*ZN(j,3)
                  End Do
                  tmp3=tmp3-gwSQ/(DSQRT(2.D0)*COS(thw))*ZSD(k,l)
     &                 *SDLeft(l,i)*(1.D0/3.D0*DCONJG(ZN(j,1))*SIN(thw)
     &                 -DCONJG(ZN(j,2))*COS(thw))
                  tmp4=tmp4-DSQRT(2.D0)*gwSQ*TAN(thw)/3.D0
     &                 *ZSD(k,l+3)*SDRight(l,i)*ZN(j,1)
               End Do
               nqsqL(i,j,k)=tmp1+tmp3
               nqsqR(i,j,k)=tmp2+tmp4
            End Do
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then
         Write(*,*) '**********************************************'
         Write(*,*) 'Down Quark Vertices'
         Write(*,*) '**********************************************'

         Do i=1,3
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqL(',i,',',j,')=',gqsqL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqR(',i,',',j,')=',gqsqR(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,2
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqL(',i,',',j,',',k,')=',cqsqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqR(',i,',',j,',',k,')=',cqsqR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,3
            Do j=1,4
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqL(',i,',',j,',',k,')=',nqsqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqR(',i,',',j,',',k,')=',nqsqR(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c Up-Quark-Squark-Gaugino/Higgsino Vertices
c***********************************************************************

c Gluino Vertices

c Note that colour algebra factors need to be included when calculating
c Wilson coefficients

      Do i=1,3
         Do j=1,6
            tmp1=DCMPLX(0.D0,0.D0)
            tmp2=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp1=tmp1-DSQRT(2.D0)*gsQG*ZSU(j,k)*SULeft(k,i)
               tmp2=tmp2+DSQRT(2.D0)*gsQG*ZSU(j,k+3)*SURight(k,i)
            End Do
            gqsqLup(i,j)=tmp1
            gqsqRup(i,j)=tmp2
         End Do
      End Do

c Chargino Vertices
      Do i=1,3
         Do j=1,2
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1+gwQH*ZSD(k,m+3)*mdbare(m,l)
     &                       *DCONJG(KMbare(n,l))*ZCU(j,2)*SULeft(n,i)
     &                       /(DSQRT(2.D0)*mw*COS(beta))
                        tmp2=tmp2+gwQH*ZSD(k,m)*DCONJG(KMbare(l,m))
     &                       *DCONJG(mubare(n,l))*ZCV(j,2)*SURight(n,i)
     &                       /(DSQRT(2.D0)*mw*SIN(beta))
                     End Do
                     tmp3=tmp3-gwSQ*ZSD(k,l)*ZCU(j,1)
     &                    *DCONJG(KMbare(i,l))*SULeft(m,i)
                  End Do
               End Do
               cqsqLup(i,j,k)=tmp1+tmp3
               cqsqRup(i,j,k)=tmp2
            End Do
         End Do
      End Do

c Neutralino Vertices
      Do i=1,3
         Do j=1,4
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1-gwQH/(DSQRT(2.D0)*mw*SIN(beta))
     &                    *ZSU(k,l+3)
     &                    *mubare(l,m)*SULeft(m,i)*DCONJG(ZN(j,4))
                     tmp2=tmp2-gwQH/(DSQRT(2.D0)*mw*SIN(beta))*ZSU(k,l)
     &                    *DCONJG(mubare(m,l))*SURight(m,i)*ZN(j,4)
                  End Do
                  tmp3=tmp3-gwSQ/(DSQRT(2.D0)*COS(thw))*ZSU(k,l)
     &                 *SULeft(l,i)*(1.D0/3.D0*DCONJG(ZN(j,1))*SIN(thw)
     &                 +DCONJG(ZN(j,2))*COS(thw))
                  tmp4=tmp4-2.D0*DSQRT(2.D0)
     &                 *gwSQ*TAN(thw)/3.D0*ZSU(k,i+3)*SURight(l,i)
     &                 *ZN(j,1)
               End Do
               nqsqLup(i,j,k)=tmp1+tmp3
               nqsqRup(i,j,k)=tmp2+tmp4
            End Do
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************
      If (debug.eq.1) Then
         Write(*,*) '**********************************************'
         Write(*,*) 'Up Quark Vertices'
         Write(*,*) '**********************************************'

         Do i=1,3
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqLup(',i,',',j,')=',gqsqLup(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqRup(',i,',',j,')=',gqsqRup(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,2
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqLup(',i,',',j,',',k,')=',cqsqLup(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqRup(',i,',',j,',',k,')=',cqsqRup(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,3
            Do j=1,4
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqLup(',i,',',j,',',k,')=',nqsqLup(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqRup(',i,',',j,',',k,')=',nqsqRup(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c W Boson vertices
c***********************************************************************

      Do i=1,2
         Do j=1,4
            WChNeL(i,j)=gwSU*(DCONJG(ZN(j,2))*ZCV(i,1)
     &           -1.D0/SQRT(2.D0)*DCONJG(ZN(j,4))*ZCV(i,2))
            WChNeR(i,j)=gwSU*(ZN(j,2)*ZCU(i,1)
     &           -1.D0/SQRT(2.D0)*ZN(j,4)*ZCU(i,2))
         End Do
      End Do

      Do i=1,6
         Do j=1,6
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               Do l=1,3
                  tmp=tmp-gwSU/SQRT(2.D0)
     &                 *DCONJG(ZSD(i,k))*ZSU(j,l)*KMbare(l,k)
               End Do
            End Do
            Wdsqusq(i,j)=tmp
         End Do
      End Do

      Return
      End
