      FUNCTION RLI2(arg) 
C----------------------------------------------------------------------
C
C  Real Li_2(x) function of the real argument
C  When translated to MPF gives MP version of the RLI2 function
C
C  Written by O.Veretin, 14.09.98
C 
C  Parameters:
C     arg --- the argument 
C             ( must be within the range (-infinity, 1.0) )
C 
C  >>
C  CHECKVALUE with precision 100 decimals:
C
C  Li_2(-3.0) = 
C -1.9393754207667089530772717191778914412225901778085784258385
C                  5746674797252831228375158904299708991596988,
C
C
C
C  MULTIPRECISION NOTES:
C
C  NOTE1:
C    * parameter "arg" in MP version
C      is defined as MULTIP REAL variable.
C
C  NOTE2:
C    * By default precision is 1.0D-16 but 
C      it is _automatically_ replaced by MP 
C      value while transforming to MPF.
C
C  NOTE3:
C    * The Pi constant (DPPIC) also will be replaced 
C      _automatically_ by the MP version of Pi.
C  

CMP+  MULTIP REAL      arg,argn,term
CMP+  MULTIP REAL      RLI2x0

      double precision arg,argn,term,DPEPS,DPPIC
      double precision RLI2,RLI2x0
      
      PARAMETER ( DPEPS = 1.0d-16 )
      PARAMETER ( DPPIC = 3.14159265358979323846264338328d+0 )

      IF ( ABS(arg-1.0d0) .LE. DPEPS )  GOTO 9  
      IF ( ABS(arg) .LT. 0.51 ) GOTO 10
      IF ( ( arg .GT. 0.5 ) .AND. ( arg .LE. 1.0 ) ) GOTO 11
      IF ( ( arg .GT. -2.0 ) .AND. ( arg .LE. -0.99 ) ) GOTO 12
      IF ( ( arg .GT. -1.0 ) .AND. ( arg .LE. -0.50 ) ) GOTO 13
      IF ( arg .LT. -1.99 )  GOTO 14
      GOTO 200
C----------------------------------------------------------------
C            arg = 1
C----------------------------------------------------------------
 9    RLI2 = DPPIC*DPPIC/6.0d0
      RETURN
C----------------------------------------------------------------
C          Region ( -0.5, +0.5 )
C----------------------------------------------------------------
 10   RLI2=RLI2x0(arg)
      RETURN
C----------------------------------------------------------------
C          Region ( +0.5, 1.0 )
C----------------------------------------------------------------
 11   argn = 1.0d0 - arg
      RLI2 = -RLI2x0(argn) 
     $        - log(arg)*log(argn) + DPPIC*DPPIC/6.0d0
      RETURN 
C----------------------------------------------------------------
C          Region ( -2.0, -1.0 )
C----------------------------------------------------------------
 12   argn = 1.0d0/(1.0d0-arg)
      term = log(1.0d0-arg)
      RLI2 = RLI2x0(argn) - term*log(-arg) + term*term*0.5d0 
     $          - DPPIC*DPPIC/6.0d0
      RETURN 
C----------------------------------------------------------------
C          Region ( -1.0, -0.5 )
C----------------------------------------------------------------
 13   argn = -arg/(1.0d0-arg)
      term = log(1.0d0-arg)
      RLI2 = -RLI2x0(argn) - term*term*0.5d0
      RETURN 
C----------------------------------------------------------------
C          Region ( -Infinity, -2.0 )
C----------------------------------------------------------------
 14   argn=1.0/arg
      RLI2 = -RLI2x0(argn) 
     $         - log(-argn)**2 * 0.5d0 - DPPIC*DPPIC/6.0d0
      RETURN 
C----------------------------------------------------------------
C          Error message
C----------------------------------------------------------------
 200  print *,'RLI2 (real RLi2 function) error: wrong parameter!'
      STOP
      END
 

C*****************************************************************
C   Zero domain li2 function
C*****************************************************************

CMP+  multip real RLI2x0

      FUNCTION RLI2x0(arg)

CMP+  MULTIP REAL      arg,argn,term
      double precision RLI2x0
      double precision arg,argn,term,DPEPS 
      integer j
      PARAMETER ( DPEPS = 1.0d-16 )
      j=0
      RLI2x0 = 0.0d0
      argn = 1.0d0

99    continue   
      j=j+1
      argn=argn*arg
      term=argn/(j**2)
      RLI2x0=RLI2x0+term
      if ( ABS(term) .GT. DPEPS ) goto 99
      RETURN
      END

C****  THE END ***






