       subroutine ddloop
c
c_______________________________________________________________________
c  some auxiliar functions used in calculating neutralino-nucleon
c  cross section when we include QCD corrections.  
c  common:
c    'dssusy.h' - file with susy common blocks
c    'dsddloops.h' - file with common blocks used in calculating
c     netralino-nucleon cross section including QCD corrections.
c  called by ddinit.
c  author: roberto ruiz
c=======================================================================
c          
       implicit none
       include 'dssusy.h'
       include 'sheffdd.h'

c-------local variables --------------

       integer a,i
       real*8 m2ln,m4ln,m6ln,m2q,m2sq,ckin,fD,sqrtfD,fL
       real*8 fun_D(41,12), fun_L(41,12)

c---- I set the initial values for every auxiliar function to zero ---


      do i=1,6
        do a=1,6
          fun_D(ksq(i,a),Kq(i)) = 0.0d0
          fun_L(ksq(i,a),kq(i)) = 0.0d0
          I1(ksq(i,a),Kq(i)) = 0.0d0
          I2(ksq(i,a),Kq(i)) = 0.0d0
          I3(ksq(i,a),Kq(i)) = 0.0d0
          I4(ksq(i,a),Kq(i)) = 0.0d0 
          I5(ksq(i,a),Kq(i)) = 0.0d0
        enddo
      enddo

          
     

c------ I define auxiliar parameters useful in this subroutine -------

      m2ln = mass(kn(kln))**2.0d0
      m4ln = mass(kn(kln))**4.0d0
      m6ln = mass(kn(kln))**6.0d0

      
     

c------- function D --------------
         
      do i=1,6
       do a=1,6
          m2sq = mass(ksq(i,a))**2.0d0
          m2q = mass(kq(i))**2.0d0
          fun_D(ksq(i,a),Kq(i)) = 
     &     2.0d0*m2ln*(m2q+m2sq)-m4ln-(m2q-m2sq)**2.0d0   
       enddo
      enddo


c-------- function L ----------------
 
      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i)) 
         sqrtfD = dsqrt(dabs(fD))
         ckin=m2q+m2sq-m2ln
         if (fD .gt. 0.0d0) then
             fun_L(ksq(i,a),Kq(i)) = 2.0d0/sqrtfD*datan(sqrtfD/ckin)
         else if(fD .lt. 0.0d0) then
             fun_L(ksq(i,a),Kq(i)) =
     &          1.0d0/sqrtfD*dlog((ckin+sqrtfD)/(ckin-sqrtfD))
         else 
             fun_L(ksq(i,a),Kq(i)) = 2.0d0/ckin
         end if
       enddo
      enddo


c-------- function I1 ----------------

      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i))
         fL = fun_L(ksq(i,a),Kq(i))
         I1(ksq(i,a),Kq(i)) = 
     &        ((m2q-m2ln)/3.0d0/m2sq
     &            -2.0d0/3.0d0*(m2sq-m2ln)/m2q
     &            -5.0d0/3.0d0 
     &           + 2.0d0*(m2sq-m2ln/3.0d0)*fL)/fD
       enddo
      enddo

      

c-------- function I2 ----------------

      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i))
         fL = fun_L(ksq(i,a),Kq(i))
         I2(ksq(i,a),Kq(i)) = 
     &      (dlog(m2sq/m2q)-(m2sq-m2q-m2ln)*fL)/2.0d0/m4ln
     &       + ((m2q*(m2q-m2sq)/m2ln-7.0d0*m2q/3.0d0
     &            +2.0d0*(m2ln-m2sq)/3.0d0)*fL 
     &          + (m2q-m2ln)/3.0d0/m2sq
     &          + (m2sq-m2q)/m2ln+2.0d0/3.0d0)/fD
       enddo
      enddo

              
c-------- function I3 ----------------

      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i))
         fL = fun_L(ksq(i,a),Kq(i))
         I3(ksq(i,a),Kq(i)) = 
     &       3.0d0*(m2ln-m2sq-m2q)/fD/fD
     &        + fL/fD*(-1.0d0+6.0d0*m2sq*m2q/fD)
       enddo
      enddo

    

c-------- function I4 ----------------

      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i))
         fL = fun_L(ksq(i,a),Kq(i))
         I4(ksq(i,a),Kq(i)) = 
     &     (dlog(m2sq/m2q)-(m2sq-m2q-m2ln)*fL)/2.0d0/m6ln
     &      - 1.0d0/m2sq/m4ln- m2q*(m2sq-m2q-m2ln)*fL/m4ln/fD
     &     + (m2q/m4ln-(1.0d0-m2q/m2ln)**2.0d0/m2sq
     &                   +1.0d0/2.0d0/m2ln)/fD
     &     + 3.0d0*m2q/fD/fD*(1.0d0+(m2sq-m2q)/m2ln
     &          +(m2q*(m2q-m2sq)/m2ln-2.0d0*m2q-m2sq+m2ln)*fL)
       enddo
      enddo

    
          
c-------- function I5 ----------------

      do i=1,6
       do a=1,6
         m2sq = mass(ksq(i,a))**2.0d0
         m2q = mass(kq(i))**2.0d0
         fD = fun_D(ksq(i,a),Kq(i))
         fL = fun_L(ksq(i,a),Kq(i))
         I5(ksq(i,a),Kq(i)) =
     &     (dlog(m2sq/m2q)-(m2sq-m2q-m2ln)*fL)/2.0d0/m4ln
     &      -((2.0d0*(m2sq-m2ln)+3.0d0*m2q
     &         +m2q*(m2sq-m2q)/m2ln)*fL
     &         -3.0d0+(m2q-m2sq)/m2ln)/fD
      
       enddo
      enddo              
     
      

      return
      end        















