      FUNCTION ALFS(Q)
C----------------------------------------------------------------------
C
C       This function returns the 1, 2, or 3-loop value of alpha_s
C       Input:
C               QSQ     = Q**2 (real)
C               ALAM4   = Lambda for 4 active quark flavors (real)
C               TMASS   = top quark mass to determine lambda-6 (real)
C               LOOP    = number of loops for alpha_s (= 1, 2 or 3)
C       Parametrization of the strong coupling constant according to
C       LOOP = 1, 2 : from the book;
C       LOOP = 3:     W. J. Marciano, Phys. Rev. D 29 (1984) 580.
C       Note : threshold at 2*Mq
C

      IMPLICIT NONE

      include 'dssusy.h'

      REAL*8 ALFS
      REAL*8 Q

C LOCAL

      INTEGER NF
      REAL*8  QSQ, LAM, LAMSQ, T, TT, B0, B1, B2, X 



C
      QSQ = Q**2.0D0

      IF (Q .LT. MBMB) THEN
        NF   = 4
        LAM=0.291D0
      ELSE IF (Q .LT. MTMT) THEN
        NF   = 5
        LAM=0.209D0
      ELSE
        NF   = 6
        LAM=0.0882D0
      END IF

      LAMSQ   = LAM**2
      T        = DLOG(QSQ/LAMSQ)
      IF (T .LE. 1.0) T = DLOG(4.0/LAMSQ)

      B0 = 11.0-2.0/3.0*NF
      B1 = 51.0-19.0/3.0*NF
      B2 = 2857.0-5033.0/9.0*NF+325.0/27.0*NF**2

      X  = 2.0D0*B1/B0**2/T
      TT = DLOG(T)

      ALFS = 4.0D0*PI/B0/T*(1.0-X*TT+X**2*((TT-0.5D0)**2.0D0
     $      +B2*B0/8.0D0/B1**2-1.25D0))


C
      RETURN
      END









