      FUNCTION RMASS(KQ,Q)
C----------------------------------------------------------------------
C
C       This function returns the running mass for quarks.
C       Computations are performanced according to hep-ph/9712201.
C--------------------------------------------------------------------

      IMPLICIT NONE
      INCLUDE 'dssusy.h'
     
      INTEGER KQ
      REAL*8 Q

      REAL*8 LOOP,RMASS,ALFS,RQCD

c local
 
      INTEGER I,N
      REAL*8 MASQ(6),K(12)
      REAL*8 ZRi3,K0,SUM,MQ,MQMQ

      DATA ZRi3/1.2021d0/


C BEGIN

C INIT. SOME PARAMETERS

      IF(KQ.EQ.KC) THEN 
        N=4
      ELSE IF(KQ.EQ.KB) THEN
        N=5
      ELSE IF(KQ.EQ.KT) THEN
        N=6
      ELSE
        WRITE(*,*) 'BAD INPUT'
        STOP
      ENDIF

      MASQ(1)=MASS(ku)
      MASQ(2)=MASS(kd)
      MASQ(3)=MASS(ks)
      MASQ(4)=MASS(kc)
      MASQ(5)=MASS(kb)
      MASQ(6)=MASS(kt) 


      DO I=1,10
       K(I)=0.0D0
      END DO

          
      k0=1./9.*PI**2*DLOG(2.0d0)+7./18.*PI**2-1./6.*ZRi3+3673./288.
     &   -(1./18.*pi**2+71./144.)*N
      
      SUM=0.0d0

      DO I=1,N-1
        SUM=SUM+LOOP(MASQ(I)/MASQ(N))
      ENDDO      


      K(KQ)=K0+4./3.*SUM


C      K(9)=14.47D0
C      K(11)=10.98D0
C      K(12)=12.94D0 !from Fusaoka et. al.

c       K(9)=-11.21d0 !10.32D0
c       K(11)=-10.17d0 !8.24D0
c       K(12)=-9.13D0 !from chetyrkin et. al.
         
      
C RUNNING MASSES AT Q=MQ
      

      IF(KQ.EQ.KC) THEN
c       MQ=MCH
        MQ=MASS(KC)
      ELSE IF(KQ.EQ.KB) THEN
c       MQ=MBT
        MQ=MASS(KB)
      ELSE IF(KQ.EQ.KT) THEN
c       MQ=MTP
        MQ=MASS(KT)
      ELSE
       WRITE(*,*) 'THE FIRST ARGUMENT OF RMASS IS WRONG' 
       STOP
      ENDIF
     
C RUNNING MASS AT POLE MASS SCALE
    

      MQMQ=MQ/(1.+4.0d0*ALFS(MQ)/3.0d0/PI
     $     + K(KQ)*(ALFS(MQ)/PI)**2.0d0)


C I HAVE TO CONSIDER SCALES BIGGER THAN RUNNING MASSES

      RMASS=MQMQ*RQCD(KQ,Q)

      RETURN
      END
 
      FUNCTION LOOP(X)
 
      IMPLICIT NONE

      REAL*8 X,LOOP
      real*8 PI

      DATA PI/3.141592653589793238d0/      

      LOOP=PI**2/8.*X-0.597*X**2+0.230*X**3

      RETURN
      END























