      function dsgm2muon()
c supersymmetric contribution to (g-2)_muon
c  output:
c    gm2amp : susy contribution to g-2 amplitude  = (g-2)/2
c according to T Moroi hep-ph/9512396 v3
c (T. Moroi, PRD 1996; (E) 1997)

c author: paolo gondolo 2001-02-08
c reference: e a baltz and p gondolo, hep-ph/0102147
c added leading two-loop SUSY contributions by Roberto Ruiz

      implicit none

      include 'dssusy.h'
 
      real*8 gf 
      parameter (gf = 1.16639d-5)
      real*8 dsgm2muon,daneu,dacha,tmp,x,dsabsq
      real*8 beta,sb,cb,c2b,sa,ca,sab,cab,mchi,msmu,mcha,clh,
     &  Msusy, gm2susy1L, gm2susy2L_QED, clbos2L, gm2susy2L_lst,
     &  gm2susy2L_bos, gm2susy2L_lsb, gm2susy2L_sf, gm2susy2L_chi,z,
     & rst
      integer i,j

      external L2F
      real*8 L2F,sq
 
      common/ratio/z
   
      sq(x)=x**2.d0

      beta=atan(tanbe)
      sb=sin(beta)
      cb=cos(beta)
      sa=sin(alpha)
      ca=cos(alpha)
      sab=sin(alpha+beta)
      cab=cos(alpha+beta)
      c2b=cos(2.d0*beta)


      mchi=0.d0
      msmu=0.d0
      mcha=0.d0

      daneu=0.d0
      do i=1,2
        do j=1,4
          x=(mass(kn(j))/mass(ksmu(i)))**2
          if((abs(x-1.d0).lt.1.d-3))then
            tmp=-(mass(kmu)/mass(ksmu(i)))**2/(6.d0)/(2.d0) *
     &       (dsabsq(gl(kmu,kn(j),ksmu(i)))+
     &        dsabsq(gr(kmu,kn(j),ksmu(i))))
     &       -mass(kmu)*mass(kn(j))/mass(ksmu(i))**2/(3.d0) *
     &       dble(gl(kmu,kn(j),ksmu(i))*conjg(gr(kmu,kn(j),ksmu(i))))
          else
            tmp=-(mass(kmu)/mass(ksmu(i)))**2/(6.d0*(1.d0-x)**4) *
     &       (dsabsq(gl(kmu,kn(j),ksmu(i)))+
     &        dsabsq(gr(kmu,kn(j),ksmu(i)))) *
     &       (1.d0-6.d0*x+3.d0*x**2+2.d0*x**3-6.d0*x**2*log(x))
     &       -mass(kmu)*mass(kn(j))/mass(ksmu(i))**2/(1.d0-x)**3 *
     &       dble(gl(kmu,kn(j),ksmu(i))*conjg(gr(kmu,kn(j),ksmu(i)))) *
     &       (1.d0-x**2+2.d0*x*log(x))
          endif
          daneu=daneu+tmp
        enddo
       msmu=msmu+mass(ksmu(i))
      enddo

      
      dacha=0.d0
      do i=1,2
        x=(mass(kcha(i))/mass(ksnumu))**2
        if((abs(x-1.d0).lt.1.d-3))then
         tmp=(mass(kmu)/mass(ksnumu))**2/12.d0 * 
     &     (dsabsq(gl(ksnumu,kmu,kcha(i)))+
     &      dsabsq(gr(ksnumu,kmu,kcha(i))))
     &      + mass(kmu)*mass(kcha(i))/mass(ksnumu)**2*2.0d0/3.0d0 *
     &     dble(gl(ksnumu,kmu,kcha(i))*conjg(gr(ksnumu,kmu,kcha(i))))
        else
         tmp=(mass(kmu)/mass(ksnumu))**2/(3.d0*(1.d0-x)**4) *
     &     (dsabsq(gl(ksnumu,kmu,kcha(i)))+
     &      dsabsq(gr(ksnumu,kmu,kcha(i)))) *
     &     (1.d0+1.5d0*x-3.d0*x**2+0.5d0*x**3+3.d0*x*log(x))
     &     -mass(kmu)*mass(kcha(i))/mass(ksnumu)**2/(1.d0-x)**3 *
     &     dble(gl(ksnumu,kmu,kcha(i))*conjg(gr(ksnumu,kmu,kcha(i)))) *
     &     (3.d0-4.d0*x+x**2+2.d0*log(x))
        endif
        dacha=dacha+tmp
        mcha=mcha+mass(kcha(i))
      enddo

c one-loop result      

      gm2susy1L=(daneu+dacha)/(16.d0*pi**2)

c      print*,'gm2susy1L= ',gm2susy1L

c two-loop inmplementation part


      do i=1,4
        mchi=mchi+mass(kn(i))
      enddo 


      Msusy= (mass(ksnumu)+msmu+mchi+mcha)/9.d0



c two-loop corrections from QED logs, hep-ph/9803384
      
      gm2susy2L_QED = - gm2susy1L * 
     &      (4.d0/(pi*137.d0) * log(Msusy/mass(kmu)))


c      print*,'gm2susy2L_QED= ',gm2susy2L_QED

c Higgs corr: Identical to the SM up a few per cent level due 
c to no-logarithm contributions hep-ph/0405255

c     x=sq(mass(kmu)/mass(kw))
 
c      clh=c2b/cb*sq(mass(kz))*(ca*cab/sq(mass(kh1))+
c     &     sa*sab/sq(mass(kh2)))

c      clbos2L=1.d0/30.d0*(98.d0+9.d0*clh+23.0*sq(1.d0-4.d0*s2thw))


c      gm2susy2L_bos = 5.d0 * gf * sq(mass(kmu)/pi) 
c     &  * alphem/24.d0/sqrt(2.d0)/pi * clbos2L*log(x)


c      print*,'gm2susy2L_bos=  ', gm2susy2L_bos

c two-loop corrections from sfer. hep-ph/0312264

c st1

      z=sq(mass(kst1)/mass(kh1))


      call dgadap(1.d-6,9.9999d-1,L2F,1.d-4,rst)


      gm2susy2L_lst = -0.013d-10/0.34*mass(kt)*mu*
     &   tanbe/sq(mass(kh1))*asoftu(3)/abs(asoftu(3))*rst

c sb1

      z=sq(mass(ksb1)/mass(kh1))


      call dgadap(1.d-6,9.9999d-1,L2F,1.d-4,rst)


      gm2susy2L_lsb = -0.0032d-10/0.34*mass(kb)*asoftd(3)*
     &   tanbe/sq(mass(kh1))*mu/abs(mu)*rst


    

      gm2susy2L_sf = gm2susy2L_lst + gm2susy2L_lsb


c      print*,'gm2susy2L_lst= ',gm2susy2L_lst,'gm2susy2L_lsb= ',
c     & gm2susy2L_lsb


c two-loop corrections from Neut-charg (hep-ph/0405255)

!approx. formula is not valid for small Msusy and tanb 
      If(tanbe < 5.d0 .and. Msusy < 100.d0) then

        gm2susy2L_chi=0.d0 

      else 

        gm2susy2L_chi=11.d-10*(tanbe/50d0)*sq(100.d0/Msusy)

      endif
     
 
c      print*,'gm2susy2L_chi= ',gm2susy2L_chi
 

      dsgm2muon=gm2susy1L + gm2susy2L_QED + gm2susy2L_sf + 
     & gm2susy2L_chi




      return
      end



c two-loop function
      real*8 function L2F(x)

      implicit none


      real*8 x

      real*8 fct, z
  
      common/ratio/z
    

      fct=x*(1.0-x)


      L2F= fct*log(z/fct)/(z-fct) 


      return

      end
