      function dsandwdcos(p,costheta)
c_______________________________________________________________________
c  annihilation differential invariant rate.
c  input:
c    p - initial cm momentum (real) for lsp annihilations
c    costheta - cosine of c.m. annihilation angle
c  common:
c    'dssusy.h' - file with susy common blocks
c  uses dsandwdcosnn, dsandwdcoscn and dsandwdcoscc
c  called by wx.
c  author: joakim edsjo (edsjo@physto.se)
c  date: 96-02-21
c  modified: 97-05-12 Joakim Edsjo (edsjo@physto.se)
c  modified: 01-01-30 paolo gondolo (paolo@mamma-mia.phys.cwru.edu)
c  modified: 02-03-09 Joakim Edsjo (edsjo@physto.se)
c=======================================================================
      implicit none
      include 'dssusy.h'
      include 'dsandwcom.h'
      real*8 dsandwdcos,dsandwdcosnn,dsandwdcoscn,dsandwdcoscc,
     &  dsasdwdcossfsf,dsasdwdcossfchi
      real*8 mx
      real*8 w,p,costheta,pnew,s,mp1,mp2,f,tmp
      integer i,j,kp1(36),kp2(36),nch,ch,kpn(4),nn,kpcha(2),ncha
      integer nsl,kpsl(6),nsnu,kpsnu(3),nsqu,kpsqu(6),nsqd,kpsqd(6)

c      write(*,*) 'dsandwdcos called with p = ',p
c      write(*,*) '            costheta = ',costheta
c...initialize dsankinvar variables
      call dsankinvar(0.0d0,0.0d0,0,0,0,0,0)

      mx = mass(kn(1))
      copart=0   ! flag to see which particles are included

c-------------------------------------- START INCLUDING COANNIHILATIONS
c...Which coannihilation processes to include are given by coproc
c...Bit  Decimal  Coannihilation processes included
c...  0        1  neu_1, neu_2, neu_3, neu_4, cha_1, cha_2
c...  1        2  sleptons, sneutrinos, squarks


      if (coproc.ne.0) then   ! include coannihilation process

c...First determine which particles that should be included
c...include the lightest neutralino
c...JE FIX Move this calculation to dsrdomega where it should be done
c...once for every model and not at two places and for each call here
c...as it is done now. Make a new routine that sets this up.
c...Note: The current setup works, but is not as convenient as it could be.

        kpn(1)=kn(1)
        nn=1
        copart=ibset(copart,0)

c...Reset all coannihilation counters
        ncha=0
        nsl=0
        nsnu=0
        nsqu=0
        nsqd=0

        if (and(coproc,1).ne.0) then ! include neutralinos and charginos

c...decide which neutralinos to include
          do i=2,4
            if (mass(kn(i))/mx.le.mcofr) then
              nn=nn+1
              kpn(nn)=kn(i)
              copart=ibset(copart,i-1)
            endif
          enddo
c...decide which charginos to include
          ncha=0
          do i=1,2
            if (mass(kcha(i))/mx.le.mcofr) then
              ncha=ncha+1
              kpcha(ncha)=kcha(i)
              copart=ibset(copart,i+3)
            endif
          enddo
        endif
        
c...decide which sfermions to include

        if (and(coproc,2).ne.0) then ! sfermions
 
c...sleptons
          nsl=0
          do i=1,6
            if (mass(ksl(i))/mx.le.mcofr) then
              nsl=nsl+1
              kpsl(nsl)=ksl(i)
              copart=ibset(copart,i+5)
            endif
          enddo
          
c...sneutrinos
          nsnu=0
          do i=1,3
            if (mass(ksnu(i))/mx.le.mcofr) then
              nsnu=nsnu+1
              kpsnu(nsnu)=ksnu(i)
              copart=ibset(copart,i+11)
            endif
          enddo

c...up squarks
          nsqu=0
          do i=1,6
            if (mass(ksqu(i))/mx.le.mcofr) then
              nsqu=nsqu+1
              kpsqu(nsqu)=ksqu(i)
              copart=ibset(copart,i+14)
            endif
          enddo

c...down squarks
          nsqd=0
          do i=1,6
            if (mass(ksqd(i))/mx.le.mcofr) then
              nsqd=nsqd+1
              kpsqd(nsqd)=ksqd(i)
              copart=ibset(copart,i+20)
            endif
          enddo

        endif


c---------------------------------------------------------------------
c----- OK, now calculate dwdcos for all combinations of initial states
c---------------------------------------------------------------------

c----------------------------------------------------------------------
c-------------------------------------------- Neutralinos and charginos
c----------------------------------------------------------------------

c---------------------------------- neutralino-neutralino annihilation
        nch=0
        do i=1,nn
          do j=i,nn
            nch=nch+1
            kp1(nch)=kpn(i)
            kp2(nch)=kpn(j)
          enddo
        enddo

        w = dsandwdcosnn(p,costheta,kp1(1),kp2(1))

c...convert given p for lsp:s to pnew and call dsandwdcosnn with pnew
        s = 4.0d0*(mx**2+p**2)
        do ch=2,nch
          mp1 = mass(kp1(ch))
          mp2 = mass(kp2(ch))
          tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
          if (tmp.gt.0.0d0) then
            pnew = sqrt(tmp)
            f=1.0d0   !  f=sum(gi*gj/g1^2) with same mass and wij
            if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
            w=w+f*pnew/p
     &        *dsandwdcosnn(pnew,costheta,kp1(ch),kp2(ch))
          endif
        enddo
  
c------------------------------------ neutralino-chargino annihilation
        if (ncha.gt.0) then
          nch=0
          do i=1,ncha
            do j=1,nn      ! i->1 je correction 97-05-12
              nch=nch+1
              kp1(nch)=kpcha(i)
              kp2(nch)=kpn(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=2.0d0           ! degrees of freedom for chargino=4
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsandwdcoscn(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo

c-------------------------------------- chargino-chargino annihilation
          nch=0
          do i=1,ncha
            do j=i,ncha
              nch=nch+1
              kp1(nch)=kpcha(i)
              kp2(nch)=kpcha(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcoscc with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=4.0d0           ! degrees of freedom for chargino=4
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsandwdcoscc(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif      ! ncha.gt.0


c----------------------------------------------------------------------
c--------------------------------------------- Sfermions with sfermions
c----------------------------------------------------------------------

c----------------------------------------- slepton-slepton annihilation
        if (nsl.gt.0) then
          nch=0
          do i=1,nsl
            do j=i,nsl
              nch=nch+1
              kp1(nch)=kpsl(i)
              kp2(nch)=kpsl(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=1.0d0           ! degrees of freedom for sleptons=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------- sneutrino-sneutrino annihilation
        if (nsnu.gt.0) then
          nch=0
          do i=1,nsnu
            do j=i,nsnu
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpsnu(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=1.0d0           ! degrees of freedom for sneutrinos=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c--------------------------------------- sneutrino-slepton annihilation
        if (nsl.gt.0.and.nsnu.gt.0) then
          nch=0
          do i=1,nsnu
            do j=1,nsl
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpsl(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=1.0d0           ! degrees of freedom for sneutrinos=2 and sleptons=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------- up-squark up-squark annihilation
        if (nsqu.gt.0) then
          nch=0
          do i=1,nsqu
            do j=i,nsqu
              nch=nch+1
              kp1(nch)=kpsqu(i)
              kp2(nch)=kpsqu(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=9.0d0           ! degrees of freedom for up-squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c--------------------------------- down-squark down-squark annihilation
        if (nsqd.gt.0) then
          nch=0
          do i=1,nsqd
            do j=i,nsqd
              nch=nch+1
              kp1(nch)=kpsqd(i)
              kp2(nch)=kpsqd(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=9.0d0           ! degrees of freedom for down-squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c----------------------------------- up-squark down-squark annihilation
        if (nsqu.gt.0.and.nsqd.gt.0) then
          nch=0
          do i=1,nsqu
            do j=1,nsqd
              nch=nch+1
              kp1(nch)=kpsqu(i)
              kp2(nch)=kpsqd(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=9.0d0           ! degrees of freedom for down-squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c--------------------------------------- slepton up-squarks annihilation
        if (nsl.gt.0.and.nsqu.gt.0) then
          nch=0
          do i=1,nsl
            do j=1,nsqu
              nch=nch+1
              kp1(nch)=kpsl(i)
              kp2(nch)=kpsqu(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for sleptons=2 & squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------- slepton down-squarks annihilation
        if (nsl.gt.0.and.nsqd.gt.0) then
          nch=0
          do i=1,nsl
            do j=1,nsqd
              nch=nch+1
              kp1(nch)=kpsl(i)
              kp2(nch)=kpsqd(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for sleptons=2 & squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------- sneutrino up-squarks annihilation
        if (nsnu.gt.0.and.nsqu.gt.0) then
          nch=0
          do i=1,nsnu
            do j=1,nsqu
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpsqu(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for sleptons=2 & squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p*
     &          dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------- sneutrino down-squarks annihilation
        if (nsnu.gt.0.and.nsqu.gt.0) then
          nch=0
          do i=1,nsnu
            do j=1,nsqd
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpsqd(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for sleptons=2 & squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfsf(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c----------------------------------------------------------------------
c----------------------------- Sfermions with neutralinos and charginos
c----------------------------------------------------------------------

c-------------------------------------- slepton neutralino annihilation
        if (nsl.gt.0.and.nn.gt.0) then
          nch=0
          do i=1,nsl
            do j=1,nn
              nch=nch+1
              kp1(nch)=kpsl(i)
              kp2(nch)=kpn(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=1.0d0           ! degrees of freedom for sleptons=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------ sneutrino neutralino annihilation
        if (nsnu.gt.0.and.nn.gt.0) then
          nch=0
          do i=1,nsnu
            do j=1,nn
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpn(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=1.0d0           ! degrees of freedom for sneutrinos=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------ up-squark neutralino annihilation
        if (nsqu.gt.0.and.nn.gt.0) then
          nch=0
          do i=1,nsqu
            do j=1,nn
              nch=nch+1
              kp1(nch)=kpsqu(i)
              kp2(nch)=kpn(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c---------------------------------- down-squark neutralino annihilation
        if (nsqd.gt.0.and.nn.gt.0) then
          nch=0
          do i=1,nsqd
            do j=1,nn
              nch=nch+1
              kp1(nch)=kpsqd(i)
              kp2(nch)=kpn(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=3.0d0           ! degrees of freedom for squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c---------------------------------------- slepton chargino annihilation
        if (nsl.gt.0.and.ncha.gt.0) then
          nch=0
          do i=1,nsl
            do j=1,ncha
              nch=nch+1
              kp1(nch)=kpsl(i)
              kp2(nch)=kpcha(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=2.0d0           ! degrees of freedom for sleptons=2, charginos=4
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c-------------------------------------- sneutrino chargino annihilation
        if (nsnu.gt.0.and.ncha.gt.0) then
          nch=0
          do i=1,nsnu
            do j=1,ncha
              nch=nch+1
              kp1(nch)=kpsnu(i)
              kp2(nch)=kpcha(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=2.0d0           ! degrees of freedom for sneutrinos=2
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c-------------------------------------- up-squark chargino annihilation
        if (nsqu.gt.0.and.ncha.gt.0) then
          nch=0
          do i=1,nsqu
            do j=1,ncha
              nch=nch+1
              kp1(nch)=kpsqu(i)
              kp2(nch)=kpcha(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=6.0d0           ! degrees of freedom for squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif

c------------------------------------ down-squark chargino annihilation
        if (nsqd.gt.0.and.ncha.gt.0) then
          nch=0
          do i=1,nsqd
            do j=1,ncha
              nch=nch+1
              kp1(nch)=kpsqd(i)
              kp2(nch)=kpcha(j)
            enddo
          enddo
c...convert given p for lsp:s to pnew and call dsandwdcosii with pnew
          s = 4.0d0*(mx**2+p**2)
          do ch=1,nch
            mp1 = mass(kp1(ch))
            mp2 = mass(kp2(ch))
            tmp=(s-(mp1-mp2)**2)*(s-(mp1+mp2)**2)/(4.0d0*s)
            if (tmp.gt.0.0d0) then
              pnew = sqrt(tmp)
                                ! f=sum(gi*gj/g1^2) with same mass and wij
              f=6.0d0           ! degrees of freedom for squarks=6
              if (kp1(ch).ne.kp2(ch)) f=2.0d0*f
              w=w+f*pnew/p
     &          *dsasdwdcossfchi(pnew,costheta,kp1(ch),kp2(ch))
            endif
          enddo
        endif
          

c----------------------------------------------------------------------
c---------------------------------------------- Without coannihilations
c----------------------------------------------------------------------

      else       ! no coannihilation
        w = dsandwdcosnn(p,costheta,kn(1),kn(1))
      endif

      dsandwdcos=w
c      write(33,*) costheta,w    ! je test
c      print*,'w= ',p,w

      end


