************************************************************************
*** SUBROUTINE dsasdwdcossfsf                                        ***
*** computes dW_{ij}/dcostheta                                       ***
*** for sfermion(1) + antisfermion(2) plus sfermion(1) + sfermion(2) ***
***                                                                  *** 
*** AUTHOR: Piero Ullio, ullio@sissa.it                              ***
*** Date: 01-08-10                                                   ***
*** modified: Joakim Edsjo, Mia Schelke to include squarks with      ***
***   gauge and Higgs boson final states and gluons                  ***
***   02-05-22                                                       ***
***   bug with switching of initial states fixed 020613 (edsjo)      ***
*** modified: Piero Ullio                                            ***
***   02-03-22                                                       ***
*** modified: Piero Ullio                                            ***
***   02-07-01                                                       ***
************************************************************************

      real*8 function dsasdwdcossfsf(p,costhe,kp1,kp2)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      include 'dsandwcom.h'
      include 'dsidtag.h'
      real*8 p,costhe
      integer kp1,kp2,kp3,kp4,itmp
      integer kp1i,kp2i
      integer f,i,kfer,kfer1,kfer2,ick1,ick2
      real*8 w,result,par,cfactini,cfactfin
      character*4 chcol
      logical wok

************************************************************************
      if (p.lt.0.0d0) then
         dsasdwdcossfsf=0.0d0
      return
      endif

***** tollerance on the position of a threshold and on a negative
***** fermion channel
      thstep=1.d-15
      fertoll=1.d-10

c...Since we want to be able to switch kp1 and kp2 without having
c...them changed on return, let's use some temporary variables
      kp1i=kp1
      kp2i=kp2

************************************************************************
*****
***** set the askin variables
      p12=p
      costheta=costhe

***** family indices:
      iifam(1)=ivfam(kp1i)
      iifam(2)=ivfam(kp2i)

***** NOTE: boson final states are computed just for kp1i up-type and 
***** kp2i down-type, not viceversa, so in the latter case switch them
      if(iifam(1).eq.(iifam(2)+1)) then
        itmp=kp1i
        kp1i=kp2i
        kp2i=itmp
        itmp=iifam(1)
        iifam(1)=iifam(2)
        iifam(2)=itmp
      endif 

***** masses in initial state:
      mass1=mass(kp1i)
      mass2=mass(kp2i)

***** kinematic variables:
      call dsaskinset1
      
***** internal degrees of freedom of the initial particles:
***** note: here we do not include particle anti-particle degrees of
*****       freedom. Only spin and colour are included here.
*****       The particle anti-particle degrees of freedom are
*****       taken into account when w is summed below.
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2)) then
        gg1=3.d0
        if(iifam(2).ge.ivfam(ksu1).and.iifam(2).le.ivfam(ksb2)) then
***** 2 squarks
          gg2=3.d0
          chcol='quar'
          call dsascolset(chcol)
          cfactini=1.d0  ! set inside the routine dsasfercol
          cfactfin=1.d0
        else  
***** 1 squark and 1 slepton
          gg2=1.d0
          chcol='lept'
          cfactini=3.d0
          cfactfin=1.d0
        endif  
      else  
        gg1=1.d0 
        if(iifam(2).ge.ivfam(ksu1).and.iifam(2).le.ivfam(ksb2)) then
***** 1 squark and 1 slepton
          gg2=3.d0
          chcol='lept'
          cfactini=3.d0
          cfactfin=1.d0
        else  
***** 2 sleptons
          gg2=1.d0
          chcol='lept'
          cfactini=1.d0
          cfactfin=3.d0
        endif  
      endif  
***** 
*****    3 cases:
***** 1) sfermions with same family index:
      if(iifam(1).eq.iifam(2)) then
c        write(*,*) 'Now in case 1...'
        do i=1,54
          prtial(i)=0.0d0
        enddo
*****
******************************************************* sf sf* -> f fbar
        do f=1,12
          kp3=f
          kp4=f
          if(chcol.eq.'lept') then
            call dsasfer(kp1i,kp2i,kp3,kp4,result)
            if(ncolor(f).gt.2.d0) then
              result=result*cfactfin 
            endif
          elseif(chcol.eq.'quar') then  
            call dsasfercol(kp1i,kp2i,kp3,kp4,result)
          else
            write(*,*) 'chcol not set correctly, = ',chcol
            stop
          endif  
          prtial(f)=result
        enddo  
*********************************************************** sf sf -> f f
        call dsasfercode('same',iifam(1),kfer)
        kp3=kfer
        kp4=kfer
        if(chcol.eq.'lept') then
          call dsasfere(kp1i,kp2i,kp3,kp4,result)
        elseif(chcol.eq.'quar') then  
          call dsasferecol(kp1i,kp2i,kp3,kp4,result)
        else
          write(*,*) 'chcol not set correctly, = ',chcol
          stop
        endif  
        prtial(13)=result
******************************************************* sf sf* ->  w+ w-
        kp3=kw
        kp4=kw
        call dsasgbgb(kp1i,kp2i,kp3,kp4,par)
        prtial(14)=par
********************************************************** sf sf* -> z z
        kp3=kz
        kp4=kz
        call dsasgbgb(kp1i,kp2i,kp3,kp4,par)
        prtial(15)=par
****************************************************** sf sf* -> z gamma
        call dsasfercode('same',iifam(1),kfer)
        if(dabs(echarg(kfer)).gt.1.d-16) then 
          kp3=kz
          kp4=kgamma
c          call dsasgbgbone(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb1exp(kp1i,kp2i,kp3,kp4,par)
          prtial(16)=par
        else  
          prtial(16)=0.d0
        endif  
************************************************** sf sf* -> gamma gamma
c same as above
c        call dsasfercode('same',iifam(1),kfer)
        if(dabs(echarg(kfer)).gt.1.d-16) then
          kp3=kgamma
          kp4=kgamma
c          call dsasgg(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb2exp(kp1i,kp2i,kp3,kp4,par)
          prtial(17)=par
        else  
          prtial(17)=0.d0
        endif  
********************************************************** sf sf* -> z h
        kp3=kz
        kp4=kh2
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(18)=par
********************************************************** sf sf* -> z H
        kp3=kz
        kp4=kh1
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(19)=par
********************************************************** sf sf* -> z a
        kp3=kz
        kp4=kh3
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(20)=par
****************************************************** sf sf* -> gamma h
c same as above
c        call dsasfercode('same',iifam(1),kfer)
        if(dabs(echarg(kfer)).gt.1.d-16) then 
          kp3=kgamma
          kp4=kh2
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(21)=par
        else  
          prtial(21)=0.d0
        endif  
****************************************************** sf sf* -> gamma H
c same as above
c        call dsasfercode('same',iifam(1),kfer)
        if(dabs(echarg(kfer)).gt.1.d-16) then
          kp3=kgamma
          kp4=kh1
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(22)=par
        else  
          prtial(22)=0.d0
        endif  
****************************************************** sf sf* -> gamma a
c same as above
c        call dsasfercode('same',iifam(1),kfer)
        if(dabs(echarg(kfer)).gt.1.d-16) then 
          kp3=kgamma
          kp4=kh3
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(23)=par
        else  
          prtial(23)=0.d0
        endif  
********************************************************** sf sf* -> h h
        kp3=kh2
        kp4=kh2
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(24)=par
********************************************************** sf sf* -> h a
        kp3=kh2
        kp4=kh3
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(25)=par
********************************************************** sf sf* -> H a
        kp3=kh1
        kp4=kh3
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(26)=par
********************************************************** sf sf* -> a a
        kp3=kh3
        kp4=kh3
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(27)=par
********************************************************** sf sf* -> H h
        kp3=kh1
        kp4=kh2
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(28)=par
********************************************************** sf sf* -> H H
        kp3=kh1
        kp4=kh1
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(29)=par
******************************************************** sf sf* -> h- w+
        kp3=kw
        kp4=khc
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(30)=2.d0*par    !factor of 2 to include sf sf* -> h+ w-
******************************************************** sf sf* -> h+ h-
        kp3=khc
        kp4=khc
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(31)=par
************************************************** sf sf* -> gluon gluon
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=kgluon
c          call dsasgg(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb2exp(kp1i,kp2i,kp3,kp4,par)
          prtial(32)=par
        else  
          prtial(32)=0.d0
        endif  
************************************************** sf sf* -> Z gluon
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kz
          kp4=kgluon
c          call dsasgbgbone(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb1exp(kp1i,kp2i,kp3,kp4,par)
          prtial(33)=par
        else  
          prtial(33)=0.d0
        endif  
************************************************** sf sf* -> gamma gluon
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgamma
          kp4=kgluon
c          call dsasgg(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb2exp(kp1i,kp2i,kp3,kp4,par)
          prtial(34)=par
        else  
          prtial(34)=0.d0
        endif  
************************************************** sf sf* -> gluon H1
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=kh1
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(35)=par
        else  
          prtial(35)=0.d0
        endif  
************************************************** sf sf* -> gluon H2
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=kh2
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(36)=par
        else  
          prtial(36)=0.d0
        endif  
************************************************** sf sf* -> gluon H3
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=kh3
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(37)=par
        else  
          prtial(37)=0.d0
        endif  
**************************************************** sum partial results
        w=0.d0
        w=w+prtial(1)            ! sf sf* -> nue nuebar
        w=w+prtial(2)            ! sf sf* -> e+ e-
        w=w+prtial(3)            ! sf sf* -> numu numubar
        w=w+prtial(4)            ! sf sf* -> mu+ mu-
        w=w+prtial(5)            ! sf sf* -> nutau nutaubar
        w=w+prtial(6)            ! sf sf* -> tau+ tau-
        w=w+prtial(7)            ! sf sf* -> u ubar
        w=w+prtial(8)            ! sf sf* -> d dbar
        w=w+prtial(9)            ! sf sf* -> c cbar
        w=w+prtial(10)           ! sf sf* -> s sbar
        w=w+prtial(11)           ! sf sf* -> t tbar
        w=w+prtial(12)           ! sf sf* -> b bbar
        w=w+prtial(13)           ! sf sf -> f f
        w=w+prtial(14)           ! sf sf* ->  w+ w-
        w=w+prtial(15)           ! sf sf* -> z z
        w=w+prtial(16)           ! sf sf* -> z gamma
        w=w+prtial(17)           ! sf sf* -> gamma gamma
        w=w+prtial(18)           ! sf sf* -> z h
        w=w+prtial(19)           ! sf sf* -> z H
        w=w+prtial(20)           ! sf sf* -> z a
        w=w+prtial(21)           ! sf sf* -> gamma h
        w=w+prtial(22)           ! sf sf* -> gamma H
        w=w+prtial(23)           ! sf sf* -> gamma a
        w=w+prtial(24)           ! sf sf* -> h h
        w=w+prtial(25)           ! sf sf* -> h a
        w=w+prtial(26)           ! sf sf* -> H a
        w=w+prtial(27)           ! sf sf* -> a a
        w=w+prtial(28)           ! sf sf* -> H h
        w=w+prtial(29)           ! sf sf* -> H H
        w=w+prtial(30)           ! sf sf* -> h- w+ and h+ w-
        w=w+prtial(31)           ! sf sf* -> h+ h-
        w=w+prtial(32)           ! sf sf* -> gluon gluon
        w=w+prtial(33)           ! sf sf* -> Z gluon
        w=w+prtial(34)           ! sf sf* -> gamma gluon
        w=w+prtial(35)           ! sf sf* -> gluon H
        w=w+prtial(36)           ! sf sf* -> gluon h
        w=w+prtial(37)           ! sf sf* -> gluon A

c
c check for large negative terms in the fermion final states
c
        wok=.true.
        do i=1,13
          if (prtial(i).lt.0.0d0) then
            if (dabs(prtial(i)/w).gt.fertoll) then
              wok=.false.
            else
              w=w-prtial(i)
              prtial(i)=0.0d0
            endif  
          endif  
        enddo
c
c write error message:
c
        if (.not.wok) then
          write(*,*) 'Model: ',idtag
          write(*,*) 'large negative term'
          write(*,*) 'dsasdwdcossfsf called with kp1i=',kp1i,
     &    ' kp2i=',kp2i
          write(*,*) '  p=',p,' costh=',costhe,' w=',w
          do i=1,37
            write(*,*) 'i=',i,' prtial=',prtial(i)
          enddo
        endif
c
        dsasdwdcossfsf = w*0.5d0 ! 0.5d0 <- we should return weff for sf sf ann
                                 ! with sf combined part. and anti-particle state
************************************************************************
*****
***** 2) sfermions with different family indices, 
*****    but in the same doublet:
      elseif((iifam(1).eq.(iifam(2)-1))
     &       .or.(iifam(1).eq.(iifam(2)+1))) then  
c        write(*,*) 'Now in case 2...'
        do i=1,54
          prtial(i)=0.0d0
        enddo
*************************************************** sfu sfd* -> fu fdbar
        do f=1,6
          kp3=2*f-1
          kp4=2*f
          if(chcol.eq.'lept') then
            call dsasfer(kp1i,kp2i,kp3,kp4,result)
            if(ncolor(kp3).gt.2.d0) then
              result=result*cfactfin
            endif  
          elseif(chcol.eq.'quar') then  
            call dsasfercol(kp1i,kp2i,kp3,kp4,result)
          else
            write(*,*) 'chcol not set correctly, = ',chcol
            stop
          endif  
          prtial(f)=result
        enddo  
******************************************************* sfu sfd -> fu fd
        call dsasfercode('same',iifam(1),kfer)
        kp3=kfer
        kp4=kfer+1
        if(chcol.eq.'lept') then
          call dsasfere(kp1i,kp2i,kp3,kp4,result)
        elseif(chcol.eq.'quar') then  
          call dsasferecol(kp1i,kp2i,kp3,kp4,result)
        else
          write(*,*) 'chcol not set correctly, = ',chcol
          stop
        endif  
        prtial(7)=result
******************************************************* sfu sfd* -> w+ z
        kp3=kw
        kp4=kz
        call dsasgbgb(kp1i,kp2i,kp3,kp4,par)
        prtial(8)=par
*************************************************** sfu sfd* -> w+ gamma
        kp3=kw
        kp4=kgamma
c        call dsasgbgbone(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
        call dsasgbgb1exp(kp1i,kp2i,kp3,kp4,par)
        prtial(9)=par
******************************************************* sfu sfd* -> w+ h
        kp3=kw
        kp4=kh2
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(10)=par
******************************************************* sfu sfd* -> w+ H
        kp3=kw
        kp4=kh1
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(11)=par
******************************************************* sfu sfd* -> w+ a
        kp3=kw
        kp4=kh3
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(12)=par
*************************************************** sfu sfd* -> h+ gamma
        kp3=kgamma
        kp4=khc
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(13)=par
******************************************************* sfu sfd* -> h+ z
        kp3=kz
        kp4=khc
        call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(14)=par
******************************************************* sfu sfd* -> h+ h
        kp3=khc
        kp4=kh2
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(15)=par
******************************************************* sfu sfd* -> h+ H
        kp3=khc
        kp4=kh1
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(16)=par
******************************************************* sfu sfd* -> h+ a
        kp3=khc
        kp4=kh3
        call dsashbhb(kp1i,kp2i,kp3,kp4,par)
        prtial(17)=par
******************************************************* sfu sfd* -> W+ g
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kw
          kp4=kgluon
c          call dsasgbgbone(kp1i,kp2i,kp3,kp4,par)  ! Old slower routine
          call dsasgbgb1exp(kp1i,kp2i,kp3,kp4,par)
          prtial(18)=par
        else  
          prtial(18)=0.d0
        endif  
******************************************************* sfu sfd* -> g H+
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=khc
          call dsasgbhb(kp1i,kp2i,kp3,kp4,par)
          prtial(19)=par
        else  
          prtial(19)=0.d0
        endif  
**************************************************** sum partial results
        w=0.d0
        w=w+prtial(1)            ! sfu sfd* -> nue e+
        w=w+prtial(2)            ! sfu sfd* -> numu mu+
        w=w+prtial(3)            ! sfu sfd* -> nutau tau+
        w=w+prtial(4)            ! sfu sfd* -> u dbar
        w=w+prtial(5)            ! sfu sfd* -> c sbar
        w=w+prtial(6)            ! sfu sfd* -> t bbar
        w=w+prtial(7)            ! sfu sfd -> fu fd
        w=w+prtial(8)            ! sfu sfd* -> w+ z
        w=w+prtial(9)            ! sfu sfd* -> w+ gamma
        w=w+prtial(10)           ! sfu sfd* -> w+ h
        w=w+prtial(11)           ! sfu sfd* -> w+ H
        w=w+prtial(12)           ! sfu sfd* -> w+ a
        w=w+prtial(13)           ! sfu sfd* -> h+ gamma
        w=w+prtial(14)           ! sfu sfd* -> h+ z
        w=w+prtial(15)           ! sfu sfd* -> h+ h
        w=w+prtial(16)           ! sfu sfd* -> h+ H
        w=w+prtial(17)           ! sfu sfd* -> h+ a
        w=w+prtial(18)           ! sfu sfd* -> W+ g
        w=w+prtial(19)           ! sfu sfd* -> g H+
c
c check for large negative terms in the fermion final states
c
        wok=.true.
        do i=1,7
          if (prtial(i).lt.0.0d0) then
            if (dabs(prtial(i)/w).gt.fertoll) then
              wok=.false.
            else
              w=w-prtial(i)
              prtial(i)=0.0d0
            endif  
          endif  
        enddo
c
c write error message:
c
        if (.not.wok) then
          write(*,*) 'Model: ',idtag
          write(*,*) 'large negative term'
          write(*,*) 'dsasdwdcossfsf called with kp1i=',kp1i,
     &    ' kp2i=',kp2i
          write(*,*) '  p=',p,' costh=',costhe,' w=',w
          do i=1,19
            write(*,*) 'i=',i,' prtial=',prtial(i)
          enddo
        endif
c
        dsasdwdcossfsf = w*0.5d0 ! 0.5d0 <- we should return weff for sf sf ann
                                 ! with sf combined part. and anti-particle state
************************************************************************
*****
***** 3) sfermions with different family indices, 
*****    and in the different doublets:
***** NOTE: states which occur through squark mixing between the families
***** are not yet included, otherwise, we could get W+g and H+g 
***** final states as well
      else   
c        write(*,*) 'Now in case 3...'
        do i=1,54
          prtial(i)=0.0d0
        enddo
*****
*************************************************** sf1 sf2* -> f1 f2bar
        call dsasfercode('same',iifam(1),kfer1)
        call dsasfercode('same',iifam(2),kfer2)
        kp3=kfer1
        kp4=kfer2
        if(chcol.eq.'lept') then
          call dsasfer(kp1i,kp2i,kp3,kp4,result)
        elseif(chcol.eq.'quar') then  
          call dsasfercol(kp1i,kp2i,kp3,kp4,result)
        else
          write(*,*) 'chcol not set correctly, = ',chcol
          stop
        endif
        prtial(1)=result*cfactini  
******************************************************* sf1 sf2 -> f1 f2
        if(chcol.eq.'lept') then
          call dsasfere(kp1i,kp2i,kp3,kp4,result)
        elseif(chcol.eq.'quar') then  
          call dsasferecol(kp1i,kp2i,kp3,kp4,result)
        else
          write(*,*) 'chcol not set correctly, = ',chcol
          stop
        endif
        prtial(2)=result*cfactini
*************************************************** sf1 sf2* -> f3 f4bar
*****  f1 and f3 in the same doublet,  f2 and f4 in the same doublet
*****  in case f1 and f2 are both up or both down
        ick1=iifam(1)/10
        ick1=iifam(1)-ick1*10
        ick2=iifam(2)/10
        ick2=iifam(2)-ick2*10
        if(ick1.eq.ick2) then
          call dsasfercode('diff',iifam(1),kfer1)
          call dsasfercode('diff',iifam(2),kfer2)
          kp3=kfer1
          kp4=kfer2
          if(chcol.eq.'lept') then
            call dsasfer(kp1i,kp2i,kp3,kp4,result)
          elseif(chcol.eq.'quar') then  
            call dsasfercol(kp1i,kp2i,kp3,kp4,result)
          else
            write(*,*) 'chcol not set correctly, = ',chcol
            stop
          endif
          prtial(3)=result*cfactini
        else
          prtial(3)=0.d0
        endif  
******************************************************* sf1 sf2 -> f3 f4
*****  f1 and f3 in the same doublet,  f2 and f4 in the same doublet
*****  in case f1 are f2 are one up and the other down
        if(ick1.ne.ick2) then
          call dsasfercode('diff',iifam(1),kfer1)
          call dsasfercode('diff',iifam(2),kfer2)
          kp3=kfer1
          kp4=kfer2
          if(chcol.eq.'lept') then
            call dsasfere(kp1i,kp2i,kp3,kp4,result)
          elseif(chcol.eq.'quar') then  
            call dsasferecol(kp1i,kp2i,kp3,kp4,result)
          else
            write(*,*) 'chcol not set correctly, = ',chcol
            stop
          endif
          prtial(4)=result*cfactini
        else
          prtial(4)=0.d0
        endif  
**************************************************** sum partial results
        w=0.d0
        w=w+prtial(1)            ! sf1 sf2* -> f1 f2bar
        w=w+prtial(2)            ! sf1 sf2 -> f1 f2
        w=w+prtial(3)            ! sf1 sf2* -> f3 f4bar
        w=w+prtial(4)            ! sf1 sf2 -> f3 f4
c
c check for large negative terms in the fermion final states
c
        wok=.true.
        do i=1,3
          if (prtial(i).lt.0.0d0) then
            if (dabs(prtial(i)/w).gt.fertoll) then
              wok=.false.
            else
              w=w-prtial(i)
              prtial(i)=0.0d0
            endif  
          endif  
        enddo
c
c write error message:
c
        if (.not.wok) then
          write(*,*) 'Model: ',idtag
          write(*,*) 'large negative term'
          write(*,*) 'dsasdwdcossfsf called with kp1i=',kp1i,
     &    ' kp2i=',kp2i
          write(*,*) '  p=',p,' costh=',costhe,' w=',w
          do i=1,2
            write(*,*) 'i=',i,' prtial=',prtial(i)
          enddo
        endif
c
        dsasdwdcossfsf = w*0.5d0 ! 0.5d0 <- we should return weff for sf sf ann
                                 ! with sf combined part. and anti-particle state
************************************************************************
***** if statement corresponding to the 3 cases closed
      endif

      if (w.lt.0.0d0) then
        write(*,*) 'Model: ',idtag
        write(*,*) 'dsasdwdcossfsf called with kp1i=',kp1i,
     &    ' kp2i=',kp2i
        write(*,*) '  p=',p,' costh=',costhe,' w=',w
        write(*,*) '  negative w, program stopped!'
        stop
      endif  

      return
      end








