**************************************************************
*** SUBROUTINE dsasfere                                    ***
*** computes dW_{ij}/dcostheta                             ***
*** sfermion(i) + sfermion(j)                              ***
*** -> fermion(k1) + fermion(k2)                           ***
*** version to be used if i or j has non-zero lepton #     ***
***                                                        *** 
*** input askin variables: p12,costheta                    ***
*** kpk1 and kpk2 are the fermion code                     ***
*** AUTHOR: Piero Ullio, ullio@sissa.it                    ***
*** Date: 01-08-10                                         ***
**************************************************************

      SUBROUTINE dsasfere(kpi,kpj,kpk1,kpk2,result)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      integer kpi,kpj,kpk1,kpk2
      integer iii
      real*8 result
      real*8 ampl2
c....check if the final state is kinematically allowed
      if((Svar-(mass(kpk1)+mass(kpk2))**2)/Svar.lt.thstep) then
        result=0.d0
        return
      endif      
***** define family indices
      iifam(3)=ivfam(kpk1)
      iifam(4)=ivfam(kpk2)
***** equal / non equal particle final state: 
      if(kpk1.eq.kpk2) then
        s34=2.d0
      else  
        s34=1.d0
      endif  
***** fermion type in final state:
      q3=2
      q4=1
***** masses in final state:
      mass3=mass(kpk1)
      mass4=mass(kpk2)
***** define the kinematic variables  
      call dsaskinset2
      call dsaskinset3
***** check if the final state is allowed
      if((Svar-(mass3+mass4)**2)/Svar.lt.1.d-16) then
        result=0.d0
        return
      endif 
***** initialize ASx and ASy to zero
      do iii=1,4
      ASxpl(iii)=(0.d0,0.d0)
      ASxpr(iii)=(0.d0,0.d0)
      enddo
      ASyl=(0.d0,0.d0)
      ASyr=(0.d0,0.d0)
***** then fill them in:
*****
*************** case 1 ****************************************
*****
***** i. s.: particle - particle ******************************
      if(iifam(1).eq.iifam(2)) then
***** f. s.: particle - particle? *****************************
        if(iifam(1).ne.iifam(3).or.iifam(1).ne.iifam(4)) then 
          write(*,*) 'dsasfere called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif  
***** neutralinos in t-channel **+*****************************
        do iii=1,4
          call dsasscscTCff(kpi,kpj,kpk1,kpk2,kn(iii))
        enddo
***** neutralinos in u-channel **+*****************************
        do iii=1,4
          call dsasscscUCff(kpi,kpj,kpk1,kpk2,kn(iii))
        enddo
*****
*************** case 2 ****************************************
*****
***** i. s.: particle - particle in same doublet **************
      elseif(iifam(1).eq.(iifam(2)-1)
     &       .or.iifam(1).eq.(iifam(2)+1)) then
***** f. s.: particle - particle in same doublet? *************
        if(iifam(1).eq.iifam(3).and.iifam(2).eq.iifam(4)) then 
***** neutralinos in t-channel **+*****************************
          do iii=1,4
            call dsasscscTCff(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo  
***** charginos in u-channel **********************************
          do iii=1,2
            call dsasscscUCff(kpi,kpj,kpk1,kpk2,kcha(iii))
          enddo
        elseif(iifam(1).eq.iifam(4).and.iifam(2).eq.iifam(3)) then 
***** neutralinos in u-channel **+*****************************
          do iii=1,4
            call dsasscscUCff(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo  
***** charginos in t-channel **********************************
          do iii=1,2
            call dsasscscTCff(kpi,kpj,kpk1,kpk2,kcha(iii))
          enddo
        else  
          write(*,*) 'dsasfere called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif  
*****
*************** case 3 ****************************************
*****
***** i. s.: particle - particle in different doublet *********
      else  
        if(iifam(1).eq.iifam(3).and.iifam(2).eq.iifam(4)) then  
***** neutralinos in t-channel **+*****************************
          do iii=1,4
            call dsasscscTCff(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo
        elseif(((iifam(1).eq.(iifam(3)-1))
     &    .or.(iifam(1).eq.(iifam(3)+1))).and.
     &    ((iifam(2).eq.(iifam(4)-1)).or.(iifam(2).eq.(iifam(4)+1)))) 
     &    then  
***** charginos in t-channel **********************************
          do iii=1,2
            call dsasscscTCff(kpi,kpj,kpk1,kpk2,kcha(iii))
          enddo
        else
***** f. s.: not the same families as i. s. *******************
          write(*,*) 'dsassferfer called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif 
      endif  
*****
*****
***** compute the amplitude squared 
      call dsasff(ampl2)
***** computes dW_{ij}/dcostheta
      result=ampl2*k34/(8.d0*pi*gg1*gg2*s34*dsqrt(Svar))
      return
      end

