      subroutine dsddffsi(q,a,z,ff)
c_______________________________________________________________________
c  Spin-independent form factor for direct detection.
c  input:
c    q : real*8  : momentum transfer in GeV ( q=sqrt(M*E/2/mu^2) )
c    a : integer : atomic number
c    z : integer : charge number
c  output:
c    ff : |F(q)|^2, the square of the form factor
c  author: paolo gondolo (paolo@physics.utah.edu) 2004
c=======================================================================
      implicit none
      include 'dsddcom.h'
      real*8 etmp
      common /dsddffexp/ etmp

      real*8 q
      integer a,z
      real*8 ff
      real*8 x,x2,fermiGeV,s,y,f,r,r1,mni,mp,mn,r1tmp,e0
      parameter (fermiGeV = 1.d0/0.1973269602d0)
      parameter (mp=0.93827231d0)
      parameter (mn=0.931494013d0)
      
c Helm form factor aka Woods-Saxon form factor

      if (ddffsi.eq.'W-S'.or.ddffsi.eq.'Helm') then
         if (a.eq.1) then
            ff = 1.d0
         else
            mni = z*mp+(a-z)*mn
            r = (0.91d0*exp(log(mni)/3.d0)+0.3d0)
            s = 1.d0
            r1tmp = r*r-5.d0*s*s
            if (r1tmp.gt.0.d0) then
               r1 = sqrt(r1tmp)
            else
               r1 = r
            endif 
            x = dabs(q*r1*fermiGeV)
            y = q*s*fermiGeV
            if (x.gt.5.d-8) then
               f = (3.d0*(sin(x)-x*cos(x))/x**3)**2
            else
               x2 = x*x
               f = 1.d0+x2*(-1.d0+x2*(1.d0/280.d0+x2*(-1.d0/15120.d0+
     &              x2/1330560.d0)))
            endif
            ff = f*exp(-y**2)
         endif

c exponential form factor

      else if (ddffsi.eq.'exp') then
         mni = z*mp+(a-z)*mn
         r = (0.91d0*exp(log(mni)/3.d0)+0.3d0)
         e0 = 3.d0/(2.d0*mni*r)*0.038919d0
         ff = exp(-etmp/e0)

c unrecognized form factor label

      else
         write (*,*) 
     &        'Unrecognized spin-independent form factor',ddffsi
         write (*,*) 'DarkSUSY stops'
         stop
      endif
      
      return
      end
