      real*8 function dsepspecm(eps,how)
****************************************************************
***                                                          ***
*** function which computes the differential flux of         ***
*** positrons for the energy eps as a result of              ***
*** neutralino annihilation in the halo.                     ***
*** input: eps - positron energy in gev                      ***
***        how = 2 - diffusion model is tabulated on first   ***
***                  call, and then interpolated             ***
***              3 - as 2, but also write the table to disk  ***
***                  at the first call                       ***
***              4 - read table from disk on first call, and ***
***                  use the subsequent calls. If the file   ***
***                  does not exist, it will be created      ***
***                  (as in 3). (default)                    ***
***                                                          ***
*** rescaling is not included                                ***
***                                                          ***
*** Author: Piero Ullio                                      ***
*** Date: 2004-02-03                                         ***
*** Modified: Joakim Edsjo, modifications to file loading    ***
****************************************************************
      implicit none
      include 'dsepcom.h'
      include 'dsprep.h'
      include 'dshmcom.h'
      integer i,how
      real*8 lowloc,uploc,checkres,par,dssigmav
      real*8 epsabs
      integer ndigits
      common/epspecsetcom/epsabs,ndigits

      integer dhow
      common/ephow/dhow
c
      real*8 eps,v,dsepvofeps,factor,vep,pi
      real*8 deltavprime,dsepintgreentab,kpc
      parameter(kpc=3.08567802d0) 
c
      real*8 up,low
      real*8 abserr,alist,blist,elist,epsrel,rlist,result
      integer ier,iord,last,limit,neval
      dimension alist(5000),blist(5000),elist(5000),iord(5000),
     & rlist(5000)
      external dsepspecmint
c
      real*8 vint
      common/epclintcom/vint
c

c...Check if we should be here
      if (dsepdiffmethod.ne.1) then
        write(*,*) 'ERROR in dsepspecm.f:'
        write(*,*) 'You have called this routine with dsepdiffmethod=',
     &    dsepdiffmethod
        write(*,*) 'which is not supported here. Call dsepdiff instead.'
        stop
      endif

c...Transfer how flag to common block
      dhow=how

c...Set up integration accuracy
      if (epcl_load) then   ! first call with new diff. model
        ndigits=5
        epsabs=1.d-10
      endif  

      epsrel=10.d0**(-ndigits)
      limit=5000

c...Perform energy integration (done in v)
      v=dsepvofeps(eps)
      vint=v
c continuum contribution
      low=dsepvofeps(mx)
      up=v
      par=0.d0
      do i=10,1,-1
      lowloc=low+(up-low)/10.d0*(i-1)
      uploc=low+(up-low)/10.d0*i
 20   call dqagse(dsepspecmint,lowloc,uploc,epsabs,epsrel,limit,
     &   result,abserr,neval,ier,alist,blist,rlist,elist,iord,last)
      checkres=par+result
      if(dabs(checkres)/10.d0**ndigits.lt.epsabs) then
        epsabs=dabs(checkres)/10.d0**(ndigits+2) !too low accuracy
        goto 20
      elseif(dabs(checkres)/10.d0**(ndigits+2).gt.epsabs) then
        epsabs=dabs(checkres)/10.d0**(ndigits+2)  !too high accuracy
      endif
      par=par+result
      enddo
      dsepspecm=par*dssigmav(0)
c sum the line contribution
      deltavprime=up-low
      DeltaVprime=deltavprime*4.0d0*k27*tau16*10.d0/kpc**2 
      dsepspecm=dsepspecm+sigv(15)*dsepintgreentab(DeltaVprime)
dsepspecmint(low)
      factor=(rho0/mx)**2*0.5d0
      dsepspecm=dsepspecm*factor*tau16*1.0d16/eps**2  !kpc**-3 cm**-3 GeV**-1
c...convert density to flux
      vep=2.99792458d10 !cm s**-1
c dsepspec_cl in kpc**-3 cm**-2 s**-1 GeV**-1 sr**-1
      pi=4.d0*datan(1.d0)
      dsepspecm=dsepspecm*vep/(4.0d0*pi)
      return
      end



      real*8 function dsepspecmint(vprime)
      implicit none
      include 'dshacom.h'
      include 'dsepcom.h'
      real*8 vprime
      real*8 dshaloyield,epsprime,deltavprime,dsepintgreentab,
     &  dsepepsofv,dsepwofu,uprime,dsepuofeps
      integer istat
      real*8 vint
      common/epclintcom/vint
      real*8 kpc
      parameter(kpc=3.08567802d0) 
      deltavprime=vint-vprime
      DeltaVprime=deltavprime*4.0d0*k27*tau16*10.d0/kpc**2 
      dsepspecmint=dsepintgreentab(DeltaVprime)
      epsprime=dsepepsofv(vprime)
      uprime=dsepuofeps(epsprime)
c dsepspecpuint in GeV**-1
      hasmooth=2  ! 2 to smooth well
      dsepspecmint=dsepspecmint*dsepwofu(uprime)
     &   *dshaloyield(epsprime,151,istat)
      hasmooth=0
      return
      end




      real*8 function dsepintgreentab(DeltaV)
****************************************************************
***                                                          ***
*** tabulated version of dsepintgreen, see header of the     ***
*** latter for details. on first call the table is either    ***
*** computed or read from file provided by the user.         ***
*** This is handled by the flag dhow in common block /ephow/ ***
***       dhow = 2 - table is tabulated on first             ***
***                  call, and then interpolated             ***
***              3 - as 2, but also write the table to disk  ***
***                  at the first call                       ***
***              4 - read table from disk on first call, and ***
***                  use the subsequent calls. If the file   ***
***                  does not exist, it will be created      ***
***                  (as in 3). (use as default)             ***
***                                                          ***
*** To load another table reset variable epcl_load           ***
*** in dsepcom.h to .true. (done automatically with call     ***
*** to dsepset.f)                                            ***
***                                                          ***
*** Author: Piero Ullio                                      ***
*** Date: 2004-02-03                                         ***
*** Modified: Joakim Edjso, modified table loading           ***
****************************************************************
      implicit none
      include 'dsepcom.h'
      include 'dssusy.h'  ! for dsroot 
      include 'dshmcom.h' ! for haloid
      integer ii
      real*8 deltav,result,dsepintgreen
      real*8 xdv(1000),ydv(1000),ydv2(1000),realndv
      real*8 xmax,xmin,xint,ymax
      character*200 epfile,scr
      integer i,k,kk,npoints
      common/bigItab2com/xdv,ydv,ydv2,realndv

      integer dhow
      common/ephow/dhow


 5    if (epcl_load) then    ! make a new table or load a new table from disk

c...Generate file name
        epfile=dsroot
        call dscharadd(epfile,'dat/epgretab-')
        call dscharadd(epfile,epc)
        call dscharadd(epfile,'-')
        call dscharadd(epfile,haloid)
        call dscharadd(epfile,'.dat')

        if (dhow.eq.2.or.dhow.eq.3) then  ! generate a new table
          if (Debug) write(*,*) 'starting dsepintgreen tabulation'  
          ymax=0.d0
          xmin=0.1d0
          xmax=10000.d0
 110      continue
          ii=0
          do i=0,100
            deltav=dexp(dlog(xmin)+(dlog(xmax)-dlog(xmin))/100.d0*i)
            result=dsepintgreen(DeltaV)
            if(i.eq.0.and.dabs(1.d0-result).gt.1.d-2) then
              xmin=xmin/10.d0
              goto 110
            endif
            ii=ii+1
            xdv(ii)=dlog(deltav)
            ydv(ii)=result
            if(ydv(ii).gt.ymax) ymax=ydv(ii)
           if (Debug) write(*,*) 'ii,deltav, result = ',ii,deltav,result
            if(ydv(ii).lt.1.d-8) goto 111
          enddo
 111      npoints=ii
          if (Debug) write(*,*) 'Number of points: ',npoints
 130      continue
          do k=1,npoints-1
            if(dabs(ydv(k)-ydv(k+1)).gt.0.1d0*max(ydv(k),ydv(k+1)).and.
     &        max(ydv(k),ydv(k+1)).gt.1.d-4*ymax) then
              xint=(xdv(k)+xdv(k+1))/2.d0
              deltav=dexp(xint)
              result=dsepintgreen(deltav)
              do kk=npoints,k+1,-1
                xdv(kk+1)=xdv(kk)
                ydv(kk+1)=ydv(kk)
              enddo
              xdv(k+1)=xint
              ydv(k+1)=result
          if (Debug) write(*,*) 'adding deltav, result = ',deltav,result
              npoints=npoints+1  
              if(npoints.le.1000) then
                goto 130 
              else
                write(*,*) 'in dsepintgreentab exceeded the maximum dim'
                write(*,*) 'allowed for vectors in the compspcom block'
                write(*,*) 'which is set equal to 1000'
                stop
              endif
            endif  
          enddo
          if (Debug) write(*,*) 'dsepintgreen tabulation is over'  
          if (dhow.eq.3) then ! write table to disk
           if (Debug) write(*,*) 'Writing ep table to file ',epfile
            open(unit=13,file=epfile,status='unknown',form='formatted')
            write(13,1001) dsversion
            write(13,1002) 
 1001       format('# Made with DarkSUSY version ',A)
 1002       format('#','..log(deltav).',1x,'...table(i)...')
            do i=1,npoints
              write(13,1000)  xdv(i),ydv(i) 
            enddo  
            close(13)
            if (Debug) write(*,*) 'Done.'
          endif

        elseif (dhow.eq.4) then  ! read table from file
          if (Debug) write(*,*) 'Reading ep table from file ',epfile
          ii=0
          open(unit=13,file=epfile,status='old',form='formatted',
     &       err=200)
          read(13,'(a)') scr  ! read header line
          read(13,'(a)') scr  ! read header line
 10       ii=ii+1
          if(ii.gt.1000) then
            write(*,*) 'in dsepintgreentab epfile has more than 1000'
            write(*,*) 'entries, increase size of var in bigItab2com'
            write(*,*) 'program stopped'
            stop
          endif  
          read(13,1000,end=11)  xdv(ii),ydv(ii) 
          goto 10
 1000     format(60(1x,e14.8))
 11       close(13)
          npoints=ii-1  ! for later spline setup
          if (Debug) write(*,*) 'Done.'
        else
          write(*,*) 'ERROR in dsepspecm: how = ',dhow,
     &     'requested, but this is out of range. Stopping.'
          stop
        endif

c...Now set up splines
        realndv=dble(npoints)
        call spline(xdv,ydv,int(realndv),1.d31,1.d31,ydv2)

        epcl_load=.false.  ! do not reload on next call

      endif

c...Now do the actual table lookup
      if(deltav.lt.dexp(xdv(1))) then
        if(dabs(1.d0-ydv(1)).lt.1.d-2) then 
          dsepintgreentab=1.d0
        else  
          dsepintgreentab=ydv(1)
        endif
      elseif(deltav.gt.dexp(xdv(int(realndv)))) then
        dsepintgreentab=0.d0
      else   
        call splint(xdv,ydv,ydv2,int(realndv),dlog(deltav),result)
        dsepintgreentab=result
      endif

      return

c----------------------------------------------------------------------

c...Now take care of missing file
c...We get here if dhow=4, but file does not exist
 200  close(13)
      write(*,*) 'The requested ep table file ',epfile
      write(*,*) 'does not exist. Will recreate if for you.'
      dhow=3
      goto 5


      end

