*****************************************************************************
*** function dshayieldh4 calculates the yield above threshold (yieldk=1) or the
*** differential yield (yieldk=2) from a given higgs
*** boson decaying in flight, the energy of the higgs boson should be given
*** in eh.
*** higgses hno = 1-4 are supported (h10, h20, h30 and h+/h-)
*** units: 1.0e-30 m**-2 (annihilation)**-1
*****************************************************************************

      real*8 function dshayieldh4(eh,emuth,hno,yieldk,istat)
      implicit none
      include 'dshacom.h'
      include 'dsprep.h'
      include 'dsidtag.h'

c------------------------ variables ------------------------------------

      real*8 eh,emuth
      integer hno,istat,yieldk
      integer dch

      real*8 yield

c------------------------ functions ------------------------------------

      real*8 dshayieldfth,dshayielddec

c-----------------------------------------------------------------------


c...loop through different decay channels
      yield=0.0d0

c---------- neutral higgs bosons ----------
      if (hno.le.3) then

c..."fundamental" channels
c        do 100 dch=1,6
c          if (hdw(dch,hno).gt.0.0d0) then
c            yield=yield+hdw(dch,hno)*2.0d0*
c     &        dshayieldfth(eh,mh(hno),map(dch),map(dch),emuth,
c     &          dch,yieldk,istat)
c          endif
c  100   continue

         yield=yield+dshayielddec(eh,hno,emuth,yieldk,istat)

c        if (hdw(15,hno).gt.0.0d0) then  ! mu+ mu-
c          yield=yield+hdw(15,hno)*2.0d0*
c     &      dshayieldfth(eh,mh(hno),map(7),map(7),emuth,
c     &        7,yieldk,istat)
c        endif


c---------- charged higgs bosons ----------
      else

        if (hdw(3,hno).gt.0.0d0) then ! u b-bar
          yield=yield+hdw(3,hno)*
     &    dshayieldfth(eh,mh(hno),map(2),mqu,emuth,2,yieldk,istat)
        endif

        if (hdw(4,hno).gt.0.0d0) then ! c d-bar
          yield=yield+hdw(4,hno)*
     &    dshayieldfth(eh,mh(hno),map(1),mqd,emuth,1,yieldk,istat)
        endif

        if (hdw(5,hno).gt.0.0d0) then ! c s-bar
          yield=yield+hdw(5,hno)*
     &    dshayieldfth(eh,mh(hno),map(1),mqs,emuth,1,yieldk,istat)
        endif

        if (hdw(6,hno).gt.0.0d0) then ! c b-bar
          yield=yield+hdw(6,hno)*
     &    dshayieldfth(eh,mh(hno),map(1),map(2),emuth,
     &    1,yieldk,istat)
          yield=yield+hdw(6,hno)*
     &    dshayieldfth(eh,mh(hno),map(2),map(1),emuth,
     &    2,yieldk,istat)
        endif

        if (hdw(7,hno).gt.0.0d0) then ! t d-bar
          yield=yield+hdw(7,hno)*
     &    dshayieldfth(eh,mh(hno),map(3),mqd,emuth,
     &    3,yieldk,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then ! t s-bar
          yield=yield+hdw(8,hno)*
     &    dshayieldfth(eh,mh(hno),map(3),mqs,emuth,
     &    3,yieldk,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then ! t b-bar
          yield=yield+hdw(9,hno)*
     &    dshayieldfth(eh,mh(hno),map(3),map(2),emuth,
     &    3,yieldk,istat)
          yield=yield+hdw(9,hno)*
     &    dshayieldfth(eh,mh(hno),map(2),map(3),emuth,
     &    2,yieldk,istat)
        endif

        if (hdw(12,hno).gt.0.0d0) then ! tau nu_tau
          yield=yield+hdw(12,hno)*
     &    dshayieldfth(eh,mh(hno),map(4),0.0d0,emuth,
     &    4,yieldk,istat)
        endif

      endif

      dshayieldh4 = yield

c...check for inconsistent declarations in hdw
      if (hno.le.3) then  ! neutral higgses
        do dch=7,14
          if (hdw(dch,hno).gt.0.0d0) then
          write(6,*) 'error in dshayieldh4: inconsistent decay widths',
     +      ' declared in hdw/hdwidth.'
          write(6,*) 'model: ',idtag
          endif
        enddo
      else                ! charged higgs
        do dch=13,15
          if (hdw(dch,hno).gt.0.0d0) then
          write(6,*) 'error in dshayieldh4: inconsistent decay widths',
     +      ' declared in hdw/hdwidth.'
          write(6,*) 'model: ',idtag
          endif
        enddo
      endif

      end




