subroutine ds(Flag, InP, InN, Outsoft, DSO)

!----------------------------------------------
! interfacing routine DarkSusy with MCMC  
! author: Roberto Ruiz
!---------------------------------------------

use parameters

implicit none  

include'dssusy-f90.h'  
include'dsidtag-f90.h' 
include'dssoftsusy-f90.h' 
 

Type(FLAGS), INTENT(IN) :: Flag
Type(Input_params), INTENT(IN) :: InP
Type(Nuisance_params), INTENT(IN) :: InN
Type(Softout), INTENT(IN)  :: Outsoft
Type(DS_Output), INTENT(INOUT) :: DSO


real (8) :: xf, dsrdomega                             ! relic density
real (8) :: sigsip, sigsin, sigsdp, sigsdn            ! direct detec.
real (8) :: dsgm2muon                                 ! g-2 function
integer  :: softerr, unphys, hwarning, iend, ierr, iwar, nfc, &
            acceptable                                ! err or war flags

integer  :: ihiggs                                    ! HDECAY flag 
real(8)  :: bsg_SM, rbsg(2)                           ! bsg parameters
real (8) :: brr_SM, brr_MSSM    ! Higgs branching ratios part
real(8)  :: bsgamma_intp        ! new SM bsgamma value from Misiak et al 


real (8) :: dsabsq ! useful function: sqrt(abs(x))


DSO%PrintOutParams=.false. !reseting


!EW coupling constants
alphem = InN%alphamz
alph3mz = InN%alphasmz

!strange quark hadronic elements
ftp_s = InN%ftp_s
ftn_s = InN%ftn_s

!bottom and top pole masses
mass(kb) = outsoft%run%mbpole 
mass(kt) = InN%mtop 

!other masses
!leptons are pole ones
mass(ke)  =  outsoft.sm.mElectron
mass(kmu) = outsoft.sm.mMuon
mass(ktau)= outsoft.sm.mTau
!quarks are running ones 
mass(ku) = outsoft.sm.mUp 
mass(kd) = outsoft.sm.mDown
mass(ks) = outsoft.sm.mStrange
mcmc = outsoft.sm.mCharm 
mbmb = InN%mbottom


!     This call initializes the DarkSUSY package. This call should
!     be the first call in any program using DarkSUSY. This call initial
!     some global variables and calls various other modules to initializ
!     them with their default values. Check out src/ini/dsinit.f if you
!     want to see what it does.

call dsinit  


!
!     The amount of output onto standard output can be controlled by the
!     variable prtlevel. Setting prtlevel=0 suppresses all messages,
!     except fatal errors that terminate the program. Setting prtlevel=1
!     displays informational messages. Setting prtlevel to higher values
!     displays more and more information. In this example, we set
!     prtlevel=1 to have a message from dsreadpar. We reset it to 0
!     later. Had we set prtlevel=2 here, we would also have messages for
!     each variable which is (re)set in dsreadpar. Caution: if prtlevel
!     is reset in dsreadpar, the new value applies to subsequent
!     messages.
!


prtlevel = 0 !to get avoid tags

!     All the SUSY spectrum passed to DarkSUSY is computed by Softsusy. 
!     The  supersymmetric parameters are: the gaugino masses m1, m2, m3; the
!     Higgs pseudoscalar mass ma; the ratio of Higgs vacuum expectation
!     values tanbe; the square of the squark and slepton mass parameters
!     mass2q(i), mass2u(i), mass2d(i), mass2l(i), mass2e(i); the
!     trilinear soft parameters asofte(i), asoftu(i), asoftd(i). Here
!     i=1,2,3 is a generation index. They are defined at the 
!     weak scale. Physical parameters are passed too.    
!


tanbe = InP%tanb

! Soft masses
mu = Outsoft%soft%mu
m1 = Outsoft%soft%m1
m2 = Outsoft%soft%m2

asofte = Outsoft%soft%asofte
asoftu = Outsoft%soft%asoftu
asoftd = Outsoft%soft%asoftd

mass2l = Outsoft%soft%mass2l
mass2e = Outsoft%soft%mass2e
mass2q = Outsoft%soft%mass2q
mass2u = Outsoft%soft%mass2u
mass2d = Outsoft%soft%mass2d


! Gluino mass
m3 = outsoft%mgluino


! Higgs sector
Mass(kh1)=outsoft%higg%mhh
Mass(kh2)=outsoft%higg%mlh
Mass(kh3) = outsoft%higg%ma
Mass(khc)=outsoft%higg%mch
alpha=outsoft%higg%halpha


! Sfermion sector
msu=outsoft%sfer%msu
msd=outsoft%sfer%msd
mse=outsoft%sfer%mse

msnu=outsoft%sfer%msnu

! Mixings
thetat = outsoft%sfer%thetat
thetab = outsoft%sfer%thetab
thetatau = outsoft%sfer%thetatau


! Neutralino sector
mchi=outsoft%neut%mneut
mixchi=outsoft%neut%mixchi


! Chargino sector
mcha=outsoft%charg%mcha

thetaL=outsoft%charg%thetaL
thetaR=outsoft%charg%thetaR


! Fermions third generation stuff 

! Running masses including SUSY threshold corr.
runmass(ktau) = outsoft%run%mtaumz
runmass(kb) = outsoft%run%mbmz
runmass(kt) =outsoft%run%mtmt


! Yukawa couplings at 2*mchi 
ylsp_tau = outsoft%run%ylsp_tau
ylsp_b = outsoft%run%ylsp_b
ylsp_t = outsoft%run%ylsp_top

! Init errors

softerr = 0

DSO%err%unphys = 0


! Error flags from SoftSusy 

if(DSO%err%muSqWrongSign.eq.1 .or. DSO%err%b.eq.1 .or. &
     DSO%err%noRhoConvergence.eq.1 .or. DSO%err%tachyon.eq.1 .or. & 
     DSO%err%nonperturbative.eq.1 .or. DSO%err%noMuConvergence.eq.1 .or. &
     DSO%err%higgsUfb.eq.1 .or. DSO%err%irqfp.eq.1 .or. &
     DSO%err%badConvergence.eq.1) then

     softerr = 1

else
    call dssusy (unphys, hwarning) 
    DSO%err%unphys = unphys
endif




!For serious debugging
! writes the spectrum in dstest1.tmp
!        open (unit=30,file='dsspect.tmp')
!        write(30,*) ' '
!        call dswspectrum(30)
!        close(30)

! writes the vertex in dsvtx.tmp
!        open (unit=40,file='dsvtx.tmp')
!        call dswvertx(40)
!        close(40)

!
!


if ( (softerr.eq.0) .and. (unphys.eq.0) ) then  
   acceptable = 0  
else  
   acceptable = 1  
endif  

if (Flag%Debug) write ( * , * ) 'acceptable =', acceptable, ' (0=OK, 1=not OK)'  

if (acceptable.ne.0) then  
   DSO%DM%oh2 = 1.0d+40  
!
   DSO%DD%sigsi_nuc = 1.0d+40
!
   DSO%Coll%mw =  1.0d+40
   DSO%Coll%sweff2 = 1.0d+40
   DSO%Coll%gm2amp = 1.0d+40  
   DSO%Coll%bsgamma = 1.0d+40 
   DSO%Coll%brbsmm = 1.0d+40
   DSO%Coll%brr = 1.0d+40
   return 
endif  

!
!
!if (Flag%Debug) then 
!      write (*,*) '  Neutralino mass = ', mass (kn (1))
!      write ( * ,  * ) '  Gaugino fraction = ', &
!          dsabsq (neunmx (1, 1) ) + dsabsq (neunmx (1, 2) )
!      write (* ,*) ' Stau mass ', mass (kstau1) , mass (kstau2)
!endif
!


!     We are now ready to calculate the relic density and different
!     direct or indirect detection rates. We will start with the relic
!     density. The relic density is calculated by the function dsrdomega
!     The first argument determines if coannihilations should be include
!     If the argument is 0, no coannihilations are included, if it is 1,
!     all relevant coannihilations are included (charginos, neutralinos
!     and sfermions if light enough). One can also choose to only
!     include a partial set of coannihilations: argument=2, only chargin
!     and neutralino coannihilaitons, argument=3, only sfermion
!     coannihiltions.
!     The second argument determines the
!     accuracy, if set to 1 a faster calculation (e.g. only including
!     coannihilations if the mass difference is less then 30%) is
!     performed and if set to 0 a more accurate calculation is performed
!     In practice, the fast option is more than adequate (better than
!     5% accuracy). The function returns the relic density, omega h^2,
!     the freeze-out temperature, xf (x=m/T), one error flag, ierr and
!     one warning flag, iwar, both of which are 0 if everything is OK.
!     nfc is the number points in momentum where the cross section had
!     to be calculated. Note that the omega calculation can be very
!     time consuming, needing up to several minutes for tricky models
!     (with many resonances, thresholds, coannihilations etc).
!

!reseting error and warnings related with oh2 calculation.

iwar = 0
ierr = 0


if (Flag%DM_predict) then 

  if (Flag%Debug) write ( * ,  * ) 'Calculating omega h^2 with & 
        &coannihilations, ', 'please be patient...'

  DSO%DM%oh2 = dsrdomega (1, 1, xf, ierr, iwar, nfc)  

  if (Flag%Debug) write ( * ,  * ) '  with coannihilations Oh2 = ', & 
        DSO%DM%oh2, ierr, iwar  

! Gaugino fraction 
  DSO%DM%gf =  dsabsq (neunmx (1, 1) ) + dsabsq (neunmx (1, 2))

endif


!----- error from oh2 calculation, Params are written ---- !
DSO%err%oh2 = ierr 



! Direct Detection rates

if (Flag%DD_predict) then

  if (Flag%Debug) write ( * , * ) 'Calculating scattering cross sections...'  

  call sheffddneunuc (sigsip, sigsin, sigsdp, sigsdn)


! Spin-independent nucleon: taked as the maximun of proton
! and neutron.    
  DSO%DD%sigsi_nuc = max(sigsip, sigsin) * 1.0d36

! Spin-dependent proton and neutron
  DSO%DD%sigsdp = sigsdp * 1.0d36 
  DSO%DD%sigsdn = sigsdn * 1.0d36 !in pb


  if (Flag%Debug) then 
   write ( * ,  * ) '  sigsip (pb)   = ', sigsip * 1.0d36 
   write ( * ,  * ) '  sigsin (pb)   = ', sigsin * 1.0d36 
   write ( * ,  * ) '  sigsi_nuc(pb) = ', DSO%DD%sigsi_nuc
   write ( * ,  * ) '  sigsdp (pb)   = ', DSO%DD%sigsdp
   write ( * ,  * ) '  sigsdn (pb)   = ', DSO%DD%sigsdn
  endif


  if(Flag%DD_rescal) then !oh2 < oh2_exp needs rescaling...
     stop 'Need to write the rescaling in dsinterface.f90'
  end if

endif



!Now we calculate the collider parameters


if (Flag%Collider_predict) then

  call EWPO(DSO%Coll%mw, DSO%Coll%sweff2) 

  if (Flag%Debug) write ( * , * ) 'MW: ', DSO%Coll%mw
  if (Flag%Debug) write ( * , * ) 'sw2: ', DSO%Coll%sweff2

! BR(B -> Xs gamma) + BR(B -> Xs l^+ l^-) through 
! sign(Ampl(B -> Xs gamma)_tot/Ampl(B -> Xs gamma)_SM) 

  bsg_SM = bsgamma_intp(mass(kt), alph3mz) * 1.d-4

  call sheffbsgamma (rbsg, DSO%Coll%sgn_MSSMoSM, 1)

! We substract the old SM value from the MSSM one and add the new one.

  DSO%Coll%bsgamma = rbsg(2) - rbsg(1) + bsg_SM

  if (Flag%Debug) write ( * , * ) 'b->s gamma: ', DSO%Coll%bsgamma  

  if (Flag%Debug) write ( * , * ) 'sgn_MSSMoSM: ', DSO%Coll%sgn_MSSMoSM

! BR(Bs -> mu^+ mu^-)  

  call bsmm (DSO%Coll%brbsmm)  

  if (Flag%Debug)   write ( * , * ) 'b->mu+ mu-: ', DSO%Coll%brbsmm

! (g-2)_SUSY

  DSO%Coll%gm2amp = dsgm2muon ()  

  if (Flag%Debug) write ( * , * ) 'g-2 amplitude [a_mu = (g-2)/2 = ]: ', &
      DSO%Coll%gm2amp  

! [BR(h-> b bbar)/BR(h->WW*])_MSSM/[BR(h-> b bbar)/BR(h->WW*)]_SM
! as computed by HDECAY

  DSO%Coll%brr = 0.d0

  if (.false.) then !this part deactivated, use Feynhiggs instead
     ihiggs = 0  
     call dshdecay(ihiggs, brr_SM) !SM branching ratios
     ihiggs = 1  
     call dshdecay(ihiggs, brr_MSSM) !MSSM branching ratios
     DSO%Coll%brr=brr_MSSM/brr_SM
     if (Flag%Debug)  write ( * , * ) 'ratio branching ratios: ', & 
     DSO%Coll%brr 
  end if


      

end if !collider predict

! ------ WARNING FLAGS -------


if(iwar.ne.0.or.suwar.ne.0.or.aswar.ne.0.or.hdecwar.ne.0 ) then
   DSO%PrintOutParams =.true.
    write(*,*) ' Warning flags from DS interface: '
    print*,'  iwar    = ', iwar
    print*,'  suwar   = ', suwar
    print*,'  aswar   = ', aswar
    print*,'  hdecwar = ', hdecwar

endif

end subroutine ds

