function  sbma2_intp(mlh, choice)

!---------------------------------------------
! extract the LEPII bounds for \zeta^2 vs Higgs
! mass from fig 10 in hep-ex/0306033
! author: Roberto Ruiz
!---------------------------------------------

implicit none
include 'dssusy-f90.h'

integer, parameter :: dim = 217
real(8), intent(in) :: mlh
integer, optional :: choice

!local
integer :: n
real(8), dimension(dim), save :: zeta2_p2s, zeta2_p1s, zeta2_med, zeta2_m2s, &
                           zeta2_m1s, zeta2_obs

real(8), dimension(dim) :: sbma2a, mlha, y2a

real(8) :: sbma2_intp
character(160) :: filein
logical, save :: first = .true.


filein = trim(dsroot)//'src/interf/Higgs-LEPII.dat'

!Column    file contents
!1         Higgs test mass
!2         predicted background at -2sigma
!3         predicted background at -1sigma
!4         predicted median
!5         predicted background at +1 sigma
!6         predicted background at +2 sigma
!7         the observed signal

if (first) then
   open(10,file=filein,status='old',err=111)
   read(10,*)
   
   do n = 1,dim
      read(10,*) mlha(n), zeta2_p2s(n), zeta2_p1s(n), zeta2_med(n), zeta2_m1s(n), &
           zeta2_m2s(n), zeta2_obs(n)
   enddo

   first = .false.
endif

if (.not. present(choice)) choice = 6 !default
select case(choice)
   case(1)
      sbma2a(:) = zeta2_p2s(:)
   case(2)
      sbma2a(:) = zeta2_p1s(:)
   case(3)
      sbma2a(:) = zeta2_med(:)
   case(4)
      sbma2a(:) = zeta2_m1s(:)
   case(5)
      sbma2a(:) = zeta2_m2s(:)
   case(6)
      sbma2a(:) = zeta2_obs(:)  
   case default
      sbma2a(:) = zeta2_obs(:)  
end select


call spline(mlha, sbma2a, DIM, 1.0e30, 1.0e30, y2a)
call splint(mlha, sbma2a, y2a, DIM, mlh, sbma2_intp)

return

111 write(*,*) 'Error reading in the LEPII Higgs file ', trim(filein)
    write(*,*) 'dsroot in higgs.f90 is ', trim(dsroot)
    stop

end function sbma2_intp
