*****************************************************************************
*** function dsmuyieldf calculates the muon yield above threshold and within
*** the selected angular window (yieldk=1) or the differential yield
*** (yieldk=2) for channel ch.
*** yieldv=1 (nu flux), yieldv=2 (nu-to-mu conv.rate), yieldv=3 (mu flux)
*** only channels ch = 1-6 are supported.
*** units: 1.0e-30 m**-2 (annihilation)**-1  integrated
*** units: 1.0e-30 m**-2 gev**-1 (degree)**-1 (annihilation)**1 differential
*** author: joakim edsjo, edsjo@physics.berkeley.edu
*** date: 1995
*** modified: dec 03, 1997.
*****************************************************************************

      real*8 function dsmuyieldf(mneu,emuthr,thmax,ch,wh,yieldk,
     &  yieldv,istat)
      implicit none
      include 'dsmucom.h'
      include 'dsidtag.h'
      include 'dsio.h'

c------------------------ variables ------------------------------------

      real*8 mneu,emuthr,thmax,phi1,phi2,mp1,mp2,zpl,thpl,mn,z,th,
     &  tmp
      integer ch,istat,zi,thi,m1i,m2i,whi,yieldk,yieldv
      character*2 wh
      logical wb
      external dsmucom ! set up common block variables

c-----------------------------------------------------------------------

      dsmuyieldf=0.0d0

      wb=.true.
      if (wh.eq.'su'.or.wh.eq.'SU') then
        whi=1
      else
        whi=2
      endif

      if (emuthr.ge.mneu) then
        dsmuyieldf=0.0d0
        return
      endif

      mn = mneu
c...take care of the case where mneu is between the mass of the annihilation
c...product and the lower bound of the simulations (there might be a small
c...gap of less than a gev) for tt-bar,ww and zz.
      if (ch.eq.3.or.ch.eq.5.or.ch.eq.6) then
        if (mn.lt.lb(ch).and.mn.gt.(0.99*lb(ch))) then
          mn=lb(ch)
        endif
      endif
c...check if mneu within correct bounds
      if (mn.lt.lb(ch).and.emuthr.lt.lb(ch)) then
        if (prtlevel.gt.0) then
          write(6,*)
          write(6,5000) 
     &      'warning in dsmuyieldf: a neutralino mass of ',mn,
     +      ' gev wants to be used,'
          write(6,5010) 'while the lower bound for channel ',ch,
     +      ' is ',lb(ch),' gev.'
          write(6,*) 'the yield is put to 0.0 for these too low masses.'
          write(6,*) 'the results can thus only be trusted as a',
     +      ' lower bound.'
          write(6,*) 'model: ',idtag
        endif
        wb=.false.
        dsmuyieldf=0.0d0
        istat=(istat/2)*2+1
      endif

      if (mn.gt.ub(ch)) then
        if (prtlevel.gt.0) then
          write(6,*)
          write(6,5000) 
     &      'warning in dsmuyieldf: a neutralino mass of ',mn,
     +      ' gev wants to be used,'
          write(6,5010) 'while the upper bound for channel ',ch,
     +      ' is ',ub(ch),' gev.'
          write(6,5020) 'a neutralino mass of ',ub(ch),' gev is used',
     +      ' instead for these too high masses.'
          write(6,*) 'the results can thus only be trusted as a',
     +      ' lower bound.'
          write(6,*) 'model: ',idtag
        endif
        istat=(istat/2)*2+1
      endif


c---------------------------------------------------- integrated yields
      if (wb.and.yieldk.eq.1) then
c...determine which entries in dsmuyieldfnt to use and how
        z=emuthr/mn
c        zi=5
        call dsmuifind(z,zindex(0,1),zpl,zi,0,zn-1)
c        do i=0,zn-1
c          if (z.ge.zindex(i,1).and.z.lt.zindex(i+1,1)) then
c            zi=i
c            zpl=(z-zindex(i,1))/(zindex(i+1,1)-zindex(i,1))
c            goto 21
c          endif
c        enddo
c 21     continue

        if (zi.eq.-5.or.zi.ge.zn) then
          dsmuyieldf=0.0d0
          return
        endif

c        thi=-5
        th=thmax
        call dsmuifind(th,thindex(0,1),thpl,thi,0,thn-1)
c        do i=0,thn-1
c          if (th.ge.thindex(i,1).and.th.lt.thindex(i+1,1)) then
c            thi=i
c            thpl=(th-thindex(i,1))/(thindex(i+1,1)-thindex(i,1))
c            goto 22
c          endif
c        enddo
c 22     continue

        if (thmax.gt.30.0d0) then
          thi=thn-1
          thpl=1.0d0
        endif
        if (thi.eq.-5) then
          dsmuyieldf=0.0d0
          return
        endif

        call dsmuifind(mn,mi(1),tmp,m1i,1,17)
        mp1=mi(m1i)
        m2i=m1i+1
        mp2=mi(m2i)

c        do i=1,17
c          if (mn.ge.mi(i).and.mn.lt.mi(i+1)) then
c            mp1=mi(i)
c            m1i=i
c            mp2=mi(i+1)
c            m2i=i+1
c          endif
c        enddo
        if (mn.ge.mi(18)) then
          m1i=18
          m2i=18
          mp1=mi(18)
          mp2=mp1

            dsmuyieldf = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi,zi+1,m1i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi+1,zi+1,m1i,ch,whi,yieldv)))
        else
          phi1 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi,zi+1,m1i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi+1,zi+1,m1i,ch,whi,yieldv)))
          phi2 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m2i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi,zi+1,m2i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m2i,ch,whi,yieldv))+
     &      zpl*dble(phiint(thi+1,zi+1,m2i,ch,whi,yieldv)))
          dsmuyieldf = phi1 + (phi2-phi1)*(mn-mp1)*(mn+mp1)/
     &      ((mp2-mp1)*(mp2+mp1))
        endif
        if (mn.ne.mneu) then
          dsmuyieldf=mneu**2/mn**2*dsmuyieldf
        endif
      endif

c-------------------------------------------------- differential yields
      if (wb.and.yieldk.eq.2) then
c...determine which entries in phidiff to use and how
        z=emuthr/mn
        call dsmuifind(z,zindex(-1,2),zpl,zi,-1,zn-1)

        if (zi.eq.-5.or.zi.ge.zn) then
          dsmuyieldf=0.0d0
          return
        endif

c        zi=-5
c        do i=-1,zn-1
c          if (z.ge.zindex(i,2).and.z.lt.zindex(i+1,2)) then
c            zi=i
c            zpl=(z-zindex(i,2))/(zindex(i+1,2)-zindex(i,2))
c            goto 24
c          endif
c        enddo
c 24     continue

        if ((emuthr.gt.mn).or.(emuthr.le.0.0).or.zi.eq.-5) then
          dsmuyieldf=0.0d0
          return
        endif

c        thi=-5
        th=thmax
        call dsmuifind(th,thindex(-1,2),thpl,thi,-1,thn-1)

c        do i=-1,thn-1
c          if (th.ge.thindex(i,2).and.th.lt.thindex(i+1,2)) then
c            thi=i
c            thpl=(th-thindex(i,2))/(thindex(i+1,2)-thindex(i,2))
c            goto 25
c          endif
c        enddo
c 25     continue

        if (thmax.gt.30.0d0.or.thi.eq.-5) then
          dsmuyieldf=0.0d0
          return
        endif

        call dsmuifind(mn,mi(1),tmp,m1i,1,17)
        mp1=mi(m1i)
        m2i=m1i+1
        mp2=mi(m2i)
c        do i=1,17
c          if (mn.ge.mi(i).and.mn.lt.mi(i+1)) then
c            mp1=mi(i)
c            m1i=i
c            mp2=mi(i+1)
c            m2i=i+1
c          endif
c        enddo
        if (mn.ge.mi(18)) then
          m1i=18
          m2i=18
          mp1=mi(18)
          mp2=mp1
          dsmuyieldf = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi,zi+1,m1i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi+1,zi+1,m1i,ch,whi,yieldv)))
        else
          phi1 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi,zi+1,m1i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m1i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi+1,zi+1,m1i,ch,whi,yieldv)))
          phi2 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m2i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi,zi+1,m2i,ch,whi,yieldv)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m2i,ch,whi,yieldv))+
     &      zpl*dble(phidiff(thi+1,zi+1,m2i,ch,whi,yieldv)))
          dsmuyieldf = phi1 + (phi2-phi1)*(mn-mp1)*(mn+mp1)/
     &      ((mp2-mp1)*(mp2+mp1))
        endif
        if (mn.ne.mneu) then
          dsmuyieldf=mneu**2/mn**2*dsmuyieldf
        endif
c... convert from dyield / dz dtheta to dyield / de dtheta
        dsmuyieldf=dsmuyieldf/mneu
      endif


 5000 format(' ',a,f8.2,a)
 5010 format(' ',a,i2,a,f8.2,a)
 5020 format(' ',a,f8.2,a,a)

      return

      end
