*****************************************************************************
*** function dsmuyieldh2 calculates the yield above threshold (yieldk=1) or the
*** differential yield (yieldk=2) from a given higgs
*** boson decaying in flight, the energy of the higgs boson should be given
*** in eh.
*** higgses hno = 1-4 are supported (h10, h20, h30 and h+/h-)
*** units: 1.0e-30 m**-2 (annihilation)**-1
*****************************************************************************

      real*8 function dsmuyieldh2(eh,emuth,thmax,hno,wh,
     &  yieldk,yieldv,istat)
      implicit none
      include 'dsmucom.h'
      include 'dsprep.h'

c------------------------ variables ------------------------------------

      real*8 eh,emuth,thmax
      integer hno,istat,yieldk,yieldv
      character*2 wh

      real*8 e1,e2,yield
      integer dch

c------------------------ functions ------------------------------------

      real*8 dsmuyieldh3,dsmuemean,dsmuyieldfth

c-----------------------------------------------------------------------


c...loop through different decay channels
      yield=0.0d0

c---------- neutral higgs bosons ----------
      if (hno.le.3) then

c..."fundamental" channels
        do 100 dch=1,6
          if (hdw(dch,hno).gt.0.0) then
            yield=yield+hdw(dch,hno)*2.0d0*
     &        dsmuyieldfth(eh,mh(hno),map(dch),map(dch),emuth,thmax,
     &          dch,wh,yieldk,yieldv,istat)
          endif
  100   continue

c..."complex" channels
        if (hdw(7,hno).gt.0.0d0) then    ! h10 h10 channel
          e1=dsmuemean(eh,mh(hno),mh(1),mh(1))
          yield=yield+2.0*hdw(7,hno)*dsmuyieldh3(e1,emuth,thmax,
     &      1,wh,yieldk,yieldv,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then    ! h20 h20 channel
          e1=dsmuemean(eh,mh(hno),mh(2),mh(2))
          yield=yield+2.0*hdw(8,hno)*dsmuyieldh3(e1,emuth,thmax,
     &      2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then    ! h30 h30 channel
          e1=dsmuemean(eh,mh(hno),mh(3),mh(3))
          yield=yield+2.0*hdw(9,hno)*dsmuyieldh3(e1,emuth,thmax,
     &      3,wh,yieldk,yieldv,istat)
        endif

        if (hdw(10,hno).gt.0.0d0) then    ! h+ h- channel
          e1=dsmuemean(eh,mh(hno),mh(4),mh(4))
          yield=yield+2.0*
     &      hdw(10,hno)*dsmuyieldh3(e1,emuth,thmax,4,wh,
     &      yieldk,yieldv,istat)
        endif

        if (hdw(11,hno).gt.0.0d0) then    ! z0 h10 channel
          e2=dsmuemean(eh,mh(hno),mh(1),map(6))
          yield=yield+hdw(11,hno)*
     +    dsmuyieldfth(eh,mh(hno),map(6),mh(1),emuth,thmax,
     &      6,wh,yieldk,yieldv,istat)
          yield=yield+hdw(11,hno)*dsmuyieldh3(e2,emuth,thmax,
     &      1,wh,yieldk,yieldv,istat)
        endif

        if (hdw(12,hno).gt.0.0d0) then    ! z0 h20 channel
          e2=dsmuemean(eh,mh(hno),mh(2),map(6))
          yield=yield+hdw(12,hno)*
     +    dsmuyieldfth(eh,mh(hno),map(6),mh(2),emuth,thmax,
     &      6,wh,yieldk,yieldv,istat)
          yield=yield+hdw(12,hno)*dsmuyieldh3(e2,emuth,thmax,
     &      2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(13,hno).gt.0.0d0) then    ! z0 h30 channel
          e2=dsmuemean(eh,mh(hno),mh(3),map(6))
          yield=yield+hdw(13,hno)*
     +    dsmuyieldfth(eh,mh(hno),map(6),mh(3),emuth,thmax,
     &      6,wh,yieldk,yieldv,istat)
          yield=yield+hdw(13,hno)*dsmuyieldh3(e2,emuth,thmax,
     &      3,wh,yieldk,yieldv,istat)
        endif

        if (hdw(14,hno).gt.0.0d0) then    ! w-h+ w+h- channel
          e2=dsmuemean(eh,mh(hno),mh(4),map(5))
          yield=yield+hdw(14,hno)*
     +    dsmuyieldfth(eh,mh(hno),map(5),mh(4),emuth,thmax,
     &      5,wh,yieldk,yieldv,istat)
          yield=yield+hdw(14,hno)*dsmuyieldh3(e2,emuth,thmax,
     &      4,wh,yieldk,yieldv,istat)
        endif

c---------- charged higgs bosons ----------
      else

        if (hdw(3,hno).gt.0.0d0) then ! u b-bar
          yield=yield+hdw(3,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(2),mqu,emuth,thmax,
     &      2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(4,hno).gt.0.0d0) then ! c d-bar
          yield=yield+hdw(4,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(1),mqd,emuth,thmax,
     &      1,wh,yieldk,yieldv,istat)
        endif

        if (hdw(5,hno).gt.0.0d0) then ! c s-bar
          yield=yield+hdw(5,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(1),mqs,emuth,thmax,
     &      1,wh,yieldk,yieldv,istat)
        endif

        if (hdw(6,hno).gt.0.0d0) then ! c b-bar
          yield=yield+hdw(6,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(1),map(2),emuth,thmax,
     &    1,wh,yieldk,yieldv,istat)
          yield=yield+hdw(6,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(2),map(1),emuth,thmax,
     &    2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(7,hno).gt.0.0d0) then ! t d-bar
          yield=yield+hdw(7,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(3),mqd,emuth,thmax,
     &    3,wh,yieldk,yieldv,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then ! t s-bar
          yield=yield+hdw(8,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(3),mqs,emuth,thmax,
     &    3,wh,yieldk,yieldv,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then ! t b-bar
          yield=yield+hdw(9,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(3),map(2),emuth,thmax,
     &    3,wh,yieldk,yieldv,istat)
          yield=yield+hdw(9,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(2),map(3),emuth,thmax,
     &    2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(12,hno).gt.0.0d0) then ! tau nu_tau
          yield=yield+hdw(12,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(4),0.0d0,emuth,thmax,
     &    4,wh,yieldk,yieldv,istat)
        endif

        if (hdw(13,hno).gt.0.0d0) then ! w+ h1
          yield=yield+hdw(13,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(5),mh(1),emuth,thmax,
     &    5,wh,yieldk,yieldv,istat)
          e2=dsmuemean(eh,mh(hno),mh(1),map(5))
          yield=yield+hdw(13,hno)*
     &      dsmuyieldh3(e2,emuth,thmax,1,wh,yieldk,yieldv,istat)
        endif

        if (hdw(14,hno).gt.0.0d0) then ! w+ h2
          yield=yield+hdw(14,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(5),mh(2),emuth,thmax,
     &    5,wh,yieldk,yieldv,istat)
          e2=dsmuemean(eh,mh(hno),mh(2),map(5))
          yield=yield+hdw(14,hno)*
     &      dsmuyieldh3(e2,emuth,thmax,2,wh,yieldk,yieldv,istat)
        endif

        if (hdw(15,hno).gt.0.0d0) then ! w+ h3
          yield=yield+hdw(15,hno)*
     &    dsmuyieldfth(eh,mh(hno),map(5),mh(3),emuth,thmax,
     &    5,wh,yieldk,yieldv,istat)
          e2=dsmuemean(eh,mh(hno),mh(3),map(5))
          yield=yield+hdw(15,hno)*
     &      dsmuyieldh3(e2,emuth,thmax,3,wh,yieldk,yieldv,istat)
        endif

      endif

      dsmuyieldh2 = yield

      end


