      subroutine dssfesct(unphys)
c_______________________________________________________________________
c  sfermion masses and mixings.
c  options:
c    msquarks=0  : squark masses and mixing from mass matrix
c    msquarks>0  : all squark masses equal to msquarks, no mixing
c    msquarks<0  : all squark masses = max(lsp,-msquarks), no mixing
c    msleptons=0  : squark masses and mixing from mass matrix
c    msleptons>0  : all squark masses equal to msleptons, no mixing
c    msleptons<0  : all squark masses = max(lsp,-msleptons), no mixing
c  common:
c    'dssusy.h' - file with susy common blocks
c  called by susyin.
c  needs neusct.
c  author: paolo gondolo 1994-1999
c     981000 correction to diagonalization of mass matrices
c     941100 addition of generation-mixing for squarks
c     950100 correction to charged slepton mass matrix
c     990715 pg options msquarks and msleptons added
c     990724 pg drop chankowski constants
c     020219 pg better reporting of unphys
c     020405 pg matrix diagonalization rewritten to handle degenerate case
c=======================================================================
      implicit none
      include 'dssusy.h'
      real*8 auxa,d,m2ll,m2rr,m2lr,msq1,msq2,theta
      integer i,g,unphys
      character*80 message
      do g=1,3
        do i=1,3
          slulmx(i,g) = dcmplx(0.d0,0.d0)
        enddo
        do i=1,6
          sldlmx(i,g) = dcmplx(0.d0,0.d0)
          sldrmx(i,g) = dcmplx(0.d0,0.d0)
          squlmx(i,g) = dcmplx(0.d0,0.d0)
          squrmx(i,g) = dcmplx(0.d0,0.d0)
          sqdlmx(i,g) = dcmplx(0.d0,0.d0)
          sqdrmx(i,g) = dcmplx(0.d0,0.d0)
        enddo
        slulmx(g,g) = dcmplx(1.d0,0.d0)
        sldlmx(g,g) = dcmplx(1.d0,0.d0)
        sldrmx(g+3,g) = dcmplx(1.d0,0.d0)
        squlmx(g,g) = dcmplx(1.d0,0.d0)
        squrmx(g+3,g) = dcmplx(1.d0,0.d0)
        sqdlmx(g,g) = dcmplx(1.d0,0.d0)
        sqdrmx(g+3,g) = dcmplx(1.d0,0.d0)
      enddo
c------------------------------------- special options

      d = mass(kz)**2*(1.d0-tanbe*tanbe)/(1.d0+tanbe*tanbe) ! mz^2 cos(2beta)

      if (msleptons.gt.0.d0) then
         do i=1,6
            mass(ksnu(i)) = msleptons
            mass(ksl(i)) = msleptons
         enddo
         do i=1,3
            asofte(i) = mu*tanbe
         enddo
      else if (msleptons.lt.0.d0) then
         if (neuloop.ne.0) then
            call dswrite(0,1,
     &    'susfesct: neuloop set to zero since msfermions=mlsp etc')
            neuloop=0
         endif
         call dsneusct
         auxa = abs(mass(kn(kln)))
         if (auxa.lt.-msleptons) auxa = -msleptons
         do i=1,6
            mass(ksnu(i)) = auxa
            mass(ksl(i)) = auxa
         enddo
         do i=1,3
            asofte(i) = mu*tanbe
         enddo
      else
         do i=1,3
c-------------------------------------------------------------sneutrinos
            msq1 = mass2l(i)+d*0.5d0
            if (prtlevel.ge.2) then
               write (*,'('' sneutrino('',i1,'') mass^2 = '',f12.2)')
     &              i,msq1
            endif
            if (msq1.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksnu(i)),
     &              ' mass squared (',real(msq1),')'
               call dswrite(1,1,message)
               unphys = -10-i
            endif
            mass(ksnu(i)) = sqrt(abs(msq1))
            slulmx(i,i)=dcmplx(1.d0,0.d0)
c--------------------------------------------------------charged sleptons
            m2ll = mass2l(i)+mass(kl(i))**2+d*(-0.5d0+s2thw)
            m2rr = mass2e(i)+mass(kl(i))**2+d*(-s2thw)
            m2lr = mass(kl(i))*(asofte(i)-mu*tanbe)
            if (prtlevel.ge.2) then
               write (*,
     &            '('' slepton('',i1,'') mass^2 matrix = '',2(f12.2))')
     &              i,m2ll,m2lr
               write (*,
     &              '(''                            '',2(f12.2))')
     &              m2lr,m2rr
            endif
            call dsdiag2by2(m2ll,m2lr,m2rr,msq1,msq2,theta)
            if (msq1.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksl(i)),
     &              ' mass squared (',real(msq1),')'
               call dswrite(1,1,message)
               unphys = -10-3-i
            endif
            mass(ksl(i)) = sqrt(abs(msq1))
            if (msq2.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksl(i+3)),
     &              ' mass squared (',real(msq2),')'
               call dswrite(1,1,message)
               unphys = -10-3-i
            endif
            mass(ksl(i+3)) = sqrt(abs(msq2))
            sldlmx(i,i) = dcmplx(cos(theta),0.d0)
            sldlmx(i+3,i) = dcmplx(-sin(theta),0.d0)
            sldrmx(i,i) = dcmplx(sin(theta),0.d0)
            sldrmx(i+3,i) = dcmplx(cos(theta),0.d0)
         enddo
      endif

      if (msquarks.gt.0.d0) then
         do i=1,6
            mass(ksqu(i)) = msquarks
            mass(ksqd(i)) = msquarks
         enddo
         do i=1,3
            asoftu(i) = mu/tanbe
            asoftd(i) = mu*tanbe
         enddo
      else if (msquarks.lt.0.d0) then
         if (neuloop.ne.0) then
            call dswrite(0,1,
     &       'susfesct: neuloop set to zero since msfermions=mlsp etc')
            neuloop=0
         endif
         call dsneusct
         auxa = abs(mass(kn(kln)))
         if (auxa.lt.-msquarks) auxa = -msquarks
         do i=1,6
            mass(ksqu(i)) = auxa
            mass(ksqd(i)) = auxa
         enddo
         do i=1,3
            asoftu(i) = mu/tanbe
            asoftd(i) = mu*tanbe
         enddo
      else
c-------------------------------------------------------up-type squarks
         do i=1,3
            m2ll = mass2q(i)+mass(kqu(i))**2+d*(0.5d0-2.d0/3.d0*s2thw)
            m2rr = mass2u(i)+mass(kqu(i))**2+d*(2.d0/3.d0*s2thw)
            m2lr = mass(kqu(i))*(asoftu(i)-mu/tanbe)
            if (prtlevel.ge.2) then
               write (*,
     &            '('' squarku('',i1,'') mass^2 matrix = '',2(f12.2))')
     &              i,m2ll,m2lr
               write (*,
     &              '(''                            '',2(f12.2))')
     &              m2lr,m2rr
            endif
            call dsdiag2by2(m2ll,m2lr,m2rr,msq1,msq2,theta)
            if (msq1.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksqu(i)),
     &              ' mass squared (',real(msq1),')'
               call dswrite(1,1,message)
               unphys = -10-6-i
            endif
            mass(ksqu(i)) = sqrt(abs(msq1))
            if (msq2.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksqu(i+3)),
     &              ' mass squared (',real(msq2),')'
               call dswrite(1,1,message)
               unphys = -10-6-i
            endif
            mass(ksqu(i+3)) = sqrt(abs(msq2))
            squlmx(i,i) = dcmplx(cos(theta),0.d0)
            squlmx(i+3,i) = dcmplx(-sin(theta),0.d0)
            squrmx(i,i) = dcmplx(sin(theta),0.d0)
            squrmx(i+3,i) = dcmplx(cos(theta),0.d0)
c------------------------------------------------------down-type squarks
            m2ll = mass2q(i)+mass(kqd(i))**2+d*(-0.5d0+1.d0/3.d0*s2thw)
            m2rr = mass2d(i)+mass(kqd(i))**2+d*(-1.d0/3.d0*s2thw)
            m2lr = mass(kqd(i))*(asoftd(i)-mu*tanbe)
            if (prtlevel.ge.2) then
               write (*,
     &            '('' squarkd('',i1,'') mass^2 matrix = '',2(f12.2))')
     &              i,m2ll,m2lr
               write (*,
     &              '(''                            '',2(f12.2))')
     &              m2lr,m2rr
            endif
            call dsdiag2by2(m2ll,m2lr,m2rr,msq1,msq2,theta)
            if (msq1.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksqd(i)),
     &              ' mass squared (',real(msq1),')'
               call dswrite(1,1,message)
               unphys = -10-9-i
            endif
            mass(ksqd(i)) = sqrt(abs(msq1))
            if (msq2.lt.0.0d0) then
               write (message,*) 'susfesct: negative ',pname(ksqd(i)),
     &              ' mass squared (',real(msq2),')'
               call dswrite(1,1,message)
               unphys = -10-9-i
            endif
            mass(ksqd(i+3)) = sqrt(abs(msq2))
            sqdlmx(i,i) = dcmplx(cos(theta),0.d0)
            sqdlmx(i+3,i) = dcmplx(-sin(theta),0.d0)
            sqdrmx(i,i) = dcmplx(dsin(theta),0.d0)
            sqdrmx(i+3,i) = dcmplx(cos(theta),0.d0)
         enddo
      endif

      if (unphys.lt.0) then
         write (message,*) 'unphysical sfermion sector (',unphys,')'
         call dswrite(1,1,message)
      endif

      end



      subroutine dsdiag2by2(a,b,c,l1,l2,theta)
c     diagonalize the 2 x 2 symmetric matrix {{a,b},{b,c}}
c     l1 and l2 are the eigenvalues, theta the rotation angle
c     -pi/4 < theta < pi/4
c     a = (l1+l2)/2 + (l1-l2)/2 cos(2theta)
c     b = (l1-l2)/2 sin(2theta)
c     c = (l1+l2)/2 - (l1-l2)/2 cos(2theta)
      implicit none
      real*8 a,b,c,l1,l2,theta
      real*8 auxa,auxb,auxd,eps,pi
      parameter (eps=1.d-9, pi=3.141592653589793238462643d0)
      auxa = 0.5d0*(a+c)
      auxd = 0.5d0*(a-c)
      auxb = b*b+auxd*auxd
      if (dabs(auxd).le.eps*dabs(b)) then
         if (auxd.ne.0.d0) then
            theta = sign(0.25d0*pi,auxd)
            l1 = auxa+sign(sqrt(auxb),auxd)
            l2 = auxa-sign(sqrt(auxb),auxd)
         else
            if (b.ne.0.d0) then
               theta = 0.25d0*pi
               l1 = a+b
               l2 = a-b
            else
               theta = 0.d0
               l1 = a
               l2 = a
            endif
         endif
      else
         theta = 0.5d0*atan(b/auxd)
         l1 = auxa+sign(sqrt(auxb),auxd)
         l2 = auxa-sign(sqrt(auxb),auxd)
      endif
      return
      end
