:Evaluate: BeginPackage["FeynHiggs`"]

:Evaluate: FHSetFlags::usage =
	"FHSetFlags sets the flags for FeynHiggs."

:Evaluate: FHSetPara::usage =
	"FHSetPara sets the parameters for FeynHiggs."

:Evaluate: FHSetSLHA::usage =
	"FHSetSLHA reads the parameters from the given file in SLHA format."

:Evaluate: FHSetCKM::usage =
	"FHSetCKM sets the CKM parameters."

:Evaluate: FHSetNMFV::usage =
	"FHSetNMFV sets the non-minimal flavour-violating parameters."

:Evaluate: FHSetDebug::usage =
	"FHSetDebug sets the FeynHiggs debug level."

:Evaluate: FHGetPara::usage =
	"FHGetPara returns some of the parameters computed by FHSetPara."

:Evaluate: FHHiggsCorr::usage =
	"FHHiggsCorr computes the Higgs masses and mixings."

:Evaluate: FHUncertainties::usage =
	"FHUncertainties computes error estimates for the Higgs masses and mixings."

:Evaluate: FHCouplings::usage =
	"FHCouplings computes the Higgs couplings, widths, and branching ratios."

:Evaluate: FHSelectUZ::usage =
	"FHSelectUZ chooses which of UHiggs (= 1) or ZHiggs (= 2) to use for internal and external Higgs bosons, i.e. in the couplings and the decays, respectively."

:Evaluate: FHConstraints::usage =
	"FHConstraints evaluates electroweak precision observables as further constraints on the MSSM parameter space."

:Evaluate: FHHiggsProd::usage =
	"FHHiggsProd computes (approximate) Higgs production cross-sections."

:Evaluate: FHGetSelf::usage =
	"FHGetSelf[p2] computes the various Higgs self-energies plus their derivatives at momentum-square p2."

:Evaluate: FHError::usage =
	"FHError is an error message returned by FeynHiggs."

:Evaluate: MapIndexed[(Key[#] = 2^(#2[[1]] - 1))&,
	SelfID = {h0h0, h0HH, HHHH, h0A0,
	  HHA0, A0A0, HmHp,
	  G0G0, h0G0, HHG0, A0G0,
	  GmGp, HmGp}]

:Evaluate: {MSf, USf, MCha, UCha, VCha, MNeu, ZNeu, DeltaMB,
	MHiggs, SAeff, UHiggs, ZHiggs,
	DeltaMHiggs, DeltaSAeff, DeltaUHiggs, DeltaZHiggs,
	Coupling, LCoupling, RCoupling,
	CouplingSM, LCouplingSM, RCouplingSM,
	gm2, DeltaRho, MW, MWSM, SW2, SW2SM,
	EDMeTh, EDMn, EDMHg, Bsgamma, BsgammaSM,
	Gamma, BR, GammaTot, GammaSM, BRSM, GammaSMTot,
	Excluded,
	bbhTev, bbhTevSM, bbhLHC, bbhLHCSM,
	btagbhTev, btagbhTevSM, btagbhLHC, btagbhLHCSM,
	gghTev, gghTevSM, gghLHC, gghLHCSM,
	qqhTev, qqhTevSM, qqhLHC, qqhLHCSM,
	tthTev, tthTevSM, tthLHC, tthLHCSM,
	WhTev, WhTevSM, WhLHC, WhLHCSM,
	ZhTev, ZhTevSM, ZhLHC, ZhLHCSM,
	StSthLHC,
	Sigma, DSigma}

:Evaluate: Begin["`Private`"]

:Begin:
:Function: mFHSetFlags
:Pattern:
  FHSetFlags[mssmpart_, fieldren_, tanbren_,
    higgsmix_, p2approx_, looplevel_,
    tlRunningMT_, tlBotResum_]
:Arguments: {
  mssmpart, fieldren, tanbren,
  higgsmix, p2approx, looplevel,
  tlRunningMT, tlBotResum }
:ArgumentTypes: {
  Integer, Integer, Integer,
  Integer, Integer, Integer,
  Integer, Integer }
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSetPara
:Pattern:
  FHSetPara[scalefactor_, AlfasMZ_,
    MC_, MT_, MB_, MW_, MZ_,
    TB_, MA0_, MHp_,
    M3SL_, M3SE_, M3SQ_, M3SU_, M3SD_,
    M2SL_, M2SE_, M2SQ_, M2SU_, M2SD_,
    M1SL_, M1SE_, M1SQ_, M1SU_, M1SD_,
    MUE_,
    Atau_, At_, Ab_,
    Amu_, Ac_, As_,
    Ae_, Au_, Ad_,
    M1_, M2_, M3_,
    Qtau_, Qt_, Qb_]
:Arguments: {
  N[scalefactor], N[AlfasMZ],
  N[MC], N[MT], N[MB], N[MW], N[MZ],
  N[TB], N[MA0], N[MHp], 
  N[M3SL], N[M3SE], N[M3SQ], N[M3SU], N[M3SD],
  N[M2SL], N[M2SE], N[M2SQ], N[M2SU], N[M2SD],
  N[M1SL], N[M1SE], N[M1SQ], N[M1SU], N[M1SD],
  N[Re[MUE]], N[Im[MUE]], 
  N[Re[Atau]], N[Im[Atau]], N[Re[At]], N[Im[At]], N[Re[Ab]], N[Im[Ab]],
  N[Re[Amu]], N[Im[Amu]], N[Re[Ac]], N[Im[Ac]], N[Re[As]], N[Im[As]],
  N[Re[Ae]], N[Im[Ae]], N[Re[Au]], N[Im[Au]], N[Re[Ad]], N[Im[Ad]],
  N[Re[M1]], N[Im[M1]], N[Re[M2]], N[Im[M2]], N[Re[M3]], N[Im[M3]], 
  N[Qtau], N[Qt], N[Qb] }
:ArgumentTypes: {
  Real, Real,
  Real, Real, Real, Real, Real,
  Real, Real, Real,
  Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real,
  Real, Real,
  Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real,
  Real, Real, Real }
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSetSLHA
:Pattern: FHSetSLHA[file_, scalefactor_]
:Arguments: {file, N[scalefactor]}
:ArgumentTypes: {String, Real}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSetCKM
:Pattern: FHSetCKM[theta12_, theta23_, theta13_, delta13_]
:Arguments: {N[theta12], N[theta23], N[theta13], N[delta13]}
:ArgumentTypes: {Real, Real, Real, Real}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSetNMFV
:Pattern: FHSetNMFV[
  deltaLLuc_, deltaLRuc_, deltaRLuc_, deltaRRuc_,
  deltaLLct_, deltaLRct_, deltaRLct_, deltaRRct_,
  deltaLLut_, deltaLRut_, deltaRLut_, deltaRRut_,
  deltaLLds_, deltaLRds_, deltaRLds_, deltaRRds_,
  deltaLLsb_, deltaLRsb_, deltaRLsb_, deltaRRsb_,
  deltaLLdb_, deltaLRdb_, deltaRLdb_, deltaRRdb_]
:Arguments: {
  N[Re[deltaLLuc]], N[Im[deltaLLuc]],
  N[Re[deltaLRuc]], N[Im[deltaLRuc]],
  N[Re[deltaRLuc]], N[Im[deltaRLuc]],
  N[Re[deltaRRuc]], N[Im[deltaRRuc]],
  N[Re[deltaLLct]], N[Im[deltaLLct]],
  N[Re[deltaLRct]], N[Im[deltaLRct]],
  N[Re[deltaRLct]], N[Im[deltaRLct]],
  N[Re[deltaRRct]], N[Im[deltaRRct]],
  N[Re[deltaLLut]], N[Im[deltaLLut]],
  N[Re[deltaLRut]], N[Im[deltaLRut]],
  N[Re[deltaRLut]], N[Im[deltaRLut]],
  N[Re[deltaRRut]], N[Im[deltaRRut]],
  N[Re[deltaLLds]], N[Im[deltaLLds]],
  N[Re[deltaLRds]], N[Im[deltaLRds]],
  N[Re[deltaRLds]], N[Im[deltaRLds]],
  N[Re[deltaRRds]], N[Im[deltaRRds]],
  N[Re[deltaLLsb]], N[Im[deltaLLsb]],
  N[Re[deltaLRsb]], N[Im[deltaLRsb]],
  N[Re[deltaRLsb]], N[Im[deltaRLsb]],
  N[Re[deltaRRsb]], N[Im[deltaRRsb]],
  N[Re[deltaLLdb]], N[Im[deltaLLdb]],
  N[Re[deltaLRdb]], N[Im[deltaLRdb]],
  N[Re[deltaRLdb]], N[Im[deltaRLdb]],
  N[Re[deltaRRdb]], N[Im[deltaRRdb]] }
:ArgumentTypes: {
  Real, Real, Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real, Real, Real,
  Real, Real, Real, Real, Real, Real, Real, Real }
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSetDebug
:Pattern: FHSetDebug[debuglevel_]
:Arguments: {debuglevel}
:ArgumentTypes: {Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHGetPara
:Pattern: FHGetPara[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHHiggsCorr
:Pattern: FHHiggsCorr[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHUncertainties
:Pattern: FHUncertainties[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHCouplings
:Pattern: FHCouplings[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHSelectUZ
:Pattern: FHSelectUZ[uzint_, uzext_]
:Arguments: {uzint, uzext}
:ArgumentTypes: {Integer, Integer}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHConstraints
:Pattern: FHConstraints[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHHiggsProd
:Pattern: FHHiggsProd[]
:Arguments: {}
:ArgumentTypes: {}
:ReturnType: Manual
:End:

:Begin:
:Function: mFHGetSelf
:Pattern: FHGetSelf[p2_, key_, dkey_]
:Arguments: {N[p2], key, dkey}
:ArgumentTypes: {Real, Integer, Integer}
:ReturnType: Manual
:End:

:Evaluate: ComplexTensor[a_, dims_] :=
  RealTensor[Apply[Complex, Partition[a, 2], 1], dims]

:Evaluate: RealTensor[a_, dims_] :=
  Transpose[Fold[Partition, Chop[a], dims][[1]], Range[Length[dims], 1, -1]]

:Evaluate: End[]

:Evaluate: EndPackage[]



/*
	MFeynHiggs.tm
		the Mathematica frontend for FeynHiggs
		this file is part of FeynHiggs
		last modified 15 Nov 06 th
*/


#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>

#include "mathlink.h"
#ifndef MLCONST
#define MLCONST
#endif

typedef struct { double re, im; } dcomplex;

typedef double real;
typedef dcomplex cplx;

typedef MLCONST char cchar;
typedef const int cint;
typedef const long clong;
typedef const real creal;
typedef const cplx ccplx;

#include "CFeynHiggs.h_"
#include "SLHADefs.h"

#if UNDERSCORE
#define flush flush_
#endif

extern void flush(cint *);


#define BeginRedirect() \
  cint prevstdout = dup(1); \
  dup2(2, 1)

#define EndRedirect() \
  { cint fortranstdout = 6; flush(&fortranstdout); } \
  fflush(stdout); \
  dup2(prevstdout, 1); \
  close(prevstdout)


/******************************************************************/

static void MLPutStatus(MLINK mlp, int error)
{
  if( error ) {
    MLPutFunction(mlp, "FHError", 1);
    MLPutInteger(mlp, error);
  }
  else MLPutSymbol(mlp, "True");
}

/******************************************************************/

static inline void MLPutRule(MLINK mlp, cchar *s)
{
  MLPutFunction(mlp, "Rule", 2);
  MLPutSymbol(mlp, s);
}

/******************************************************************/

static void MLPutComplex(MLINK mlp, ccplx c)
{
  if( c.im == 0 ) MLPutReal(mlp, c.re);
  else {
    MLPutFunction(mlp, "Complex", 2);
    MLPutReal(mlp, c.re);
    MLPutReal(mlp, c.im);
  }
}

/******************************************************************/

static void MLPutRealTensor(MLINK mlp, real *a, clong len,
  int *dims, clong depth)
{
  MLPutFunction(mlp, "FeynHiggs`Private`RealTensor", 2);
  MLPutRealList(mlp, a, len);
  MLPutIntegerList(mlp, dims, depth);
}

/******************************************************************/

static void MLPutComplexTensor(MLINK mlp, cplx *a, cint len,
  int *dims, cint depth)
{
  MLPutFunction(mlp, "FeynHiggs`Private`ComplexTensor", 2);
  MLPutRealList(mlp, (real *)a, 2*len);
  MLPutIntegerList(mlp, dims, depth);
}

/******************************************************************/

static void mFHSetFlags(cint mssmpart, cint fieldren, cint tanbren,
  cint higgsmix, cint p2approx, cint looplevel,
  cint tlRunningMT, cint tlBotResum)
{
  int error;

  BeginRedirect();

  FHSetFlags(&error, mssmpart, fieldren, tanbren,
    higgsmix, p2approx, looplevel,
    tlRunningMT, tlBotResum);

  EndRedirect();

  MLPutStatus(stdlink, error);
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHSetPara(creal scalefactor, creal AlfasMZ,
  creal MC, creal MT, creal MB, creal MW, creal MZ,
  creal TB, creal MA0, creal MHp,
  creal M3SL, creal M3SE, creal M3SQ, creal M3SU, creal M3SD,
  creal M2SL, creal M2SE, creal M2SQ, creal M2SU, creal M2SD,
  creal M1SL, creal M1SE, creal M1SQ, creal M1SU, creal M1SD,
  creal reMUE, creal imMUE,
  creal reAtau, creal imAtau,
  creal reAt, creal imAt,
  creal reAb, creal imAb,
  creal reAmu, creal imAmu,
  creal reAc, creal imAc,
  creal reAs, creal imAs,
  creal reAe, creal imAe,
  creal reAu, creal imAu,
  creal reAd, creal imAd,
  creal reM1, creal imM1,
  creal reM2, creal imM2,
  creal reM3, creal imM3,
  creal Qtau, creal Qt, creal Qb)
{
  int error;
  ccplx MUE = {reMUE, imMUE};
  ccplx Atau = {reAtau, imAtau};
  ccplx At = {reAt, imAt};
  ccplx Ab = {reAb, imAb};
  ccplx Amu = {reAmu, imAmu};
  ccplx Ac = {reAc, imAc};
  ccplx As = {reAs, imAs};
  ccplx Ae = {reAe, imAe};
  ccplx Au = {reAu, imAu};
  ccplx Ad = {reAd, imAd};
  ccplx M1 = {reM1, imM1};
  ccplx M2 = {reM2, imM2};
  ccplx M3 = {reM3, imM3};

  BeginRedirect();

  FHSetPara(&error, scalefactor,
    AlfasMZ, MC, MT, MB, MW, MZ,
    TB, MA0, MHp,
    M3SL, M3SE, M3SQ, M3SU, M3SD,
    M2SL, M2SE, M2SQ, M2SU, M2SD,
    M1SL, M1SE, M1SQ, M1SU, M1SD,
    MUE,
    Atau, At, Ab, Amu, Ac, As, Ae, Au, Ad,
    M1, M2, M3,
    Qtau, Qt, Qb);

  EndRedirect();

  MLPutStatus(stdlink, error);
  MLEndPacket(stdlink);
}

/******************************************************************/

#if UNDERSCORE
#define slhaclear slhaclear_
#define slharead slharead_
#endif

void slhaclear(cplx *);
void slharead(int *, cplx *, cchar *, cint *, cint);

static void mFHSetSLHA(cchar *file, creal scalefactor)
{
  cint abort = 0;
  int error;
  cplx slhadata[nslhadata];

  BeginRedirect();

  slharead(&error, slhadata, file, &abort, strlen(file));
  if( error == 0 ) FHSetSLHA(&error, slhadata, scalefactor);

  EndRedirect();

  MLPutStatus(stdlink, error);
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHSetCKM(creal theta12, creal theta23, creal theta13,
  creal delta13)
{
  int error;

  BeginRedirect();

  FHSetCKM(&error, theta12, theta23, theta13, delta13);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else MLPutSymbol(stdlink, "Null");
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHSetNMFV(
  creal redeltaLLuc, creal imdeltaLLuc, 
  creal redeltaLRuc, creal imdeltaLRuc,
  creal redeltaRLuc, creal imdeltaRLuc,
  creal redeltaRRuc, creal imdeltaRRuc,
  creal redeltaLLct, creal imdeltaLLct,
  creal redeltaLRct, creal imdeltaLRct,
  creal redeltaRLct, creal imdeltaRLct,
  creal redeltaRRct, creal imdeltaRRct,
  creal redeltaLLut, creal imdeltaLLut,
  creal redeltaLRut, creal imdeltaLRut,
  creal redeltaRLut, creal imdeltaRLut,
  creal redeltaRRut, creal imdeltaRRut,
  creal redeltaLLds, creal imdeltaLLds,
  creal redeltaLRds, creal imdeltaLRds,
  creal redeltaRLds, creal imdeltaRLds,
  creal redeltaRRds, creal imdeltaRRds,
  creal redeltaLLsb, creal imdeltaLLsb,
  creal redeltaLRsb, creal imdeltaLRsb,
  creal redeltaRLsb, creal imdeltaRLsb,
  creal redeltaRRsb, creal imdeltaRRsb,
  creal redeltaLLdb, creal imdeltaLLdb,
  creal redeltaLRdb, creal imdeltaLRdb,
  creal redeltaRLdb, creal imdeltaRLdb,
  creal redeltaRRdb, creal imdeltaRRdb)
{
  int error;
  ccplx deltaLLuc = {redeltaLLuc, imdeltaLLuc};
  ccplx deltaLRuc = {redeltaLRuc, imdeltaLRuc};
  ccplx deltaRLuc = {redeltaRLuc, imdeltaRLuc};
  ccplx deltaRRuc = {redeltaRRuc, imdeltaRRuc};
  ccplx deltaLLct = {redeltaLLct, imdeltaLLct};
  ccplx deltaLRct = {redeltaLRct, imdeltaLRct};
  ccplx deltaRLct = {redeltaRLct, imdeltaRLct};
  ccplx deltaRRct = {redeltaRRct, imdeltaRRct};
  ccplx deltaLLut = {redeltaLLut, imdeltaLLut};
  ccplx deltaLRut = {redeltaLRut, imdeltaLRut};
  ccplx deltaRLut = {redeltaRLut, imdeltaRLut};
  ccplx deltaRRut = {redeltaRRut, imdeltaRRut};
  ccplx deltaLLds = {redeltaLLds, imdeltaLLds};
  ccplx deltaLRds = {redeltaLRds, imdeltaLRds};
  ccplx deltaRLds = {redeltaRLds, imdeltaRLds};
  ccplx deltaRRds = {redeltaRRds, imdeltaRRds};
  ccplx deltaLLsb = {redeltaLLsb, imdeltaLLsb};
  ccplx deltaLRsb = {redeltaLRsb, imdeltaLRsb};
  ccplx deltaRLsb = {redeltaRLsb, imdeltaRLsb};
  ccplx deltaRRsb = {redeltaRRsb, imdeltaRRsb};
  ccplx deltaLLdb = {redeltaLLdb, imdeltaLLdb};
  ccplx deltaLRdb = {redeltaLRdb, imdeltaLRdb};
  ccplx deltaRLdb = {redeltaRLdb, imdeltaRLdb};
  ccplx deltaRRdb = {redeltaRRdb, imdeltaRRdb};

  BeginRedirect();

  FHSetNMFV(&error,
    deltaLLuc, deltaLRuc, deltaRLuc, deltaRRuc,
    deltaLLct, deltaLRct, deltaRLct, deltaRRct,
    deltaLLut, deltaLRut, deltaRLut, deltaRRut,
    deltaLLds, deltaLRds, deltaRLds, deltaRRds,
    deltaLLsb, deltaLRsb, deltaRLsb, deltaRRsb,
    deltaLLdb, deltaLRdb, deltaRLdb, deltaRRdb);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else MLPutSymbol(stdlink, "Null");
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHSetDebug(cint debuglevel)
{
  BeginRedirect();

  FHSetDebug(debuglevel);

  EndRedirect();

  MLPutSymbol(stdlink, "Null");
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHGetPara(void)
{
  int error, nmfv, t, g;
  real MSf[2*4*3], MASf[6*2], MCha[2], MNeu[4];
  cplx USf[2*2*4*3], UASf[6*6*2];
  cplx UCha[2*2], VCha[2*2], ZNeu[4*4];
  cplx DeltaMB;

  BeginRedirect();

  FHGetPara(&error, &nmfv, MSf, USf, MASf, UASf,
    MCha, UCha, VCha, MNeu, ZNeu, &DeltaMB);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List",
      4*3 + 3*3 - nmfv*(2*2 + 2*2) + 3 + 2 + 1);

    for( t = 0; t < 4 - 2*nmfv; ++t ) {
      for( g = 0; g < 3; ++g ) {
        MLPutFunction(stdlink, "Rule", 2);
        MLPutFunction(stdlink, "MSf", 2);
        MLPutInteger(stdlink, t + 1);
        MLPutInteger(stdlink, g + 1);
        if( t == 0 ) MLPutRealList(stdlink, &MSf[2*4*g], 1);
        else {
          MLPutRealList(stdlink, &MSf[2*(t + 4*g)], 2);

          MLPutFunction(stdlink, "Rule", 2);
          MLPutFunction(stdlink, "USf", 2);
          MLPutInteger(stdlink, t + 1);
          MLPutInteger(stdlink, g + 1);
          MLPutComplexTensor(stdlink, &USf[2*2*(t + 4*g)],
            2*2, (int[]){2, 2}, 2);
        }
      }
    }

    if( nmfv ) {
      for( t = 2; t < 4; ++t ) {
        MLPutFunction(stdlink, "Rule", 2);
        MLPutFunction(stdlink, "MASf", 1);
        MLPutInteger(stdlink, t + 1);

        MLPutRealList(stdlink, &MASf[6*(t - 2)], 6);

        MLPutFunction(stdlink, "Rule", 2);
        MLPutFunction(stdlink, "UASf", 1);
        MLPutInteger(stdlink, t + 1);
        MLPutComplexTensor(stdlink, &UASf[6*6*(t - 2)],
          6*6, (int[]){6, 6}, 2);
      }
    }

    MLPutRule(stdlink, "MCha");
    MLPutRealList(stdlink, MCha, 2);

    MLPutRule(stdlink, "UCha");
    MLPutComplexTensor(stdlink, UCha, 2*2, (int[]){2, 2}, 2);

    MLPutRule(stdlink, "VCha");
    MLPutComplexTensor(stdlink, VCha, 2*2, (int[]){2, 2}, 2);

    MLPutRule(stdlink, "MNeu");
    MLPutRealList(stdlink, MNeu, 4);

    MLPutRule(stdlink, "ZNeu");
    MLPutComplexTensor(stdlink, ZNeu, 4*4, (int[]){4, 4}, 2);

    MLPutRule(stdlink, "DeltaMB");
    MLPutComplex(stdlink, DeltaMB);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHHiggsCorr(void)
{
  int error;
  real MHiggs[4];
  cplx SAeff, UHiggs[3*3], ZHiggs[3*3];

  BeginRedirect();

  FHHiggsCorr(&error, MHiggs, &SAeff, UHiggs, ZHiggs);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List", 4);

    MLPutRule(stdlink, "MHiggs");
    MLPutRealList(stdlink, MHiggs, 4);

    MLPutRule(stdlink, "SAeff");
    MLPutComplex(stdlink, SAeff);

    MLPutRule(stdlink, "UHiggs");
    MLPutComplexTensor(stdlink, UHiggs, 3*3, (int[]){3, 3}, 2);

    MLPutRule(stdlink, "ZHiggs");
    MLPutComplexTensor(stdlink, ZHiggs, 3*3, (int[]){3, 3}, 2);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHUncertainties(void)
{
  int error;
  real DeltaMHiggs[4];
  cplx DeltaSAeff, DeltaUHiggs[3*3], DeltaZHiggs[3*3];

  BeginRedirect();

  FHUncertainties(&error, DeltaMHiggs,
    &DeltaSAeff, DeltaUHiggs, DeltaZHiggs);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List", 3);

    MLPutRule(stdlink, "DeltaMHiggs");
    MLPutRealList(stdlink, DeltaMHiggs, 4);

    MLPutRule(stdlink, "DeltaSAeff");
    MLPutComplex(stdlink, DeltaSAeff);

    MLPutRule(stdlink, "DeltaUHiggs");
    MLPutComplexTensor(stdlink, DeltaUHiggs, 3*3, (int[]){3, 3}, 2);

    MLPutRule(stdlink, "DeltaZHiggs");
    MLPutComplexTensor(stdlink, DeltaZHiggs, 3*3, (int[]){3, 3}, 2);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHCouplings(void)
{
  int error;
  cplx couplings[ncouplings], couplingsms[ncouplingsms];
  real gammas[ngammas], gammasms[ngammasms];
  int excl;

  BeginRedirect();

  FHCouplings(&error, couplings, couplingsms, gammas, gammasms, &excl);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List", 48);

#define Array(array, channel) \
  MLPutFunction(stdlink, "Rule", 2); \
  MLPutFunction(stdlink, #array, 1); \
  MLPutSymbol(stdlink, #channel); \
  MLPutTensor(stdlink, &array(channel

#define Array1(array, channel, i) \
  Array(array, channel) (1)), i, (int[]){i}, 1)

#define Array2(array, channel, i,j) \
  Array(array, channel) (1,1)), i*j, (int[]){i,j}, 2)

#define Array3(array, channel, i,j,k) \
  Array(array, channel) (1,1,1)), i*j*k, (int[]){i,j,k}, 3)

#define Array4(array, channel, i,j,k,l) \
  Array(array, channel) (1,1,1,1)), i*j*k*l, (int[]){i,j,k,l}, 4)

#define Array5(array, channel, i,j,k,l,m) \
  Array(array, channel) (1,1,1,1,1)), i*j*k*l*m, (int[]){i,j,k,l,m}, 5)

/* COUPLINGS */

#define MLPutTensor MLPutComplexTensor

    Array2(Coupling,  H0VV,     3,5);
    Array3(LCoupling, H0FF,     3,4,3);
    Array3(RCoupling, H0FF,     3,4,3);
    Array2(LCoupling, HpFF,     2,3);
    Array2(RCoupling, HpFF,     2,3);
    Array3(LCoupling, H0ChaCha, 3,2,2);
    Array3(RCoupling, H0ChaCha, 3,2,2);
    Array3(LCoupling, H0NeuNeu, 3,4,4);
    Array3(RCoupling, H0NeuNeu, 3,4,4);
    Array2(LCoupling, HpNeuCha, 4,3);
    Array2(RCoupling, HpNeuCha, 4,3);
    Array2(Coupling,  H0HV,     3,3);
    Array1(Coupling,  HpHV,     3);
    Array3(Coupling,  H0HH,     3,4,4);
    Array5(Coupling,  H0SfSf,   3,2,2,4,3);
    Array4(Coupling,  HpSfSf,   2,2,2,3);

    Array2(CouplingSM,  H0VV,   3,5);
    Array3(LCouplingSM, H0FF,   3,4,3);
    Array3(RCouplingSM, H0FF,   3,4,3);

#undef MLPutTensor

/* DECAY WIDTHS */

#define MLPutTensor MLPutRealTensor

    MLPutRule(stdlink, "GammaTot");
    MLPutRealList(stdlink, &GammaTot(1), 4);

    Array2(Gamma, H0VV,     3,5);
    Array3(Gamma, H0FF,     3,4,3);
    Array2(Gamma, HpFF,     2,3);
    Array3(Gamma, H0ChaCha, 3,2,2);
    Array3(Gamma, H0NeuNeu, 3,4,4);
    Array2(Gamma, HpNeuCha, 4,3);
    Array2(Gamma, H0HV,     3,3);
    Array1(Gamma, HpHV,     3);
    Array3(Gamma, H0HH,     3,4,4);
    Array5(Gamma, H0SfSf,   3,2,2,4,3);
    Array4(Gamma, HpSfSf,   2,2,2,3);

    Array2(BR, H0VV,     3,5);
    Array3(BR, H0FF,     3,4,3);
    Array2(BR, HpFF,     2,3);
    Array3(BR, H0ChaCha, 3,2,2);
    Array3(BR, H0NeuNeu, 3,4,4);
    Array2(BR, HpNeuCha, 4,3);
    Array2(BR, H0HV,     3,3);
    Array1(BR, HpHV,     3);
    Array3(BR, H0HH,     3,4,4);
    Array5(BR, H0SfSf,   3,2,2,4,3);
    Array4(BR, HpSfSf,   2,2,2,3);

    MLPutRule(stdlink, "GammaSMTot");
    MLPutRealList(stdlink, &GammaSMTot(1), 3);

    Array2(GammaSM, H0VV, 3,5);
    Array3(GammaSM, H0FF, 3,4,3);

    Array2(BRSM,    H0VV, 3,5);
    Array3(BRSM,    H0FF, 3,4,3);

#undef MLPutTensor
    MLPutRule(stdlink, "Excluded");
    MLPutInteger(stdlink, excl);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHSelectUZ(cint uzint, cint uzext)
{
  int error;

  BeginRedirect();

  FHSelectUZ(&error, uzint, uzext);

  EndRedirect();

  MLPutStatus(stdlink, error);
  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHConstraints(void)
{
  int error;
  real gm2, Deltarho, MWMSSM, MWSM, SW2MSSM, SW2SM;
  real edmeTh, edmn, edmHg;
  real bsgMSSM, bsgSM;

  BeginRedirect();

  FHConstraints(&error, &gm2,
    &Deltarho, &MWMSSM, &MWSM, &SW2MSSM, &SW2SM,
    &edmeTh, &edmn, &edmHg,
    &bsgMSSM, &bsgSM);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List", 11);

    MLPutRule(stdlink, "gm2");
    MLPutReal(stdlink, gm2);

    MLPutRule(stdlink, "DeltaRho");
    MLPutReal(stdlink, Deltarho);

    MLPutRule(stdlink, "MW");
    MLPutReal(stdlink, MWMSSM);

    MLPutRule(stdlink, "MWSM");
    MLPutReal(stdlink, MWSM);

    MLPutRule(stdlink, "SW2");
    MLPutReal(stdlink, SW2MSSM);

    MLPutRule(stdlink, "SW2SM");
    MLPutReal(stdlink, SW2SM);

    MLPutRule(stdlink, "EDMeTh");
    MLPutReal(stdlink, edmeTh);

    MLPutRule(stdlink, "EDMn");
    MLPutReal(stdlink, edmn);

    MLPutRule(stdlink, "EDMHg");
    MLPutReal(stdlink, edmHg);

    MLPutRule(stdlink, "Bsgamma");
    MLPutReal(stdlink, bsgMSSM);

    MLPutRule(stdlink, "BsgammaSM");
    MLPutReal(stdlink, bsgSM);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHHiggsProd(void)
{
  int error;
  real prodxs[nprodxs];

  BeginRedirect();

  FHHiggsProd(&error, prodxs);

  EndRedirect();

#define ProdXS(channel) \
  MLPutRule(stdlink, #channel); \
  MLPutRealList(stdlink, &channel(1), 3)

  if( error ) MLPutStatus(stdlink, error);
  else {
    MLPutFunction(stdlink, "List", 29);
    ProdXS(bbhTev);
    ProdXS(bbhTevSM);
    ProdXS(bbhLHC);
    ProdXS(bbhLHCSM);
    ProdXS(btagbhTev);
    ProdXS(btagbhTevSM);
    ProdXS(btagbhLHC);
    ProdXS(btagbhLHCSM);
    ProdXS(gghTev);
    ProdXS(gghTevSM);
    ProdXS(gghLHC);
    ProdXS(gghLHCSM);
    ProdXS(qqhTev);
    ProdXS(qqhTevSM);
    ProdXS(qqhLHC);
    ProdXS(qqhLHCSM);
    ProdXS(tthTev);
    ProdXS(tthTevSM);
    ProdXS(tthLHC);
    ProdXS(tthLHCSM);
    ProdXS(WhTev);
    ProdXS(WhTevSM);
    ProdXS(WhLHC);
    ProdXS(WhLHCSM);
    ProdXS(ZhTev);
    ProdXS(ZhTevSM);
    ProdXS(ZhLHC);
    ProdXS(ZhLHCSM);
    ProdXS(StSthLHC);
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

static void mFHGetSelf(creal p2, cint key, cint dkey)
{
  enum { semax = 13 };
  int error, i;
  cplx sig[semax], dsig[semax];

  BeginRedirect();

  FHGetSelf(&error, p2, key, sig, dkey, dsig);

  EndRedirect();

  if( error ) MLPutStatus(stdlink, error);
  else {
    int n = 0;
    for( i = 0; i < semax; ++i )
      n += ((key >> i) & 1) + ((dkey >> i) & 1);
    MLPutFunction(stdlink, "List", n);

    for( i = 0; i < semax; ++i ) {
      if( (key >> i) & 1 ) {
        MLPutFunction(stdlink, "Rule", 2);
        MLPutFunction(stdlink, "Sigma", 1);
        MLPutFunction(stdlink, "Part", 2);
        MLPutSymbol(stdlink, "SelfID");
        MLPutInteger(stdlink, i + 1);
        MLPutComplex(stdlink, sig[i]);
      }
      if( (dkey >> i) & 1 ) {
        MLPutFunction(stdlink, "Rule", 2);
        MLPutFunction(stdlink, "DSigma", 1);
        MLPutFunction(stdlink, "Part", 2);
        MLPutSymbol(stdlink, "SelfID");
        MLPutInteger(stdlink, i + 1);
        MLPutComplex(stdlink, dsig[i]);
      }
    }
  }

  MLEndPacket(stdlink);
}

/******************************************************************/

int main(int argc, char **argv)
{
  return MLMain(argc, argv);
}


