! Interfacing routine FH-MCMC
! Author: Roberto Ruiz 
subroutine FeynHiggs(Flag, Out_err, Leshouches, Prodxbr)

use parameters

implicit none

Type(FLAGS), INTENT(IN) :: Flag
Type(Error_Out), INTENT(INOUT) :: Out_err
Type(Leshouches_Out), INTENT(IN):: Leshouches
Type(Prodxbr_Out), INTENT(OUT):: Prodxbr

#include "FeynHiggs-f90.h"
#include "FHCouplings.h"
#include "SLHAIntern.h"
#include "SLHAReadBlocks.h"


real(8) :: mHiggs(4)

integer :: i, j, h
integer :: mssmpart, fieldren, tanbren, higgsmix, p2approx
integer :: looplevel, tl_running_mt, tl_bot_resum
parameter (mssmpart = 4)
parameter (fieldren = 0)
parameter (tanbren = 0)
parameter (higgsmix = 2)
parameter (p2approx = 0)
parameter (looplevel = 2)
parameter (tl_running_mt = 1)
parameter (tl_bot_resum = 1)

real(8) :: scalefactor
real(8) :: sd(nslhadata)
real(8) :: gammas(ngammas), gammasms(ngammasms)
real(8) :: prodxst(nprodxs)

integer :: errort
complex(8) ::  SAeff, UHiggs(3,3)


character(2) :: Higgs_type(3)


higgs_type(1) = "h0"
higgs_type(2) = "HH"
higgs_type(3) = "A0"




!init
do h=1,3
   Prodxbr%Wh%bbar_Tev(h) = 0.0
   Prodxbr%Zh%bbar_Tev(h) = 0.0
   Prodxbr%bbh%bbar_Tev(h) = 0.0
   Prodxbr%ggh%gamgam_LHC(h) = 0.0
   Prodxbr%tth%bbar_LHC(h) = 0.0
   Prodxbr%qqh%tautau_LHC(h) = 0.0
enddo


if (.not.Flag%FH_predict) return
!RT small change: predic-> predict

!if (Flag%Debug) call FHSetDebug(5)


if (Out_err%unphys.ne.0.or.Out_err%oh2.ne.0 &
    .or.Out_err%muSqWrongSign.ne.0.or.Out_err%b.ne.0 &
    .or.Out_err%noRhoConvergence.ne.0.or.Out_err%tachyon.ne.0 &
    .or.Out_err%nonperturbative.ne.0.or.Out_err%noMuConvergence.ne.0 &
    .or.Out_err%higgsUfb.ne.0.or.Out_err%irqfp.ne.0 &
    .or.Out_err%badConvergence.ne.0) goto 100


call FHSetFlags(errort, mssmpart, fieldren, tanbren, higgsmix, &
                p2approx, looplevel, tl_running_mt, tl_bot_resum)

if( errort .ne. 0 ) goto 100

!Filling FH parameters through Leshouches accord format from softsusy

! Block SMINPUTS
        SMInputs_GF =  Leshouches.SMinputs.GFermi
        SMInputs_AlfasMZ = Leshouches.SMinputs.alpha_s
        SMInputs_MZ = Leshouches.SMinputs.MZ 
        SMInputs_Mb = Leshouches.SMinputs.mb
        SMInputs_Mt = Leshouches.SMinputs.mtop
        SMInputs_Mtau = Leshouches.SMinputs.mtau      

        

! Block MASSES
!	snu_e1     
        Mass_MSf(1,1,1) = Leshouches.masses.msnue 
!        print*,Mass_MSf(1,1,1)
!       selectron1   
        Mass_MSf(1,2,1) = Leshouches.masses.msel
!        print*,Mass_MSf(1,2,1)
!       selectron2   
        Mass_MSf(2,2,1) = Leshouches.masses.mser
!        print*,Mass_MSf(2,2,1)
!       sup1         
        Mass_MSf(1,3,1) = Leshouches.masses.msul
!        print*,Mass_MSf(1,3,1)
!       sup2         
        Mass_MSf(2,3,1) = Leshouches.masses.msur
!        print*,Mass_MSf(2,3,1)
!       sdown1       
        Mass_MSf(1,4,1) = Leshouches.masses.mssl
!        print*,Mass_MSf(1,4,1)
!       sdown2       
        Mass_MSf(2,4,1) = Leshouches.masses.msdr
!        print*,Mass_MSf(2,4,1)
!       snu_mu1      
        Mass_MSf(1,1,2) = Leshouches.masses.msnumu 
!        print*,Mass_MSf(1,1,2)
!	smuon1       
        Mass_MSf(1,2,2) = Leshouches.masses.msmul
!        print*,Mass_MSf(1,2,2)
!	smuon2      
        Mass_MSf(2,2,2) = Leshouches.masses.msmur
!        print*,Mass_MSf(2,2,2)
!       scharm1      
        Mass_MSf(1,3,2) = Leshouches.masses.mscl
!        print*,Mass_MSf(1,3,2)
!	scharm2      
        Mass_MSf(2,3,2) = Leshouches.masses.mscr
!        print*,Mass_MSf(2,3,2)
!       sstrange1    
        Mass_MSf(1,4,2) = Leshouches.masses.mssl
!        print*,Mass_MSf(1,4,2)
!       sstrange2    
        Mass_MSf(2,4,2) = Leshouches.masses.mssr
!        print*,Mass_MSf(2,4,2)
!       snu_tau1     
        Mass_MSf(1,1,3) = Leshouches.masses.msnutau
!        print*,Mass_MSf(1,1,3)
!       stau1        
        Mass_MSf(1,2,3) = Leshouches.masses.mstau1
!        print*,Mass_MSf(1,2,3)
!       stau2       
        Mass_MSf(2,2,3) = Leshouches.masses.mstau2
!        print*,Mass_MSf(2,2,3)
!	stop1       
	Mass_MSf(1,3,3) = Leshouches.masses.mst1 
!        print*,Mass_MSf(1,3,3)
!	stop2       
        Mass_MSf(2,3,3) = Leshouches.masses.mst2
!        print*,Mass_MSf(2,3,3)
!	sbottom1    
        Mass_MSf(1,4,3) = Leshouches.masses.msb1
!        print*,Mass_MSf(1,4,3)
!       sbottom2   
      	Mass_MSf(2,4,3) = Leshouches.masses.msb2
!        print*,Mass_MSf(2,4,3)



!W            
	Mass_MW = Leshouches.masses.MW
!h0           
	Mass_Mh0 = Leshouches.masses.mlh
!HH          
	Mass_MHH = Leshouches.masses.mhh
!A0      
	Mass_MA0 = Leshouches.masses.mA0
!Hp          
	Mass_MHp = Leshouches.masses.mHp


!neutralino1  
	Mass_MNeu(1) = Leshouches.masses.mneut1
!neutralino2  
	Mass_MNeu(2) = Leshouches.masses.mneut2
!neutralino3 
	Mass_MNeu(3) = Leshouches.masses.mneut3
!neutralino4  
	Mass_MNeu(4) = Leshouches.masses.mneut4
!chargino1   
	Mass_MCha(1) = Leshouches.masses.mch1
!chargino2   
	Mass_MCha(2) = Leshouches.masses.mch2
!gluino     
        Mass_MGl = Leshouches.masses.mgluino


! Block NMIX
        do i = 1, 4
         do j= 1, 4
           NMix_ZNeu(i, j) = Leshouches.mix.NMix(j, i)
         enddo
        enddo


! Block UMIX
       do i = 1, 2
         do j= 1, 2
           UMix_UCha(i, j) = Leshouches.mix.UMix(j, i)
         enddo
       enddo


! Block VMIX
       do i = 1, 2
         do j= 1, 2
          VMix_VCha(i, j) = Leshouches.mix.VMix(j, i)
         enddo
       enddo


! Block STAUMIX
       do i = 1, 2
         do j= 1, 2
          StauMix_USf(i, j) = Leshouches.mix.StauMix(j, i)
         enddo
       enddo


! Block STOPMIX
       do i = 1, 2
         do j= 1, 2
          StopMix_USf(i, j) = Leshouches.mix.StopMix(j, i)
         enddo
       enddo


! Block SBOTMIX
       do i = 1, 2
         do j= 1, 2
          SbotMix_USf(i, j) = Leshouches.mix.SbotMix(j, i)
         enddo
       enddo


! Block ALPHA
        Alpha_Alpha = Leshouches.masses.alpha

       
! Block HMIX
	HMix_Q = Leshouches.scale.Q
        HMix_MUE = Leshouches.susy.mu
        HMix_TB = Leshouches.susy.tanb
        HMix_VEV = Leshouches.susy.hvev
        HMix_MA02 = Leshouches.soft.m3sq


! Block GAUGE
	Gauge_Q = Leshouches.scale.Q
        Gauge_g1 = Leshouches.coup.gp
	Gauge_g2 = Leshouches.coup.g2
	Gauge_g3 = Leshouches.coup.g3	 


! Block MSOFT
	MSoft_Q = Leshouches.scale.Q
	MSoft_M1 = Leshouches.soft.M1
	MSoft_M2 = Leshouches.soft.M2
	MSoft_M3 = Leshouches.soft.M3
        MSoft_MHd2 = Leshouches.soft.mH12
        MSoft_MHu2 = Leshouches.soft.mH22
        MSoft_MSL(1) = Leshouches.soft.meL
        MSoft_MSL(2) = Leshouches.soft.mmuL
        MSoft_MSL(3) = Leshouches.soft.mtauL
        MSoft_MSE(1) = Leshouches.soft.meR
        MSoft_MSE(2) = Leshouches.soft.mmuR
        MSoft_MSE(3) = Leshouches.soft.mtauR
        MSoft_MSQ(1) = Leshouches.soft.mqL1
        MSoft_MSQ(2) = Leshouches.soft.mqL2
        MSoft_MSQ(3) = Leshouches.soft.mqL3
        MSoft_MSU(1) = Leshouches.soft.muR
        MSoft_MSU(2) = Leshouches.soft.mcR
        MSoft_MSU(3) = Leshouches.soft.mtR
        MSoft_MSD(1) = Leshouches.soft.mdR
        MSoft_MSD(2) = Leshouches.soft.msR
        MSoft_MSD(3) = Leshouches.soft.mbR


! Block AE
	Ae_Q = Leshouches.scale.Q
        Ae_Atau = Leshouches.soft.atau


        Af_Af(1, 1, 2) =  Leshouches.soft.ael
        Af_Af(2, 2, 2) =  Leshouches.soft.amu


! Block AU
	Au_Q = Leshouches.scale.Q
        Au_At = Leshouches.soft.at


        Af_Af(1, 1, 3) =  Leshouches.soft.aup
        Af_Af(2, 2, 3) =  Leshouches.soft.ach

! Block AD
	Ad_Q = Leshouches.scale.Q
	Ad_Ab = Leshouches.soft.ab    


        Af_Af(1, 1, 4) =  Leshouches.soft.ado
        Af_Af(2, 2, 4) =  Leshouches.soft.ast

! Block YE
	Ye_Q = Leshouches.scale.Q
	Ye_Ytau	= Leshouches.coup.ytauq

! Block YU
	Yu_Q = Leshouches.scale.Q
	Yu_Yt = Leshouches.coup.ytq 

! Block YD
	Yd_Q = Leshouches.scale.Q
	Yd_Yb = Leshouches.coup.ybq    



!call to FH routines for Higgs production quantities

!scalefactor = Leshouches.scale.Q/Leshouches.SMinputs.mtop
!better fixed to 1
scalefactor = 1.d0

call SLHAMode(errort, scalefactor, sd, gammas, gammasms, prodxst)


!for uncertainties determination
!DMass_Mh0 = DMass_DeltaMh0


if( errort .ne. 0 ) goto 100


!Refs: hep-ph/0203056 pg. 6...., hep-ph/0503173....


do h= 1, 3

! Tevatron
! gluon gluon fusion:  g + g -> h

    prodxbr%ggh%sigma_Tev(h) = prodxst(h+24)

    prodxbr%ggh%sigmaSM_Tev(h) = prodxst(h+27)


! W,Z Bremsstrahlung -> b bbar
! Dominant

    prodxbr%Wh%sigma_Tev(h) = prodxst(h+60)

    prodxbr%Wh%sigmaSM_Tev(h) = prodxst(h+63)


    prodxbr%Wh%bbar_Tev(h) = prodxst(h+60)*BR(H0FF(h,4,3))

    prodxbr%Wh%bbarSM_Tev(h) = prodxst(h+63)*BRSM(H0FF(h,4,3))


    prodxbr%Wh%bbar_Tev_r(h) = &
       prodxst(h+66)*BR(H0FF(h,4,3))/(prodxst(h+69)*BRSM(H0FF(h,4,3)))

!

    prodxbr%Zh%sigma_Tev(h) = prodxst(h+72)

    prodxbr%Zh%sigmaSM_Tev(h) = prodxst(h+75)


    prodxbr%Zh%bbar_Tev(h) = prodxst(h+72)*BR(H0FF(h,4,3))

    prodxbr%Zh%bbarSM_Tev(h) = prodxst(h+75)*BRSM(H0FF(h,4,3))


    prodxbr%Zh%bbar_Tev_r(h) = &
       prodxst(h+72)*BR(H0FF(h,4,3))/(prodxst(h+75)*BRSM(H0FF(h,4,3)))



! bb fusion; bbar -> (h, H or A) -> b + bbar
! Not so important

     
    prodxbr%bbh%sigma_Tev(h) = prodxst(h)

    prodxbr%bbh%sigmaSM_Tev(h) = prodxst(h+3)


    prodxbr%bbh%bbar_Tev(h) = prodxst(h)*BR(H0FF(h,4,3)) 

    prodxbr%bbh%bbarSM_Tev(h) = prodxst(h+3)*BRSM(H0FF(h,4,3))
  

    prodxbr%bbh%bbar_Tev_r(h) = &
        prodxst(h)*BR(H0FF(h,4,3))/(prodxst(h+3)*BRSM(H0FF(h,4,3)))


! bb fusion; bbar -> b (h, H or A) 

    prodxbr%bh%sigma_Tev(h) = prodxst(h+12)   

    prodxbr%bh%sigmaSM_Tev(h) = prodxst(h+15)     


! LHC
! Gluon fusion: g + g -> h, H or A -> gamma + gamma 


    prodxbr%ggh%sigma_LHC(h) = prodxst(h+30)

    prodxbr%ggh%sigmaSM_LHC(h) = prodxst(h+33)


    prodxbr%ggh%gamgam_LHC(h) = prodxst(h+30)*BR(H0VV(h,1))

    prodxbr%ggh%gamgamSM_LHC(h) = prodxst(h+33)*BRSM(H0VV(h,1))


    prodxbr%ggh%gamgam_LHC_r(h) = & 
       prodxst(h+30)*BR(H0VV(h,1))/(prodxst(h+33)*BRSM(H0VV(h,1))) 


! tt (h, H or A) -> b + bbar


    prodxbr%tth%sigma_LHC(h) = prodxst(h+54)

    prodxbr%tth%sigmaSM_LHC(h) = prodxst(h+57)

 
    prodxbr%tth%bbar_LHC(h) = prodxst(h+54)*BR(H0FF(h,4,3))

    prodxbr%tth%bbarSM_LHC(h) = prodxst(h+57)*BRSM(H0FF(h,4,3))
  

    prodxbr%tth%bbar_LHC_r(h) = &
       prodxst(h+54)*BR(H0FF(h,4,3))/(prodxst(h+57)*BRSM(H0FF(h,4,3)))


! VBF: qq (h, H or A) -> tau + tau


    prodxbr%qqh%sigma_LHC(h) = prodxst(h+42)

    prodxbr%qqh%sigmaSM_LHC(h) = prodxst(h+45)


    prodxbr%qqh%tautau_LHC(h) = prodxst(h+42)*BR(H0FF(h,2,3))

    prodxbr%qqh%tautauSM_LHC(h) = prodxst(h+45)*BRSM(H0FF(h,2,3))
  

    prodxbr%qqh%tautau_LHC_r(h) = &
          prodxst(h+42)*BR(H0FF(h,2,3))/(prodxst(h+45)*BRSM(H0FF(h,2,3)))


! bb fusion; bbar -> b (h, H or A) 

    prodxbr%bh%sigma_LHC(h) = prodxst(h+18)   

    prodxbr%bh%sigmaSM_LHC(h) = prodxst(h+21)  


! BRs 

    prodxbr%branch%BRtobbar(h) =  BR(H0FF(h,4,3))
    prodxbr%branch%BRtobbarSM(h) = BRSM(H0FF(h,4,3))
    prodxbr%branch%BRtogamgam(h) = BR(H0VV(h,1))
    prodxbr%branch%BRtogamgamSM(h) = BRSM(H0VV(h,1))
    prodxbr%branch%BRtotautau(h) = BR(H0FF(h,2,3))
    prodxbr%branch%BRtotautauSM(h) = BRSM(H0FF(h,2,3))
    prodxbr%branch%BRtoWW(h) = BR(H0VV(h,4))
    prodxbr%branch%BRtoWWSM(h) = BRSM(H0VV(h,4))


! Gammas

    prodxbr%gamma%tobbar(h) = Gamma(H0FF(h,4,3))
    prodxbr%gamma%tobbarSM(h) = GammaSM(H0FF(h,4,3))
    prodxbr%gamma%togamgam(h) = Gamma(H0VV(h,1))
    prodxbr%gamma%togamgamSM(h) = GammaSM(H0VV(h,1))
    prodxbr%gamma%totautau(h) = Gamma(H0FF(h,2,3))
    prodxbr%gamma%totautauSM(h) = GammaSM(H0FF(h,2,3))
    prodxbr%gamma%toWW(h) = Gamma(H0VV(h,4))
    prodxbr%gamma%toWWSM(h) = GammaSM(H0VV(h,4))


enddo
   

return
100 continue

Out_err%FH = 1  


end subroutine FeynHiggs

