* CalcSelf.F
* Calculation of the renormalized Higgs self-energies
* this file is part of FeynHiggs
* last modified 27 Nov 06 th

#include "externals.h"
#include "debug.h"


	subroutine CalcSelf(k2, key, dkey)
	implicit none
	double precision k2
	integer key, dkey

#include "FH.h"
#include "OneLoop.h"

#define SelfEnergy(i,f) if( ibits(key,i-1,1) .eq. 1 ) \
seR(i) = f(p2) + dZ(i)*(p2 - Msq(i)) - dMsq(i)

#define DSelfEnergy(i,f) if( ibits(dkey,i-1,1) .eq. 1 ) \
dseR(i) = f(p2) + dZ(i)

	double precision p2
	logical onshell
	integer i

* p2approx decides at which p^2 the self-energies are to be evaluated
	p2 = k2
	onshell = p2approx .eq. 2 .or.
     &    ((p2approx .eq. 0 .or. p2approx .eq. 3) .and. p2 .eq. -1)
       	if( p2approx .eq. 1 .or. p2 .eq. -1) p2 = 0


	do i = 1, semax
	  seR(i) = 0
	  dseR(i) = 0
	enddo

* 1) the CP-even bosons h & H + the mixing h-H

	if( iand(key, cpeven) .ne. 0 .or.
     &      iand(dkey, cpeven) .ne. 0 ) then
	  if( mssmpart .eq. 0 ) then
	    call CPevenSEs_MTapprox
	  else
	    if( onshell ) p2 = Msq(h0h0)
	    SelfEnergy(h0h0, se_h0h0)
	    DSelfEnergy(h0h0, dse_h0h0)

	    if( onshell ) p2 = Msq(HHHH)
	    SelfEnergy(HHHH, se_HHHH)
	    DSelfEnergy(HHHH, dse_HHHH)

	    if( onshell ) p2 = Msq(h0HH)
	    SelfEnergy(h0HH, se_h0HH)
	    DSelfEnergy(h0HH, dse_h0HH)
	  endif

	  if( looplevel .gt. 1 ) then
	    seR(h0h0) = seR(h0h0) + se2R(h0h02)
	    seR(HHHH) = seR(HHHH) + se2R(HHHH2)
	    seR(h0HH) = seR(h0HH) + se2R(h0HH2)
	  endif
	endif

* 2) the CP-odd boson A

	if( onshell ) p2 = Msq(A0A0)
	SelfEnergy(A0A0, se_A0A0)
	DSelfEnergy(A0A0, dse_A0A0)

* 3) the CP-violating h-A & H-A mixing

	if( onshell ) p2 = Msq(h0A0)
	SelfEnergy(h0A0, se_h0A0)
	DSelfEnergy(h0A0, dse_h0A0)

	if( onshell ) p2 = Msq(HHA0)
	SelfEnergy(HHA0, se_HHA0)
	DSelfEnergy(HHA0, dse_HHA0)

* 4) the charged boson H+

	if( onshell ) p2 = Msq(HmHp)
	SelfEnergy(HmHp, se_HmHp)
	DSelfEnergy(HmHp, dse_HmHp)

* 5) the Goldstones

	if( onshell ) p2 = Msq(G0G0)
	SelfEnergy(G0G0, se_G0G0)
	DSelfEnergy(G0G0, dse_G0G0)

	if( onshell ) p2 = Msq(h0G0)
	SelfEnergy(h0G0, se_h0G0)
	DSelfEnergy(h0G0, dse_h0G0)

	if( onshell ) p2 = Msq(HHG0)
	SelfEnergy(HHG0, se_HHG0)
	DSelfEnergy(HHG0, dse_HHG0)

	if( onshell ) p2 = Msq(A0G0)
	SelfEnergy(A0G0, se_A0G0)
	DSelfEnergy(A0G0, dse_A0G0)


	if( p2approx .eq. 3 ) then
	  do i = 1, semax
	    seR(i) = DBLE(seR(i))
	    dseR(i) = DBLE(dseR(i))
	  enddo
	endif
	end


************************************************************************
* CPevenSEs_MTapprox: the one-loop CP-even renormalized self-energies
* in the MT^4-approximation as specified in Sven Heinemeyer's PhD
* thesis, p. 74.

	subroutine CPevenSEs_MTapprox
	implicit none

#include "FH.h"

	double precision mlog, x, x2, c
	double precision sigma_t, lam_t, omega_t

	mlog = log(MSf2(2,3,3)/MSf2(1,3,3))

	x = DBLE(Xf(3,3,3))/MSf2(3,3,3)
	x2 = x**2*(1 - .5D0*MSf2(4,3,3)/MSf2(3,3,3)*mlog)
	x = x*mlog

	c = 3*GF*MT2**2/(sqrt2*pi**2*SB2)

	sigma_t = c*MUE**2*x2

	lam_t = c*MUE*(-.5D0*x - Af(3,3,3)*x2)

	omega_t = c*(log(MSf(1,3,3)*MSf(2,3,3)/MT2) +
     &    Af(3,3,3)*(x + Af(3,3,3)*x2))

	seR(h0h0) = -(SA2*sigma_t + CA2*omega_t - 2*SA*CA*lam_t)
	seR(HHHH) = -(CA2*sigma_t + SA2*omega_t + 2*SA*CA*lam_t)
	seR(h0HH) = -((CA2 - SA2)*lam_t + SA*CA*(omega_t - sigma_t))
	end


************************************************************************
* CalcCTs: calculate the relevant counterterms in the Higgs sector.

	subroutine CalcCTs
	implicit none

#include "FH.h"
#include "OneLoop.h"

	double precision d1, d2, c
	double precision dZH1, dZH2, dTB, dMZ2, dMW2, dTHH, dTh0, dTA0

	Msq(h0h0) = Mh02
	Msq(HHHH) = MHH2
	Msq(A0A0) = MA02
	Msq(HmHp) = MHp2
	Msq(h0HH) = .5D0*(Msq(h0h0) + Msq(HHHH))
	Msq(h0A0) = .5D0*(Msq(h0h0) + Msq(A0A0))
	Msq(HHA0) = .5D0*(Msq(HHHH) + Msq(A0A0))
	Msq(G0G0) = 0
	Msq(h0G0) = .5D0*(Msq(h0h0) + Msq(G0G0))
	Msq(HHG0) = .5D0*(Msq(HHHH) + Msq(G0G0))
	Msq(A0G0) = .5D0*(Msq(A0A0) + Msq(G0G0))
	Msq(GmGp) = 0
	Msq(HmGp) = .5D0*(Msq(HmHp) + Msq(GmGp))

* choose the field renormalization:
	if( fieldren .eq. 4 ) then
* on-shell field renormalization IV goes like this...
	  dZ(h0h0) = -DBLE(dse_h0h0(0D0))
	  dZ(HHHH) = -DBLE(dse_HHHH(0D0))
	  dZ(A0A0) = -DBLE(dse_A0A0(0D0))
	  dZ(HmHp) = -DBLE(dse_HmHp(0D0))
	  dZ(h0HH) = -DBLE(dse_h0HH(0D0))
	  dZ(h0A0) = -DBLE(dse_h0A0(0D0))
	  dZ(HHA0) = -DBLE(dse_HHA0(0D0))
	  dZ(G0G0) = -DBLE(dse_G0G0(0D0))
	  dZ(h0G0) = -DBLE(dse_h0G0(0D0))
	  dZ(HHG0) = -DBLE(dse_HHG0(0D0))
	  dZ(A0G0) = -DBLE(dse_A0G0(0D0))
	  dZ(GmGp) = -DBLE(dse_GmGp(0D0))
	  dZ(HmGp) = -DBLE(dse_HmGp(0D0))
	else
* ... all others use these relations:
	  if( fieldren .eq. 0 ) then
	    dZH1 = 0
	    dZH2 = 0
	  else if( fieldren .eq. 1 ) then
	    d1 = DBLE(dse_A0A0(MA02))
	    d2 = DBLE(se_A0Z(MA02))
	    dZH1 = -d1 - d2/(MZ*TB)
	    dZH2 = -d1 + d2*TB/MZ
	  else if( fieldren .eq. 2 ) then
	    d1 = DBLE(dse_G0G0(MZ2))
	    d2 = DBLE(dse_A0A0(MA02))
	    dZH1 = -( CB2*d1 - SB2*d2)/C2B
	    dZH2 = -(-SB2*d1 + CB2*d2)/C2B
	  else
	    d1 = DBLE(dse_HHHH(MHH2))
	    d2 = DBLE(dse_h0h0(Mh02))
	    dZH1 = -( CA2*d1 - SA2*d2)/C2A
	    dZH2 = -(-SA2*d1 + CA2*d2)/C2A
	  endif

	  dZ(h0h0) = SA2*dZH1 + CA2*dZH2
	  dZ(HHHH) = CA2*dZH1 + SA2*dZH2
	  dZ(A0A0) = SB2*dZH1 + CB2*dZH2
	  dZ(HmHp) = dZ(A0A0)
	  dZ(h0HH) = SA*CA*(dZH2 - dZH1)
	  dZ(h0A0) = 0
	  dZ(HHA0) = 0
	  dZ(G0G0) = CB2*dZH1 + SB2*dZH2
	  dZ(GmGp) = dZ(G0G0)
	  dZ(h0G0) = 0
	  dZ(HHG0) = 0
	  dZ(A0G0) = SB*CB*(dZH2 - dZH1)
	  dZ(HmGp) = dZ(A0G0)
	endif

* renormalization of TB:
	if( tanbren .eq. 0 ) then
	  dTB = 0
	else if( tanbren .eq. 1 ) then
	  dTB = .5D0*(dZH2 - dZH1)
	else
	  dTB = (DBLE(se_A0Z(MA02)) + .5D0*dZ(A0G0)*MZ)/(SB*CB*MZ)
	endif

* on-shell MZ^2, MW^2 and tadpole counterterms:
	dMZ2 = DBLE(se_ZZ(MZ2))
	dMW2 = DBLE(se_WW(MW2))
	dTh0 = -DBLE(td_h0())
	dTHH = -DBLE(td_HH())
	dTA0 = -DBLE(td_A0())

* depending on `inputmass', either MA0 or MHp is defined on-shell:
	if( inputmass .eq. 3 ) then
	  dMsq(A0A0) = DBLE(se_A0A0(MA02))
	  dMsq(HmHp) = dMsq(A0A0) + dMW2
	else
	  dMsq(HmHp) = DBLE(se_HmHp(MHp2))
	  dMsq(A0A0) = dMsq(HmHp) - dMW2
	endif

	c = EL/(2*MZ*SW*CW)

* and finally the Higgs mass counterterms:
	dMsq(h0h0) = dMsq(A0A0)*CBA**2 + dMZ2*SAB**2 +
     &    c*SBA*(dTHH*CBA*SBA - dTh0*(1 + CBA**2)) -
     &    dTB*S2B*(MA02*SBA*CBA - MZ2*SAB*CAB)

	dMsq(HHHH) = dMsq(A0A0)*SBA**2 + dMZ2*CAB**2 -
     &    c*CBA*(dTHH*(1 + SBA**2) - dTh0*SBA*CBA) +
     &    dTB*S2B*(MA02*SBA*CBA - MZ2*SAB*CAB)

	dMsq(h0HH) = -dMsq(A0A0)*SBA*CBA - dMZ2*SAB*CAB -
     &    c*(dTHH*SBA**3 + dTh0*CBA**3) -
     &    dTB*SB*CB*(MA02*(CBA - SBA)*(CBA + SBA) +
     &      MZ2*(CAB - SAB)*(CAB + SAB))

	dMsq(h0A0) = -c*dTA0*SBA
	dMsq(HHG0) = dMsq(h0A0)
	dMsq(h0G0) = c*dTA0*CBA
	dMsq(HHA0) = -dMsq(h0G0)

	dMsq(G0G0) = -c*(dTHH*CBA + dTh0*SBA)
	dMsq(GmGp) = dMsq(G0G0)
	dMsq(A0G0) = c*(dTHH*SBA - dTh0*CBA) - dTB*SB*CB*MA02
	dMsq(HmGp) = c*(dTHH*SBA - dTh0*CBA - cI*dTA0) -
     &    dTB*SB*CB*MHp2

	if( debuglevel .lt. 2 ) return

	DSELF "========================"	ENDL
	DSELF "counter terms by CalcCTs"	ENDL
	DSELF "------------------------"	ENDL
	DSELF "dMsq(h0h0) =", dMsq(h0h0)	ENDL
	DSELF "dZ(h0h0)   =", dZ(h0h0)		ENDL
	DSELF "dMsq(HHHH) =", dMsq(HHHH)	ENDL
	DSELF "dZ(HHHH)   =", dZ(HHHH)		ENDL
	DSELF "dMsq(A0A0) =", dMsq(A0A0)	ENDL
	DSELF "dZ(A0A0)   =", dZ(A0A0)		ENDL
	DSELF "dMsq(HmHp) =", dMsq(HmHp)	ENDL
	DSELF "dZ(HmHp)   =", dZ(HmHp)		ENDL
	DSELF "dMsq(h0HH) =", dMsq(h0HH)	ENDL
	DSELF "dZ(h0HH)   =", dZ(h0HH)		ENDL
	DSELF "dMsq(h0A0) =", dMsq(h0A0)	ENDL
	DSELF "dZ(h0A0)   =", dZ(h0A0)		ENDL
	DSELF "dMsq(HHA0) =", dMsq(HHA0)	ENDL
	DSELF "dZ(HHA0)   =", dZ(HHA0)		ENDL
	DSELF "dMsq(G0G0) =", dMsq(G0G0)	ENDL
	DSELF "dZ(G0G0)   =", dZ(G0G0)		ENDL
	DSELF "dMsq(h0G0) =", dMsq(h0G0)	ENDL
	DSELF "dZ(h0G0)   =", dZ(h0G0)		ENDL
	DSELF "dMsq(HHG0) =", dMsq(HHG0)	ENDL
	DSELF "dZ(HHG0)   =", dZ(HHG0)		ENDL
	DSELF "dMsq(A0G0) =", dMsq(A0G0)	ENDL
	DSELF "dZ(A0G0)   =", dZ(A0G0)		ENDL
	DSELF "dMsq(GmGp) =", dMsq(GmGp)	ENDL
	DSELF "dZ(GmGp)   =", dZ(GmGp)		ENDL
	DSELF "dMsq(HmGp) =", dMsq(HmGp)	ENDL
	DSELF "dZ(HmGp)   =", dZ(HmGp)		ENDL
	end


************************************************************************
* calculate the two-loop parts of Sigma_hh, Sigma_hH, and Sigma_HH

* Note: this subroutine is actually called from DeltaMB.F since it also
* computes the DeltaMB compatible with the two-loop corrections.

	subroutine CalcRenSETL(error)
	implicit none
	integer error

#include "FH.h"
#include "looptools.h"

	double precision dPhi(3)

	call TLCorr(error, getmudim(), dPhi)

	se2R(h0h02) = SA2*dPhi(1) + CA2*dPhi(3) - S2A*dPhi(2)
	se2R(HHHH2) = CA2*dPhi(1) + SA2*dPhi(3) + S2A*dPhi(2)
	se2R(h0HH2) = C2A*dPhi(2) + SA*CA*(dPhi(3) - dPhi(1))

	if( debuglevel .lt. 2 ) return

	DSELF "========================"	ENDL
	DSELF "two-loop self-energies"		ENDL
	DSELF "------------------------"	ENDL
	DSELF "Sigma_11 =", dPhi(1)		ENDL
	DSELF "Sigma_12 =", dPhi(2)		ENDL
	DSELF "Sigma_22 =", dPhi(3)		ENDL
	end

