* DRbartoOS.F
* this file is part of FeynHiggs
* last modified 9 May 05 th

* Code by Pietro Slavich
* 28/05/2004: error in charg12 corrected
* 18/06/2004: rewritten by TH for FeynHiggs

#include "externals.h"
#include "debug.h"


	subroutine DRbartoOS(MSt2, QMSt1, QMSt2, stt, Qstt,
     &    MSb2, USt, USb, Xt, Xb, TB_, MB_, MT_)
	implicit none
	double precision MSt2(2), QMSt1, QMSt2, stt, Qstt
	double precision MSb2(2), TB_, MB_, MT_
	double complex USt(2,2), USb(2,2), Xt, Xb

#define __SUBROUTINE__ "FHSetPara"

	double precision dMSt2(2), dstt
	double precision m1s, m1y, m2s, m2y, ss, sy

	call StrongShifts(dMSt2, dstt,
     &    MSt2, QMSt1, QMSt2, USt(1,1), USt(1,2), Qstt, MT_)

	m1s = MSt2(1) - dMSt2(1)
	m2s = MSt2(2) - dMSt2(2)
	ss = DBLE(USt(1,2)) - dstt

	call YukawaShifts(dMSt2, dstt,
     &    MSt2, QMSt1, QMSt2, USt(1,1), USt(1,2), Qstt,
     &    MSb2, USb(1,1), USb(1,2), Xt, Xb, TB_, MB_, MT_)

	m1y = m1s - dMSt2(1)
	m2y = m2s - dMSt2(2)
	sy = ss - dstt

	if( m1y .ge. 0 .and. m2y .ge. 0 .and. abs(sy) .le. 1 ) then
	  MSt2(1) = m1y
	  MSt2(2) = m2y
	  stt = sy
	else if( m1s .ge. 0 .and. m2s .ge. 0 .and. abs(ss) .le. 1 ) then
	  Warning("Using DRbartoOS without Yukawa corrections")
	  MSt2(1) = m1s
	  MSt2(2) = m2s
	  stt = ss
	else
	  Warning("DRbartoOS transition failed")
	endif
	end


************************************************************************

	subroutine StrongShifts(dMSt2, dstt,
     &    MSt2, QMSt1, QMSt2, ctt, stt, Qstt, MT_)
	implicit none
	double precision dMSt2(2), dstt
	double precision MSt2(2), QMSt1, QMSt2, ctt, stt, Qstt, MT_

#include "FH.h"
#include "looptools.h"

	double precision MT2_, c2tt, s2tt

	MT2_ = MT_**2

	c2tt = (1 - sqrt2*stt)*(1 + sqrt2*stt)
	s2tt = 2*ctt*stt

* Eq. (B3) of DSZ
	dMSt2(1) = Alfas/(3*pi)*(
     &    MSt2(1)*((3 - c2tt**2)*(log(MSt2(1)/QMSt1) - 1) - 4) -
     &    MSt2(2)*s2tt**2*(log(MSt2(2)/QMSt1) - 1) +
     &    2*(MGl2*(log(MGl2/QMSt1) - 1) +
     &       MT2_*(log(MT2_/QMSt1) - 1) +
     &       (MSt2(1) - MGl2 - MT2_ + 2*s2tt*MT_*MGl)*
     &         myB0(MSt2(1), MT2_, MGl2, QMSt1))
     &  )

* Eq. (B4) of DSZ
	dMSt2(2) = Alfas/(3*pi)*(
     &    MSt2(2)*((3 - c2tt**2)*(log(MSt2(2)/QMSt2) - 1) - 4) -
     &    MSt2(1)*s2tt**2*(log(MSt2(1)/QMSt2) - 1) +
     &    2*(MGl2*(log(MGl2/QMSt2) - 1) +
     &       MT2_*(log(MT2_/QMSt2) - 1) +
     &       (MSt2(2) - MGl2 - MT2_ - 2*s2tt*MT_*MGl)*
     &         myB0(MSt2(2), MT2_, MGl2, QMSt2))
     &  )

* On-Shell theta-stop: asymmetric definition used in FeynHiggs
	dstt = Alfas/(3*pi)*ctt*c2tt/(MSt2(1) - MSt2(2))*
     &    (4*MT_*MGl*myB0(MSt2(1), MT2_, MGl2, Qstt) +
     &      s2tt*(MSt2(2)*(1 - log(MSt2(2)/Qstt)) -
     &            MSt2(1)*(1 - log(MSt2(1)/Qstt))))
	end


************************************************************************

	subroutine YukawaShifts(dMSt2, dstt,
     &    MSt2, QMSt1, QMSt2, ctt, stt, Qstt,
     &    MSb2, ctb, stb, Xt, Xb, TB_, MB_, MT_)
	implicit none
	double precision dMSt2(2), dstt
	double precision MSt2(2), QMSt1, QMSt2, ctt, stt, Qstt
	double precision MSb2(2), ctb, stb, Xt, Xb, TB_, MB_, MT_

#include "FH.h"

	double precision Pitt11, Pitt12
	external Pitt11, Pitt12

	double precision c2tt, s2tt, c2tb, s2tb

	s2tt = 2*ctt*stt
	c2tt = (ctt - stt)*(ctt + stt)

	s2tb = 2*ctb*stb
	c2tb = (ctb - stb)*(ctb + stb)

	dMSt2(1) = 1/(16*pi**2)*
     &    Pitt11(MSt2(1), QMSt1, TB_, MB_, MT_,
     &      MSt2(1), MSt2(2), MSb2(1), MSb2(2),
     &      s2tt, c2tt, s2tb, c2tb, Xt, Xb)

	dMSt2(2) = 1/(16*pi**2)*
     &    Pitt11(MSt2(2), QMSt2, TB_, MB_, MT_,
     &      MSt2(2), MSt2(1), MSb2(1), MSb2(2),
     &      -s2tt, -c2tt, s2tb, c2tb, Xt, Xb)

	dstt = .5D0/(16*pi**2)*ctt/(MSt2(1) - MSt2(2))*(
     &    Pitt12(MSt2(1), Qstt, TB_, MB_, MT_,
     &      MSt2(1), MSt2(2), MSb2(1), MSb2(2),
     &      s2tt, c2tt, s2tb, c2tb, Xt, Xb) +
     &    Pitt12(MSt2(2), Qstt, TB_, MB_, MT_,
     &      MSt2(1), MSt2(2), MSb2(1), MSb2(2),
     &      s2tt, c2tt, s2tb, c2tb, Xt, Xb)
     &  )
	end


************************************************************************

	double precision function Pitt11(p, Q, TB_, MB_, MT_,
     &    MSt12, MSt22, MSb12, MSb22,
     &    s2tt, c2tt, s2tb, c2tb, Xt, Xb)
	implicit none
	double precision p, Q, TB_, MB_, MT_
	double precision MSt12, MSt22, MSb12, MSb22
	double precision s2tt, c2tt, s2tb, c2tb, Xt, Xb

#include "FH.h"
#include "looptools.h"

	integer Nc
	parameter (Nc = 3)

	double precision MT2_, MB2_, CB_, SB_, CB2_, SB2_
	double precision ht, hb, Yt, Yb, MUE2
	double precision ctt2, stt2, ctb2, stb2

	double precision Charge11
	Charge11(s2tb, c2tb, stb2, ctb2, SB_, CB_, Xt, Xb) =
     &    (ht*SB_)**2*(ctt2*ctb2*MT2_ + stt2*stb2*MB2_ +
     &      .5D0*s2tt*s2tb*MB_*MT_ +
     &      Xt*(ctb2*s2tt*MT_ + stt2*s2tb*MB_ + stt2*ctb2*Xt)) +
     &    (hb*CB_)**2*(ctb2*ctt2*MB2_ + stb2*stt2*MT2_ +
     &      .5D0*s2tb*s2tt*MB_*MT_ +
     &      Xb*(ctt2*s2tb*MB_ + stb2*s2tt*MT_ + stb2*ctt2*Xb)) -
     &    ht*hb*SB_*CB_*(2*(ctt2*ctb2 + stt2*stb2)*MT_*MB_ +
     &      .5D0*s2tt*s2tb*(MT2_ + MB2_ + Xt*Xb) +
     &      s2tb*MT_*(ctt2*Xb + stt2*Xt) +
     &      s2tt*MB_*(ctb2*Xt + stb2*Xb))

	MT2_ = MT_**2
	MB2_ = MB_**2

	CB2_ = 1/(1 + TB_**2)
	CB_ = sqrt(CB2_)
	SB_ = CB_*TB_
	SB2_ = SB_**2
	
	ht = sqrt2/vev*MT_/SB_
	hb = sqrt2/vev*MB_/CB_
	      
	Yt = Xt + reMUE/(SB_*CB_)
	Yb = Xb + reMUE/(SB_*CB_)

	MUE2 = reMUE**2

	ctt2 = .5D0*(1 + c2tt)
	stt2 = .5D0*(1 - c2tt)
	ctb2 = .5D0*(1 + c2tb)
	stb2 = .5D0*(1 - c2tb)

	Pitt11 =
     &    ht**2*(myG(p, MT2_, MUE2, Q) +
     &      stt2*(ctb2*myAA(MSb12, Q) + stb2*myAA(MSb22, Q)) +
     &      (c2tt**2 - .5D0*(Nc - 1)*s2tt**2)*myAA(MSt22, Q) +
     &      .5D0*(Nc + 1)*s2tt**2*myAA(MSt12, Q) +
     &      CB2_*(1 + stt2)*myAA(MA02, q) +
     &      .5D0*(SB2_*(2*MT_ + s2tt*Xt)**2*myB0(p, MSt12, 0D0, Q) +
     &         CB2_*(2*MT_ + s2tt*Yt)**2*myB0(p, MSt12, MA02, Q) +
     &         (1 + c2tt**2)*(SB2_*Xt**2*myB0(p, MSt22, 0D0, Q) +
     &                        CB2_*Yt**2*myB0(p, MSt22, MA02, Q)))) +
     &    (ht**2*stt2 + hb**2*ctt2)*myG(p, MB2_, MUE2, Q) -
     &    2*ht*hb*MB_*reMUE*s2tt*myB0(p, MB2_, MUE2, Q) +
     &    hb**2*ctt2*(SB2_*myAA(MA02, Q) +
     &      stb2*myAA(MSb12, Q) + ctb2*myAA(MSb22, Q)) +
     &    Charge11(s2tb, c2tb, stb2, ctb2, SB_, CB_, Xt, Xb)*
     &      myB0(p, MSb12, 0D0, Q) +
     &    Charge11(-s2tb, -c2tb, ctb2, stb2, SB_, CB_, Xt, Xb)*
     &      myB0(p, MSb22, 0D0, Q) +
     &    Charge11(s2tb, c2tb, stb2, ctb2, CB_, -SB_, Yt, Yb)*
     &      myB0(p, MSb12, MA02, Q) +
     &    Charge11(-s2tb, -c2tb, ctb2, stb2, CB_, -SB_, Yt, Yb)*
     &      myB0(p, MSb22, MA02, Q)
	end


************************************************************************

	double precision function Pitt12(p, Q, TB_, MB_, MT_,
     &    MSt12, MSt22, MSb12, MSb22,
     &    s2tt, c2tt, s2tb, c2tb, Xt, Xb)
	implicit none
	double precision p, Q, TB_, MB_, MT_
	double precision MSt12, MSt22, MSb12, MSb22
	double precision s2tt, c2tt, s2tb, c2tb, Xt, Xb

#include "FH.h"
#include "looptools.h"

	integer Nc
	parameter (Nc = 3)

	double precision MT2_, MB2_, CB_, SB_, CB2_, SB2_
	double precision ht, hb, Yt, Yb, MUE2
	double precision ctt2, stt2, ctb2, stb2

	double precision Charge12
	Charge12(s2tb, c2tb, stb2, ctb2, SB_, CB_, Xt, Xb) =
     &    .5D0*(ht*SB_)**2*(s2tt*(stb2*MB2_ - ctb2*MT2_) + 
     &      Xt*(2*c2tt*ctb2*MT_ + s2tt*(s2tb*MB_ + ctb2*Xt)) +
     &      c2tt*s2tb*MT_*MB_) +
     &    .5D0*(hb*CB_)**2*(s2tt*(stb2*MT2_ - ctb2*MB2_) + 
     &      Xb*(2*c2tt*ctb2*MT_ - s2tt*(s2tb*MB_ + stb2*Xb)) +
     &      c2tt*s2tb*MT_*MB_) -
     &    .5D0*ht*hb*SB_*CB_*(c2tt*s2tb*(MT2_ + MB2_ + Xt*Xb) +
     &      s2tt*s2tb*MT_*(Xt - Xb) +
     &      2*MB_*(c2tt*(ctb2*Xt + stb2*Xb) - s2tt*c2tb*MT_))

	MT2_ = MT_**2
	MB2_ = MB_**2

	CB2_ = 1/(1 + TB_**2)
	SB2_ = 1 - CB2_
	CB_ = sqrt(CB2_)
	SB_ = CB_*TB_
	
	ht = sqrt2/vev*MT_/SB_
	hb = sqrt2/vev*MB_/CB_
	      
	Yt = Xt + reMUE/(SB_*CB_)
	Yb = Xb + reMUE/(SB_*CB_)

	MUE2 = reMUE**2

	ctt2 = .5D0*(1 + c2tt)
	stt2 = .5D0*(1 - c2tt)
	ctb2 = .5D0*(1 + c2tb)
	stb2 = .5D0*(1 - c2tb)

	Pitt12 =
     &    .5D0*ht**2*(
     &      s2tt*(CB2_*myAA(MA02, Q) +
     &        (Nc + 1)*c2tt*(myAA(MSt12, Q) - myAA(MSt22, Q)) +
     &        ctb2*myAA(MSb12, Q) + stb2*myAA(MSb22, Q)) +
     &      c2tt*(SB2_*Xt*(2*MT_ + s2tt*Xt)*myB0(p, MSt12, 0D0, Q) +
     &        CB2_*Yt*(2*MT_ + s2tt*Yt)*myB0(p, MSt12, MA02, Q) +
     &        SB2_*Xt*(2*MT_ - s2tt*Xt)*myB0(p, MSt22, 0D0, Q) +
     &        CB2_*Yt*(2*MT_ - s2tt*Yt)*myB0(p, MSt22, MA02, Q))) +
     &    .5D0*(ht**2 - hb**2)*s2tt*myG(p, MB2_, MUE2, Q) -
     &    2*ht*hb*MB_*reMUE*c2tt*myB0(p, MB2_, MUE2, Q) -
     &    .5D0*hb**2*s2tt*(SB2_*myAA(MA02, Q) +
     &      stb2*myAA(MSb12, Q) + ctb2*myAA(MSb22, Q)) +
     &    Charge12(s2tb, c2tb, stb2, ctb2, SB_, CB_, Xt, Xb)*
     &      myB0(p, MSb12, 0D0, Q) +
     &    Charge12(-s2tb, -c2tb, ctb2, stb2, SB_, CB_, Xt, Xb)*
     &      myB0(p, MSb22, 0D0, Q) +
     &    Charge12(s2tb, c2tb, stb2, ctb2, CB_, -SB_, Yt, Yb)*
     &      myB0(p, MSb12, MA02, Q) +
     &    Charge12(-s2tb, -c2tb, ctb2, stb2, CB_, -SB_, Yt, Yb)*
     &      myB0(p, MSb22, MA02, Q)
	end

