* DeltaMB.F
* calculation of the Delta-MB corrections
* this file is part of FeynHiggs
* last modified 1 Nov 06 th

#include "externals.h"
#include "debug.h"


	subroutine MBfromPara(error)
	implicit none
	integer error

#include "FH.h"
#define __SUBROUTINE__ "MBfromPara"

	double precision oldMBy
	integer i
	logical fail

	if( looplevel .gt. 1 ) then
	  call TLShifts(error,
     &      Alfas, TB, MA02, M_3, MUE, Xf, DSS2, DSf,
     &      tl_running_mt, MTrun, MT,
     &      tl_bot_resum, MBy, DeltaMBc)

	  if( debuglevel .ge. 2 )
     &      DPARA "MBfromPara: MBy =", MBy ENDL

	  call CalcRenSETL(error)
	  return
	endif

	if( tl_bot_resum .eq. 0 ) return

	do i = 1, DeltaMB_maxpass
	  call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &      DSS2(1,4,3), MBy, Xf(4,3,3))

	  oldMBy = MBy
	  call ResumMB

	  if( debuglevel .ge. 2 )
     &      DPARA "MBfromPara: MBy =", MBy ENDL

	  if( abs(MBy - oldMBy) .lt. 1D-8 ) return
	enddo

	if( DeltaMB_maxpass .gt. 1 )
     &    Warning("resummation of MB did not converge")
	end

#undef __SUBROUTINE__


************************************************************************

	subroutine MBfromMasses(error, X)
	implicit none
	integer error
	double complex X(2,2)

#include "FH.h"
#define __SUBROUTINE__ "MBfromMasses"

	double precision oldMBy
	integer i, err
	logical fail

	if( tl_bot_resum .eq. 1 ) call ResumMB

	if( debuglevel .ge. 2 )
     &    DPARA "MBfromMasses: MBy =", MBy ENDL

	if( looplevel .lt. 2 ) then
	  DSS2(1,4,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	  DSS2(1,5,3) = DSS2(1,4,3)
	  DSS2(2,4,3) = DBLE(X(2,2)) - MBy2
	  DSS2(2,5,3) = DSS2(2,4,3)
c	  Xf(4,3,3) = DCONJG(X(1,2))/MBy
	  return
	endif

	do i = 1, DeltaMB_maxpass
	  DSS2(1,4,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	  DSS2(1,5,3) = DSS2(1,4,3)
	  DSS2(2,4,3) = DBLE(X(2,2)) - MBy2
	  DSS2(2,5,3) = DSS2(2,4,3)
c	  Xf(4,3,3) = DCONJG(X(1,2))/MBy

	  oldMBy = MBy
	  err = error
	  call TLShifts(err,
     &      Alfas, TB, MA02, M_3, MUE, Xf, DSS2, DSf,
     &      tl_running_mt, MTrun, MT,
     &      tl_bot_resum, MBy, DeltaMBc)

	  if( debuglevel .ge. 2 )
     &      DPARA "MBfromMasses: MBy =", MBy ENDL

	  if( abs(MBy - oldMBy) .lt. 1D-8 ) goto 1
	enddo

	if( DeltaMB_maxpass .gt. 1 )
     &    Warning("resummation of MB did not converge")

1	error = err
	call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &    DSS2(1,4,3), MBy, Xf(4,3,3))
	if( fail ) Error("negative sbottom mass squares")

	call CalcRenSETL(error)
	end


************************************************************************

	subroutine ResumMB
	implicit none

#include "FH.h"

	double precision Tdmb
	external Tdmb

	DeltaMBc(1) = TB*DCONJG(MUE)*(
     &    2/(3*pi)*Alfas*DCONJG(M_3)*
     &      Tdmb(MSf2(1,4,3), MSf2(2,4,3), Sq(M_3)) +
     &    (MTrun/174.1D0)**2/(4*pi)**2*DCONJG(Af(3,3,3))*
     &      Tdmb(MSf2(1,3,3), MSf2(2,3,3), Sq(MUE)) )
	DeltaMBc(2) = DeltaMBc(1)

	Mdy(3,1) = MBrun/abs(1 + DeltaMBc(1))
	Mdy(3,2) = Mdy(3,1)

	call BUpdate
	end


************************************************************************
	
	double precision function Tdmb(x, y, z)
	implicit none
	double precision x, y, z

	double precision n, d

	n = 0
	d = 1

	if( x .ne. y ) then
	  if( x*y .ne. 0 ) n = n + x*y*log(x/y)
	  d = d*(x - y)
	endif
	if( x .ne. z ) then
	  if( x*z .ne. 0 ) n = n + x*z*log(z/x)
	  d = d*(x - z)
	endif
	if( y .ne. z ) then
	  if( y*z .ne. 0 ) n = n + y*z*log(y/z)
	  d = d*(y - z)
	endif

	Tdmb = n/d
	end       


************************************************************************
	
	subroutine BUpdate
	implicit none

#include "FH.h"

	Mdy2(3,1) = Mdy(3,1)**2

	Mdy(3,2) = MBrun/abs(1 + DeltaMBc(2))
	Mdy2(3,2) = Mdy(3,2)**2

	Hff(1,3,1) = 1 - CA/(SA*TB)*DeltaMBc(1)
	Hff(2,3,1) = 1 + SA/(CA*TB)*DeltaMBc(1)

	Hff(1,3,2) = 1 - CA/(SA*TB)*DeltaMBc(2)
	Hff(2,3,2) = 1 + SA/(CA*TB)*DeltaMBc(2)
	end

