* HiggsProd.F
* approximate calculation of the Higgs production cross-sections
* all cross section data is from Fabio Maltoni
* see http://maltoni.home.cern.ch/maltoni/TeV4LHC
* this file is part of FeynHiggs
* last modified 15 Nov 06 th

#include "externals.h"
#include "debug.h"
#include "FHCouplings.h"


	subroutine FHHiggsProd(error, prodxs)
	implicit none
	integer error
	double precision prodxs(nprodxs)

#include "FH.h"
#define __SUBROUTINE__ "FHHiggsProd"

	integer i, h
	double precision sqrtm, c

	character*2 higgs(4)
	data higgs /"h0", "HH", "A0", "Hp"/

	call CheckCouplings(error)
	if( error .ne. 0 ) return

	do i = 1, nprodxs
	  prodxs(i) = -999
	enddo

	do h = 1, 3
	  if( MHiggs(h) .gt. 80 ) then
	    sqrtm = sqrt(MHiggs(h))

#include "HiggsProdFits.f"

	    c = GammaSM(H0FF(h,4,3))
	    if( c .eq. 0 ) then
	      bbhTev(h) = 0
	      bbhTevSM(h) = 0
	      bbhLHC(h) = 0
	      bbhLHCSM(h) = 0
	      btagbhTev(h) = 0
	      btagbhTevSM(h) = 0
	      btagbhLHC(h) = 0
	      btagbhLHCSM(h) = 0
	    else
	      c = Gamma(H0FF(h,4,3))/c
	      bbhTev(h) = c*bbhTevSM(h)
	      bbhLHC(h) = c*bbhLHCSM(h)
	      btagbhTev(h) = c*btagbhTevSM(h)
	      btagbhLHC(h) = c*btagbhLHCSM(h)
	    endif

	    c = ratios(H0FF(h,3,3))
	    tthTev(h) = c*tthTevSM(h)
	    tthLHC(h) = c*tthLHCSM(h)

	    c = GammaSM(H0VV(h,5))
	    if( c .eq. 0 ) then
	      gghTev(h) = 0
	      gghTevSM(h) = 0
	      gghLHC(h) = 0
	      gghLHCSM(h) = 0
	    else
	      c = Gamma(H0VV(h,5))/c
	      gghTev(h) = c*gghTevSM(h)
	      gghLHC(h) = c*gghLHCSM(h)
	    endif

	    c = ratios(H0VV(h,4))
	    qqhTev(h) = c*qqhTevSM(h)
	    qqhLHC(h) = c*qqhLHCSM(h)
	    WhTev(h) = c*WhTevSM(h)
	    WhLHC(h) = c*WhLHCSM(h)

	    c = ratios(H0VV(h,3))
	    ZhTev(h) = c*ZhTevSM(h)
	    ZhLHC(h) = c*ZhLHCSM(h)

	    if( h .eq. 1 ) then
	      StSthLHC(h) = StSthLHC(h)*
     &          Sq(Coupling(H0SfSf(h,1,1,3,3)))
	    endif
	  endif

	  if( debuglevel .ge. 2 ) then
	    DPROD higgs(h), " production cross-sections (MSSM, SM):" ENDL
	    DPROD "bbhTev    =", bbhTev(h), bbhTevSM(h)		ENDL
	    DPROD "bbhLHC    =", bbhLHC(h), bbhLHCSM(h)		ENDL
	    DPROD "btagbhTev =", btagbhTev(h), btagbhTevSM(h)	ENDL
	    DPROD "btagbhLHC =", btagbhLHC(h), btagbhLHCSM(h)	ENDL
	    DPROD "gghTev    =", gghTev(h), gghTevSM(h)		ENDL
	    DPROD "gghLHC    =", gghLHC(h), gghLHCSM(h)		ENDL
	    DPROD "qqhTev    =", qqhTev(h), qqhTevSM(h)		ENDL
	    DPROD "qqhLHC    =", qqhLHC(h), qqhLHCSM(h)		ENDL
	    DPROD "tthTev    =", tthTev(h), tthTevSM(h)		ENDL
	    DPROD "tthLHC    =", tthLHC(h), tthLHCSM(h)		ENDL
	    DPROD "WhTev     =", WhTev(h), WhTevSM(h)		ENDL
	    DPROD "WhLHC     =", WhLHC(h), WhLHCSM(h)		ENDL
	    DPROD "ZhTev     =", ZhTev(h), ZhTevSM(h)		ENDL
	    DPROD "ZhLHC     =", ZhLHC(h), ZhLHCSM(h)		ENDL
	    DPROD "StSthLHC  =", StSthLHC(h)			ENDL
	  endif
	enddo
	end

