* SetCKM.F
* initialization routines for the CKM matrix
* this file is part of FeynHiggs
* last modified 27 May 06 th

#include "externals.h"
#include "debug.h"


	subroutine FHSetCKM(error, theta12, theta23, theta13, delta13)
	implicit none
	integer error
	double precision theta12, theta23, theta13, delta13

#include "FH.h"
#define __SUBROUTINE__ "FHSetCKM"

	double precision sin12, sin23, sin13, d13
	double precision cos12, cos23, cos13
	double complex sin13c

	error = 0

	if( .not. flags_valid )
     &    Error("must call FHSetFlags before")
	if( .not. para_valid )
     &    Error("must call FHSetPara before")

	if( error .ne. 0 ) return

	entry SetCKM(theta12, theta23, theta13, delta13)

* put the PDG values as default
	if( theta12 .ge. 0 ) then
	  sin12 = sin(theta12)
	else
	  sin12 = .2243D0
	endif

	if( theta23 .ge. 0 ) then
	  sin23 = sin(theta23)
	else
	  sin23 = .0413D0
	endif

	if( theta13 .ge. 0 ) then
	  sin13 = sin(theta13)
	else
	  sin13 = .0037D0
	endif

	d13 = delta13
	if( d13 .lt. 0 ) d13 = 60

	cos12 = sqrt((1 - sin12)*(1 + sin12))
	cos23 = sqrt((1 - sin23)*(1 + sin23))
	cos13 = sqrt((1 - sin13)*(1 + sin13))
	sin13c = sin13*exp(cI*degree*d13)

	CKM(1,1) = cos12*cos13
	CKM(2,1) = -sin12*cos23 - cos12*sin23*sin13c
	CKM(3,1) = sin12*sin23 - cos12*cos23*sin13c
	CKM(1,2) = sin12*cos13
	CKM(2,2) = cos12*cos23 - sin12*sin23*sin13c
	CKM(3,2) = -cos12*sin23 - sin12*cos23*sin13c
	CKM(1,3) = DCONJG(sin13c)
	CKM(2,3) = sin23*cos13
	CKM(3,3) = cos23*cos13

	if( debuglevel .lt. 1 ) return

	DPARA "============================"	ENDL
	DPARA "parameters set by FHSetCKM"	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "CKM(1,1) =", CKM(1,1)		ENDL
	DPARA "CKM(1,2) =", CKM(1,2)		ENDL
	DPARA "CKM(1,3) =", CKM(1,3)		ENDL
	DPARA "CKM(2,1) =", CKM(2,1)		ENDL
	DPARA "CKM(2,2) =", CKM(2,2)		ENDL
	DPARA "CKM(2,3) =", CKM(2,3)		ENDL
	DPARA "CKM(3,1) =", CKM(3,1)		ENDL
	DPARA "CKM(3,2) =", CKM(3,2)		ENDL
	DPARA "CKM(3,3) =", CKM(3,3)		ENDL
	end

