* TLCorr.F
* computes the two-loop corrections
* this file is part of FeynHiggs
* last modified 7 Nov 06 th

#include "externals.h"
#include "debug.h"


	subroutine TLShifts(error,
     &    Alfas_, TB_, MA02_, MGl_, MUE_,
     &    Xf_, DSS2_, DSf_,
     &    runmt, MTrun_, MT_,
     &    runmb, MBy_, DeltaMBc_)
	implicit none
	integer error, runmt, runmb
	double precision Alfas_, TB_, MA02_
	double complex MGl_, MUE_, Xf_(2:4,3,3)
	double precision DSS2_(2,2:5,3), DSf_(2,4)
	double precision MTrun_, MT_, MBy_
	double complex DeltaMBc_(2)

#include "TL.h"

	double precision db(2), eb(2), dhbhb(2), dMSQ2(2), dvevvev

	Alfas = Alfas_
	TB = TB_
	MA02 = MA02_

	MGl = DBLE(MGl_)
	MGl2 = MGl**2

* note that {MUE,At,Ab} \equiv Re({MUEc,Atc,Abc}) (see TL.h)

	MUEc = MUE_
	if( abs(MUEc) .lt. 1D-7 ) MUEc = 1D-7
	MUE2 = MUE**2

	Xtc = Xf_(3,3,3)
	Atc = Xtc + MUEc/TB
	Xbc = Xf_(4,3,3)
	Abc = Xbc + MUEc*TB

	DSS2(1,3) = DSS2_(1,3,3)
	DSS2(2,3) = DSS2_(2,3,3)
	DSS2(1,4) = DSS2_(1,5,3)
	DSS2(2,4) = DSS2_(2,5,3)

	DSf(1,3) = DSf_(1,3)
	DSf(2,3) = DSf_(2,3)
	DSf(1,4) = DSf_(1,4)
	DSf(2,4) = DSf_(2,4)

	if( runmt .eq. 1 ) then
	  MTrun = MTrun_
	else
	  MTrun = MT_
	endif

	if( runmb .eq. 1 ) then
	  call TLNormalTB(error, MT_, MBrun)

* Note: this is 175, NOT MT (to match the 2.74 for MBrun)
	  Q = 175**2

* For computing the bottom shifts, we use MTrun only in ht.
* In all other places, we use MT proper.
	  ht = ht*MTrun/MT
	  ht2 = ht**2

	  call TLStrongShifts(db(1), dhbhb(1), dMSQ2(1))

	  eb(1) = CF/(4*pi)*Alfas*2*MUE*MGl*TB*
     &      (MSb22/(MSb22 - MGl2)*log(MSb22/MGl2) -
     &       MSb12/(MSb12 - MGl2)*log(MSb12/MGl2))/MSb2diff

	  eb(2) = 1/(16*pi**2)*ht2*At*MUE*TB*
     &      (MSt22/(MSt22 - MUE2)*log(MSt22/MUE2) -
     &       MSt12/(MSt12 - MUE2)*log(MSt12/MUE2))/MSt2diff

	  call TLLargeTB(error)

	  call TLYukawaShifts(db(2), dhbhb(2), dMSQ2(2), dvevvev)

	  DSS2(1,4) = DSS2(1,4) + dMSQ2(1) + dMSQ2(2)
	  DSS2_(1,4,3) = DSS2(1,4)

	  MBy_ = MBrun*((1 + db(1) + db(2))/
     &        ((1 + dvevvev)*abs(1 + eb(1) + eb(2))) -
     &      dhbhb(1) - dhbhb(2))
	  DeltaMBc_(1) = MBrun/MBy_ - 1
	  DeltaMBc_(2) = eb(1) + eb(2)
	  call BUpdate
	endif

	MBy = MBy_
	end


************************************************************************

	subroutine TLCorr(error, mudim, dPhi)
	implicit none
	integer error
	double precision mudim, dPhi(3)

#include "TL.h"

	double precision asat(3), asab(3), atat(3)
	double precision atab(3), atat_largeTB(3)
	integer i

	Q = mudim
	call TLNormalTB(error, MTrun, MBy)
	if( error .ne. 0 ) return

	call TLatat(atat)
	call TLasat(asat)

	call TLLargeTB(error)
	if( error .ne. 0 ) return

	call TLasab(asab)
	call TLatab(atab)

* the O(at^2) part at large TB will be subtracted:
	call TLatat(atat_largeTB)

	do i = 1, 3
	  dPhi(i) = atat_largeTB(i) - atab(i) -
     &      atat(i) - asat(i) - asab(i)
	enddo
	end


************************************************************************

	subroutine TLNormalTB(error, MT_, MB_)
	implicit none
	integer error
	double precision MT_, MB_

#include "TL.h"

	CB2 = 1/(1 + TB**2)
	SB2 = 1 - CB2
	C2B = CB2 - SB2
	CB = sqrt(CB2)
	SB = CB*TB

	MT = MT_
	MT2 = MT**2

	ht = sqrt2/vev*MT/SB
	ht2 = ht**2

	call TLMSf(error, StopPara, DSS2(1,3), MT, Xtc)

	Xt = -.5D0*MSt2diff*s2tt/MT
	Yt = At + MUE*TB

	MB = MB_
	MB2 = MB**2

	hb = sqrt2/vev*MB/CB
	hb2 = hb**2

	call TLMSf(error, SbotPara, DSS2(1,4), MB, Xbc)
	end


************************************************************************

	subroutine TLLargeTB(error)
	implicit none
	integer error

#include "TL.h"

	double precision TBlarge
	parameter (TBlarge = 1000)

	double precision x, DSS2large(2)

	CB2 = 1/(1 + TBlarge**2)
	SB2 = 1

* Note: for convenience, SB, CB, and C2B keep their old
* (normal TB) values -- they are not used in the TLa?a? terms.

	Xt = At
	Yt = At + MUE*TBlarge

	ht = sqrt2/vev*MT
	ht2 = ht**2

	x = 1 + 1/C2B
	DSS2large(1) = DSS2(1,3) - x*(DSf(1,3) - DSf(2,3))
	DSS2large(2) = DSS2(2,3) - x*DSf(2,3)

	call TLMSf(error, StopPara, DSS2large, MT, Atc)

	DSS2large(1) = DSS2(1,4) - x*(DSf(1,4) - DSf(2,4))
	DSS2large(2) = DSS2(2,4) - x*DSf(2,4)

	call TLMSf(error, SbotPara, DSS2large, MB, Abc - MUEc/CB)
c	call TLMSf(error, SbotPara, DSS2large, MB, -MUEc/CB)
	end


************************************************************************

	subroutine TLMSf(error, StopPara, M11, Mf, Xf)
	implicit none
	integer error
	double precision StopPara(*), M11(2), Mf
	double complex Xf

#define __SUBROUTINE__ "TLCorr"

	logical flip
	common /sfdiag/ flip

	double precision MSf(2)
	double complex USf(2,2)
	logical fail

	call EigenSf(fail, MSf, MSt12, USf, M11, Mf, Xf)
	if( fail ) Error("negative sfermion mass squares")

	if( flip ) then
	  ctt = DBLE(USf(1,1))
	  ctt2 = ctt**2
	  stt2 = 1 - ctt2
	  stt = sign(sqrt(stt2), DBLE(USf(1,2)))
	else
	  stt = DBLE(USf(1,2))
	  stt2 = stt**2
	  ctt2 = 1 - stt2
	  ctt = sign(sqrt(ctt2), DBLE(USf(1,1)))
	endif
	s2tt = 2*stt*ctt
	c2tt = ctt2 - stt2
	c2tt2 = c2tt**2
	s2tt2 = 1 - c2tt2
	end


************************************************************************

	subroutine TLStrongShifts(db, dhbhb, dMSQ2)
	implicit none
	double precision db, dhbhb, dMSQ2

#include "TL.h"

	double precision logMT2, logMGl2
	double precision logMSt12, logMSt22, logMSb12, logMSb22
	double precision b0MSt12, b0MSt22
	double precision stop11, stop12, stop22
	double precision sbot11, sbot12, sbot22, top

	logMT2 = log(MT2/Q)
	logMGl2 = log(MGl2/Q)
	logMSt12 = log(MSt12/Q)
	logMSt22 = log(MSt22/Q)
	logMSb12 = log(MSb12/Q)
	logMSb22 = log(MSb22/Q)
	b0MSt12 = myB0(MSt12, MT2, MGl2, Q)
	b0MSt22 = myB0(MSt22, MT2, MGl2, Q)

	db = CF/(4*pi)*Alfas*(3/2D0 - logMGl2 + .5D0*(
     &    MSb12/(MGl2 - MSb12)*(1 - (logMGl2 - logMSb12)*
     &      (1 + MGl2/(MGl2 - MSb12) + 4*MGl*Ab/MSb2diff)) +
     &    MSb22/(MGl2 - MSb22)*(1 - (logMGl2 - logMSb22)*
     &      (1 + MGl2/(MGl2 - MSb22) - 4*MGl*Ab/MSb2diff)) ))

	dhbhb = CF/(4*pi)*Alfas*(-4 - 2*logMGl2 -
     &    2*MSb12/MSb2diff*(2*logMSb12 -
     &      (1 - MGl2/MSb12)**2*log(abs(1 - MSb12/MGl2))) +
     &    2*MSb22/MSb2diff*(2*logMSb22 -
     &      (1 - MGl2/MSb22)**2*log(abs(1 - MSb22/MGl2))))

	top = MT*(3*logMT2 + MSDR) +
     &    .5D0/MT*(2*MGl2*(logMGl2 - 1) -
     &      MSt12*(logMSt12 - 1) - MSt22*(logMSt22 - 1) +
     &      (MGl2 + MT2 - MSt12 - 2*s2tt*MT*MGl)*
     &        myB0(MT2, MGl2, MSt12, Q) +
     &      (MGl2 + MT2 - MSt22 + 2*s2tt*MT*MGl)*
     &        myB0(MT2, MGl2, MSt22, Q))

	stop11 = MSt12*(3*logMSt12 - 7 - c2tt2*(logMSt12 - 1)) -
     &    s2tt2*MSt22*(logMSt22 - 1) +
     &    2*(MGl2*(logMGl2 - 1) + MT2*(logMT2 - 1) +
     &      (MSt12 - MGl2 - MT2 + 2*s2tt*MT*MGl)*b0MSt12)

	stop22 = MSt22*(3*logMSt22 - 7 - c2tt2*(logMSt22 - 1)) -
     &    s2tt2*MSt12*(logMSt12 - 1) +
     &    2*(MGl2*(logMGl2 - 1) + MT2*(logMT2 - 1) +
     &      (MSt22 - MGl2 - MT2 - 2*s2tt*MT*MGl)*b0MSt22)

	stop12 =
     &    c2tt*s2tt*(MSt22*(1 - logMSt22) - MSt12*(1 - logMSt12)) +
     &    2*MT*MGl*c2tt*(b0MSt12 + b0MSt22)

	sbot11 = MSb12*(3*logMSb12 - 3 - c2tb2*(logMSb12 - 1)) -
     &    s2tb2*MSb22*(logMSb22 - 1) - 6*MGl2 -
     &    2*(MSb12 - 2*MGl2)*logMGl2 -
     &    2*(MSb12 - MGl2)**2/MSb12*log(abs(1 - MSb12/MGl2))

	sbot22 = MSb22*(3*logMSb22 - 3 - c2tb2*(logMSb22 - 1)) -
     &    s2tb2*MSb12*(logMSb12 - 1) - 6*MGl2 -
     &    2*(MSb22 - 2*MGl2)*logMGl2 -
     &    2*(MSb22 - MGl2)**2/MSb22*log(abs(1 - MSb22/MGl2))

	sbot12 =
     &    c2tb*s2tb*(MSb22*(1 - logMSb22) - MSb12*(1 - logMSb12))

	dMSQ2 = CF/(4*pi)*Alfas*(
     &    ctt2*stop11 + stt2*stop22 - 2*stt*ctt*stop12 -
     &    ctb2*sbot11 - stb2*sbot22 + 2*stb*ctb*sbot12 -
     &    2*MT*top )
	end


************************************************************************

	subroutine TLYukawaShifts(db, dhbhb, dMSQ2, dvevvev)
	implicit none
	double precision db, dhbhb, dMSQ2, dvevvev

#include "TL.h"

	double precision logMT2, logMA02, top, delta, t1, t2
	double precision stop11, stop12, stop22
	double precision sbot11, sbot12, sbot22

	double precision Pi11t, Pi22t, Pi12t
	double precision Pi11b, Pi22b, Pi12b, Pitt
	external Pi11t, Pi22t, Pi12t, Pi11b, Pi22b, Pi12b, Pitt

	double precision x, y, fW
	fW(x, y) = .25D0*(x + y) - .5D0*x*y/(x - y)*log(x/y)

	logMT2 = log(MT2/Q)
	logMA02 = log(MA02/Q)

	top = Pitt(MT2)

	stop11 = Pi11t(MSt12)
	stop22 = Pi22t(MSt22)
	stop12 = .5D0*(Pi12t(MSt12) + Pi12t(MSt22))

	sbot11 = Pi11b(MSb12)
	sbot22 = Pi22b(MSb22)
	sbot12 = .5D0*(Pi12b(MSb12) + Pi12b(MSb22))

	delta = MT2 - MA02
	if( abs(delta) .lt. 1D-13 ) then
	  t1 = -logMT2
	  t2 = -.5D0
	else
	  t1 = 1 - (MT2*logMT2 - MA02*logMA02)/delta
	  t2 = MT2*(delta - MT2*(logMT2 - logMA02))/delta**2
	endif

	db = 1/(32*pi**2)*(
     &    ht2*(6/4D0*logMT2 - 5/4D0 + 2*t1 +
     &         2*MUE2/MSt2diff*(myB0(0D0, MUE2, MSt12, Q) -
     &                          myB0(0D0, MUE2, MSt22, Q))) +
     &    hb2*(.5D0*t2 + 3/4D0 - 3/2D0*logMA02 +
     &         myB1(0D0, MUE2, MSb12, Q) +
     &         myB1(0D0, MUE2, MSb22, Q)) +
     &    (hb2*ctt2 + ht2*stt2)*myB1(0D0, MUE2, MSt12, Q) +
     &    (hb2*stt2 + ht2*ctt2)*myB1(0D0, MUE2, MSt22, Q) )

	dvevvev = Nc/(16*pi**2)*ht2*(.5D0*logMT2 - .25D0 -
     &    (ctt2*ctb2*fW(MSt12, MSb12) + 
     &     ctt2*stb2*fW(MSt12, MSb22) +
     &     stt2*ctb2*fW(MSt22, MSb12) +
     &     stt2*stb2*fW(MSt22, MSb22))/MT2)

	dhbhb = (sbot22 - sbot11 - 2*c2tb/s2tb*sbot12)/
     &      (16*pi**2*MSb2diff) -
     &    dvevvev

	dMSQ2 = 1/(16*pi**2)*(
     &    ctt2*stop11 + stt2*stop22 - 2*stt*ctt*stop12 -
     &    ctb2*sbot11 - stb2*sbot22 + 2*stb*ctb*sbot12 -
     &    2*MT*top )
	end

