* TLMA.F
* Two-loop O(at^2 + at ab + ab^2) corrections to the CP-odd Higgs mass
* and to the minimization conditions of the effective potential.
* Written by P. Slavich (e-mail: pietro.slavich@durham.ac.uk).
* Based on A. Dedes, G. Degrassi and P. Slavich,
* Nucl. Phys. B672 (2003) 144.
* this file is part of FeynHiggs
* last modified 10 May 05 th

* Revision history:
* 01/04/2005: correction to the CP-odd Higgs mass extracted
*             from the full code for use in FeynHiggs
* 25/09/2003: numerical value of Pi corrected (10th digit)
* 17/09/2003: error corrected in F1q, F6q and FAq;
*             optimization: calls to makederiv reduced to 2
* 20/07/2003: routines optimized, bug for mu < 0 corrected

* Notice: we assume that the 1-loop part is computed in terms of
*         running (DRbar) parameters, evaluated at the scale Q.
*         The parameters in the bottom/sbottom sector should be
*         computed in term of the "resummed" bottom Yukawa coupling.

	subroutine TLMA(dMA0)
	implicit none

#include "TL.h"

	double precision x34, x36, x20, x38, x40, x42, x22, x24, x12
	double precision x26, x28, x14, x16, x44, x18, x30, x31, x19
	double precision x32, x1, x3, x8, x10, x46, x11, x4, x6, x7
	double precision x33, x9, x5, x35, x37, x21, x39, x41, x43
	double precision x23, x25, x13, x27, x29, x45, x15, x17, x47
	double precision Ft, Fb, FAp, FA

	double precision tmp1, tmp2, tmp3, tmp4, tmp5

	x34 = InvTri(MA02,MSb12,MSt12)

	x36 = InvTri(MA02,MSb12,MSt22)

	x20 = InvTri(MA02,MSb22,MSb22)

	x38 = InvTri(MA02,MSb22,MSt12)

	x40 = InvTri(MA02,MSb22,MSt22)

	x42 = InvTri(MA02,MSt12,MSt12)

	x22 = InvTri(MA02,MSt22,MSb12)

	x24 = InvTri(MA02,MSt22,MSb22)

	x12 = InvTri(MA02,MSt22,MSt22)

	x26 = InvTri(MB2,MSb12,MUE2)

	x28 = InvTri(MB2,MSb22,MUE2)

	x14 = InvTri(MB2,MSt12,MUE2)

	x16 = InvTri(MB2,MSt22,MUE2)

	x44 = InvTri(MSb22,MT2,MUE2)

	x18 = Li2(MSb2diff/MSb22)

	x30 = Li2(1 - MSt12/MSb12)

	x31 = Li2(1 - MSt12/MSb22)

	x19 = Li2(1 - MSb22/MSt22)

	x32 = Li2(1 - MSt22/MSb12)

	x1 = Li2(1 - MSt22/MSb22)

	x3 = Li2(MSt2diff/MSt22)

	x8 = Log(MA02)

	x10 = Log(MB2)

	x46 = Log(MGl2)

	x11 = Log(MSb12)

	x4 = Log(MSb22)

	x6 = Log(MSt12)

	x7 = Log(MSt22)

	x33 = Log(MT2)

	x9 = Log(MUE2)

	x5 = Log(Q)

	x35 = Phi(MA02,MSb12,MSt12)

	x37 = Phi(MA02,MSb12,MSt22)

	x21 = Phi(MA02,MSb22,MSb22)

	x39 = Phi(MA02,MSb22,MSt12)

	x41 = Phi(MA02,MSb22,MSt22)

	x43 = Phi(MA02,MSt12,MSt12)

	x23 = Phi(MA02,MSt22,MSb12)

	x25 = Phi(MA02,MSt22,MSb22)

	x13 = Phi(MA02,MSt22,MSt22)

	x27 = Phi(MSb12,MB2,MUE2)

	x29 = Phi(MSb22,MB2,MUE2)

	x45 = Phi(MSb22,MT2,MUE2)

	x15 = Phi(MSt12,MB2,MUE2)

	x17 = Phi(MSt22,MB2,MUE2)

	x47 = Phi(MT2,MSt22,MGl2)

        tmp1 = 1/2.D0*(c2tt2*
     -       (ht2*s2tt*(4*MSt12**2*(1 + Nc) + 
     -            2*CB2*(MA02 - MSt2sum)*(x5 - x7)*Yt**2 - 
     -            MSt2diff*(SB2*(1 + x5 - x7)*Xt**2 + CB2*Yt**2) - 
     -            4*MSt12*(MSt22*(1 + Nc)*(1 + x5 - x7) + 
     -               SB2*(2 + x5 - x7)*Xt**2 + 2*CB2*Yt**2)) + 
     -         4*MB*(hb*ht*MUE*(MB2 - MSt12 - MUE2)*(x5 - x9))) + 
     -      c2tt*s2tt*(2*CB2*ht2*MA02*MSt12 - 
     -         hb2*(2*ctb2*MSb22*MSt12 + 4*MSt12**2 - 
     -            6*MSt12*MUE2 + 2*MA02*MSt12*SB2 + 
     -            2*MSb12*MSt12*stb2 + MSt12**2*x5 - 
     -            2*MSt12*MUE2*x5 - MUE2**2*x5 - MSt12**2*x9 + 
     -            2*MSt12*MUE2*x9 + MUE2**2*x9 + 
     -            MB2**2*(x5 - x9) - 2*MB2*(MSt12*(3 + x5 - x9)))+
     -           ht2*(2*ctb2*MSb12*MSt12 + 4*MSt12**2 - 
     -            6*MSt12*MUE2 + 2*MSb22*MSt12*stb2 + 
     -            MSt12**2*x5 - 2*MSt12*MUE2*x5 - MUE2**2*x5 - 
     -            MSt12**2*x9 + 2*MSt12*MUE2*x9 + MUE2**2*x9 + 
     -            MB2**2*(x5 - x9) - 2*MB2*(MSt12*(3 + x5 - x9))))-
     -        MSt2diff*(4*CB2*ht2*MT2*s2tt + 4*ht2*MT2*s2tt*SB2 + 
     -         6*ht2*MSt12*s2tt*stt2 - 4*ht2*MUE2*s2tt*stt2 + 
     -         2*ctb2*hb2*MT2*s2tt*SB2*stt2 + 
     -         2*hb2*MT2*s2tt*SB2*stb2*stt2 + 
     -         2*ht2*MSt12*s2tt*stt2*x5 - 
     -         2*ht2*MUE2*s2tt*stt2*x5 - 
     -         2*ht2*MSt12*s2tt*stt2*x9 + 
     -         2*ht2*MUE2*s2tt*stt2*x9 + 4*ht2*MT*s2tt2*SB2*Xt + 
     -         ht2*s2tt*SB2*Xt**2 + ht2*s2tt*s2tt2*SB2*Xt**2 + 
     -         ht2*s2tt*SB2*x5*Xt**2 - ht2*s2tt*SB2*x7*Xt**2 + 
     -         2*ctb2*hb2*MT*s2tt2*SB2*Yb + 
     -         2*hb2*MT*s2tt2*SB2*stb2*Yb + 
     -         2*ctt2*s2tt*
     -          (CB2*ht2*MT2 + 
     -            hb2*(MA02*SB2 + MSb12*stb2 + 
     -               MSt12*(3 + x5 - x9) + MUE2*(-2 - x5 + x9) + 
     -               ctb2*SB2*Yb**2 + SB2*stb2*Yb**2)) + 
     -         4*CB2*ht2*MT*s2tt2*Yt + 
     -         2*CB2*ctb2*ht2*MT*s2tt2*Yt + 
     -         2*CB2*ht2*MT*s2tt2*stb2*Yt + CB2*ht2*s2tt*Yt**2 + 
     -         CB2*ht2*s2tt*s2tt2*Yt**2 + 
     -         2*CB2*ctb2*ht2*s2tt*stt2*Yt**2 + 
     -         2*CB2*ht2*s2tt*stb2*stt2*Yt**2 - 
     -         4*MB*(hb*ht*MUE*s2tt2) + 
     -         4*MB*(CB*ctb2*hb*ht*MT*s2tt*SB*stt2) + 
     -         4*MB*(CB*hb*ht*MT*s2tt*SB*stb2*stt2) - 
     -         2*MB*(hb*ht*MUE*s2tt2*x5) + 
     -         2*MB*(hb*ht*MUE*s2tt2*x9) + 
     -         2*MB2*(s2tt*
     -            (1 + ht2*stt2*(-2 + CB2 - x5 + x9) + 
     -              ctt2*hb2*(-2 + ctb2*SB2 + SB2*stb2 - x5 + x9)))
     -           + 2*MB*(CB*ctb2*hb*ht*s2tt2*SB*Yb) + 
     -         2*MB*(CB*hb*ht*s2tt2*SB*stb2*Yb) + 
     -         2*MB*(CB*ctb2*hb*ht*s2tt2*SB*Yt) + 
     -         2*MB*(CB*hb*ht*s2tt2*SB*stb2*Yt)))/(MSt2diff*s2tt)

        tmp1 = tmp1 + (2*MSt12*
     -      (c2tt*s2tt*(SB2*
     -            (ht2*(MB2 - MT2 + Xt**2) - 
     -              hb2*(MB2 - MT2 + Yb**2)) + 
     -           CB2*(-(hb2*(MB2 - MT2 + Xb**2)) + 
     -              ht2*(MB2 - MT2 + Yt**2))) + 
     -        2*c2tt2*(SB2*(ht2*Xt*(3*MT + s2tt*Xt) + hb2*MT*Yb) + 
     -           CB2*(hb2*MT*stb2*Xb + ctb2*MT*(hb2*Xb + ht2*Yt) + 
     -              ht2*Yt*(MT*(2 + stb2) + s2tt*Yt)) + 
     -           MB*(hb*ht*(-2*MUE + CB*SB*(-Xb - Xt + Yb + Yt)))))
     -      )/(MSt2diff*s2tt)

        tmp2 = c2tt*(MB2 + hb2*MA02*SB2) + 
     -   (2*(2*MB2 + c2tt*MA02*
     -         (-MB2 + CB2*ht2*
     -            (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -              s2tt*(ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                 MB*(s2tb*Yt))) - 
     -           CB*hb*ht*SB*
     -            (MT*s2tt*
     -               (s2tb*(Yb - Yt) + 2*MB*ctb2 - 2*MB*stb2) - 
     -              c2tt*(s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                 2*MB*(stb2*Yb + ctb2*Yt))))))/
     -    (MSt2diff*s2tt) + 
     -   1/2.D0*(MB2 + (x4 - x5)*
     -       (MB2 + (c2tt*(MB2 + 
     -              (-MB2 - MSb22 + MSt12)*
     -               (-MB2 - 
     -                 CB2*ht2*
     -                  (c2tt*MT*(-2*stb2*Yt + MB*s2tb) + 
     -                    s2tt*
     -                     (MT2*stb2 - stb2*Yt**2 - MB2*ctb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                 CB*hb*ht*SB*
     -                  (MT*s2tt*
     -                     (s2tb*(-Yb + Yt) - 2*MB*ctb2 + 
     -                       2*MB*stb2) + 
     -                    c2tt*
     -                     (s2tb*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(ctb2*Yb + stb2*Yt))))))/
     -          (MSt2diff*s2tt))) + 
     -   (-x5 + x7)*(-(hb2*MA02*SB2*stt2) + 
     -      (MB2 + 1/2.D0*(c2tt*
     -             (MB2 + (-MB2 + MSb12 - MSt22)*
     -                (-MB2 - 
     -                  CB2*ht2*
     -                   (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(-Yb + Yt) - 2*MB*ctb2 + 
     -                       2*MB*stb2) + 
     -                     c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stb2*Yb + ctb2*Yt))))))/s2tt)/
     -       MSt2diff + 1/4.D0*
     -       (-(2*MB2*(1 + CB2*ctt2*ht2 + ctb2*hb2*SB2*stt2)) + 
     -         hb2*SB2*(-2*stb2*
     -             (ctt2*MT2 + Yb*(-(MT*s2tt) + stt2*Yb)) + 
     -            MB*(s2tb*(MT*s2tt - 2*stt2*Yb))) - 
     -         2*(CB2*ht2*(MT2*stb2*stt2 + 
     -               Yt*(-(MT*s2tt*stb2) + Yt + c2tt2*Yt + 
     -                  ctt2*stb2*Yt) + 
     -               ctb2*(MT2*stt2 + Yt*(-(MT*s2tt) + ctt2*Yt)))+
     -              MB*(CB*hb*ht*SB*(2*MT*stt2 - s2tt*(Yb + Yt))))
     -         ))

        tmp2 = tmp2 + (-x5 + x6)*
     -    (MA02*(MB2 - (c2tt2*CB2*ht2*Yt*(2*MT + s2tt*Yt))/
     -          (MSt2diff*s2tt)) + 
     -      (MB2 + 1/2.D0*(c2tt*
     -             (MB2 + (-MB2 + MSb12 - MSt12)*
     -                (-MB2 + 
     -                  CB2*ht2*
     -                   (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(Yb - Yt) + 2*MB*ctb2 - 
     -                       2*MB*stb2) - 
     -                     c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stb2*Yb + ctb2*Yt))))))/s2tt)/
     -       MSt2diff + 1/2.D0*
     -       (hb2*(MB2 + SB2*
     -             (MT2*stb2*stt2 + 
     -               Yb*(MT*s2tt*stb2 + ctt2*(MB2 + stb2*Yb)) + 
     -               MB2*(ctb2*ctt2) + MB/2.D0*(MT*s2tb*s2tt))) + 
     -         CB2*(2*MB2 + 
     -            ht2*(MB2 + stb2*Yt*(MT*s2tt + stt2*Yt) - 
     -               MB2*(ctb2*ctt2) - 
     -               MB/2.D0*(s2tb*(MT*s2tt + 2*stt2*Yt)))) + 
     -         CB*(MB2 + hb*ht*SB*
     -             (MB*(MB2 + s2tt*stb2*Yb + ctb2*s2tt*Yt) + 
     -               s2tb*(MB2 + 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt)))))
     -         ))

        Ft = c2tt*(2*MB2 - hb2*MSb12*stb2) + tmp1*(-x5 + x6) + 
     -   tmp2*(-x5 + x8) + 
     -   (c2tt*(MB2 + (-hb2 + ht2)*MUE2) - 
     -      2*(MB2 + ctt2*hb2*MUE2 + ht2*MUE2*stt2))*(-x5 + x9) - 
     -   ht2*s2tt2*SB2*x3*Xt**2 + 
     -   s2tt*(MB2 + 2*ht2*MT*(SB2*Xt + CB2*Yt)) + 
     -   (x10 - x5)*(MB2*(c2tt*(1 - hb2 + ht2)) + 
     -      (-x5 + x6)*((MB2 + MSt12 - MUE2)*
     -          (ctt2*hb2 + ht2*stt2) + 
     -         c2tt*(MB2 - 
     -            1/2.D0*((hb2 - ht2)*(MB2 + MSt12 - MUE2)*
     -                (MB2 - MSt12 + MUE2))/MSt2diff) + 
     -         hb*(MB2 - MB*(ht*MUE*s2tt))) + 
     -      (-x5 + x9)*(-((MB2 - MSt22 + MUE2)*(MB2 + hb2*stt2)) + 
     -         (MB2 - MSt12 + MUE2)*(ctt2*hb2 + ht2*stt2) - 
     -         1/2.D0*(c2tt*(hb2 - ht2)*(MB2 - MSt22 + 2*MUE2)) + 
     -         2*(MB2 + MB*(hb*ht*MUE*s2tt))) + 
     -      (-x5 + x7)*((-MSt22 + MUE2)*(ctt2*ht2 + hb2*stt2) + 
     -         c2tt*(MB2 + 
     -            1/2.D0*((hb2 - ht2)*(MB2 + MSt22 - MUE2)*
     -                (MB2 - MSt22 + MUE2))/MSt2diff) + 
     -         hb*(MB2 - MB*(ht*MUE*s2tt)) + MB2*(MB2 + hb2*stt2))-
     -        2*MB2*(1 + ctt2*hb2 + ht2*stt2)) + 
     -   (MB2 + (-5*c2tt2*(ht2*MSt12*SB2*Xt*(2*MT + s2tt*Xt) + 
     -            2*MB*(hb*ht*MSt2diff*MUE)) + 
     -         c2tt*(MB2 + 
     -            (-MSb22 + MSt22)*x1*
     -             (CB2*hb2*
     -                (c2tt*MT*(-2*ctb2*Xb + MB*s2tb) + 
     -                  s2tt*
     -                   (ctb2*(-MT2 + Xb**2) + MB2*stb2 - 
     -                     MB*(s2tb*Xb))) + 
     -               ht2*SB2*
     -                (c2tt*MT*(-2*stb2*Xt + MB*s2tb) + 
     -                  s2tt*
     -                   (-MB2 + MT2*stb2 - stb2*Xt**2 + 
     -                     MB*(s2tb*Xt))) + 
     -               CB*hb*ht*SB*
     -                (MT*s2tt*
     -                   (s2tb*(MB2 - Xt) + MB*(MB2 - 2*stb2)) - 
     -                  c2tt*
     -                   (s2tb*(MB2 + MT2 + Xb*Xt) - 
     -                     2*MB*(ctb2*Xb + stb2*Xt))))))/s2tt)/
     -    MSt2diff + (x17*(MB2 + 
     -        1/2.D0*(s2tt*(c2tt*(hb2 - ht2)*(MB2 - MSt22 + MUE2) - 
     -               MSt2diff*(ctt2*ht2 + hb2*stt2)) - 
     -            4*MB*(c2tt2*hb*ht*MUE))/(MSt2diff*s2tt*x16)))/
     -    MUE2 + 1/2.D0*(MB2 + 
     -      x15*(MB2 + (s2tt*
     -             (-(c2tt*(hb2 - ht2)*(MB2 - MSt12 + MUE2)) + 
     -               MSt2diff*(ctt2*hb2 + ht2*stt2)) + 
     -            4*MB*(c2tt2*hb*ht*MUE))/(MSt2diff*MUE2*s2tt*x14))
     -      )

        Ft = Ft + CB2*(MB2*(2 + x13) + 
     -      1/8.D0*(ht2*x13*(-4*c2tt2*MSt22*Yt*(-2*MT + s2tt*Yt) + 
     -            MSt2diff*
     -             (4*MT2*s2tt + s2tt2*Yt*(-4*MT + s2tt*Yt))))/
     -        (MSt22**2*MSt2diff*s2tt*x12)) + 
     -   (x5 - x6)**2*(c2tt2*
     -       (MB2 - (ht2*MSt12*SB2*Xt*(2*MT + s2tt*Xt))/
     -          (MSt2diff*s2tt)) + 
     -      1/4.D0*(ht2*(SB2*(4*MT2 + Xt*(4*MT*s2tt + s2tt2*Xt)) + 
     -           CB2*(4*MT2 + Yt*(4*MT*s2tt + s2tt2*Yt)))) + 
     -      1/2.D0*(c2tt2*CB2*ht2*(MA02 - 2*MSt12)*Yt*
     -           (2*MT + s2tt*Yt) + 
     -          c2tt*(2*MB2 + 
     -             (-MSb12 + MSt12)*
     -              (-(CB2*hb2*
     -                   (-(c2tt*MT*(2*stb2*Xb + MB*s2tb)) + 
     -                     s2tt*
     -                      (stb2*(MB2 + Xb**2) + MB2*ctb2 + 
     -                       MB*(s2tb*Xb)))) + 
     -                ht2*SB2*
     -                 (c2tt*MT*(2*ctb2*Xt + MB*s2tb) + 
     -                   s2tt*
     -                    (ctb2*(-MT2 + Xt**2) + MB2*stb2 + 
     -                      MB*(s2tb*Xt))) + 
     -                CB*hb*ht*SB*
     -                 (MT*s2tt*
     -                    (s2tb*(MB2 - Xt) + MB*(MB2 - 2*stb2)) - 
     -                   c2tt*
     -                    (s2tb*(MB2 + MT2 + Xb*Xt) + 
     -                      2*MB*(stb2*Xb + ctb2*Xt))))))/
     -        (MSt2diff*s2tt)) + 
     -   1/4.D0*((x5 - x7)**2*
     -      (ht2*(3*MB2 - CB2*
     -            (4*MT2 + Yt*(-4*MT*s2tt + s2tt2*Yt))) + 
     -        (2*c2tt*(2*MB2 + 
     -             (-MSb12 + MSt22)*
     -              (-MB2 - 
     -                ht2*SB2*
     -                 (c2tt*MT*(2*ctb2*Xt + MB*s2tb) + 
     -                   s2tt*
     -                    (ctb2*(-MT2 + Xt**2) + MB2*stb2 + 
     -                      MB*(s2tb*Xt))) + 
     -                CB*hb*ht*SB*
     -                 (MT*s2tt*
     -                    (s2tb*(-Xb + Xt) - 2*MB*ctb2 + 2*MB*stb2)
     -                     + c2tt*
     -                    (s2tb*(MB2 + MT2 + Xb*Xt) + 
     -                      2*MB*(stb2*Xb + ctb2*Xt))))))/
     -         (MSt2diff*s2tt)))

        Ft = Ft + (-x5 + x7)*
     -    (-((2*MB2 - MSt22 + MUE2)*(MB2 + hb2*stt2)) + 
     -      (-x5 + x9)*((MB2 - MSt22)*(ctt2*ht2 + hb2*stt2) + 
     -         MUE2*(MB2 + hb2*stt2) + 
     -         c2tt*(MB2 + 
     -            1/2.D0*((hb2 - ht2)*MUE2*(MB2 - MSt22 + MUE2))/
     -              MSt2diff) + hb*(MB2 - MB*(ht*MUE*s2tt))) + 
     -      hb2*(2*MB2 + SB2*
     -          (ctb2*(MB2 + Yb*(-(MT*s2tt) + stt2*Yb)) + 
     -            MB2*(stb2*stt2) + 
     -            MB/2.D0*(s2tb*(MT*s2tt - 2*stt2*Yb)))) + 
     -      CB2*(MB2 + ht2*
     -          (stb2*(MB2 - MT*s2tt*Yt + ctt2*Yt**2) + 
     -            MB2*(ctb2*ctt2) + 
     -            MB/2.D0*(s2tb*(MT*s2tt - 2*ctt2*Yt)))) + 
     -      (MB2 + (2*MSt22*
     -            (MB2 + c2tt*
     -               (hb2*SB2*
     -                  (c2tt*MT*(-2*ctb2*Yb + MB*s2tb) + 
     -                    s2tt*
     -                     (ctb2*(-MT2 + Yb**2) + MB2*stb2 - 
     -                       MB*(s2tb*Yb))) + 
     -                 CB2*ht2*
     -                  (c2tt*MT*(-2*stb2*Yt + MB*s2tb) + 
     -                    s2tt*
     -                     (-MB2 + MT2*stb2 - stb2*Yt**2 + 
     -                       MB*(s2tb*Yt))) - 
     -                 CB*hb*ht*SB*
     -                  (MT*s2tt*
     -                     (s2tb*(MB2 - Yt) + MB*(MB2 - 2*stb2)) - 
     -                    c2tt*
     -                     (s2tb*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(ctb2*Yb + stb2*Yt))))))/s2tt)/
     -       MSt2diff + hb*
     -       (MB2 + CB*ht*SB*
     -          (s2tb*(MB2 - 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt))) + 
     -            MB*(MB2 - s2tt*(stb2*Yb + ctb2*Yt)))) + 
     -      1/2.D0*(ht2*(MB2 + 
     -            CB2*(4*MT2 + Yt*(-4*MT*s2tt + s2tt2*Yt))) + 
     -         CB*hb*ht*SB*
     -          (s2tb*(MB2 + s2tt*Yb*Yt) + 
     -            MT*(MB2 - 2*s2tb*(stt2*Yb + ctt2*Yt)) - 
     -            2*MB*(s2tt*(ctb2*Yb + stb2*Yt)))))

        Ft = Ft + (x4 - x5)*
     -    (c2tt*(MB2 - ht2*MSb22*stb2) + 
     -      (2*(2*MB2 + c2tt*MSb22*
     -            (hb2*SB2*
     -               (c2tt*MT*(-2*ctb2*Yb + MB*s2tb) + 
     -                 s2tt*
     -                  (ctb2*(-MT2 + Yb**2) + MB2*stb2 - 
     -                    MB*(s2tb*Yb))) + 
     -              CB2*ht2*
     -               (c2tt*MT*(-2*stb2*Yt + MB*s2tb) + 
     -                 s2tt*
     -                  (-MB2 + MT2*stb2 - stb2*Yt**2 + 
     -                    MB*(s2tb*Yt))) - 
     -              CB*hb*ht*SB*
     -               (MT*s2tt*
     -                  (s2tb*(MB2 - Yt) + MB*(MB2 - 2*stb2)) - 
     -                 c2tt*
     -                  (s2tb*(MB2 + MT2 + Yb*Yt) - 
     -                    2*MB*(ctb2*Yb + stb2*Yt))))))/
     -       (MSt2diff*s2tt) + 
     -      (-x5 + x6)*(MSb22*
     -          (MB2 + (c2tt*MSt12*(ctb2*hb2 - ht2*stb2))/MSt2diff)
     -           + (MB2 + 1/2.D0*
     -             (c2tt*(MA02 - MB2 - MSt12)*
     -                (-MB2 - 
     -                  CB2*ht2*
     -                   (c2tt*MT*(-2*stb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (MT2*stb2 - stb2*Yt**2 - MB2*ctb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(-Yb + Yt) - 2*MB*ctb2 + 
     -                       2*MB*stb2) + 
     -                     c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(ctb2*Yb + stb2*Yt)))))/s2tt)/
     -          MSt2diff + 
     -         1/2.D0*(hb2*SB2*
     -             (ctb2*(MB2 + Yb*(MT*s2tt + ctt2*Yb)) + 
     -               MB2*(ctt2*stb2) - 
     -               MB/2.D0*(s2tb*(MT*s2tt + 2*ctt2*Yb))) + 
     -            CB2*ht2*(MB2 + stb2*Yt*(MT*s2tt + stt2*Yt) - 
     -               MB2*(ctb2*ctt2) - 
     -               MB/2.D0*(s2tb*(MT*s2tt + 2*stt2*Yt))) + 
     -            CB*hb*ht*SB*
     -             (MB*(MB2 + ctb2*s2tt*Yb + s2tt*stb2*Yt) + 
     -               s2tb*(MB2 - 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt)))))
     -         ) + (-x5 + x7)*
     -       (MSb22*(MB2 - ctb2*hb2*stt2) + 
     -         (MB2 + 1/2.D0*
     -             (c2tt*(MA02 - MB2 - MSt22)*
     -                (hb2*SB2*
     -                   (c2tt*MT*(-2*ctb2*Yb + MB*s2tb) + 
     -                     s2tt*
     -                      (ctb2*(-MT2 + Yb**2) + MB2*stb2 - 
     -                       MB*(s2tb*Yb))) + 
     -                  CB2*ht2*
     -                   (c2tt*MT*(-2*stb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (-MB2 + MT2*stb2 - stb2*Yt**2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(MB2 - Yt) + MB*(MB2 - 2*stb2))-
     -                       c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(ctb2*Yb + stb2*Yt)))))/s2tt)/
     -          MSt2diff + 
     -         1/2.D0*(-MB2 - 
     -            CB2*ht2*(MT2*stb2*stt2 + 
     -               stb2*Yt*(-(MT*s2tt) + ctt2*Yt) + 
     -               MB2*(ctb2*ctt2) + 
     -               MB/2.D0*(s2tb*(MT*s2tt - 2*ctt2*Yt))) - 
     -            1/2.D0*(CB*hb*ht*SB*
     -               ((MB2 + MT2)*s2tb*s2tt + s2tb*s2tt*Yb*Yt - 
     -                 2*MT*s2tb*(stt2*Yb + ctt2*Yt) + 
     -                 4*MB*(MT*stb2*stt2) - 
     -                 2*MB*(s2tt*(ctb2*Yb + stb2*Yt)))))))

        Ft = Ft + (x11 - x5)*
     -    (c2tt*(MB2 + hb2*MSb12*stb2) + 
     -      (2*(2*MB2 + c2tt*MSb12*
     -            (-MB2 - CB2*ht2*
     -               (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -                 s2tt*
     -                  (ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                    MB*(s2tb*Yt))) - 
     -              CB*hb*ht*SB*
     -               (MT*s2tt*
     -                  (s2tb*(-Yb + Yt) - 2*MB*ctb2 + 2*MB*stb2)+
     -                   c2tt*
     -                  (s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                    2*MB*(stb2*Yb + ctb2*Yt))))))/
     -       (MSt2diff*s2tt) + 
     -      (-x5 + x6)*(MSb12*
     -          (MB2 + (c2tt*MSt12*(-(ctb2*ht2) + hb2*stb2))/
     -             MSt2diff) + 
     -         (MB2 + 1/2.D0*
     -             (c2tt*(MA02 - MB2 - MSt12)*
     -                (-MB2 + 
     -                  CB2*ht2*
     -                   (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(Yb - Yt) + 2*MB*ctb2 - 
     -                       2*MB*stb2) - 
     -                     c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stb2*Yb + ctb2*Yt)))))/s2tt)/
     -          MSt2diff + 
     -         1/2.D0*(hb2*SB2*
     -             (MT2*stb2*stt2 + 
     -               Yb*(MT*s2tt*stb2 + ctt2*(MB2 + stb2*Yb)) + 
     -               MB2*(ctb2*ctt2) + MB/2.D0*(MT*s2tb*s2tt)) + 
     -            CB2*ht2*(ctb2*
     -                (ctt2*MT2 + MT*s2tt*Yt + stt2*Yt**2) + 
     -               MB2*(stb2*stt2) + MB*(MB2 + s2tb*stt2*Yt)) + 
     -            CB*hb*ht*SB*
     -             (MB*(MB2 + s2tt*stb2*Yb + ctb2*s2tt*Yt) + 
     -               s2tb*(MB2 + 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt)))))
     -         ) + (-x5 + x7)*
     -       (MSb12*(MB2 - hb2*stb2*stt2) + 
     -         (MB2 + 1/2.D0*
     -             (c2tt*(MA02 - MB2 - MSt22)*
     -                (-MB2 - 
     -                  CB2*ht2*
     -                   (c2tt*MT*(2*ctb2*Yt + MB*s2tb) + 
     -                     s2tt*
     -                      (ctb2*(-MT2 + Yt**2) + MB2*stb2 + 
     -                       MB*(s2tb*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tt*
     -                      (s2tb*(-Yb + Yt) - 2*MB*ctb2 + 
     -                       2*MB*stb2) + 
     -                     c2tt*
     -                      (s2tb*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stb2*Yb + ctb2*Yt)))))/s2tt)/
     -          MSt2diff + 
     -         1/2.D0*(-MB2 - 
     -            CB2*ht2*(ctb2*MT2*stt2 - ctb2*MT*s2tt*Yt + 
     -               ctb2*ctt2*Yt**2 - MB/2.D0*(MT*s2tb*s2tt) + 
     -               MB2*(ctt2*stb2) + MB*(ctt2*s2tb*Yt)) - 
     -            CB*hb*ht*SB*
     -             (MT*s2tb*(stt2*Yb + ctt2*Yt) + 
     -               2*MB*(ctb2*MT*stt2) - 
     -               MB*(s2tt*(stb2*Yb + ctb2*Yt)) - 
     -               1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt))))))

        tmp3 = 1/2.D0*(-2*MB2 - 2*ctb2*hb2*MA02*SB2 - 
     -     4*MSb12*(ctb2*hb2 + ht2*stb2) + 
     -     2*MUE2*(ctb2*hb2 + ht2*stb2) + 
     -     2*(MB2 - MSb12 + MUE2)*(ctb2*hb2 + ht2*stb2) - 
     -     2*ctb2*hb2*MSt12*stt2 + 
     -     (2*c2tt2*MSb12*(CB2*ht2*MA02 + 
     -          hb2*(3*MB2 - 2*MSb12 - ctt2*MSt22 + 3*MUE2 - 
     -             MA02*SB2 - MSt12*stt2) + 
     -          ht2*(-3*MB2 + 2*MSb12 + ctt2*MSt12 - 3*MUE2 + 
     -             MSt22*stt2)))/(c2tb*MSb2diff) - 
     -     2*(1 + c2tt2)*ht2*SB2*Xb**2 + 
     -     ((1 + c2tt2)*ht2*(MSb22 - MSb2diff)*SB2*Xb**2)/MSb12 - 
     -     ht2*SB2*(4*MT2 + Xb*(4*MT*s2tb + s2tt2*Xb)) - 
     -     (1 + c2tt2)*CB2*ht2*Yb**2 - 
     -     (4*c2tt2*ht2*MSb12*
     -        (MSb2diff + MSb2diff*Nc + 2*SB2*Xb**2 + 2*CB2*Yb**2))
     -       /MSb2diff - CB2*ht2*
     -      (4*MT2 + Yb*(4*MT*s2tb + s2tt2*Yb)) + 
     -     (ht2*(x4 - x5)*(MSb2diff*SB2*Xb**2 + 
     -          c2tt2*(4*MSb12*MSb22*(1 + Nc) + 
     -             (4*MSb22 - 3*MSb2diff)*SB2*Xb**2 + 
     -             2*CB2*(-MA02 + MSb2sum)*Yb**2)))/MSb2diff + 
     -     (8*c2tt2*MSb12*(ht2*SB2*Xb*(2*MT + s2tb*Xb) + 
     -          CB2*ht2*Yb*(2*MT + s2tb*Yb) - 2*MB*(hb*ht*MUE)))/
     -      (MSb2diff*s2tb) + 4*MB*(hb*ht*MUE*s2tb) + 
     -     ((x5 - x9)*(-(c2tt2*(hb2 - ht2)*s2tb*
     -             (MB2**2 + MSb12**2 - 2*MSb12*MUE2 - MUE2**2 - 
     -               2*MB2*MSb12)) + 
     -          2*c2tb*(2*MB*
     -              (c2tt2*hb*ht*MUE*(MB2 - MSb12 - MUE2)) + 
     -             MSb2diff*
     -              (ctb2*hb2*(MB2 - MSb12 + MUE2)*s2tb + 
     -                ht2*(MB2 - MSb12 + MUE2)*s2tb*stb2 + 
     -                MB*(hb*ht*MUE*s2tb2)))))/(c2tb*MSb2diff*s2tb)
     -       - 2*hb2*SB2*(MT2*stb2*stt2 + MT*s2tb*stt2*Yt + 
     -        ctb2*(MB2*ctt2 + Yt*(stt2*Yt + MB*s2tt)) + 
     -        MB/2.D0*(MT*s2tb*s2tt)) + 
     -     2*MB2*(ctb2*hb2 + ht2*stb2) - 
     -     2*CB2*ht2*(ctb2*ctt2*MT2 + MB/2.D0*(MT*s2tb*s2tt) + 
     -        Yb*(ctt2*MT*s2tb + ctt2*stb2*Yb + MB*(s2tt*stb2)) + 
     -        MB2*(stb2*stt2)) - 
     -     2*CB2*ht2*(ctb2*MT2*stt2 + 
     -        stt2*Yb*(MT*s2tb + stb2*Yb) + MB2*(ctt2*stb2) - 
     -        MB/2.D0*(s2tt*(MT*s2tb + 2*stb2*Yb))) - 
     -     2*hb2*SB2*(ctt2*MT2*stb2 + 
     -        ctt2*Yt*(MT*s2tb + ctb2*Yt) + MB2*(ctb2*stt2) - 
     -        MB/2.D0*(s2tt*(MT*s2tb + 2*ctb2*Yt))) - 
     -     2*CB*hb*ht*SB*(-(MT*s2tt*(stb2*Yb + ctb2*Yt)) + 
     -        2*MB*(ctt2*MT*stb2) + 
     -        MB*(s2tb*(stt2*Yb + ctt2*Yt)) - 
     -        1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt))) - 
     -     2*CB*hb*ht*SB*(MT*s2tt*(stb2*Yb + ctb2*Yt) + 
     -        2*MB*(MT*stb2*stt2) + 
     -        MB*(s2tb*(ctt2*Yb + stt2*Yt)) + 
     -        1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt))))

        tmp3 = tmp3 + 1/2.D0*
     -    ((-x5 + x7)*(2*ctb2*ctt2*hb2*MSt22 + 
     -        (2*c2tt2*MSb12*MSt22*(ctt2*hb2 - ht2*stt2))/
     -         (c2tb*MSb2diff) - 
     -        (2*c2tt2*MSb12*
     -           (ht2*SB2*(-(MT2*s2tb*stt2) + 2*c2tb*MT*stt2*Xb + 
     -                s2tb*stt2*Xb**2 + MB2*(ctt2*s2tb) - 
     -                MB*(c2tb*MT*s2tt) - MB*(s2tb*s2tt*Xb)) + 
     -             CB2*hb2*
     -              (ctt2*(MT2*s2tb + Xt*(2*c2tb*MT - s2tb*Xt)) - 
     -                MB*(c2tb*MT*s2tt) - MB2*(s2tb*stt2) + 
     -                MB*(s2tb*s2tt*Xt)) + 
     -             CB*hb*ht*SB*
     -              (MT*s2tb*(s2tt*Xb - s2tt*Xt + 2*MB*stt2) + 
     -                c2tb*
     -                 (MT2*s2tt + s2tt*Xb*Xt + MB2*s2tt - 
     -                   2*MB*(stt2*Xb)) - 
     -                2*MB*(ctt2*(MT*s2tb + c2tb*Xt)))))/
     -         (c2tb*MSb2diff*s2tb) + 
     -        CB2*ht2*(ctb2*MT2*stt2 + 
     -           stt2*Yb*(MT*s2tb + stb2*Yb) + MB2*(ctt2*stb2) - 
     -           MB/2.D0*(s2tt*(MT*s2tb + 2*stb2*Yb))) + 
     -        (c2tt2*(MA02 - MSb12 - MSt22)*
     -           (CB2*ht2*(-(MT2*s2tb*stt2) + 2*c2tb*MT*stt2*Yb + 
     -                s2tb*stt2*Yb**2 + MB2*(ctt2*s2tb) - 
     -                MB*(c2tb*MT*s2tt) - MB*(s2tb*s2tt*Yb)) + 
     -             hb2*SB2*
     -              (ctt2*(MT2*s2tb + Yt*(2*c2tb*MT - s2tb*Yt)) - 
     -                MB*(c2tb*MT*s2tt) - MB2*(s2tb*stt2) + 
     -                MB*(s2tb*s2tt*Yt)) - 
     -             CB*hb*ht*SB*
     -              (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                c2tb*
     -                 (s2tt*(MB2 + MT2 + Yb*Yt) - 2*MB*(stt2*Yb))-
     -                  2*MB*(ctt2*(MT*s2tb + c2tb*Yt)))))/
     -         (c2tb*MSb2diff*s2tb) + 
     -        hb2*SB2*(ctt2*MT2*stb2 + 
     -           ctt2*Yt*(MT*s2tb + ctb2*Yt) + MB2*(ctb2*stt2) - 
     -           MB/2.D0*(s2tt*(MT*s2tb + 2*ctb2*Yt))) + 
     -        CB*hb*ht*SB*(-(MT*s2tt*(stb2*Yb + ctb2*Yt)) + 
     -           2*MB*(ctt2*MT*stb2) + 
     -           MB*(s2tb*(stt2*Yb + ctt2*Yt)) - 
     -           1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt)))))

        tmp3 = tmp3 + (2*c2tt2*MSb12*
     -      (-(ht2*MT2*s2tb*SB2) + 2*c2tb*ht2*MT*SB2*Xb + 
     -        ht2*s2tb*SB2*Xb**2 + 
     -        hb2*SB2*(MT2*s2tb + 2*c2tb*MT*Yt - s2tb*Yt**2 - 
     -           MB2*s2tb) + 
     -        CB2*(hb2*(MT2*s2tb + 2*c2tb*MT*Xt - s2tb*Xt**2 - 
     -              MB2*s2tb) + 
     -           ht2*(-(MT2*s2tb) + 2*c2tb*MT*Yb + s2tb*Yb**2 + 
     -              MB2*s2tb)) + MB2*(ht2*s2tb*SB2) - 
     -        2*MB*(c2tb*CB*hb*ht*SB*Xb) - 
     -        2*MB*(c2tb*CB*hb*ht*SB*Xt) + 
     -        2*MB*(c2tb*CB*hb*ht*SB*Yb) + 
     -        2*MB*(c2tb*CB*hb*ht*SB*Yt)))/(c2tb*MSb2diff*s2tb)

        tmp3 = tmp3 + 1/2.D0*
     -    ((-x5 + x6)*(2*ctb2*hb2*MSt12*stt2 + 
     -        (2*c2tt2*MSb12*MSt12*(-(ctt2*ht2) + hb2*stt2))/
     -         (c2tb*MSb2diff) + 
     -        hb2*SB2*(MT2*stb2*stt2 + MT*s2tb*stt2*Yt + 
     -           ctb2*(MB2*ctt2 + Yt*(stt2*Yt + MB*s2tt)) + 
     -           MB/2.D0*(MT*s2tb*s2tt)) + 
     -        CB2*ht2*(ctb2*ctt2*MT2 + MB/2.D0*(MT*s2tb*s2tt) + 
     -           Yb*(ctt2*MT*s2tb + ctt2*stb2*Yb + 
     -              MB*(s2tt*stb2)) + MB2*(stb2*stt2)) + 
     -        (2*c2tt2*MSb12*
     -           (-(ht2*SB2*
     -                (ctt2*
     -                   (-(MT2*s2tb) + Xb*(2*c2tb*MT + s2tb*Xb))+
     -                    MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) + 
     -                  MB*(s2tb*s2tt*Xb))) + 
     -             CB2*hb2*
     -              (MB2*(ctt2*s2tb) - 
     -                c2tb*MT*(2*stt2*Xt + MB*s2tt) + 
     -                s2tb*
     -                 (-(MT2*stt2) + stt2*Xt**2 + MB*(s2tt*Xt)))+
     -               CB*hb*ht*SB*
     -              (MT*s2tb*(s2tt*Xb - s2tt*Xt + 2*MB*stt2) + 
     -                MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Xb)) + 
     -                c2tb*
     -                 (MT2*s2tt + s2tt*Xb*Xt + MB2*s2tt + 
     -                   2*MB*(stt2*Xt)))))/(c2tb*MSb2diff*s2tb) + 
     -        (c2tt2*(MA02 - MSb12 - MSt12)*
     -           (CB2*ht2*(ctt2*
     -                 (-(MT2*s2tb) + Yb*(2*c2tb*MT + s2tb*Yb)) + 
     -                MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) + 
     -                MB*(s2tb*s2tt*Yb)) + 
     -             hb2*SB2*
     -              (-(MB2*(ctt2*s2tb)) + 
     -                c2tb*MT*(2*stt2*Yt + MB*s2tt) + 
     -                s2tb*(MT2*stt2 - stt2*Yt**2 - MB*(s2tt*Yt)))+
     -               CB*hb*ht*SB*
     -              (MT*s2tb*(s2tt*Yb - s2tt*Yt + 2*MB*stt2) + 
     -                MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Yb)) + 
     -                c2tb*
     -                 (MT2*s2tt + s2tt*Yb*Yt + MB2*s2tt + 
     -                   2*MB*(stt2*Yt)))))/(c2tb*MSb2diff*s2tb) + 
     -        CB*hb*ht*SB*(MT*s2tt*(stb2*Yb + ctb2*Yt) + 
     -           2*MB*(MT*stb2*stt2) + 
     -           MB*(s2tb*(ctt2*Yb + stt2*Yt)) + 
     -           1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt)))))

        tmp4 = -((2*MB2 - MSb22 + MUE2)*(MB2 + hb2*stb2)) + 
     -   (-x5 + x9)*((MB2 - MSb22)*(ctb2*ht2 + hb2*stb2) + 
     -      MUE2*(MB2 + hb2*stb2) + 
     -      (MB2 + 1/2.D0*(c2tt2*(hb2 - ht2)*MUE2*
     -             (MB2 - MSb22 + MUE2))/MSb2diff)/c2tb + 
     -      hb*(MB2 - MB*(ht*MUE*s2tb))) + 
     -   CB2*(MB2 + ht2*(MT2*stb2*stt2 - MT*s2tb*stt2*Yb + 
     -         ctb2*(MB2 + Yb*(stt2*Yb - MB*s2tt)) + 
     -         MB/2.D0*(MT*s2tb*s2tt))) + 
     -   (MB2 + (2*(2*MB2 + 
     -           c2tt2*MSb22*
     -            (-MB2 + hb2*SB2*
     -               (ctt2*
     -                  (-(MT2*s2tb) + Yt*(-2*c2tb*MT + s2tb*Yt))+
     -                   MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) - 
     -                 MB*(s2tb*s2tt*Yt)) - 
     -              CB*hb*ht*SB*
     -               (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) - 
     -                 c2tb*
     -                  (s2tt*(MB2 + MT2 + Yb*Yt) - 2*MB*(stt2*Yb))
     -                   + 2*MB*(ctt2*(MT*s2tb + c2tb*Yt))))))/
     -       (c2tb*s2tb))/MSb2diff + 
     -   hb*(MB2 + CB*ht*SB*
     -       (MT*(MB2 + ctb2*s2tt*Yb + s2tt*stb2*Yt) + 
     -         s2tb*(MB2 - 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt))))) + 
     -   hb2*(SB2*Yt*(-(MT*s2tb*stt2) + stb2*stt2*Yt + 
     -         MB*(s2tt*stb2)) + 
     -      MB2*(2 + SB2*(stb2 + stt2) + MB*SB2)) + 
     -   1/2.D0*(ht2*(MB2 + CB2*
     -          (4*MT2 + Yb*(-4*MT*s2tb + s2tt2*Yb))) + 
     -      CB*hb*ht*SB*(-2*MT*s2tt*(ctb2*Yb + stb2*Yt) + 
     -         s2tb*(MB2 + s2tt*Yb*Yt) + 
     -         MB*(MB2 - 2*s2tb*(stt2*Yb + ctt2*Yt)))) + 
     -   (-x5 + x7)*(MSt22*
     -       (MB2 + (c2tt2*MSb22*(-(ctt2*hb2) + ht2*stt2))/
     -          (c2tb*MSb2diff)) + 
     -      (MB2 + 1/2.D0*(c2tt2*(MA02 - MB2 - MSt22)*
     -             (-MB2 + 
     -               hb2*SB2*
     -                (ctt2*
     -                   (-(MT2*s2tb) + Yt*(-2*c2tb*MT + s2tb*Yt))+
     -                    MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) - 
     -                  MB*(s2tb*s2tt*Yt)) - 
     -               CB*hb*ht*SB*
     -                (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) - 
     -                  c2tb*
     -                   (s2tt*(MB2 + MT2 + Yb*Yt) - 
     -                     2*MB*(stt2*Yb)) + 
     -                  2*MB*(ctt2*(MT*s2tb + c2tb*Yt)))))/
     -           (MSb2diff*s2tb))/c2tb + 
     -      1/2.D0*(-MB2 - hb2*SB2*
     -          (ctb2*ctt2*MT2 + ctt2*Yt*(-(MT*s2tb) + stb2*Yt) + 
     -            MB2*(stb2*stt2) + 
     -            MB/2.D0*(s2tt*(MT*s2tb - 2*stb2*Yt))) - 
     -         1/2.D0*(CB*hb*ht*SB*
     -            ((MB2 + MT2)*s2tb*s2tt + s2tb*s2tt*Yb*Yt - 
     -              2*MT*s2tt*(ctb2*Yb + stb2*Yt) + 
     -              4*MB*(MT*stb2*stt2) - 
     -              2*MB*(s2tb*(stt2*Yb + ctt2*Yt))))))

        tmp4 = tmp4 + (-x5 + x6)*
     -    (MSt12*(MB2 + (c2tt2*MSb22*(ctt2*ht2 - hb2*stt2))/
     -          (c2tb*MSb2diff)) + 
     -      (MB2 + 1/2.D0*(c2tt2*(MA02 - MB2 - MSt12)*
     -             (-MB2 + 
     -               hb2*SB2*
     -                (MB2*(ctt2*s2tb) - 
     -                  c2tb*MT*(2*stt2*Yt + MB*s2tt) + 
     -                  s2tb*
     -                   (-(MT2*stt2) + stt2*Yt**2 + MB*(s2tt*Yt)))
     -                 - CB*hb*ht*SB*
     -                (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                  MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Yb)) + 
     -                  c2tb*
     -                   (s2tt*(MB2 + MT2 + Yb*Yt) + 
     -                     2*MB*(stt2*Yt)))))/(MSb2diff*s2tb))/c2tb
     -        + 1/2.D0*(-MB2 - 
     -         hb2*SB2*(ctb2*MT2*stt2 - MT*s2tb*stt2*Yt + 
     -            stb2*stt2*Yt**2 - MB/2.D0*(MT*s2tb*s2tt) + 
     -            MB2*(ctt2*stb2) + MB*(s2tt*stb2*Yt)) - 
     -         CB*hb*ht*SB*
     -          (MT*s2tt*(ctb2*Yb + stb2*Yt) + 
     -            2*MB*(ctt2*MT*stb2) - 
     -            MB*(s2tb*(ctt2*Yb + stt2*Yt)) - 
     -            1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt)))))

        tmp5 = (MB2 + (2*
     -         (2*MB2 + c2tt2*MA02*
     -            (-MB2 + hb2*SB2*
     -               (MB2*(ctt2*s2tb) - 
     -                 c2tb*MT*(2*stt2*Yt + MB*s2tt) + 
     -                 s2tb*
     -                  (-(MT2*stt2) + stt2*Yt**2 + MB*(s2tt*Yt)))-
     -                CB*hb*ht*SB*
     -               (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                 MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Yb)) + 
     -                 c2tb*
     -                  (s2tt*(MB2 + MT2 + Yb*Yt) + 2*MB*(stt2*Yt))
     -                 ))))/(MSb2diff*s2tb))/c2tb + 
     -   1/2.D0*(MB2 + (-x5 + x7)*
     -       (MB2 + (c2tt2*
     -            (MB2 + (-MB2 + MSb12 - MSt22)*
     -               (-MB2 + 
     -                 hb2*SB2*
     -                  (ctt2*
     -                     (MT2*s2tb + Yt*(2*c2tb*MT - s2tb*Yt)) - 
     -                    MB*(c2tb*MT*s2tt) - MB2*(s2tb*stt2) + 
     -                    MB*(s2tb*s2tt*Yt)) - 
     -                 CB*hb*ht*SB*
     -                  (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                    c2tb*
     -                     (s2tt*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(stt2*Yb)) - 
     -                    2*MB*(ctt2*(MT*s2tb + c2tb*Yt))))))/
     -          (c2tb*MSb2diff*s2tb))) + 
     -   (x11 - x5)*(MA02*(MB2 - 
     -         (c2tt2*CB2*ht2*Yb*(2*MT + s2tb*Yb))/(MSb2diff*s2tb))
     -        + (MB2 + 1/2.D0*
     -          (c2tt2*(MB2 + 
     -               (-MB2 - MSb12 + MSt12)*
     -                (-MB2 + 
     -                  CB2*ht2*
     -                   (ctt2*
     -                      (-(MT2*s2tb) + 
     -                       Yb*(2*c2tb*MT + s2tb*Yb)) + 
     -                     MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) + 
     -                     MB*(s2tb*s2tt*Yb)) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) + 
     -                     2*MB*(ctt2*(MT*s2tb - c2tb*Yb)) - 
     -                     c2tb*
     -                      (s2tt*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stt2*Yt))))))/(c2tb*s2tb))/
     -       MSb2diff + 1/2.D0*
     -       (hb2*(MB2 + SB2*
     -             (MT2*stb2*stt2 + MT*s2tb*stt2*Yt + MB2*MT + 
     -               ctb2*(MB2*ctt2 + Yt*(stt2*Yt + MB*s2tt)))) + 
     -         CB2*(2*MB2 + 
     -            ht2*(stt2*(MB2 + MT*s2tb*Yb + stb2*Yb**2) + 
     -               MB2*(ctt2*stb2) - 
     -               MB/2.D0*(s2tt*(MT*s2tb + 2*stb2*Yb)))) + 
     -         CB*(MB2 + hb*ht*SB*
     -             (MT*(MB2 + s2tt*stb2*Yb + ctb2*s2tt*Yt) + 
     -               s2tb*(MB2 + 1/2.D0*(s2tt*(MB2 + MT2 + Yb*Yt)))))
     -         ))

        tmp5 = tmp5 + (x4 - x5)*
     -    (MA02*(MB2 - hb2*SB2*stb2) + 
     -      (MB2 + 1/2.D0*(c2tt2*
     -             (MB2 + (-MB2 - MSb22 + MSt12)*
     -                (-MB2 + 
     -                  hb2*SB2*
     -                   (MB2*(ctt2*s2tb) - 
     -                     c2tb*MT*(2*stt2*Yt + MB*s2tt) + 
     -                     s2tb*
     -                      (-(MT2*stt2) + stt2*Yt**2 + 
     -                       MB*(s2tt*Yt))) - 
     -                  CB*hb*ht*SB*
     -                   (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                     MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Yb)) + 
     -                     c2tb*
     -                      (s2tt*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stt2*Yt))))))/(c2tb*s2tb))/
     -       MSb2diff + 1/4.D0*
     -       (-2*ctb2*ctt2*hb2*MT2*SB2 + 
     -         2*ctt2*hb2*MT*s2tb*SB2*Yt - 
     -         2*ctt2*hb2*SB2*stb2*Yt**2 - 
     -         MB*(hb2*MT*s2tb*s2tt*SB2) - 
     -         4*MB*(CB*ctt2*hb*ht*MT*SB*stb2) - 
     -         4*MB*(CB*hb*ht*MT*SB*stb2*stt2) - 
     -         2*MB2*(1 + hb2*SB2*stb2*stt2 + 
     -            CB2*(-2 + ctb2*ht2*stt2)) + 
     -         2*MB*(CB*ctt2*hb*ht*s2tb*SB*Yb) + 
     -         2*MB*(CB*hb*ht*s2tb*SB*stt2*Yb) + 
     -         CB2*ht2*(-2*ctt2*
     -             (MT2*stb2 - MT*s2tb*Yb + ctb2*Yb**2) + 
     -            MB*(s2tt*(MT*s2tb - 2*ctb2*Yb))) + 
     -         2*MB*(CB*ctt2*hb*ht*s2tb*SB*Yt) + 
     -         2*MB*(hb2*s2tt*SB2*stb2*Yt) + 
     -         2*MB*(CB*hb*ht*s2tb*SB*stt2*Yt)))

        Fb = tmp3*(x11 - x5) + tmp4*(x4 - x5) + 
     -   tmp5*(-x5 + x8) + 
     -   ((c2tt2*(MB2 + (-hb2 + ht2)*MUE2))/c2tb - 
     -      2*(MB2 + ctb2*hb2*MUE2 + ht2*MUE2*stb2))*(-x5 + x9) - 
     -   ht2*s2tt2*SB2*x18*Xb**2 + 
     -   s2tb*(MB2 + 2*ht2*MT*(SB2*Xb + CB2*Yb)) + 
     -   (x10 - x5)*(MB2*(c2tt2*(1 - hb2 + ht2))/c2tb + 
     -      (x11 - x5)*((MB2 + MSb12 - MUE2)*
     -          (ctb2*hb2 + ht2*stb2) + 
     -         (MB2 - 1/2.D0*
     -             (c2tt2*(hb2 - ht2)*(MB2 + MSb12 - MUE2)*
     -                (MB2 - MSb12 + MUE2))/MSb2diff)/c2tb + 
     -         hb*(MB2 - MB*(ht*MUE*s2tb))) + 
     -      (-x5 + x9)*(-((MB2 - MSb22 + MUE2)*(MB2 + hb2*stb2)) + 
     -         (MB2 - MSb12 + MUE2)*(ctb2*hb2 + ht2*stb2) + 
     -         c2tt2*(MB2 + 2*MB*(hb*ht*MUE)/s2tb) + 
     -         2*MB*(hb*ht*MUE*s2tb)) + 
     -      (x4 - x5)*((-MSb22 + MUE2)*(ctb2*ht2 + hb2*stb2) + 
     -         (MB2 + 1/2.D0*
     -             (c2tt2*(hb2 - ht2)*(MB2 + MSb22 - MUE2)*
     -                (MB2 - MSb22 + MUE2))/MSb2diff)/c2tb + 
     -         hb*(MB2 - MB*(ht*MUE*s2tb)) + MB2*(MB2 + hb2*stb2))-
     -        2*MB2*(1 + ctb2*hb2 + ht2*stb2)) + 
     -   (MB2 + (c2tt2*(MB2 + 
     -           (MSb22 - MSt22)*x19*
     -            (-MB2 + CB2*hb2*
     -               (ctt2*
     -                  (-(MT2*s2tb) + Xt*(-2*c2tb*MT + s2tb*Xt))+
     -                   MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) - 
     -                 MB*(s2tb*s2tt*Xt)) + 
     -              CB*hb*ht*SB*
     -               (MT*s2tb*(s2tt*(-Xb + Xt) - 2*MB*stt2) - 
     -                 c2tb*
     -                  (s2tt*(MB2 + MT2 + Xb*Xt) - 2*MB*(stt2*Xb))
     -                   + 2*MB*(ctt2*(MT*s2tb + c2tb*Xt))))))/
     -       (MSb2diff*s2tb))/c2tb + 
     -   c2tt2*(MB2 - 5/2.D0*
     -       (CB2*ht2*(MA02 + 2*MSb12)*Yb*(2*MT + s2tb*Yb))/
     -        (MSb2diff*s2tb)) + 
     -   (x11 - x5)**2*(c2tt2*
     -       (2*MB2 + 1/2.D0*
     -          (CB2*ht2*(MA02 - 2*MSb12)*Yb*(2*MT + s2tb*Yb))/
     -           (MSb2diff*s2tb)) + 
     -      1/4.D0*(ht2*(SB2*(4*MT2 + Xb*(4*MT*s2tb + s2tt2*Xb)) + 
     -           CB2*(4*MT2 + Yb*(4*MT*s2tb + s2tt2*Yb))))) + 
     -   (x29*(MB2 + 1/2.D0*(c2tt2*(hb2 - ht2)*(MB2 - MSb22 + MUE2)*
     -             s2tb - c2tb*
     -             (MSb2diff*s2tb*(ctb2*ht2 + hb2*stb2) + 
     -               4*MB*(c2tt2*hb*ht*MUE)))/
     -          (c2tb*MSb2diff*s2tb*x28)))/MUE2 + 
     -   1/2.D0*(MB2 + x27*(MB2 + 
     -         (-(c2tt2*(hb2 - ht2)*(MB2 - MSb12 + MUE2)*s2tb) + 
     -            c2tb*(MSb2diff*s2tb*(ctb2*hb2 + ht2*stb2) + 
     -               4*MB*(c2tt2*hb*ht*MUE)))/
     -          (c2tb*MSb2diff*MUE2*s2tb*x26)))

        Fb = Fb + CB2*(MB2*(2 + x21) + 
     -      1/8.D0*(ht2*x21*(-4*c2tt2*MSb22*Yb*(-2*MT + s2tb*Yb) + 
     -            MSb2diff*
     -             (4*MT2*s2tb - 4*MT*s2tb2*Yb + s2tb*s2tt2*Yb**2))
     -          )/(MSb22**2*MSb2diff*s2tb*x20)) + 
     -   1/4.D0*((x4 - x5)**2*
     -      (ht2*(3*MB2 - CB2*
     -            (4*MT2 + Yb*(-4*MT*s2tb + s2tt2*Yb))) + 
     -        (2*c2tt2*(2*MB2 + 
     -             (-MSb22 + MSt12)*
     -              (-MB2 + 
     -                CB2*hb2*
     -                 (MB2*(ctt2*s2tb) - 
     -                   c2tb*MT*(2*stt2*Xt + MB*s2tt) + 
     -                   s2tb*
     -                    (-(MT2*stt2) + stt2*Xt**2 + MB*(s2tt*Xt))
     -                   ) + 
     -                CB*hb*ht*SB*
     -                 (MT*s2tb*(s2tt*(Xb - Xt) + 2*MB*stt2) + 
     -                   MB*(ctt2*(-2*MT*s2tb + 2*c2tb*Xb)) + 
     -                   c2tb*
     -                    (s2tt*(MB2 + MT2 + Xb*Xt) + 
     -                      2*MB*(stt2*Xt))))))/
     -         (c2tb*MSb2diff*s2tb))) + 
     -   (MB2 + x23*(MB2 - 
     -         1/2.D0*(c2tt2*
     -             (-MB2 + 
     -               hb2*SB2*
     -                (ctt2*
     -                   (MT2*s2tb + Yt*(2*c2tb*MT - s2tb*Yt)) - 
     -                  MB*(c2tb*MT*s2tt) - MB2*(s2tb*stt2) + 
     -                  MB*(s2tb*s2tt*Yt)) - 
     -               CB*hb*ht*SB*
     -                (MT*s2tb*(s2tt*(Yb - Yt) + 2*MB*stt2) + 
     -                  c2tb*
     -                   (s2tt*(MB2 + MT2 + Yb*Yt) - 
     -                     2*MB*(stt2*Yb)) - 
     -                  2*MB*(ctt2*(MT*s2tb + c2tb*Yt)))))/
     -           (c2tb*MSb2diff*s2tb*x22)))/MSb12 + 
     -   (MB2 + x25*(MB2 - 
     -         1/2.D0*(c2tt2*
     -             (-MB2 + 
     -               hb2*SB2*
     -                (ctt2*
     -                   (-(MT2*s2tb) + Yt*(-2*c2tb*MT + s2tb*Yt))+
     -                    MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) - 
     -                  MB*(s2tb*s2tt*Yt)) - 
     -               CB*hb*ht*SB*
     -                (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) - 
     -                  c2tb*
     -                   (s2tt*(MB2 + MT2 + Yb*Yt) - 
     -                     2*MB*(stt2*Yb)) + 
     -                  2*MB*(ctt2*(MT*s2tb + c2tb*Yt)))))/
     -           (c2tb*MSb2diff*s2tb*x24)))/MSb22 + 
     -   (-x5 + x7)*((MB2 + 
     -         (2*(2*MB2 + 
     -              c2tt2*MSt22*
     -               (-MB2 + 
     -                 hb2*SB2*
     -                  (ctt2*
     -                     (-(MT2*s2tb) + 
     -                       Yt*(-2*c2tb*MT + s2tb*Yt)) + 
     -                    MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) - 
     -                    MB*(s2tb*s2tt*Yt)) - 
     -                 CB*hb*ht*SB*
     -                  (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) - 
     -                    c2tb*
     -                     (s2tt*(MB2 + MT2 + Yb*Yt) - 
     -                       2*MB*(stt2*Yb)) + 
     -                    2*MB*(ctt2*(MT*s2tb + c2tb*Yt))))))/
     -          (MSb2diff*s2tb))/c2tb + 
     -      1/2.D0*(MB2 + ((-MA02 + MB2 + MSb12 + MSt22)*
     -            (-MB2 - hb2*SB2*
     -               (ctt2*MT2*stb2 + 
     -                 ctt2*Yt*(MT*s2tb + ctb2*Yt) + 
     -                 MB2*(ctb2*stt2) - 
     -                 MB/2.D0*(s2tt*(MT*s2tb + 2*ctb2*Yt))) - 
     -              CB*hb*ht*SB*
     -               (-(MT*s2tt*(stb2*Yb + ctb2*Yt)) + 
     -                 2*MB*(ctt2*MT*stb2) + 
     -                 MB*(s2tb*(stt2*Yb + ctt2*Yt)) - 
     -                 1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt)))))/
     -          MSb12))

        Fb = Fb + (MB2 + 
     -      1/2.D0*(c2tt2*(MB2 + 
     -             (x5 - x7)**2*
     -              (MB2 + 
     -                (MSb12 - MSt22)*
     -                 (-MB2 + 
     -                   CB2*hb2*
     -                    (ctt2*
     -                       (MT2*s2tb + Xt*(2*c2tb*MT - s2tb*Xt))-
     -                        MB*(c2tb*MT*s2tt) - 
     -                      MB2*(s2tb*stt2) + MB*(s2tb*s2tt*Xt)) + 
     -                   CB*hb*ht*SB*
     -                    (MT*s2tb*(s2tt*(Xb - Xt) + 2*MB*stt2) + 
     -                      c2tb*
     -                       (s2tt*(MB2 + MT2 + Xb*Xt) - 
     -                       2*MB*(stt2*Xb)) - 
     -                      2*MB*(ctt2*(MT*s2tb + c2tb*Xt)))))) - 
     -          5*(2*MB2 + 
     -             c2tt2*(MA02 + MSb12 + MSt12)*
     -              (-MB2 + 
     -                CB2*ht2*
     -                 (ctt2*
     -                    (-(MT2*s2tb) + Yb*(2*c2tb*MT + s2tb*Yb))+
     -                     MB*(c2tb*MT*s2tt) + MB2*(s2tb*stt2) + 
     -                   MB*(s2tb*s2tt*Yb)) - 
     -                CB*hb*ht*SB*
     -                 (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) + 
     -                   2*MB*(ctt2*(MT*s2tb - c2tb*Yb)) - 
     -                   c2tb*
     -                    (s2tt*(MB2 + MT2 + Yb*Yt) + 
     -                      2*MB*(stt2*Yt))))))/(c2tb*s2tb))/
     -    MSb2diff + (-x5 + x6)*
     -    ((MB2 + (2*(2*MB2 + 
     -              c2tt2*MSt12*
     -               (-MB2 + 
     -                 CB2*ht2*
     -                  (ctt2*
     -                     (-(MT2*s2tb) + Yb*(2*c2tb*MT + s2tb*Yb))
     -                      + MB*(c2tb*MT*s2tt) + 
     -                    MB2*(s2tb*stt2) + MB*(s2tb*s2tt*Yb)) - 
     -                 CB*hb*ht*SB*
     -                  (MT*s2tb*(s2tt*(-Yb + Yt) - 2*MB*stt2) + 
     -                    2*MB*(ctt2*(MT*s2tb - c2tb*Yb)) - 
     -                    c2tb*
     -                     (s2tt*(MB2 + MT2 + Yb*Yt) + 
     -                       2*MB*(stt2*Yt))))))/(MSb2diff*s2tb))/
     -       c2tb + 1/2.D0*(MB2 + 
     -         ((-MA02 + MB2 + MSb12 + MSt12)*
     -            (-MB2 - hb2*SB2*
     -               (MT2*stb2*stt2 + MT*s2tb*stt2*Yt + 
     -                 ctb2*(MB2*ctt2 + Yt*(stt2*Yt + MB*s2tt)) + 
     -                 MB/2.D0*(MT*s2tb*s2tt)) - 
     -              CB*hb*ht*SB*
     -               (MT*s2tt*(stb2*Yb + ctb2*Yt) + 
     -                 2*MB*(MT*stb2*stt2) + 
     -                 MB*(s2tb*(ctt2*Yb + stt2*Yt)) + 
     -                 1/2.D0*(s2tb*s2tt*(MB2 + MT2 + Yb*Yt)))))/
     -          MSb12))

        FAp = 5*(MB2 + (At**2*hb*ht*MT*MUE)/(MSt2diff*s2tt)) + 
     -   (At**2*hb*ht*MT*MUE*x17)/(MSt2diff**2*MUE2*s2tt*x16) + 
     -   MB2/s2tt2 + (MB2 + 
     -      MB*(Ab**2*hb*ht*MUE*x45)/(MUE2*s2tb*x44))/MSb2diff**2+
     -     (MB2*s2tb + MB*(Ab**2*hb*ht*MUE*(x33 - x5)*(x5 - x9)))/
     -    (MSb2diff*s2tb) + 
     -   1/(2.D0*MB)*(-(5*MB2*(At**2*CB*hb*ht*MT*SB)) + 
     -       MB*(5*At**2*CB*hb*ht*MT*SB*(ctb2*Xb + stb2*Xt) - 
     -          5*MB2*(MSt2diff*s2tt)))/(MSt2diff*s2tt) + 
     -   1/MB*(x30*(-(MB2*((MSb12 - MSt12)*MSt2diff*s2tt*
     -              (MSb2diff*(MB2 + ht2*MT2*MUE2*s2tb*SB2) + 
     -                Ab**2*CB*hb*ht*MSt2diff*SB*
     -                 (MB2 + MT*stt2*Xt + MB2*MT)))) + 
     -         s2tb*(MB2 + 
     -            MB*(MSb2diff*
     -               (-(At**2*CB*hb*ht*MSb2diff*(MSb12 - MSt12)*
     -                    MT2*SB*(stb2*Xb + ctb2*Xt)) + 
     -                 MB2*
     -                  (MSt2diff*MT*s2tt*
     -                    (At*CB*hb*ht*(MSb12 - MSt12)*MUE*SB*TB + 
     -                      MSt2diff*
     -                       (1 + 2*MSb2diff*MSt12 + MB2*MSb2diff))
     -                    ))))))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt) + 
     -   1/(2.D0*MB)*(x32*((MSb12 - MSt22)*
     -          (MB2 - 2*MB2*
     -             (MSt2diff*s2tt*
     -               (-MB2 - ht2*MSb2diff*MT2*MUE2*s2tb*SB2 + 
     -                 Ab**2*CB*hb*ht*MSt2diff*SB*
     -                  (MB2 + MT*stt2*Xb + ctt2*MT*Xt)))) + 
     -         2*MB*(MSb2diff*s2tb*
     -            (At**2*CB*hb*ht*MSb2diff*(MSb12 - MSt22)*MT2*SB*
     -               (stb2*Xb + ctb2*Xt) + 
     -              MB2*(MSt2diff*MT*s2tt*
     -                 (At*CB*hb*ht*(-MSb12 + MSt22)*MUE*SB*TB + 
     -                   MSt2diff*
     -                    (1 + 2*MSb2diff*MSt22 + MB2*MSb2diff)))))
     -         ))/(MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt) + 
     -   1/(2.D0*MB)*(x1*(-((MSb22 - MSt22)*MUE2*
     -            (CB*hb*ht*MSb2diff**2*MT2**2*s2tb2*SB + 
     -              2*MB2*(CB2*hb2*MSb2diff*MSt2diff*MT2*s2tb*s2tt*
     -                 TB2) + 
     -              ht2*SB2*
     -               (2*MSb2diff**2*MT*MT2*s2tb*stb2*Xt + 
     -                 MB2*MT2 + 
     -                 MB**3*
     -                  (MSt2diff**2*TB2**2*
     -                    (-(MT*s2tt2) + 2*ctt2*s2tt*Xt))))) + 
     -         MB2*(MSb2diff**2*MT*s2tb*
     -             (MSb22*MT2*MUE2 - 
     -               4*MB*(MSb22*MSt2diff**2*s2tt*TB2) + 
     -               4*MB*(MSt22*MSt2diff**2*s2tt*TB2)) + 
     -            MB2*(MT*(1 + MSb22 + 
     -                  MSb2diff**2*s2tb*
     -                   (MSt22 + 2*MSt2diff**2*s2tt*TB2)) + 
     -               (MSb22 - MSt22)*MSt2diff**2*MUE2*TB2**2*
     -                (-(CB*hb*ht*s2tt2*SB) + MB*s2tt)))))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt*TB2)

        FAp = FAp + 1/4.D0*
     -    (2*(MB2 + 1/MB*(CB2*ht2*MT2*MUE2*x43*Yt)/
     -           (MSt12*MSt2diff**2*s2tt*TB2*x42)) + 
     -      (2*x31*(-((MSb22 - MSt12)*MSt2diff**2*
     -              (MB2*(ht2*MUE2*SB2*TB2*
     -                   (MT*s2tt2 + 2*s2tt*stt2*Xt)) + 
     -                MB2*(s2tt*
     -                   (2*MSb2diff**2*MT*s2tb + MB2*(MUE2*TB2))))
     -              ) + CB*
     -            (MB*(hb*ht*(MSb22 - MSt12)*SB*
     -                 (4*MT2*MUE2**2*Xb*Xt + 
     -                   Ab**2*
     -                    (4*At**2*MT2*Xb*Xt + 
     -                      MSt2diff**2*
     -                       (MT2*s2tt2 + 
     -                       2*MT*s2tt*(ctt2*Xb + stt2*Xt))) + 
     -                   MB2*MT2)) + 
     -              MB2*(MSt2diff**2*
     -                 (2*MSb2diff**2*MT*s2tb*s2tt + 
     -                   MB*
     -                    (hb*ht*(MSb22 - MSt12)*MUE2*s2tt2*SB*TB2)
     -                   )))))/
     -       (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt) + 
     -      1/MB*((x4 - x5)**2*
     -          (s2tt2*(MT2*
     -                (MB2 + 
     -                  CB*hb*ht*SB*
     -                   (4*MB2*
     -                      (MSt2diff*(-(Ab*At) + MUE2)**2*TB2*Xb*
     -                       Xt) + 
     -                     MSb2diff**2*
     -                      (MB2 - MSt12*MT2*MUE2*s2tb2 - 
     -                       2*MB*
     -                       (At**2*MSt2diff*s2tb*TB2*
     -                       (ctb2*Xb + stb2*Xt))))) + 
     -               MUE2*SB2*
     -                (-(ht2*MSb2diff**2*(MSb22 - MSt12)*MT*MT2*
     -                     (-2*s2tb*stb2*Xt + MB*s2tb2)) + 
     -                  2*MB**3*
     -                   (hb2*(MA02 - 2*MSb22)*MSt2diff**2*s2tt*
     -                     TB2**2*Yb))) + 
     -            MB2*(2*MSt2diff**2*s2tt*s2tt2*TB2*
     -                (MB2 + 2*MB*(MT*s2tb)) + 
     -               MSb2diff**2*
     -                (At**2*CB*hb*ht*MSt2diff*MT2*s2tb2*s2tt2*SB*
     -                   TB2 + 
     -                  MT*s2tb*s2tt*
     -                   (-1 + 
     -                     4*MB*
     -                      (MSt2diff*(1 + CB*MSt2diff)*s2tt2*TB2))
     -                  ))))/
     -        (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt*s2tt2*TB2) + 
     -      1/MB*((x11 - x5)**2*
     -          (s2tb2*s2tt2*
     -             (CB*hb*ht*SB*
     -                (-(MSb2diff**2*MSt22*MT2**2*MUE2*s2tb2) + 
     -                  MSb12*
     -                   (MSb2diff**2*MT2**2*MUE2*s2tb2 - 
     -                     2*MB2*
     -                      (Ab**2*MSt2diff**2*MT*s2tt*TB2*
     -                       (ctt2*Xb + stt2*Xt))) + 
     -                  2*MB2*
     -                   (Ab**2*MSt12*MSt2diff**2*MT*s2tt*TB2*
     -                     (ctt2*Xb + stt2*Xt))) + 
     -               MUE2*(ht2*SB2*
     -                   (MSb2diff**2*
     -                      (MB2 + 
     -                       2*ctb2*(-MSb12 + MSt22)*MT*MT2*s2tb*Xt
     -                       ) + 
     -                     MB**3*
     -                      ((MSb12 - MSt12)*MSt2diff**2*TB2**2*
     -                       (MT*s2tt2 + 2*s2tt*stt2*Xt))) + 
     -                  MB**3*
     -                   (hb2*MSt2diff**2*TB2**2*
     -                     (CB2*(MSb12 - MSt12)*
     -                       (MT*s2tt2 + 2*ctt2*s2tt*Xb) - 
     -                       2*(MA02 - 2*MSb12)*s2tt*SB2*Yb)))) + 
     -            MB2*(MSb2diff**2*s2tb*s2tb2*
     -                (MT2*
     -                   (s2tt + (-MSb12 + MSt22)*MT*MUE2*s2tt2) + 
     -                  2*(2 + MB)*(MSt2diff**2*MT*s2tt*s2tt2*TB2))
     -                 + MB*
     -                (MSt2diff**2*s2tt2*TB2*
     -                  (s2tb*(-1 + 2*Ab**2 + 4*MT*s2tb2)*s2tt - 
     -                    MB*
     -                     (CB*hb*ht*(MSb12 - MSt12)*s2tb2*SB*
     -                       (2*Ab**2*s2tt + MUE2*s2tt2*TB2)))))))/
     -        (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tb2*s2tt*s2tt2*
     -          TB2))

        FAp = FAp + 1/(4.D0*MB)*
     -    (((x5 - x6)**2*(2*At**2*MSb2diff**2*s2tb*s2tb2*TB2*
     -             (MB2*(2*MT*s2tt + 
     -                  CB*hb*ht*(-MSb22 + MSt12)*MT2*s2tt2*SB) - 
     -               MB*(CB*hb*ht*(MSb22 - MSt12)*MT2*s2tt2*SB*
     -                  (ctb2*Xb + stb2*Xt))) + 
     -            s2tt2*(-2*CB2*ht2*MSb2diff**2*(MA02 - 2*MSt12)*
     -                MT*MT2*MUE2*s2tb*s2tb2*Yt + 
     -               TB2*(MB2 + 
     -                  MB2*
     -                   (Ab**2*CB*hb*ht*(MSb22 - MSt12)*
     -                     MSt2diff**2*s2tb2*SB*
     -                     (MT2*s2tt2 + 
     -                       2*MT*s2tt*(ctt2*Xb + stt2*Xt)))))))/
     -        (MSb2diff**2*MT*s2tb*s2tb2*s2tt2*TB2) + 
     -       5*(-(MB2*(At**2*CB*hb*ht*(MA02 + MSt22)*MT*s2tb*
     -               SB)) + 
     -          MB*(2*At**2*CB*hb*ht*MT*SB*
     -              (MSb12*(stb2*(Xb - Yb) + ctb2*(Xt - Yt)) - 
     -                (MA02 + MSt22)*(stb2*Yb + ctb2*Yt)) + 
     -             MB2*(MSt2diff**2*s2tt))) + 
     -       (4*(-x5 + x7)*
     -          (At**2*(MB2 + 
     -               MB*(hb*ht*MSb2diff**2*MT2*MUE*MUE2*s2tb*s2tb2*
     -                  s2tt2*(-x5 + x9))) + 
     -            MSt2diff**2*s2tt*s2tt2*
     -             (MSb2diff**2*
     -                (MB2 - 
     -                  MB*
     -                   (CB*ctb2*hb*ht*MSt22*MT*s2tb*s2tb2*SB*
     -                     stt2)) + 
     -               2*MB2*
     -                (Ab**2*CB*hb*ht*MSt22*MT*s2tb2*SB*
     -                   (stt2*Yb + ctt2*Yt) + 
     -                  MB2*(s2tb - Ab**2*CB*hb*ht*MSt22*s2tb2*SB))
     -               )))/(MSb2diff**2*MT*s2tb*s2tb2*s2tt2) + 
     -       ((x5 - x7)**2*
     -          (MUE2*s2tb2*s2tt2*
     -             (-(CB*hb*ht*MSb2diff**3*MT2**2*s2tb2*SB) + 
     -               CB2*(2*ht2*MSb2diff**2*(MA02 - 2*MSt22)*MT*
     -                   MT2*s2tb*Yt + 
     -                  hb2*
     -                   (MT*
     -                      (MB2 + 
     -                       2*MSb2diff**2*(MB2 + MSt22)*MT2*s2tb*
     -                       (MB2 + stb2)*Xb) + 
     -                     MB**3*
     -                      ((MSb22 - MSt22)*MSt2diff**2*TB2**2*
     -                       (MT*s2tt2 - 2*s2tt*stt2*Xb)))) + 
     -               ht2*SB2*
     -                (MT*(MB2 + 
     -                     2*MSb2diff**2*(MB2 + MSt22)*MT2*s2tb*
     -                      (MB2 + stb2)*Xt) + 
     -                  MB**3*
     -                   ((MSb22 - MSt22)*MSt2diff**2*TB2**2*
     -                     (MT*s2tt2 - 2*ctt2*s2tt*Xt)))) + 
     -            MB2*(MSb2diff**2*s2tb*s2tb2*s2tt*
     -                (MT2 + 
     -                  MT*
     -                   (-1 + 2*MB*((2 + MSt2diff**2)*s2tt2*TB2)))
     -                 - MSt2diff**2*s2tt2*TB2*
     -                (MB*(s2tb*s2tt) + 
     -                  MB2*
     -                   (CB*hb*ht*(MSb22 - MSt22)*MUE2*s2tb2*
     -                     s2tt2*SB*TB2)))))/
     -        (MSb2diff**2*MT*s2tb*s2tb2*s2tt2*TB2) - 
     -       (x39*(MB**3*(MSt2diff**2*MUE2*TB2*
     -               (hb2*SB2*(MT*s2tt2 + 2*ctt2*s2tt*Yb) + 
     -                 CB2*ht2*(MT*s2tt2 + 2*s2tt*stt2*Yt))) - 
     -            2*MB*(MSb2diff**2*s2tb*
     -               (At**2*CB*hb*ht*MT2*SB*(ctb2*Yb + stb2*Yt) + 
     -                 MB2*((2 + MSt12)*MSt2diff**2*MT*s2tt))) + 
     -            MB2*(MB2 + 
     -               MB2*(MSt2diff**2*
     -                  (s2tt + CB*hb*ht*MUE2*s2tt2*SB*TB2)))))/
     -        (MSb2diff**2*MSt12*MT*s2tb*x38))/(MSt2diff**2*s2tt)

        FAp = FAp + 1/(4.D0*MB)*
     -    (-((x37*(MSt2diff**2*
     -               (MB2 + 
     -                 MB**3*
     -                  (MUE2*TB2*
     -                    (hb2*SB2*(MT*s2tt2 - 2*s2tt*stt2*Yb) + 
     -                      CB2*ht2*(MT*s2tt2 - 2*ctt2*s2tt*Yt))))+
     -                MSb2diff**2*
     -               (MB2 + 
     -                 MB*(s2tb*
     -                    (2*At**2*CB*hb*ht*MT2*SB*
     -                       (stb2*Yb + ctb2*Yt) + 
     -                      MB2*
     -                       ((-4 + MSt22 - 4*CB*MSt22)*
     -                       MSt2diff**2*MT*s2tt))))))/(MSt22*x36))
     -         - (4*(x5 - x6)*
     -          (At**2*(MB2 + 
     -               MB*(hb*ht*MSb2diff**2*MT2*MUE*MUE2*s2tb*s2tb2*
     -                  s2tt2*(x5 - x9))) + 
     -            MSt2diff**2*s2tt2*
     -             (MSb2diff**2*MT*s2tb*s2tb2*s2tt*
     -                (MB2 + MB*(CB*ctb2*hb*ht*MSt12*SB*stt2)) + 
     -               MB2*(Ab**2*
     -                  (CB*hb*ht*MSt12*s2tb2*SB*
     -                     (MT2*s2tt2 + 
     -                       2*MT*s2tt*(ctt2*Yb + stt2*Yt)) + 
     -                    2*MB*(s2tb*s2tt))))))/(s2tb2*s2tt2) + 
     -       (2*x35*(CB*(MB2 + 
     -               MB2*(Ab**2*hb*ht*MSt2diff**2*s2tt*SB*
     -                  (MB2 + MT*stt2*Yt + MB2*MT))) + 
     -            MB*(MSb2diff*s2tb*
     -               (At**2*CB*hb*ht*MSb2diff*MT2*SB*
     -                  (stb2*Yb + ctb2*Yt) + 
     -                 MB2*
     -                  (MSt2diff*MT*s2tt*
     -                    (-1 + 2*MB2 + 
     -                      2*MSb2diff*(2 + CB*MSt12)*MSt2diff - 
     -                      At*CB*hb*ht*MUE*SB*TB))))))/(MSt12*x34)
     -         + (x41*(MUE2*
     -             (CB*hb*ht*MSb2diff**2*MT2**2*s2tb2*SB + 
     -               CB2*ht2*
     -                (MSb2diff**2*(MB2 - 2*MT*MT2*s2tb*stb2*Yt) + 
     -                  MB**3*
     -                   (MSt2diff**2*TB2**2*
     -                     (MT*s2tt2 - 2*ctt2*s2tt*Yt)))) + 
     -            MB2*(MT*(MB2 + 
     -                  MSb2diff**2*s2tb*
     -                   (MT2*MUE2 + 
     -                     4*MB*
     -                      (MSt2diff**2*s2tt*TB2*(2 + MB*MSt22))))
     -                 + MB2*
     -                (MSt2diff**2*MUE2*TB2**2*
     -                  (CB*hb*ht*s2tt2*SB + MB*s2tt)))))/
     -        (MSt22*TB2*x40))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tt)

        FAp = FAp + 1/(4.D0*MB)*
     -    ((-x5 + x8)*(MSb2diff**2*
     -          (MB2 + CB*hb*ht*MSb22*MT2**2*MUE2*s2tb2**2*s2tt2*
     -             SB*(x5 - x7)) + 
     -         s2tb2*(MB2 + 
     -            MB2**2*(MSb2diff**2*MT*MT2*MUE2*s2tb*s2tt2*
     -               (-x5 + x7))) + 
     -         s2tt2*(MB2 + 
     -            4*MB*MB2*
     -             (MSb2diff*MSt2diff**2*MT*s2tb*s2tb2*s2tt*TB2*
     -               (-x5 + x7))) + 
     -         4*(MB2 + MB*MB2*
     -             (CB*MSb2diff**2*MSt2diff**2*MT*s2tb*s2tb2*s2tt*
     -               s2tt2*TB2*(-x5 + x7))) + 
     -         MSt2diff**2*
     -          (2*MB2 + MB2**3*
     -             (CB*hb*ht*MUE2*s2tb2*s2tt2**2*SB*TB2**2*
     -               (-x5 + x7))) + 
     -         2*(2*MB2 + CB*
     -             (MB2 + MB2*
     -                (Ab**2*hb*ht*MSb2diff*MSt2diff**2*MT*s2tb2*
     -                  s2tt*s2tt2*SB*stt2*TB2*(-x5 + x7)*Yb))) + 
     -         2*CB2*(MB2 + 
     -            MB**3*(ctt2*ht2*MSt22*MSt2diff**2*MUE2*s2tb2*
     -               s2tt*s2tt2*TB2**2*(x5 - x7)*Yt)) + 
     -         MT*(MB2 + 2*MB2*
     -             (Ab**2*CB*ctt2*hb*ht*MSb2diff*MSt2diff**2*s2tb2*
     -               s2tt*s2tt2*SB*TB2*(-x5 + x7)*Yt)) - 
     -         2*(MB2 + MB**3*MB2*
     -             (CB2*ctt2*ht2*MSt2diff**2*MUE2*s2tb2*s2tt*s2tt2*
     -               TB2**2*(-x5 + x7)*Yt)) - 
     -         8*MB2*(2 + Ab**2*CB*hb*ht*MA02*MSt2diff**2*MT*s2tb2*
     -             s2tt*s2tt2*SB*stt2*TB2*Yt) - 
     -         16*MB2*(1 + 
     -            MB2*(Ab**2*CB*hb*ht*MA02*MSt2diff**2*s2tb2*s2tt*
     -               s2tt2*SB*TB2)) + 
     -         MB*(MB2 + MB2*
     -             (CB2*ht2*MSb2diff**2*MT*MT2*MUE2*s2tb2**2*s2tt2*
     -               (-x5 + x7))) + 
     -         MB2*(MB2 + MB**3*
     -             (MB2 + MSt22*MSt2diff**2*MUE2*s2tb2*s2tt*s2tt2*
     -                TB2**2*(-x5 + x7))) + 
     -         (x11 - x5)*(s2tb2*
     -             (MB2 + s2tt2*
     -                (-(CB*hb*ht*MSb2diff**2*(MSb12 - MSt22)*MT2*
     -                     SB*
     -                     (MT2*MUE2*s2tb2 + 
     -                       2*MB*
     -                       (At**2*s2tb*TB2*(stb2*Yb + ctb2*Yt))))
     -                    + CB2*ht2*MUE2*
     -                   (MSb2diff**2*
     -                      (MB2 + 
     -                       2*ctb2*(-MSb12 + MSt22)*MT*MT2*s2tb*Yt
     -                       ) + 
     -                     MB**3*
     -                      ((MSb12 - MSt12)*MSt2diff**2*TB2**2*
     -                       (MT*s2tt2 + 2*s2tt*stt2*Yt))))) + 
     -            MB2*(-(MSb2diff**2*s2tb2*
     -                  (CB*
     -                     (MB2 - 
     -                       8*MB*
     -                       (MSt2diff**2*MT*s2tb*s2tt*s2tt2*TB2))+
     -                      MT2*
     -                     (s2tb2*s2tt2*
     -                       (At**2*CB*hb*ht*(MSb12 - MSt22)*SB*
     -                       TB2 + MB*(CB2*ht2*MT*MUE2)) + 
     -                       s2tb*
     -                       (-s2tt + 
     -                       s2tt2*
     -                       (MB2*(2 + MT) + 
     -                       2*ctb2*Yt*
     -                       (CB2*ht2*MT*MUE2 + 
     -                       MB*(At**2*CB*hb*ht*SB*TB2))))))) + 
     -               MB*(MB2 + 
     -                  MSt2diff**2*s2tt2*TB2*
     -                   (MB*
     -                      (2*s2tb*s2tt + 
     -                       CB*hb*ht*(MSb12 - MSt12)*MUE2*s2tb2*
     -                       s2tt2*SB*TB2) + 
     -                     s2tb2*
     -                      (MB2 + 
     -                       MB2*
     -                       (MUE2*TB2*
     -                       (CB2*ht2*
     -                      (MT*s2tt2 + 2*s2tt*stt2*Yt) + MB2*s2tt)
     -                       ))))))))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tb2*s2tt*s2tt2*TB2)

        FAp = FAp + 1/(4.D0*MB)*
     -    ((x4 - x5)*(4*s2tt2*TB*TB2*
     -          (MB2*(Ab**2*hb*ht*MSt2diff**2*MT*MUE*
     -               (MB2 - MT2 + MUE2)*s2tb2*s2tt*(-x5 + x9)) + 
     -            s2tb*(MB*MB2*(Ab**2*MSt2diff**2*MT*s2tt) + 
     -               s2tb2*
     -                (MB2*(MSb2diff**2*MSt2diff**2*MT*s2tt) + 
     -                  MB*
     -                   (CB*hb*ht*MSb22*MSb2diff**2*SB*
     -                      (ctb2*
     -                       (MSt2diff**2*MT*s2tt*stt2 + 
     -                       2*At**2*MT2*(Xb - Yb)) + 
     -                       2*At**2*MT2*stb2*(Xt - Yt)) + 
     -                     MB2*(MSt2diff**2*MT*s2tt))))) + 
     -         s2tb2*(MB2 + 
     -            (-x5 + x7)*
     -             (MT2*(MB2 + 
     -                  s2tt2*
     -                   (MSb2diff*
     -                      (MB2 + 
     -                       CB2*MSt22*MUE2*TB*
     -                       (ht2*MSb2diff*MT*
     -                       (-2*s2tb*stb2*Yt + MB*s2tb2) - 
     -                       4*MB2*(hb2*MSt2diff*s2tb*s2tt*TB2)))+
     -                       CB*hb*ht*SB*
     -                      (MSb2diff**2*TB*
     -                       (MSt22*
     -                       (MT2*MUE2*s2tb2 - 
     -                       2*MB*
     -                       (At**2*s2tb*TB2*(ctb2*Yb + stb2*Yt)))-
     -                        2*MB*
     -                       (At**2*MSb22*s2tb*TB2*
     -                       (ctb2*Yb + stb2*Yt))) + 
     -                       TB2*
     -                       (MB2 - 
     -                       4*MB2*
     -                       ((-(Ab*At) + MUE2)**2*TB*
     -                       (-((MSb22 + MSt22)*Yb*Yt) + MB2*MSt22)
     -                       ))))) + 
     -               MB2*(MSb2diff*
     -                  (MSb2diff*TB*
     -                     (MT2*s2tt2*
     -                       (MSt22*
     -                       (MT*MUE2*s2tb + 
     -                       At**2*CB*hb*ht*s2tb2*SB*TB2) + 
     -                       CB2*ht2*MT*MUE2*
     -                       (-2*s2tb*stb2*Yt + MB*s2tb2)) + 
     -                       MB2*
     -                       (4 + MT + s2tt2 + 
     -                       4*MSt2diff**2*MT*s2tb*s2tt*s2tt2*TB2))
     -                      + 4*MB*
     -                     (MSt2diff*MT*s2tb*s2tt*s2tt2*TB2*
     -                       (MSt2diff*TB + 
     -                       At*CB*hb*ht*MSt22*MUE*SB*TB2))))))))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tb2*s2tt*s2tt2*TB*
     -       TB2)

        FAp = FAp + 1/(4.D0*MB)*
     -    ((x11 - x5)*(s2tb2*s2tt2*(-x5 + x6)*
     -          (MT2*(2*MB2 - 
     -               MA02*TB*
     -                (MSb2diff**2*
     -                   (MB2 + 
     -                     CB2*ht2*MT*MUE2*
     -                      (2*ctb2*s2tb*Yt + MB*s2tb2)) + 
     -                  CB*hb*ht*SB*
     -                   (MSb2diff**2*MT2*MUE2*s2tb2 + 
     -                     4*MB2*((-(Ab*At) + MUE2)**2*TB2*Yb*Yt)))
     -               ) + MB2*
     -             (MSb2diff*
     -               (MB2*(MSb2diff*(2*MSt2diff**2 + MUE2)*TB) + 
     -                 MT*(CB2*ht2*MSb2diff*MT2*MUE2*TB*
     -                     (2*ctb2*s2tb*Yt + MB*s2tb2) + 
     -                    4*MB*
     -                     (MSt2diff*s2tb*s2tt*TB2*
     -                       (MSt2diff*TB + 
     -                       Ab*MSb2diff*MSt2diff*TB + 
     -                       At*CB*hb*ht*MSt12*MUE*SB*TB2)))))) + 
     -         TB2*(4*TB*(MSt2diff**2*
     -                (MB2 + 
     -                  MB2*
     -                   (Ab**2*hb*ht*MT*MUE*(MB2 - MT2 + MUE2)*
     -                     s2tb2*s2tt*s2tt2*(x5 - x9))) + 
     -               At**2*MSb2diff**2*s2tb2*
     -                (MB2*
     -                   (2*MT*s2tb*s2tt + 
     -                     CB*hb*ht*MSb12*MT2*s2tb2*s2tt2*SB) + 
     -                  2*MB*
     -                   (CB*hb*ht*MSb12*MT2*s2tb*s2tt2*SB*
     -                     (stb2*Yb + ctb2*Yt)))) + 
     -            s2tt2*(-x5 + x7)*
     -             (s2tb2*(2*MB2 + 
     -                  MB**3*
     -                   ((MA02 - MB2 - MSt22)*MSt2diff**2*MUE2*TB*
     -                     TB2*
     -                     (CB2*ht2*(MT*s2tt2 - 2*ctt2*s2tt*Yt) + 
     -                       MB2*s2tt))) + 
     -               MB2*(TB*
     -                  (s2tb2*
     -                     (MB2 - 
     -                       2*
     -                       (MSb2diff*
     -                       (MB2 + 
     -                       hb2*(MA02 - MSt22)*MSt2diff*MT2*MUE2*
     -                       s2tb*s2tt*SB2) + 
     -                       CB*hb*ht*SB*
     -                       (MT2*
     -                       (MB2 + 
     -                       2*MUE2**2*
     -                       ((-MA02 + MSb12)*Yb*Yt + 
     -                       MSt22*(-2*Xb*Xt + Yb*Yt))) + 
     -                       Ab**2*
     -                       ((MA02 - MSt22)*MSt2diff**2*MT*s2tt*
     -                       (stt2*Yb + ctt2*Yt) + 
     -                       2*At**2*MT2*
     -                       ((MSb12 + MSt22)*Yb*Yt + 
     -                       MB2*(MSt22 + Yb)))))) + 
     -                    MB2*
     -                     (MSt2diff*
     -                       (2*MSb2diff*MT2*MUE2*s2tb*s2tb2*s2tt*
     -                       (CB2*ht2 + hb2*SB2) + 
     -                       MSt2diff*
     -                       (2*s2tb*s2tt + 
     -                       CB*hb*ht*s2tb2*SB*
     -                       ((MA02 - MSt22)*MUE2*s2tt2*TB2 + 
     -                       2*Ab**2*s2tt*
     -                       (-MA02 + MSt22 + ctt2*MT*Yt + MB2*MT))
     -                       )))))))))/
     -     (MSb2diff**2*MSt2diff**2*MT*s2tb*s2tb2*s2tt*s2tt2*TB*
     -       TB2)

        FA = At*(MB2 + 2*
     -       (MB2 + 2*MT2 + 
     -         (MT*((MGl2 - MSt22)*(2*MGl2 - 2*MSt22 + MSt2diff) + 
     -              (-4*(MGl2 + MSt22) + MSt2diff)*MT2 + 2*MT2**2)*
     -            s2tt)/(MGl*MSt2diff))*x47) + 
     -   (2*(MB2 + MGl*(MGl2 - MT2)*(MSt2diff + 2*At*MT*s2tt)*
     -          (x33 - x46)*(x5 - x7)) + 
     -      At*(MB2 + MSt2sum*s2tt2*
     -          (MSt2diff + MSt12*(-x5 + x6) + MSt22*(x5 - x7))*
     -          (x6 - x7)))/MSt2diff + 
     -   (x46 - x5)*(-2*(At + MGl)*MSt2diff*(x33 - x5) + 
     -      2*(MGl*MSt22 + 
     -         At*(-MB2 + MSt22 - MT2 + 
     -            (MGl*MSt2sum*MT*s2tt)/MSt2diff))*(-x5 + x7)) + 
     -   (-x5 + x6)*(At*(MB2 + 
     -         2*(-MB2 + MGl2 + MT2 - 
     -            (MGl*MSt2sum*MT*s2tt)/MSt2diff)*(x46 - x5)) + 
     -      (2*(MGl*(MGl2 - MT2)*(MSt2diff + 2*At*MT*s2tt)*
     -            (x33 - x46) + 2*At*MSt12*MSt22*(-x5 + x7)))/
     -       MSt2diff)

	dMA0 = 3/(16*pi**2)**2/(SB*CB)*(
     &    ht2*MUE/MSt2diff*(At*Ft + Cf*gs**2*Fts) +
     &    hb2*MUE/MSb2diff*(Ab*Fb + Cf*gs**2*Fbs) -
     &    2*ht*hb*FAp )
	end

