* TLasat.F
* Two-loop O(a_s a_t) corrections to the CP-even Higgs mass matrix.
* Routine written by P. Slavich (e-mail: slavich@mppmu.mpg.de).
* Based on G. Degrassi, P. Slavich, and F. Zwirner,
* Nucl. Phys. B611 (2001) 403 [hep-ph/0105096].
* this file is part of FeynHiggs
* last modified 30 Aug 04 th

* Revision history:
* 14/07/2004: rewritten completely by TH for FeynHiggs
* 24/02/2004: mg is given as input instead of mg^2;
*             value of pi corrected (10th digit);
*             unused variables cleaned up.
* 22/10/2002: gs is given as input.

#include "externals.h"


* dPhi(1..3) = corrections to the Higgs mass matrix
* where 1 = 11, 2 = 12, 3 = 22

	subroutine TLasat(dPhi)
	implicit none
	double precision dPhi(3)

#include "TL.h"

	double precision F1, s2ttF2, s2ttF3, F1o, F2o, F3o
	double precision s2ttAtdF2, s2ttAtdF3, s2ttdAt
	double precision dMTMT, dMSt12, dMSt22, ds2tt, k

	double precision x1, x6, x3, x4, x5, x7, x8, x9, x10, x13, x14
	double precision x11, x12
	double precision tmp1, tmp2, tmp3, tmp4, tmp5

	x1 = InvTri(MGl2,MT2,MSt12)

	x6 = InvTri(MGl2,MT2,MSt22)

	x3 = Log(MGl2)

	x4 = Log(MSt12)

	x5 = Log(MSt22)

	x7 = Log(MT2)

	x8 = Log(Q)

	x9 = Phi(MT2,MSt12,MGl2)

	x10 = Phi(MT2,MSt22,MGl2)

	x13 = myB0(MSt12,MT2,MGl2,Q)

	x14 = myB0(MSt22,MT2,MGl2,Q)

	x11 = myB0(MT2,MGl2,MSt12,Q)

	x12 = myB0(MT2,MGl2,MSt22,Q)

	tmp1 = MGl2 - MSt12 + MT2 - 2*MGl*MT*s2tt

	tmp2 = MGl2 - MSt22 + MT2 + 2*MGl*MT*s2tt

        F1 = -6 + (4*(MGl2 + MT2 - MGl*MT*s2tt) - MSt22*s2tt2)/
     -    MSt12 + (-2 + (4*MGl2*
     -         (2*MGl2*MT - MGl*(MGl2 - MSt12 + MT2)*s2tt)*x1)/MT)*
     -    (-x3 + x4) + c2tt2*(-2 + (x4 - x5)**2) + 
     -   (-x3 + x5)*(-2 + 4*MGl2*
     -       (2*MGl2 + (MGl*(MGl2 - MSt22 + MT2)*s2tt)/MT)*x6) + 
     -   (5 + 2*x5 - 2*x7)*(x4 - x7) + (x4 - x7)**2 + 
     -   5*(x5 - x7) + (x5 - x7)**2 + 
     -   x6*(4*MGl2*tmp2*x10 + 
     -      ((2*MGl2**2*MSt22*MT + 
     -           4*MGl2*MT2*
     -            (MT*(5*MSt22 + MT2) + MGl*(3*MSt22 + MT2)*s2tt)-
     -             2*(MSt22 - MT2)**2*
     -            (MSt22*MT + 2*MT2*(MT + MGl*s2tt)))*(-x3 + x7))/
     -       (MSt22*MT)) + 
     -   (4*(MGl2 + MT2 + MGl*MT*s2tt) - MSt12*s2tt2 + 
     -      ((-4*MSt2sum*(MGl2 + MT2) + 4*MGl*MSt2diff*MT*s2tt)*
     -         (x3 - x8))/MSt12)/MSt22 + 
     -   8*(-x3 + x7 + (x7 - x8)**2) + 
     -   (1 - (MSt2diff*s2tt2)/MSt22 + (4*MGl*s2tt*(x7 - x8))/MT)*
     -    (x4 - x8) + (-4 + (2*MGl*s2tt)/MT)*(x4 - x8)**2 + 
     -   (1 + (MSt2diff*s2tt2)/MSt12 - (4*MGl*s2tt*(x7 - x8))/MT)*
     -    (x5 - x8) + (-4 - (2*MGl*s2tt)/MT)*(x5 - x8)**2 + 
     -   6*(x7 - x8) + (4*MGl*s2tt*(x10 - x9))/MT + 
     -   x1*(((2*MGl2**2*MSt12*MT + 
     -           4*MGl2*MT2*
     -            (MT*(5*MSt12 + MT2) - MGl*(3*MSt12 + MT2)*s2tt)-
     -             2*(MSt12 - MT2)**2*
     -            (MSt12*MT + 2*MT2*(MT - MGl*s2tt)))*(-x3 + x7))/
     -       (MSt12*MT) + 4*MGl2*tmp1*x9)

        s2ttF2 = (-2*s2tt + 
     -      4*MGl2*((MGl2 - MSt12 + MT2)*s2tt - 2*MGl*MT*s2tt2)*x1)
     -     *(-x3 + x4) + (-x3 + x5)*
     -    (2*s2tt - 4*MGl2*
     -       ((MGl2 - MSt22 + MT2)*s2tt + 2*MGl*MT*s2tt2)*x6) + 
     -   x6*(-(((8*MGl*MGl2*MT*MT2*s2tt + 
     -             (MGl2 - MSt22 + MT2)**3*s2tt2)*x10)/(MGl*MT)) - 
     -      (4*(-((MSt22 - MT2)**2*(MT2*s2tt + MGl*MT*s2tt2)) + 
     -           MGl2*(MT2*(3*MSt22 + MT2)*s2tt + 
     -              MGl*MT*(MSt22 + MT2)*s2tt2))*(-x3 + x7))/MSt22)
     -     + ((4*MSt2diff*MT*(MGl2 + MT2)*s2tt - 
     -         (4*MGl*(3*MSt12*MSt22 + MSt2sum*MT2) + 
     -            MSt2diff*MSt2sum*MT*s2tt)*s2tt2 + 
     -         ((4*MSt2diff*
     -               (-(MSt2diff*MT*(MGl2 + MT2)*s2tt) + 
     -                 MGl*MSt2sum*MT2*s2tt2) + 
     -              MGl*MSt12*MSt22*(2*MGl2 + MSt2sum - 2*MT2)*
     -               s2tt2*(x4 - x5))*(x3 - x8))/MSt2diff)/MSt22 + 
     -      (((2*MGl*MSt12*(4*MSt22 + MSt2sum) + 
     -              MSt22*MSt2diff*MT*s2tt)*s2tt2 + 
     -           MGl*MSt12*
     -            (-8*c2tt2*MT2 + 
     -              (8*MT2 - (-2*MGl2 + MSt2sum + 10*MT2)*s2tt2)*
     -               (x7 - x8)))*(x5 - x8))/MSt2diff)/(MSt12*MT) + 
     -   (-3*s2tt + (MGl*MSt2sum*s2tt2)/(MSt2diff*MT))*
     -    (x4 - x8)**2 + (3*s2tt - 
     -      (MGl*MSt2sum*s2tt2)/(MSt2diff*MT))*(x5 - x8)**2 + 
     -   x1*((4*((MSt12 - MT2)**2*(-(MT2*s2tt) + MGl*MT*s2tt2) + 
     -           MGl2*(MT2*(3*MSt12 + MT2)*s2tt - 
     -              MGl*MT*(MSt12 + MT2)*s2tt2))*(-x3 + x7))/MSt12+
     -        ((8*MGl*MGl2*MT*MT2*s2tt - 
     -           (MGl2 - MSt12 + MT2)**3*s2tt2)*x9)/(MGl*MT)) + 
     -   (s2tt*(x4 - x5)*(5*(1 + c2tt2)*MSt2diff + 
     -         c2tt2*(MSt2sum*x4 - 2*MSt22*x5 - MSt2sum*x5 + 
     -            2*MSt12*(x4 - x8) + 2*MSt22*x8)) + 
     -      (((-((2*MGl*MSt22*(4*MSt12 + MSt2sum) + 
     -                   MSt12*MSt2diff*MT*s2tt)*s2tt2) + 
     -              MGl*MSt22*
     -               (8*c2tt2*MT2 + 
     -                 (-8*MT2 + 
     -                    (-2*MGl2 + MSt2sum + 10*MT2)*s2tt2)*
     -                  (x7 - x8)))*(x4 - x8))/MSt22 + 
     -         (8*c2tt2*MGl2*MT2*(-x10 + x9) + 
     -            s2tt2*((-2*x10)/x6 + (2*x9)/x1))/MGl)/MT)/
     -    MSt2diff

	tmp5 = -((x4 - x5)*(x4 + x5 - 2*x8))

	s2ttAtdF2 = 2*MGl*s2tt*tmp5

        tmp3 = MSt2diff*(-2*MSt2sum + 4*(MGl2 + MT2))*s2tt + 
     -   4*MGl*MSt2sum*MT*(2 - 3*s2tt2)

        tmp4 = MSt2diff*(MSt2sum - 2*(MGl2 + MT2))*s2tt + 
     -   2*MGl*MSt2sum*MT*(-2 + 3*s2tt2)

        s2ttF3 = s2tt*(10 + 18*c2tt2 - 
     -      4*(c2tt2 + (MSt12*(MGl2 + MT2))/(MSt22*MSt2diff)) + 
     -      ((4*MSt22*(MGl2 + MT2))/MSt2diff - 
     -         ((MSt12**2 + MSt22**2)*s2tt2)/MSt22)/MSt12) + 
     -   ((-2 + (2*(MSt22 - MT2))/MGl2 - (8*MT2)/MSt2diff)*s2tt - 
     -      (4*MT*(MGl2 - MSt22 + MT2)*(-2 + 3*s2tt2))/
     -       (MGl*MSt2diff))*x10 + 
     -   (MGl*MT*(64 + (-88 - (4*MSt12)/MSt22 - (4*MSt22)/MSt12)*
     -          s2tt2) - 3*MSt2sum*s2tt*(x4 - x5)*(x4 + x5 - 2*x8))
     -     /MSt2diff + x6*(4*MT2*
     -       (s2tt*tmp2*x10 + 
     -         (((-(MSt22 - MT2)**2 + MGl2*(MSt22 + MT2))*s2tt + 
     -              MGl*MT*(MGl2 + MSt22 - MT2)*s2tt2)*(x7 - x8))/
     -          MSt22) + (8*MGl2*MT2*s2tt + 
     -         4*MGl*MT*(MGl2 - MSt22 + MT2)*s2tt2)*(x5 - x8)) + 
     -   (x3 - x8)*(MGl2*(((-4*MSt2sum*s2tt)/MSt22 + 
     -            4*(-((MGl2 + MSt12 - MT2)*MT2*s2tt) - 
     -               MGl*MT*(-MGl2 + MSt12 + MT2)*s2tt2)*x1)/MSt12+
     -           (4*(-((MGl2 + MSt22 - MT2)*MT2*s2tt) + 
     -              MGl*MT*(-MGl2 + MSt22 + MT2)*s2tt2)*x6)/MSt22)+
     -        (2*tmp4*x4 + tmp3*x5 - tmp3*x8 - 2*tmp4*x8)/
     -       MSt2diff**2) + 
     -   ((-2 + (2*(MSt12 - MT2))/MGl2 + (8*MT2)/MSt2diff)*s2tt + 
     -      (MT*(MGl2 - MSt12 + MT2)*(8 - 12*s2tt2))/(MGl*MSt2diff)
     -      )*x9 + x1*((8*MGl2*MT2*s2tt - 
     -         4*MGl*MT*(MGl2 - MSt12 + MT2)*s2tt2)*(x4 - x8) + 
     -      4*MT2*((((-(MSt12 - MT2)**2 + MGl2*(MSt12 + MT2))*
     -               s2tt - MGl*MT*(MGl2 + MSt12 - MT2)*s2tt2)*
     -            (x7 - x8))/MSt12 + s2tt*tmp1*x9)) + 
     -   ((x5 - x8)*(-8*MGl2*MSt2diff*s2tt - 
     -         3*MSt22*MSt2diff*s2tt - 
     -         13*c2tt2*MSt22*MSt2diff*s2tt - 
     -         8*MSt2diff*MT2*s2tt + 
     -         (MSt22**2*MSt2diff*s2tt*s2tt2)/MSt12 - 
     -         2*MSt2diff**2*s2tt*s2tt2 + 
     -         8*MGl*MT*(-2 + 3*s2tt2)*
     -          (2*MSt22 + MSt2sum - (MGl2 - MT2)*(x3 - x7)) + 
     -         tmp3*x7 - tmp3*x8) + 
     -      (x4 - x8)*(8*MGl2*MSt2diff*s2tt + 
     -         3*MSt12*MSt2diff*s2tt + 
     -         13*c2tt2*MSt12*MSt2diff*s2tt + 
     -         8*MSt2diff*MT2*s2tt - 
     -         (MSt12**2*MSt2diff*s2tt*s2tt2)/MSt22 - 
     -         2*MSt2diff**2*s2tt*s2tt2 - 
     -         8*MGl*MT*(-2 + 3*s2tt2)*
     -          (2*MSt12 + MSt2sum - (MGl2 - MT2)*(x3 - x7)) + 
     -         2*tmp4*x7 - 2*tmp4*x8) + 
     -      MSt2sum*s2tt*(x4 - x5)*
     -       ((3 + 9*c2tt2)*MSt2diff + 
     -         c2tt2*(MSt2sum*x4 - 6*MSt22*x5 - MSt2sum*x5 + 
     -            6*MSt12*(x4 - x8) + 6*MSt22*x8)) + 
     -      ((MGl*MSt2diff*s2tt + 2*MGl2*MT*(-2 + 3*s2tt2))*
     -         ((-4*x10)/x6 + (4*x9)/x1))/(MGl*MGl2))/MSt2diff**2

        s2ttAtdF3 = (2*MGl*s2tt*
     -     (4*MSt2diff + MSt2sum*tmp5 + 4*MSt12*x4 - 4*MSt22*x5 - 
     -       4*MSt12*x8 + 4*MSt22*x8))/MSt2diff

* COUNTER TERMS:

	if( onshell ) then
	  F1o = x4 + x5 - 2*x7
	  F2o = x4 - x5
	  F3o = 2 + (MSt2sum*(x4 - x5))/MSt2diff

	  dMTMT = 1/2.D0*(MSt2sum + 2*MSDR*MT2 + tmp1*x11 + 
     -      tmp2*x12 - MSt12*x4 - MSt22*x5 + 6*MT2*x7 + 
     -      2*MGl2*(-1 + x3 - x8) + MSt12*x8 + MSt22*x8 - 6*MT2*x8)
     -     /MT2

	  dMSt12 = -2*(MGl2 + 3*MSt12 + MT2 + tmp1*x13) + 
     -     2*(MGl2*(x3 - x8) + MT2*(x7 - x8)) - 
     -     (-3 + c2tt2)*MSt12*(x4 - x8) + 
     -     s2tt2*(MSt2diff + MSt22*(-x5 + x8))

	  dMSt22 = -(MSt2diff*s2tt2) - 
     -     2*(MGl2 + 3*MSt22 + MT2 + tmp2*x14) + 2*MGl2*(x3 - x8) - 
     -     MSt12*s2tt2*(x4 - x8) - (-3 + c2tt2)*MSt22*(x5 - x8) + 
     -     2*MT2*(x7 - x8)

	  ds2tt = (-2*c2tt2*
     -     (4*MGl*MT*x13 + 
     -       s2tt*(MSt2diff + MSt12*(x4 - x8) + MSt22*(-x5 + x8))))
     -      /MSt2diff

	  k = s2tt*(dMSt12/MSt12 - dMSt22/MSt22)

* Eq. (37)
	  F1 = F1 + dMSt12/MSt12 + dMSt22/MSt22 - 4*dMTMT*(1 - F1o)

* Eq. (38)
	  s2ttF2 = s2ttF2 + k + (3*s2tt*dMTMT + ds2tt)*F2o

* Eq. (39)
	  s2ttF3 = s2ttF3 + 2*(s2tt*dMTMT + ds2tt)*F3o +
     &      k*(2*MSt12*MSt22/MSt2diff*F2o + MSt2sum)/MSt2diff

* Eq. (B8)
	  s2ttdAt = Xt*(ds2tt +
     &      s2tt*((dMSt22 - dMSt12)/MSt2diff - dMTMT))
	  s2ttAtdF2 = s2ttAtdF2 + s2ttdAt*F2o
	  s2ttAtdF3 = s2ttAtdF3 + s2ttdAt*F3o
	endif

	k = CF*Nc/(64*pi**3)*Alfas*ht2

* Eq. (25)
	dPhi(1) = k*.5D0*MUE2*s2tt*s2ttF3

* Eq. (26)
	dPhi(2) = -k*MUE*(.5D0*s2tt*(At*s2ttF3 + s2ttAtdF3) + MT*s2ttF2)

* Eq. (27)
	dPhi(3) = k*(s2tt*At*(.5D0*At*s2ttF3 + s2ttAtdF3) +
     &    2*MT*(At*s2ttF2 + s2ttAtdF2) + 2*MT2*F1)
	end

