! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version May 2007 !
! Tester file
! Useful to produce single models, or to check an existing chain point by point etc

  program tester

    use parameters
    use ParamDef
    use calclike

    implicit none

    
    Type(FLAGS) ::  Flag
    Type(Input_params) :: InP
    Type(ParamSet) :: P
    Type(Nuisance_params) :: InN  
    Type(Susy_spec) :: Spect
    Type(Prodxbr_Out) :: Prodxbr

    Type(BDecay_out) :: BDecay
    Type(DS_Output) :: DSO
    Type(Softout) :: Outsoft 

    logical :: Debug
    logical :: getlike = .true.

    !switch this to .true. if you want to read in the elements
    !of an existing chains one by one
    logical, parameter :: test_chain = .false.

    integer :: ierr, line
    real :: like, NewLike, mult
    real :: alpha_inv_difference
    
    real (8) :: beta, alpha, sbma2_th, DeltM, oh2_exp, mlsp, sigip_bound 
    real (8) :: sbma2_exp, mchar_exp,  sbma2_intp, d1mach
    real (8) ::  SinBmASq  

    ! Debugging the code
    Flag%Debug = .true.
    Feedback = 5
    ! Flags to switch on/off the calculation of parameters

    Flag%Collider_predict = .true.
    Flag%DM_predict  = .true. 
    Flag%DD_predict  = .true.
    Flag%ID_predict  = .false.
    Flag%FH_predict  = .true. ! Higgs detection prospects part
    Flag%BD_predict  = .true.

    !Flags for likelihood computation
    Use_CDM = .true.
    Use_Nuisance = .true.
    Use_LEP = .true.
    Use_Anomalous_Mu = .true.
    Use_bsgamma = .true.
    Use_Bsmumu = .true.
    Use_Mass_W = .true.
    Use_Weak_Mixing = .true.
    Use_Bdecay = .true.

    Use_DD = .false.
    if (.not. test_chain) then
     !---------------------------------------------------------
     ! CMSSM parameters - set them by hand to the value you want
     !---------------------------------------------------------
     InP%m0 = 1000.d0
     InP%mhalf = 1000.d0
     InP%a0 = -1000.d0
     InP%tanb = 50.d0
     !fixed 
     InP%sgnmu0 = 1
     InP%scaledep = 1.0d0
     ! Nuisance parameters.
     InN%mbottom = 4.20d0
     InN%mtop = 170.9d0
     InN%alphasmz = 0.1187d0
     InN%alphamz = 1.d0/(127.918d0)

     !this is the number dumped in the chains
     !alpha_inv_difference =  -0.19737E-01 
     !InN%alphamz = 1.d0/(alpha_inv_mean + (alpha_inv_difference))   
     !InN%alphamz =   7.817508091120874E-003
     !alpha_inv_difference =  -0.1045837
     !alpha_em_inv = alpha_EM^-1 - 127.958

     !Fixed Nuisance params
     !those are fixed for the moment
     InN%ftp_s = 0.118 
     InN%ftn_s = 0.118   
     goto 111

  else
     !---------------------------------------------------------
     !chain reprocessing - useful for all sorts of checks
     !---------------------------------------------------------
     write(*,*) ' Opening chain'
     !put here by hand the chain's filename
     open(33, file='chains/test_v5_1.txt', status='old', iostat=ierr)
     if (ierr < 0) stop 'problem opening test chain'
     line = 0
     !put here the output file name
     open(outfile_unit, file="out_v3.tester", status='replace')
     do 
        read(33, *, iostat=ierr) mult, like, P%P
        if(ierr < 0) exit
        line = line + 1
        write(*,*) 'reading in line ', line
        use_log = .false.
        !converting the value of alpha_EM_inv, which has been read in
        call ConvertFromReading(P)
        call ParamsToMSSMParams(P%P, InP, InN)

111     if (Flag%Debug) then
           write(*,*) '>>>>  Input Params - read in and converted'
           write(*,*) 'm0  param    : ', InP%m0 
           write(*,*) 'mhalf  param : ', InP%mhalf
           write(*,*) 'a0 param     : ', InP%a0
           write(*,*) 'tanb param   : ', InP%tanb
           write(*,*) 'sgnmu0 param : ', InP%sgnmu0
           write(*,*) 'scale dep    : ', InP%scaledep
           write(*,*) '>>>> Nuisance Params'
           write(*,*) 'm bottom     : ', InN%mbottom 
           write(*,*) 'm top        : ', InN%mtop
           write(*,*) 'alpha_S      : ', InN%alphasmz
           write(*,*) '1/alpha_em   : ', 1d0/InN%alphamz
           write(*,*) 'alpha_em     : ', InN%alphamz
           write(*,*) '>>>> Fixed Nuisance Params'
           write(*,*) 'ftp_s        : ', InN%ftp_s
           write(*,*) 'ftp_n        : ', InN%ftn_s
        end if

        if (Flag%Debug) write(*,*) 'Now calling softsusy interface... '


        call softsusy(Flag,InP, InN, Spect, Outsoft, Leshouches, DSO%err)
        !here I compute the ratio of the couplings
        beta = atan(InP%tanb)
        alpha = Outsoft%higg%halpha
        SinBmASq = sin(beta-alpha)**2.d0
 
        if (Flag%Debug) then

           write(*,'(A,F10.2)') '>>> Softsusy intermediate spectrum'
           write(*,'(A,F10.2)') '    >>> Lights Chargino mass   : ', Spect%mch1
           write(*,'(A,F10.2)') '    >>> Lights Higgs    mass   : ', Spect%mlh
           write(*,'(A,F10.2)') '    >>> sin(a-b)^2             : ', SinBmASq
           write(*,'(A,F10.2)') '    >>> Lights neutralino mass : ', Spect%mneut1
           write(*,'(A,F10.2)') '>>> Softsusy: new sparticle masses '
           write(*,'(A,F10.2)') '    >>>  s electron R  mass    : ', Spect%mseR
           write(*,'(A,F10.2)') '    >>>  s muon R mass         : ', Spect%msmuR   
           write(*,'(A,F10.2)') '    >>>  s tau 1 mass          : ', Spect%mstau1   
           write(*,'(A,F10.2)') '    >>>  tau sneutrino mass    : ', Spect%msntau 
           write(*,'(A,F10.2)') '    >>>  stop 1 mass           : ', Spect%mst1
           write(*,'(A,F10.2)') '    >>>  sbottom 1 mass        : ', Spect%msb1
           write(*,'(A,F10.2)') '    >>>  Lights squark mass    : ', min(Spect%mscR,Spect%mssR,Spect%msuR,Spect%msdR)
           write(*,'(A,F10.2)') '    >>>  Gluino mass           : ', Spect%mgluino
           write(*,'(A,F10.2)') '>>> Softsusy Out Soft'
           write(*,'(A,F10.2)') '    >>> Lgts neutralino        : ', minval(abs(OutSoft%neut%mneut))
        end if

       
        if (Flag%Debug) write(*,*) 'Now calling DS interface...'  
        call ds(Flag, InP, InN, Outsoft, DSO)

        if (Flag%Debug) write(*,*) 'Warning from DS: ', DSO%PrintOutParams


        if (Flag%BD_Predict) then
           if (Flag%Debug) write(*,*) 'Now calling BDecay interface...'  
           call bd(Flag, InP, InN, Outsoft, BDecay, DSO%err)
        end if


        if (Flag%FH_Predict) then
           if (Flag%Debug) write(*,*) 'Now calling FH interface...' 
           call FeynHiggs(Flag, DSO%err, Leshouches, Prodxbr)
        end if
        

        if (Flag%Debug) then
           write(*,*) 'Writing physical spectrum in spectrum.out'
           call WriteDebug(Flag, InP, InN, Spect, DSO, Prodxbr, BDecay, Outsoft)
        endif

        if (.not. test_chain) then
           !converting back to Params
           call  MSSMParamsToParams(InP, P%P, InN)
        end if
 
        PreviousOutputP = DSO
        PreviousSusySp  = Spect
        PreviousSoftOut = Outsoft
        PreviousProdxbr = Prodxbr
        PreviousBdecay  = Bdecay


        if (Flag%DD_predict) then

           mlsp =minval(abs(Outsoft%neut%mneut)) 
           !mass of the lightest neutralino
           !the third argument in function "sigip_exp" is an integer 
           !which sets the DD bounds for a given experiment.
           !1: CDMS-II, 2: ZEPLIN-I and 3: EDELWEISS-I 
           call sigip_exp(mlsp, sigip_bound,3)
    
           if (Flag%Debug) then
              print*, 'Max. of the spin-indep nucleon cross sections: ', &
                   DSO%DD%sigsi_nuc
              print*, 'Exp. upper bound ', sigip_bound
           end if
        end if
        !------------------------------------------
        !computes likelihood
        !-----------------------------------------

        if (getlike) then
           !set scales
           Scales%PMin = -1E4
           Scales%PMax = 1E4
           future_errors = .false.
           GFlags = Flag 
           !Gflags are used in getloglike
           write(*,*) 'Getting Likelihood...'
           NewLike = GetLogLike(P)
              write(*,'(A,E15.5)') ' New   -lnlike: ', NewLike
           if (test_chain) then
              write(*,'(A,E15.5)') ' Old   -lnlike: ', like
              write(*,'(A,E15.5)') ' Delta -lnlike: ', NewLike - like
           end if
           Like = NewLike
        end if


        call SetFormat

        call WriteParams(P, mult, like)


        if (.not. test_chain) then
           close (33)
           close (outfile_unit)
           stop !one model only and stop
        end if

     end do !loops over chain elements
  end if
  close (33)
  close (outfile_unit)

end program tester

