!module containing all global types definitons for variables
!This version May 2007 
!SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)

module parameters


 integer, parameter:: lmax = 200

!**********************************************
! NEW TYPES DEFINITIONS
!**********************************************
!--------- Input parameters in SoftSusy ---------
! So far only the CMSSM is implemented 
 Type Input_params
   real*8 :: m0, mhalf, a0, tanb, scaledep
   integer :: sgnmu0    ! CMSSM input parameters
 end Type Input_params

 character(LEN=lmax), dimension(6):: CMSSMNames = (/ 'm_0 (GeV)', 'm_{1/2} (GeV)', 'A_0 (GeV)', 'tan \beta', 'scale', 'sgn(\mu)' /)
 logical, dimension(6), parameter :: CMSSMUsed = (/ .true. , .true., .true., .true., .false., .true. /)


 Type Nuisance_params
   real (8) :: mbottom, mtop, alphamz, alphasmz, ftp_s, ftn_s
 end Type Nuisance_params

 character(LEN=lmax), dimension(6):: NuisanceNames = (/ 'm_b(m_b) (GeV)', 'm_t (GeV)', '\alpha_s', '\alpha_{em}', 'ft^p_s', 'ft^n_s'  /)
 logical, dimension(6), parameter :: NuisanceUsed = (/ .true. , .true., .true., .true., .false., .false. /)

 character(LEN=lmax), dimension(12):: ParamsNames

!---------  Output from SoftSusy ------------
!Higgs sector
 Type Higgs
   real (8) :: mlh, mhh, ma, mch, halpha
 end Type Higgs

 character(LEN=lmax), dimension(5):: HiggsNames = (/ 'm_h (GeV)', 'm_H (GeV)', 'm_A (GeV)', 'm_{H^\pm} (GeV)', '\alpha' /)


!Sfermion sector
 Type sfermions
   real (8) :: msu(3,2), msd(3,2), mse(3,2)
   real (8) :: msnu(3)
   real (8) :: thetat, thetab, thetatau
 end Type sfermions


!Neutralino sector
 Type neutralino
   real (8) :: mneut(4)
   real (8) :: mixchi(4,4)      
 end Type neutralino

 character(LEN=lmax), dimension(4):: NeutralinoNames = (/ 'm_{\chi_1^0} (GeV)', 'm_{\chi_2^0} (GeV)', 'm_{\chi_3^0} (GeV)', 'm_{\chi_4^0} (GeV)'  /)  
  
 character(LEN=lmax), dimension(16):: NeutralinoMixNames =  (/ 'N_{11}', 'N_{12}', 'N_{13}', 'N_{14}', 'N_{21}', 'N_{22}', 'N_{23}', 'N_{24}', 'N_{31}', 'N_{32}', 'N_{33}', 'N_{34}', 'N_{41}', 'N_{42}', 'N_{43}', 'N_{44}'  /)


!Chargino sector
 Type chargino
   real (8) :: mcha(2)
   real (8) :: thetaL, thetaR
 end Type chargino

 character(LEN=lmax), dimension(2):: CharginoNames = (/ 'm_{\chi_1^\pm} (GeV)', 'm_{\chi_2^\pm} (GeV)'/)
 character(LEN=lmax), dimension(2):: CharginoMixNames = (/ '\theta_L', '\theta_R' /)


!Running masses and Yuk. couplings of the third gen. of fermions
 Type running
   real(8) :: mtaumz, mbmz, mtmt, mbpole
   real(8) :: mtpole, ylsp_tau, ylsp_b, ylsp_top
 end Type running

 character(LEN=lmax), dimension(8):: RunningNames= (/ 'm_\tau(m_Z) (GeV)', 'm_b(m_Z) (GeV)', 'm_t(m_t) (GeV)', 'm_b (GeV)', 'm_t (GeV)', 'Y_tau(2 m_\chi)', 'Y_b(2 m_\chi)','Y_t(2 m_\chi)' /)


!SM parameters
 Type standard
   real (8) :: mUp, mCharm, mDown, mStrange
   real (8) :: mElectron, mMuon, mTau
 end Type standard

 character (LEN=lmax), dimension(7):: StandardNames= (/ 'm_u (GeV)', 'm_c (GeV)', 'm_d (GeV)', 'm_s (GeV)', 'm_e (GeV)', 'm_\muon (GeV)', 'm_\tau (GeV)' /)

!soft terms
 Type Softpar
   real (8) :: mu, m1, m2
   real (8) :: asofte(3), asoftu(3), asoftd(3) & 
             , mass2l(3), mass2e(3), mass2q(3) & 
	     , mass2u(3), mass2d(3)
  end Type Softpar

 character (LEN=lmax), dimension(3):: SoftGaugNames=  (/ '\mu (GeV)', 'm_1 (GeV)', 'm_2 (GeV)' /)
 
 character (LEN=lmax), dimension(9):: SoftTrilNames=  (/ 'A_E(1) (GeV)', 'A_E(2) (GeV)', 'A_E(3) (GeV)', 'A_U(1) (GeV)', 'A_U(2) (GeV)', 'A_U(3) (GeV)', 'A_D(1) (GeV)', 'A_D(2) (GeV)', 'A_D(3) (GeV)'/)

 character (LEN=lmax), dimension(15):: SoftMassNames=  (/ 'm^2_{\tilde L}(1) (GeV)', 'm^2_{\tilde L}(2) (GeV)', 'm^2_{\tilde L}(3) (GeV)', 'm^2_{\tilde E}(1) (GeV)', 'm^2_{\tilde E}(2) (GeV)', 'm^2_{\tilde E}(3) (GeV)', 'm^2_{\tilde Q}(1) (GeV)', 'm^2_{\tilde Q}(2) (GeV)', 'm^2_{\tilde Q}(3) (GeV)', 'm^2_{\tilde U}(1) (GeV)', 'm^2_{\tilde U}(2) (GeV)', 'm^2_{\tilde U}(3) (GeV)', 'm^2_{\tilde D}(1) (GeV)', 'm^2_{\tilde D}(2) (GeV)', 'm^2_{\tilde D}(3) (GeV)' /) 


!Soft parameters
 Type Softout
   Type(standard) :: sm
   Type(Softpar) :: soft
   Type(Higgs) :: higg
   Type(sfermions) :: sfer
   Type(neutralino) :: neut
   Type(chargino) :: charg
   Type(running) :: run
   real (8) :: mgluino
 End Type Softout


! Some physical masses - intermediate spectrum
 Type Susy_spec
   real (8) :: mlh, mhh, mA, mhc &
        , mch1, mch2, mneut1, mneut2, mneut3, mneut4 & 
        , mseR, mseL, msmuR, msmuL, mstau1, mstau2 &
        , msne, msnmu, msntau & 
        , msuR, msuL, mscR, mscL, mst1, mst2 &
	, msdR, msdL, mssR, mssL, msb1, msb2 &
        , mgluino
   real(8) ::  SinBmASq 
 end Type Susy_spec

 character (LEN=lmax), dimension(33):: SusySpecNames= (/ 'm_h (GeV)', 'm_H (GeV)', 'm_A (GeV)', 'm_{H^\pm} (GeV)', 'm_{\chi_1^\pm} (GeV)', 'm_{\chi_2^\pm} (GeV)', 'm_{\chi_1^0} (GeV)', 'm_{\chi_2^0} (GeV)', 'm_{\chi_3^0} (GeV)', 'm_{\chi_4^0} (GeV)','m_{\tilde{e}_R} (GeV)', 'm_{\tilde{e}_L} (GeV)', 'm_{\tilde{\mu}_R} (GeV)', 'm_{\tilde{\mu}_L} (GeV)', 'm_{\tilde{\tau}_1} (GeV)', 'm_{\tilde{\tau}_2} (GeV)', 'm_{\tilde{\nu}_e} (GeV)', 'm_{\tilde{\nu}_\mu} (GeV)', 'm_{\tilde{\nu}_\tau} (GeV)','m_{\tilde{u}_R} (GeV)', 'm_{\tilde{u}_L} (GeV)','m_{\tilde{s}_R} (GeV)', 'm_{\tilde{s}_L} (GeV)', 'm_{\tilde{t}_1} (GeV)', 'm_{\tilde{t}_2} (GeV)', 'm_{\tilde{d}_R} (GeV)', 'm_{\tilde{d}_L} (GeV)','m_{\tilde{c}_R} (GeV)', 'm_{\tilde{c}_L} (GeV)', 'm_{\tilde{b}_1} (GeV)', 'm_{\tilde{b}_2} (GeV)', 'm_{\tilde{g}} (GeV)', 'sin(\beta-\alpha)^2' /)
    


!--------- Output from Dark Susy ---------------

!Relic Density 
 Type DM_Out
   real (8) :: oh2 = 0d0, gf = 0d0     ! relic density and gaugino fraction
 end Type DM_Out

 character(LEN=lmax), dimension(2):: DMNames= (/ '\Omega_{\chi}h^2', 'g_f' /)

!Direct Detection rates
 Type DD_Out
   real (8) ::  sigsi_nuc=0d0, sigsdp=0d0, sigsdn=0d0  ! direct detection cross sections
 end Type DD_OUT

 character(LEN=lmax), dimension(3):: DDNames= (/ '\sigma_p^{SI} (pb)', '\sigma_p^{SD} (pb)', '\sigma_n^{SD} (pb)' /)


!Indirect Detection rates
 Type ID_Out
   real (8) :: phiep=0d0, fluxgac=0d0, pb_b=0d0, rateea=0d0, ratesu=0d0  ! indirect detection
 end Type ID_OUT

 character(LEN=lmax), dimension(5):: IDNames= (/ '\Phi_{e+}', '\Phi_\gamma', '\Phi_{\bar p}', '\Phi_\mu^e', '\Phi_\mu^s'/)


!Some collider stuff
 Type Collider_Out
    real (8) :: gm2amp= 0d0   ! g-2 amplitude
    real (8) :: bsgamma= 0d0  ! bsgamma is the new comp. 
    real (8) :: brbsmm = 0d0  ! Bs -> mu+ mu-
    real (8) :: brr= 0d0  ! (BR(h->bb)/BR(h->WW))_MSSM/(BR(h->bb)/BR(h->WW))_SM
    integer  :: sgn_MSSMoSM = 0!relative sign of bsgamma amplitude
    real (8) :: mw = 0d0  !W gauge boson mass
    real (8) :: sweff2 = 0d0 !sin of the effective lept. angle
 end Type Collider_Out

 character(LEN=lmax), dimension(7):: ColliderNames= (/ '\delta a_{\mu}^{SUSY}', 'BR(B->X_s\gamma)', 'BR(B_s->\mu^+\mu^-)', 'Brr','sgn()','m_W (GeV)','sin^2\theta_{eff}' /)


!---------- Error messages from SoftSusy and Dark Susy about physical 
!and unphysical points ------------
 Type Error_Out
    integer :: unphys !set by DS when neutralino is not the LSP
    integer :: oh2 !problems with calculation of odmh2
    !all the other are set by softsust, see doc/cmssm.ps for explantions
    integer :: muSqWrongSign, b, noRhoConvergence, tachyon 
    integer :: nonperturbative,noMuConvergence, higgsUfb, irqfp
    integer :: badConvergence
    integer :: fh
    integer :: bd
 end Type Error_Out


!All output from DS is collected in one single type
!for practical purposes when manipulating things
 Type DS_Output
    !IF YOU CHANGE THIS REMEMBER TO ADJUST WriteParams FORMAT BELOW !
    Type(DM_Out) :: DM
    Type(DD_Out) :: DD
    Type(ID_Out) :: ID
    Type(Collider_Out) :: Coll
    Type(Error_Out) :: Err
    logical :: PrintOutParams
 end Type DS_Output


!-------- Some flags to decide what to compute ------------
 Type Flags
   logical :: Debug, Collider_predict, DM_predict
   logical :: DD_rescal
   logical :: DD_predict, ID_predict,  FH_predict, BD_predict
 End Type Flags

!----------- FeynHiggs ---------------------
 Type Wh_cha
  real(8) :: sigma_Tev(3), sigmaSM_Tev(3), bbar_Tev(3) &
            , bbarSM_Tev(3), bbar_Tev_r(3)
 end Type Wh_cha
 
 character(LEN=lmax), dimension(15):: FHWhNames= (/ '\sigma (q \bar{q}->W h)_{CMSSM} (fb)', '\sigma (q \bar{q}->W H)_{CMSSM} (fb)', '\sigma (q \bar{q}->W A)_{CMSSM} (fb)', '\sigma (q \bar{q}->W h)_{SM} (fb)', '\sigma (q \bar{q}->W H)_{SM} (fb)', '\sigma (q \bar{q}->W A)_{SM} (fb)', '[\sigma (q \bar{q}->W h)*BR(h->b \bar{b})]_{CMSSM} (fb)', '[\sigma (q \bar{q}->W H)*BR(H->b \bar{b})]_{CMSSM} (fb)', '[\sigma (q \bar{q}->W A)*BR(A->b \bar{b})]_{CMSSM} (fb)','[\sigma (q \bar{q}->W h)*BR(h->b \bar{b})]_{SM} (fb)', '[\sigma (q \bar{q}->W H)*BR(h->b \bar{b})]_{SM} (fb)', '[\sigma (q \bar{q}->W A)*BR(h->b \bar{b})]_{SM} (fb)','[\sigma (q \bar{q}->W h)*BR(h->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->W h)*BR(h->b \bar{b})]_{SM}', '[\sigma (q \bar{q}->W H)*BR(H->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->W H)*BR(H->b \bar{b})]_{SM}', '[\sigma (q \bar{q}->W A)*BR(A->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->W A)*BR(A->b \bar{b})]_{SM}' /) 


 Type Zh_cha
  real(8) :: sigma_Tev(3), sigmaSM_Tev(3), bbar_Tev(3) & 
            , bbarSM_Tev(3), bbar_Tev_r(3) 
 end Type Zh_cha

 character(LEN=lmax), dimension(15):: FHZhNames= (/ '\sigma (q \bar{q}->Z h)_{CMSSM} (fb)', '\sigma (q \bar{q}->Z H)_{CMSSM} (fb)', '\sigma (q \bar{q}->Z A)_{CMSSM} (fb)', '\sigma (q \bar{q}->Z h)_{SM} (fb)', '\sigma (q \bar{q}->Z H)_{SM} (fb)', '\sigma (q \bar{q}->Z A)_{SM} (fb)', '[\sigma (q \bar{q}->Z h)*BR(h->b \bar{b})]_{CMSSM} (fb)', '[\sigma (q \bar{q}->Z H)*BR(H->b \bar{b})]_{CMSSM} (fb)', '[\sigma (q \bar{q}->Z A)*BR(A->b \bar{b})]_{CMSSM} (fb)','[\sigma (q \bar{q}->Z h)*BR(h->b \bar{b})]_{SM} (fb)', '[\sigma (q \bar{q}->Z H)*BR(h->b \bar{b})]_{SM} (fb)', '[\sigma (q \bar{q}->Z A)*BR(h->b \bar{b})]_{SM} (fb)','[\sigma (q \bar{q}->Z h)*BR(h->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->Z h)*BR(h->b \bar{b})]_{SM}', '[\sigma (q \bar{q}->Z H)*BR(H->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->Z H)*BR(H->b \bar{b})]_{SM}', '[\sigma (q \bar{q}->Z A)*BR(A->b \bar{b})]_{CMSSM}/[\sigma (q \bar{q}->Z A)*BR(A->b \bar{b})]_{SM}' /) 


 Type bbh_cha
  real(8) :: sigma_Tev(3), sigmaSM_Tev(3), bbar_Tev(3) &
            , bbarSM_Tev(3), bbar_Tev_r(3) 
 end Type bbh_cha

 character(LEN=lmax), dimension(15):: FHbbhNames= (/ '\sigma (b \bar{b}->h)_{CMSSM} (fb)', '\sigma (b \bar{b}->H)_{CMSSM} (fb)', '\sigma (b \bar{b}->A)_{CMSSM} (fb)', '\sigma (b \bar{b}->h)_{SM} (fb)', '\sigma (b \bar{b}->H)_{SM} (fb)', '\sigma (b \bar{b}->A)_{SM} (fb)', '[\sigma (b \bar{b}->h)*BR(h->b \bar{b})]_{CMSSM} (fb)', '[\sigma (b \bar{b}->H)*BR(H->b \bar{b})]_{CMSSM} (fb)', '[\sigma (b \bar{b}->A)*BR(A->b \bar{b})]_{CMSSM} (fb)','[\sigma (b \bar{b}->h)*BR(h->b \bar{b})]_{SM} (fb)', '[\sigma (b \bar{b}->H)*BR(H->b \bar{b})]_{SM} (fb)', '[\sigma (b \bar{b}->A)*BR(A->b \bar{b})]_{SM} (fb)','[\sigma (b \bar{b}->h)*BR(h->b \bar{b})]_{CMSSM}/[\sigma (b \bar{b}->h)*BR(h->b \bar{b})]_{SM}', '[\sigma (b \bar{b}->H)*BR(H->b \bar{b})]_{CMSSM}/[\sigma (b \bar{b}->H)*BR(H->b \bar{b})]_{SM}', '[\sigma (b \bar{b}->A)*BR(A->b \bar{b})]_{CMSSM}/[\sigma (b \bar{b}->A)*BR(A->b \bar{b})]_{SM}'/) 


 Type ggh_cha
  real(8) :: sigma_Tev(3), sigmaSM_Tev(3), sigma_LHC(3), sigmaSM_LHC(3) &
            , gamgam_LHC(3), gamgamSM_LHC(3), gamgam_LHC_r(3)
 end Type ggh_cha


 character(LEN=lmax), dimension(21):: FHgghNames= (/ '\sigma (g g->h)_{CMSSM} (fb)', '\sigma (g g->H)_{CMSSM} (fb)', '\sigma (g g->A)_{CMSSM} (fb)', '\sigma (g g->h)_{SM} (fb)', '\sigma (g g->H)_{SM} (fb)', '\sigma (g g->A)_{SM} (fb)', '\sigma (g g->h)_{CMSSM} (fb)', '\sigma (g g->H)_{CMSSM} (fb)', '\sigma (g g->A)_{CMSSM} (fb)', '\sigma (g g->h)_{SM} (fb)', '\sigma (g g->H)_{SM} (fb)', '\sigma (g g->A)_{SM} (fb)', '[\sigma (g g->h)*BR(h->\gamma \gamma)]_{CMSSM} (fb)', '[\sigma (g g->H)*BR(H->\gamma \gamma)]_{CMSSM} (fb)', '[\sigma (g g->A)*BR(A->\gamma \gamma)]_{CMSSM} (fb)','[\sigma (g g->h)*BR(h->\gamma \gamma)]_{SM} (fb)', '[\sigma (g g->H)*BR(H->\gamma \gamma)]_{SM} (fb)', '[\sigma (g g->A)*BR(A->\gamma \gamma)]_{SM} (fb)','[\sigma (g g->h)*BR(h->\gamma \gamma)]_{CMSSM}/[\sigma (g g->h)*BR(h->\gamma \gamma)]_{SM}', '[\sigma (g g->H)*BR(H->\gamma \gamma)]_{CMSSM}/[\sigma (g g->H)*BR(H->\gamma \gamma)]_{SM}', '[\sigma (g g->A)*BR(A->\gamma \gamma)]_{CMSSM}/[\sigma (g g->A)*BR(A->\gamma \gamma)]_{SM}' /) 



 Type tth_cha
  real(8) :: sigma_LHC(3), sigmaSM_LHC(3), bbar_LHC(3) &
            , bbarSM_LHC(3), bbar_LHC_r(3)
 end Type tth_cha


 character(LEN=lmax), dimension(15):: FHtthNames= (/ '\sigma (q q->tt h)_{CMSSM} (fb)', '\sigma (q q->tt H)_{CMSSM} (fb)', '\sigma (q q->tt A)_{CMSSM} (fb)', '\sigma (q q->tt h)_{SM} (fb)', '\sigma (q q->tt H)_{SM} (fb)', '\sigma (q q->tt A)_{SM} (fb)', '[\sigma (q q->tt h)*BR(h->b \bar{b} )]_{CMSSM} (fb)', '[\sigma (q q->tt H)*BR(H->b \bar{b} )]_{CMSSM} (fb)', '[\sigma (q q->tt A)*BR(A->b \bar{b} )]_{CMSSM} (fb)','[\sigma (q q->tt h)*BR(h->b \bar{b} )]_{SM} (fb)', '[\sigma (q q->tt H)*BR(H->b \bar{b} )]_{SM} (fb)', '[\sigma (q q->tt A)*BR(A->b \bar{b} )]_{SM} (fb)','[\sigma (q q->tt h)*BR(h->b \bar{b} )]_{CMSSM}/[\sigma (q q->tt h)*BR(h->b \bar{b} )]_{SM}', '[\sigma (q q->tt H)*BR(H->b \bar{b} )]_{CMSSM}/[\sigma (q q->tt H)*BR(H->b \bar{b} )]_{SM}', '[\sigma (q q->tt A)*BR(A->b \bar{b} )]_{CMSSM}/[\sigma (q q->tt A)*BR(A->b \bar{b} )]_{SM}' /) 



 Type qqh_cha
  real(8) ::  sigma_LHC(3), sigmaSM_LHC(3), tautau_LHC(3) &
            , tautauSM_LHC(3), tautau_LHC_r(3)
 end Type qqh_cha


 character(LEN=lmax), dimension(15):: FHqqhNames= (/ '\sigma (q q->qq h)_{CMSSM} (fb)', '\sigma (q q->qq H)_{CMSSM} (fb)', '\sigma (q q->qq A)_{CMSSM} (fb)', '\sigma (q q->qq h)_{SM} (fb)', '\sigma (q q->qq H)_{SM} (fb)', '\sigma (q q->qq A)_{SM} (fb)', '[\sigma (q q->qq h)*BR(h->b \bar{b} )]_{CMSSM} (fb)', '[\sigma (q q->qq H)*BR(H->b \bar{b} )]_{CMSSM} (fb)', '[\sigma (q q->qq A)*BR(A->b \bar{b} )]_{CMSSM} (fb)','[\sigma (q q->qq h)*BR(h->b \bar{b} )]_{SM} (fb)', '[\sigma (q q->qq H)*BR(H->b \bar{b} )]_{SM} (fb)', '[\sigma (q q->qq A)*BR(A->b \bar{b} )]_{SM} (fb)','[\sigma (q q->qq h)*BR(h->b \bar{b} )]_{CMSSM}/[\sigma (q q->qq h)*BR(h->b \bar{b} )]_{SM}', '[\sigma (q q->qq H)*BR(H->b \bar{b} )]_{CMSSM}/[\sigma (q q->qq H)*BR(H->b \bar{b} )]_{SM}', '[\sigma (q q->qq A)*BR(A->b \bar{b} )]_{CMSSM}/[\sigma (q q->qq A)*BR(A->b \bar{b} )]_{SM}'/) 


 Type bh_cha
   real(8) :: sigma_Tev(3), sigmaSM_Tev(3), sigma_LHC(3), sigmaSM_LHC(3) 
 end Type bh_cha


 character(LEN=lmax), dimension(12):: FHbhNames= (/'\sigma (b g->b h)_{CMSSM} (fb)', '\sigma (b g->b H)_{CMSSM} (fb)', '\sigma (b g->b A)_{CMSSM} (fb)', '\sigma (b g->b h)_{SM} (fb)', '\sigma (b g->b H)_{SM} (fb)', '\sigma (b g->b A)_{SM} (fb)', '\sigma (b g->b h)_{CMSSM} (fb)', '\sigma (b g->b H)_{CMSSM} (fb)', '\sigma (b g->b A)_{CMSSM} (fb)', '\sigma (b g->b h)_{SM} (fb)', '\sigma (b g->b H)_{SM} (fb)', '\sigma (b g->b A)_{SM} (fb)' /)


 Type decay
  real(8) :: BRtobbar(3),  BRtobbarSM(3), BRtogamgam(3), BRtogamgamSM(3) &
        , BRtotautau(3), BRtotautauSM(3), BRtoWW(3), BRtoWWSM(3)
 end Type decay


 character(LEN=lmax), dimension(24):: FHBRNames= (/'BR(h->b \bar{b})_{CMSSM}', 'BR(H->b \bar{b})_{CMSSM}', 'BR(A->b \bar{b})_{CMSSM}', 'BR(h->b \bar{b})_{SM}', 'BR(H->b \bar{b})_{SM}', 'BR(A->b \bar{b})_{SM}', 'BR(h->\gamma \gamma)_{CMSSM}', 'BR(H->\gamma \gamma)_{CMSSM}', 'BR(A->\gamma \gamma)_{CMSSM}', 'BR(h->\gamma \gamma)_{SM}', 'BR(H->\gamma \gamma)_{SM}', 'BR(A->\gamma \gamma)_{SM}', 'BR(h->\tau \tau)_{CMSSM}',  'BR(H->\tau \tau)_{CMSSM}', 'BR(A->\tau \tau)_{CMSSM}', 'BR(h->\tau \tau)_{SM}', 'BR(H->\tau \tau)_{SM}', 'BR(A->\tau \tau)_{SM}', 'BR(h->W W^*)_{CMSSM}', 'BR(H->W W)_{CMSSM}',  'BR(A->W W)_{CMSSM}', 'BR(h->W W^*)_{SM}', 'BR(H->W W)_{SM}', 'BR(A->W W^*)_{SM}' /)


 Type widths
  real(8) :: tobbar(3),  tobbarSM(3), togamgam(3), togamgamSM(3) &
        , totautau(3), totautauSM(3), toWW(3), toWWSM(3)
 end Type widths


 character(LEN=lmax), dimension(24):: FHwidthNames= (/'\Gamma(h->b \bar{b})_{CMSSM}', '\Gamma(H->b \bar{b})_{CMSSM}', '\Gamma(A->b \bar{b})_{CMSSM}', '\Gamma(h->b \bar{b})_{SM}', '\Gamma(H->b \bar{b})_{SM}', '\Gamma(A->b \bar{b})_{SM}', '\Gamma(h->\gamma \gamma)_{CMSSM}', '\Gamma(H->\gamma \gamma)_{CMSSM}', '\Gamma(A->\gamma \gamma)_{CMSSM}', '\Gamma(h->\gamma \gamma)_{SM}', '\Gamma(H->\gamma \gamma)_{SM}', '\Gamma(A->\gamma \gamma)_{SM}', '\Gamma(h->\tau \tau)_{CMSSM}', '\Gamma(H->\tau \tau)_{CMSSM}', '\Gamma(A->\tau \tau)_{CMSSM}', '\Gamma(h->\tau \tau)_{SM}', '\Gamma(H->\tau \tau)_{SM}', '\Gamma(A->\tau \tau)_{SM}', '\Gamma(h->W W^*)_{CMSSM}', '\Gamma(H->W W)_{CMSSM}', '\Gamma(A->W W)_{CMSSM}', '\Gamma(h->W W^*)_{SM}', '\Gamma(H->W W)_{SM}', '\Gamma(A->W W^*)_{SM}' /)




 Type Prodxbr_Out
  Type(Wh_cha)  :: Wh !15 elements
  Type(Zh_cha)  :: Zh !15 elements
  Type(bbh_cha) :: bbh 
  Type(ggh_cha) :: ggh
  Type(tth_cha) :: tth
  Type(qqh_cha) :: qqh
  Type(bh_cha)  :: bh 
  Type(decay)   :: branch
  Type(widths)  :: gamma
 End Type Prodxbr_Out

 Type Reduced_Out
    !W + h -> h -> b + bbar (Tev)
    real(8) :: Wh_sigma_Tev, Wh_sigmaSM_Tev, Wh_bbar_Tev, Wh_bbarSM_Tev
    !Z + h -> h -> b + bbar (Tev) 
    real(8) :: Zh_sigma_Tev, Zh_sigmaSM_Tev, Zh_bbar_Tev, Zh_bbarSM_Tev
    !bbar + h -> h -> b + bbar (Tev)
    real(8) :: bbh_sigma_Tev, bbh_sigmaSM_Tev, bbh_bbar_Tev, bbh_bbarSM_Tev
    !gg -> h -> gamma + gamma (Tev+LHC)
    real(8) :: ggh_sigma_Tev, ggh_sigmaSM_Tev, ggh_sigma_LHC, ggh_sigmaSM_LHC, ggh_gamgam_LHC, ggh_gamgamSM_LHC
    !qq -> tt h -> b + bar (LHC)
    real(8) :: tth_sigma_LHC, tth_sigmaSM_LHC, tth_bbar_LHC, tth_bbarSM_LHC
    !qq -> VV h -> tau + tau (LHC)
    real(8) :: qqh_sigma_LHC, qqh_sigmaSM_LHC, qqh_tautau_LHC, qqh_tautauSM_LHC
    !bbar -> b h  (Tev+LHC)
    real(8) :: bh_sigma_Tev, bh_sigmaSM_Tev, bh_sigma_LHC, bh_sigmaSM_LHC
    !Branchings BR(h -> X Y)
    !BR(h-> b bar)_MSSM & SM, BR(h-> gam gam)_MSSM & SM
    real(8) :: branch_BRtobbar, branch_BRtobbarSM, branch_BRtogamgam, branch_BRtogamgamSM
    !BR(h-> tau tau)_MSSM & SM
    real(8) :: branch_BRtotautau, branch_BRtotautauSM
    !BR(h-> W W*)_MSSM & SM
    real(8) :: branch_BRtoWW, branch_BRtoWWSM
    !Widths
    real(8) :: gamma_tobbar, gamma_tobbarSM, gamma_togamgam, gamma_togamgamSM
    !Gamma(h-> tau tau)_MSSM & SM
    real(8) :: gamma_totautau, gamma_totautauSM
    !Gamma(h-> W W*)_MSSM & SM
    real(8) :: gamma_toWW, gamma_toWWSM     
 end Type Reduced_Out

 character(LEN=lmax), dimension(46):: FHReducedNames


!------------- SLHA structure for passing par. to FH ----

 Type SM
   real(8) ::GFermi, alpha_s, MZ, mb, mtop, mtau;
 end Type SM

 Type physical
   real(8) :: MW, mlh, mhh, mA0, mHp &
        , msdl, msul, mssl, mscl, msb1, mst1 & 
        , msdr , msur, mssr, mscr, msb2, mst2  &
        , msel, msnue, msmul, msnumu, mstau1, msnutau  & 
        , mser,  msmur, mstau2, mgluino  &
	, mneut1, mneut2, mneut3, mneut4, mch1  &
        , mch2 , alpha 
 end Type physical

 Type mixings
   real(8) ::StopMix(2,2), SbotMix(2,2), StauMix(2,2) & 
        , NMix(4,4), UMix(2,2), VMix(2,2)  
 end Type mixings

 Type couplings
   real(8) :: gp, g2, g3, ytq, ybq, ytauq
 end Type couplings

 Type softpars
   real(8) :: m3sq, M1, M2, M3, mH12, mH22  &
        , meL, mmuL, mtauL, meR, mmuR, mtauR &
        , mqL1, mqL2, mqL3  &
        , muR, mcR, mtR, mdR, msR, mbR &
        , aup, ach, at, ado, ast, ab, ael, amu, atau
 end Type softpars


 Type susypars
   real(8) :: mu, tanb, hvev
 end Type susypars

 Type renormalization
   real(8) :: Q
 end Type renormalization

 Type Leshouches_out
   Type(SM) :: SMinputs
   Type(physical) ::masses
   Type(mixings) :: mix 
   Type(couplings) :: coup
   Type(softpars) :: soft
   Type(susypars) :: susy
   Type(renormalization) :: scale
 end Type Leshouches_out


!------ Output from bdecay -------------------

 Type BDecay_Out
    !keep it general for future additions
    real (8) :: delMB
 End Type BDecay_Out

 character(LEN=lmax), dimension(1):: BDNames= (/ '\Delta M_{B_s}' /)
end module parameters
